Package: SPARTAAS
Type: Package
Title: Statistical Pattern Recognition and daTing using Archaeological
        Artefacts assemblageS
Version: 1.2.5
Date: 2026-03-25
Authors@R: c(person("Arthur","Coulon", role = c("aut","cre"),
                     email = "arthur-coulon@outlook.fr"),
             person("Lise","Bellanger", role = "aut",
                     email = "lise.bellanger@univ-nantes.fr"),
             person("Philippe","Husi", role = "aut",
                     email = "philippe.husi@univ-tours.fr"),
             person("Bruno","Desachy", role = "ctb"),
             person("Benjamin","Martineau", role = "ctb")
             )
Maintainer: Arthur Coulon <arthur-coulon@outlook.fr>
URL: https://spartaas.gitpages.huma-num.fr/r-package/
Description: Statistical pattern recognition and dating using archaeological artefacts assemblages.
    Package of statistical tools for archaeology.
    hclustcompro()/perioclust(): Bellanger Lise, Coulon Arthur, Husi Philippe (2021, ISBN:978-3-030-60103-4).
    mapclust(): Bellanger Lise, Coulon Arthur, Husi Philippe (2021) <doi:10.1016/j.jas.2021.105431>.
    seriograph(): Desachy Bruno (2004) <doi:10.3406/pica.2004.2396>.
    cerardat(): Bellanger Lise, Husi Philippe (2012) <doi:10.1016/j.jas.2011.06.031>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0)
Imports:
        FactoMineR,grDevices,dplyr,tidyr,ggplot2,plotly,stringr,colorspace,
        shiny,shinydashboard,shinyjs,shinyjqui,fpc,ggdendro,htmltools,htmlwidgets,
        shinythemes,explor,shinyWidgets,scatterD3,ks,grid,cluster,leaflet,ape,
        MASS,ade4,lmtest,nor1mix,shinycssloaders,scales,fastcluster,nloptr
Suggests: knitr, rmarkdown,sf
RoxygenNote: 7.1.1
NeedsCompilation: no
Packaged: 2026-03-26 09:01:43 UTC; arcoulon
Author: Arthur Coulon [aut, cre],
  Lise Bellanger [aut],
  Philippe Husi [aut],
  Bruno Desachy [ctb],
  Benjamin Martineau [ctb]
Repository: CRAN
Date/Publication: 2026-03-26 10:10:07 UTC
Built: R 4.6.0; ; 2026-04-25 14:09:45 UTC; unix

Package: SPARTAAS
Type: Package
Title: Statistical Pattern Recognition and daTing using Archaeological
        Artefacts assemblageS
Version: 1.2.7
Date: 2026-07-24
Authors@R: c(person("Arthur","Coulon", role = c("aut","cre"),
                     email = "arthur-coulon@outlook.fr"),
             person("Lise","Bellanger", role = "aut",
                     email = "lise.bellanger@univ-nantes.fr"),
             person("Philippe","Husi", role = "aut",
                     email = "philippe.husi@univ-tours.fr"),
             person("Bruno","Desachy", role = "ctb"),
             person("Benjamin","Martineau", role = "ctb")
             )
Maintainer: Arthur Coulon <arthur-coulon@outlook.fr>
URL: https://spartaas.gitpages.huma-num.fr/r-package/
Description: Statistical pattern recognition and dating using archaeological artefacts assemblages.
    Package of statistical tools for archaeology.
    hclustcompro()/perioclust(): Bellanger Lise, Coulon Arthur, Husi Philippe (2021, ISBN:978-3-030-60103-4).
    mapclust(): Bellanger Lise, Coulon Arthur, Husi Philippe (2021) <doi:10.1016/j.jas.2021.105431>.
    seriograph(): Desachy Bruno (2004) <doi:10.3406/pica.2004.2396>.
    cerardat(): Bellanger Lise, Husi Philippe (2012) <doi:10.1016/j.jas.2011.06.031>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0)
Imports:
        FactoMineR,grDevices,dplyr,tidyr,ggplot2,plotly,stringr,colorspace,shiny,
        shinydashboard,shinyjs,shinyjqui,fpc,ggdendro,htmltools,htmlwidgets,shinythemes,
        explor,shinyWidgets,scatterD3,ks,grid,cluster,leaflet,ape,caret,MASS,ade4,
        lmtest,nor1mix,shinycssloaders,scales,fastcluster,nloptr
Suggests: knitr,rmarkdown,sf
NeedsCompilation: no
Packaged: 2026-07-24 08:32:30 UTC; arcoulon
Author: Arthur Coulon [aut, cre],
  Lise Bellanger [aut],
  Philippe Husi [aut],
  Bruno Desachy [ctb],
  Benjamin Martineau [ctb]
Repository: CRAN
Date/Publication: 2026-07-24 10:00:21 UTC
Built: R 4.6.1; ; 2026-07-24 11:29:29 UTC; unix

export("CAdist","adjacency","arrondi","seriograph","serio_app","overlap","timerange",
       "cerardat","extract_results","cerardat_estim_nf","cerardat_app","arrangePlot",
       "hclustcompro","hclust","hclustcompro_cutree","hclustcompro_select_alpha",
       "perioclust","perioclust_subdivide","mapclust","mapclust_cut_tree",
       "mapclust_app","optim_select_alpha","hclustcompro_app")

importFrom("FactoMineR",      "CA")
importFrom("grDevices",       "colorRampPalette","recordPlot","rainbow","graphics.off","tiff","rgb")
importFrom("tidyr",           "gather")
importFrom("dplyr",           "mutate","group_by","summarise_all","funs","ungroup","inner_join","arrange",
                              "summarise","first","last","filter")
importFrom("ggplot2",         "ggplot","geom_bar","geom_col","aes","ggtitle","labs","xlab",
                              "ylab","facet_wrap","position_stack","coord_flip","facet_grid","scale_fill_manual",
                              "scale_y_continuous","theme","element_text","element_blank","scale_x_discrete",
                              "geom_segment","margin","geom_point","scale_colour_gradientn","geom_line",
                              "scale_x_continuous","geom_text","scale_y_reverse","ylim","element_line",
                              "geom_hline","geom_errorbar","theme_classic","geom_vline","scale_y_log10",
                              "annotation_logticks","annotate")
importFrom("ggdendro",        "dendro_data","ggdendrogram","segment","label")
importFrom("plotly",          "plot_ly","add_lines","add_segments","add_markers","layout","subplot")
importFrom("graphics",        "barplot","plot","abline","legend","lines","points","text","mtext",
                              "axis","boxplot","image","par","plot.new", "segments", "symbols","polygon")
importFrom("stats",           "median","quantile","cophenetic","cor","dist","as.dist","chisq.test","cutree",
                              "optimize","rect.hclust","reorder","coef","sd","lm","qnorm","na.omit",
                              "approx","density","predict","rstudent","shapiro.test","as.hclust","as.formula",
                              "formula","residuals","lm.influence")
importFrom("nor1mix",         "qnorMix","norMix","dnorMixL")
importFrom("lmtest",          "dwtest","bptest")
importFrom("MASS",            "boxcox")
importFrom("ade4",            "supcol","dudi.coa","inertia.dudi")
importFrom("utils",           "setTxtProgressBar","txtProgressBar","data","str","capture.output","head")
importFrom("stringr",         'str_sub','str_split','fixed','str_extract')
importFrom("colorspace",      "rainbow_hcl")
importFrom("shinydashboard",  "box")
importFrom("shinyjqui",       "jqui_resizable")
importFrom("fpc",             "cluster.stats")
importFrom("ks",              "kde","Hscv")
importFrom("cluster",         "silhouette")
importFrom("htmltools",       "htmlEscape")
importFrom("shinycssloaders", "withSpinner")
importFrom("fastcluster",     "hclust")
importFrom("scales",          "scientific")
importFrom("nloptr",          "nloptr")
importFrom("ape",             "Moran.I")

import("shinyjs",             except=c("runExample","alert"))

import('grid','shiny','leaflet','htmlwidgets','shinythemes',"shinyWidgets",
       "scatterD3","explor")

S3method("print",     "perioclust_cl")
S3method("print",     "selectAlpha_obj")
S3method("print",     "seriograph")
S3method("summary",   "selectAlpha_obj")
S3method("print",     "cerardat_obj")
S3method("plot",      "cerardat_obj")

library(shinythemes)
library(shinyjs)
library(leaflet)
#library(markdown)
#library(sp)          #for grid ...
library(cluster)      #for silhouette
library(ggplot2)
library(ape)          #for Moran.i
library(ks)           #for kde Hscv
library("shinyWidgets")
library(htmltools)

load(file = "./data/datarcheo.RData")
load(file = "./data/datacancer.RData")

#----------------------------------------------------------------------------------------------#
###########################                  - UI -                  ###########################
#----------------------------------------------------------------------------------------------#
ui <- fluidPage(useShinyjs(),style="padding-top: 150px;",theme = shinytheme("flatly"),
                tags$head(
                  tags$link(rel = "icon", type = "image/gif", href = "https://spartaas.gitpages.huma-num.fr/r-package/img/lambda.png"),
                  tags$title("mapclust")
                ),
                tags$head(tags$style(
                  type = "text/css",
                  ".leaflet-left {
                    bottom:30px;
                  }
                  .btn-default {
                    background-color:#5da3a8;
                    border-color:#5da3a8;
                  }
                  .dropdown-menu li a{
                    background-color:#fff;
                    border-color:#dce4ec;
                  }"
                )),
                #header
                absolutePanel(class = "panel panel-default",
                              style="z-index: 2000;padding: 8px; background: #ecf0f1; opacity: 1;border-bottom: 1px solid #2c3e50;",
                              top = 0, left = 0, right = 0,
                              fixed = TRUE,
                              titlePanel(h2("Divisive Hierarchical Clustering using Spatials Patches")),
                              div(span(strong("SPARTAAS | mapclust ")))
                ),
                #footer
                absolutePanel(style="z-index: 2000;padding: 0px; border-bottom: 0px solid #CCC; background: #fff;opacity: 1;",
                              bottom = 0, left = 0,
                              fixed = TRUE,
                              div(span(strong("SPARTAAS")),span("[Bellanger,Coulon,Husi]",style ="font-size:14px;"))
                ),
                #go top panel
                absolutePanel(class = "panel-hover",
                              style="opacity:0.8;z-index: 1000;padding: 8px; border-bottom: 1px solid #CCC; background: #FFFFEE;width:30px;",
                              bottom = 20, right = 0,
                              fixed = TRUE,
                              HTML("<a href=\"#top\"><span class=\"glyphicon glyphicon-chevron-up\"></span></a>")
                ),
                #-----------------------------------#
                #    NavBar
                #-----------------------------------#
                navbarPage("MapClust",id="navbar",
                           #-----------------------------------#
                           #    Home
                           #-----------------------------------#
                           tabPanel(value = "1",span(icon("home", lib = "glyphicon"),strong("Home")),style="max-width:1200px;",
                                    br(),
                                    sidebarLayout(
                                      #-----------------------------------#
                                      sidebarPanel(style="border-width:0px;border-color:#222222;",
                                                   h1("R Package:"),
                                                   br(),
                                                   HTML("<p>This method is part of the <a target='_blank' href='https://spartaas.gitpages.huma-num.fr/r-package/index.html'>SPARTAAS</a> package.</p>
                                                        <p>If you are interested you can install our R package available on the <a target='_blank' href='https://cran.r-project.org/package=SPARTAAS'>CRAN</a>.</p>

                                                        ")

                                      ),
                                      #-----------------------------------#
                                      mainPanel(
                                         HTML("
                                              <h3>Introduction</h3>
                                      <p>
                                              Hierarchical classification can also be considered by introducing constraints that allow, for example, geographical proximity to be taken into account.
                                              </p>
                                              <h4>Spatialpatches</h4>
                                              <p>
                                              The spatial distribution of data can be heterogeneous and present local aggregations or spatial patches. P. Petitgas proposed an algorithm to identify them in the context of fish population density (WOILLEZ et al., 2009). The parameters are the geographical coordinates (X,Y) and a variable of interest (var).
                                              </p>
                                              <p>
                                              The position of a patch is then determined by its center of gravity. The algorithm starts with the highest value of var and then considers each observation in decreasing order of value of var. The highest value initiates the first patch. Then, the observation considered is assigned to the nearest patch, provided that its distance from the center of gravity of the patch is smaller than the threshold distance dlim. Otherwise, the observation forms a new patch. The results on spatial patches are of course influenced by the choice of the dlim threshold and the location of the highest values of var.
                                              </p>
                                              <h4>MapClust</h4>
                                              <p>
                                              In order to better understand the spatial distribution of individuals, spatial patch construction is applied hierarchically top-down by varying the maximum acceptable distance (dlim) between observation points and the centre of gravity of a patch. This approach leads to a Hierarchical Top-down Classification (EVERITT et al., 2001) based at each step on the Spatialpatches algorithm: at each node (parent group) of the hierarchy we divide into two patches (child nodes). Spatialpatches has been developed to describe the spatial distribution patterns of a fish population based on density data, so the variable of interest is assumed to be positive. var can therefore be related to a positive count, frequency or real variable.
                                              </p>
                                              <p>
                                              In the case where the variable of interest var is real, means that it can take negative values, it was necessary to adapt our MapClust classification algorithm. We no longer work with the values of var but with the values of the probability density f associated with var. In the same way we were able to adapt the method to the case where var is multidimensional by working on a Kernel density estimate.
                                              </p>
                                              ")
                                      )
                                    )
                           ),
                           #-----------------------------------#
                           #    MapClust
                           #-----------------------------------#
                           tabPanel(value = "2",span(icon("map"),strong("MapClust")),style="max-width:1200px;",
                                    h1("MapClust"),
                                    sidebarLayout(
                                      sidebarPanel(width = 5,style="border-width:0px;border-color:#222222;",
                                                   wellPanel(width = 5, style="border-width:0px;",
                                                             selectInput("select", label = h4("Dataset:"),
                                                                         choices = list("datarcheo" = 1, "datacancer" = 2, "your data" = 3),
                                                                         selected = 1),
                                                             checkboxInput("Plabel","Print the list of names on the dendrogram.", value = FALSE),
                                                             checkboxInput("extenddend","Merge the leaves on the same level of the dendogram (activate before running the method: play button).", value = FALSE),
                                                             hr(style="border-color: #222222;"),
                                                             fluidRow(
                                                               column(width = 3,
                                                                      h4("Run:"),
                                                                      actionButton("Run",icon("play", lib = "glyphicon"))
                                                               ),
                                                               column(width = 6,
                                                                      h4("Import data:"),
                                                                      actionButton("UL",icon("import", lib = "glyphicon"))
                                                               ),
                                                               column(width = 1,
                                                                      h4("Export:"),
                                                                      dropdownButton(
                                                                        downloadButton("all.pdf","Download plots as pdf"),
                                                                        hr(style="border-color: #222222;"),
                                                                        #downloadButton("map.png", "Download map as png"),
                                                                        icon = icon("floppy-disk", lib = "glyphicon"), width = "220px",
                                                                        size = "sm",
                                                                        tooltip = tooltipOptions(title = "Download")
                                                                      )
                                                               )
                                                             )
                                                   ),hr(style="border-color: #222222;border-width:2px;"),
                                                   wellPanel(width = 5, style="border-width:0px;background:#dddddd;",
                                                             h4("Dendrogram"),
                                                             plotOutput("distPlot",click = "plot_click")
                                                   ),hr(style="border-color: #222222;border-width:2px;"),
                                                   wellPanel(width = 5, style="border-width:0px;background:#dddddd;",
                                                             h4("Partition evaluation"),
                                                             plotOutput("avgsilPlot"),
                                                             hr(style="border-color: #777777;"),
                                                             plotOutput("WSSPlot")
                                                   )
                                      ),
                                      mainPanel(width = 7,
                                                h4("Console"),
                                                hr(style="border-color: #222222;"),
                                                verbatimTextOutput("click_info"),
                                                h4("Map"),
                                                hr(style="border-color: #222222;"),
                                                leafletOutput("mymap",height = 500),br(),
                                                h4("Silhouette"),
                                                hr(style="border-color: #222222;"),
                                                plotOutput("silPlot",height = "auto"),
                                                h4("Summary"),
                                                hr(style="border-color: #222222;"),
                                                verbatimTextOutput("click_info2")
                                      )
                                    )
                           ),
                           #-----------------------------------#
                           #    Contact
                           #-----------------------------------#
                           tabPanel(value = "5",span(icon("envelope", lib = "glyphicon"),strong("Contact")),style="max-width:1200px;",
                                    HTML("<style>
                                       .carte{
                                       border-left-width: 4px;
                                       border-left-style:solid;
                                       border-color: #2ac0a2;
                                       padding-left: 10px;
                                       }
                                       </style>
                                       <h3>Authors:</h3>
                                       <hr>

                                       <div class=\"carte\"><h4>L. Bellanger</h4><h5>mail: &lt;lise.bellanger@univ-nantes.fr&gt;</h5></div>
                                       <div class=\"carte\"><h4>P. Husi</h4><h5>mail: &lt;philippe.husi@univ-tours.fr&gt;</h5></div><div class=\"carte\"><h4>A. Coulon</h4></div>

                                       <h3>Maintainer:</h3>
                                       <hr>
                                       <div class=\"carte\"><h4>A. Coulon</h4><h5>mail: &lt;arthur-coulon@outlook.fr&gt;</h5></div>
                                       ")
                                                ),
                           #-----------------------------------#
                           #    Wiki
                           #-----------------------------------#
                           tabPanel(value = "6",span(icon("question-sign", lib = "glyphicon"),strong("Wiki")),style="max-width:1200px;",
                                    HTML(
                                      "<style>body {text-align: justify}</style><h1>Get started with the MapClust application</h1>
                                      <h2>Table of content</h2>
                                      <div id=\"TOC\">
                                      <ul>
                                      <li><a href=\"#introduction\">Introduction</a><ul>
                                      <li><a href=\"#spatialpatches\">Spatialpatches</a></li>
                                      <li><a href=\"#mapclust\">MapClust</a></li>
                                      </ul></li>
                                      <li><a href=\"#data\">The App</a><ul>
                                      <li><a href=\"#first\">Run (first time)</a></li>
                                      <li><a href=\"#plabel\">Print label option</a></li>
                                      <li><a href=\"#import\">Import your data</a><ul>
                                      <li><a href=\"#csv\">CSV Format and write.table</a></li>
                                      <li><a href=\"#label\">Label</a></li>
                                      <li><a href=\"#header\">Header</a></li>
                                      <li><a href=\"#separator\">Separator</a></li>
                                      <li><a href=\"#quote\">Quote</a></li>
                                      <li><a href=\"#dec\">Decimal</a></li>
                                      <li><a href=\"#uni\">Univariate data</a></li>
                                      <li><a href=\"#multi\">Multivariate data</a></li>
                                      </ul></li>
                                      </ul></li>
                                      <li><a href=\"#eval\">Evaluation Plot</a><ul>
                                      <li><a href=\"#wssplot\">Within Sum of Square Plot</a></li>
                                      <li><a href=\"#avesilplot\">Average Silhouette Plot</a></li>
                                      </ul></li>
                                      </li>
                                      <li><a href=\"#output\">Output: select a partition</a><ul>
                                      <li><a href=\"#dendrogram\">Dendrogram</a></li>
                                      <li><a href=\"#map\">Map</a></li>
                                      <li><a href=\"#silhouette\">Silhouette</a></li>
                                      <li><a href=\"#summary\">Additionnal informations</a></li>
                                      </ul></li>
                                      <li><a href=\"#references\">References</a></li>
                                      </ul>
                                      </div>

<hr>
<h2 id=\"introduction\">Introduction</h2>
<hr>
<p>Hierarchical classification can also be considered by introducing constraints that allow, for example, geographical proximity to be taken into account.</p>
<h3 id=\"spatialpatches\">Spatialpatches</h3>
<p>The spatial distribution of data can be heterogeneous and present local aggregations or spatial patches. P. Petitgas proposed an algorithm to identify them in the context of fish population density (WOILLEZ et al., 2009). The parameters are the geographical coordinates (X,Y) and a variable of interest (var).</p>
<p>The position of a patch is then determined by its center of gravity. The algorithm starts with the highest value of var and then considers each observation in decreasing order of value of var. The highest value initiates the first patch. Then, the observation considered is assigned to the nearest patch, provided that its distance from the center of gravity of the patch is smaller than the threshold distance dlim. Otherwise, the observation forms a new patch. The results on spatial patches are of course influenced by the choice of the dlim threshold and the location of the highest values of var.</p>
<h3 id=\"mapclust\">MapClust</h3>
<p>In order to better understand the spatial distribution of individuals, spatial patch construction is applied hierarchically top-down by varying the maximum acceptable distance (dlim) between observation points and the centre of gravity of a patch. This approach leads to a Hierarchical Top-down Classification (EVERITT et al., 2001) based at each step on the Spatialpatches algorithm: at each node (parent group) of the hierarchy we divide into two patches (child nodes). Spatialpatches has been developed to describe the spatial distribution patterns of a fish population based on density data, so the variable of interest is assumed to be positive. var can therefore be related to a positive count, frequency or real variable.</p>
<p>In the case where the variable of interest var is real, means that it can take negative values, it was necessary to adapt our MapClust classification algorithm. We no longer work with the values of var but with the values of the probability density f associated with var. In the same way we were able to adapt the method to the case where var is multidimensional by working on a Kernel density estimate.</p>


<hr>
<h2 id=\"data\">The App</h2>
<hr>
<h3 id=\"first\">Run (first time)</h3>
<p>The function therefore requires as data geographical coordinates, latitude and longitude (<code>coord</code>) and the variable(s) of interest (<code>var</code>) in the same table.</p>
<p>For your first use, you can use our dataset. The first one is call datarcheo. Just select it in the MapClust tab (it will be selected by default) and run. You can also try the other dataset datacancer.</p>
<h3 id=\"plabel\">Print label option</h3>
<p>Located just below the dataset selection, it is used to define whether or not to display the labels on the dendrogram.</p>
<p><img src=\"GS/Plabel.png\">Select dataset and print label option</p>
<h3 id=\"import\">Import your data</h3>
<p>You can import your data. As they may contain only one or more variables of interest, you will need to adjust the parameters.</p>
<p>Let's have a look at the interface. There are three main parts. The import area with only one import button. The second part is more complex and has more options. This is the parameter section, where you need to configure the import tool according to your data. Finally, there is the preview area where you can see how your data should look and how your data is. There is still something left, the concordance indicator, which allows you to quickly check if your data matches what it should look like.</p>
<p><img src=\"GS/import.png\"></p>
<h4 id=\"csv\">CSV Format and write.table</h4>
<p><img src=\"GS/csv.png\"><br>A csv file.</p>
<p>It is a data.frame of 3 columns with headers separated by semicolons \";\".</p>
<p>The input format for importing data is the .csv format, but also supports the .txt format as a .csv file.</p>
<p>In R, you can export your data frame to a csv file using write.csv2 or write.table. In a csv you can choose a character to separate the columns. In the same way, you can define the character to indicate the decimal point.</p>
<code>
write.table(data,file=\"path/to/name_file.csv\",sep=\";\",dec=\".\",row.names=FALSE,quote=FALSE)
</code>
<p>In Excel you can save in csv format in order to import your data frame.</p>
<p>The import interface allows you to set these values using the 'header', 'decimal', 'separator' and 'quote' options.</p>
<p>To calculate exactly as you want, we need to know a few things. The first is the presence or absence of labels in your data. The label must be in the last column. We also need to know how many variables of interest you have in addition to the two coordinate variables.</p>
<p>If the setup doesn't match your real data frame, you can't import anything. You will have to change the data or parameters. If it was good you will see a green tick, if not you will see a red cross.</p>

<h4 id=\"label\">Label</h4>
<p>Yes or no option. Do you have labels in your data ? If you have labels, put them in the last colunm.</p>
<h4 id=\"header\">Header</h4>
<p>Yes or no option. Do you have headers on your colunms?</p>
<h4 id=\"separator\">Separator</h4>
<p>Select the character you want to use to separate the colunms.</p>
<h4 id=\"quote\">Quote</h4>
<p>Select the quotation marks to use on strings.</p>
<h4 id=\"dec\">Decimal</h4>
<p>Select the character to use to indicate the decimal point.</p>
<h4 id=\"uni\">Univariate data</h4>
<p>Move the slider to 'Univariate' and import your data. If you have labels, check the option, if not, uncheck Label.</p>
<p>Configure all the settings for the csv format. Which symbol to use to separate the columns, which decimal symbol to use, which quotes to use for the string, and whether to include a header in your columns.</p>
<p><img src=\"GS/importUni.png\"></p>
<h4 id=\"multi\">Multivariate data</h4>
<p>The only difference is that you have to set the slider to 'Multivariate'. When you do this, you have access to another slider. You must specify the number of variables of interest. Warning: You must not include the two coordinate variables.</p>
<p><img src=\"GS/importMulti.png\"></p>


<hr>
<h2 id=\"eval\">Evaluation Plot</h2>
<hr>
<p>It is essential to be able to evaluate the different partitions of the hierarchy in order to identify the most relevant one(s). In our case, the number k of classes in the partition to be retained is based on several indicators calculated for different values of K: the total within-class sum of squares (WSS) and the global average of silhouette widths.</p>
<p>When running the mapclust method, you have to make a choice. You must cut the dendrogram. This operation selects the partition. To compare all the possibilities, you can see the evaluation plots (WSSPlot and AveSilPlot). These two plots evaluate the relative quality of the partition.</p>
<h3 id=\"wssplot\">Within Sum of Square Plot (WSSPlot)</h3>
<p>This is the plot of the within group sum of squares against the number of clusters. The within group sum of squares decreases as the number of clusters increases. In this plot, the best partition is when adding one or more clusters doesn't decrease the WSS value. It's called the elbow method.</p>
<h4>Example:</h4>
<p><img style=\"width:600px;\" src=\"GS/WSS2.png\"></p>
<p>On this graph, we start by looking at the value for the lowest number of groups: 2. If I add a third group, we see that the WSS value will decrease (from 0.09 to 0.03). If I add another group, this value decreases again (from 0.03 to 0.01). After that, adding a group has no or only a small effect on the value. Adding a group is therefore not interesting, we will keep a partition with 4 groups.</p>
<h3 id=\"avesilplot\">Average silhouette Plot</h3>
<p>This graph shows the average silhouette width of each partition (ROUSSEEUW 1987). The silhouette width is a bounded index between -1 and 1, calculated for each observation. The closer the value is to 1, the better the observation is classified. We look for the average value for a partition that is closest to 1.</p>
<h4>Example:</h4>
<p><img style=\"width:600px;\" src=\"GS/AveSil2.png\"></p>
<p>On this graph we look for the maximum value. The best score corresponds to the division into 7 groups. Looking at the second best partition, we identify the one with 4 groups. Although the one with 7 groups is higher for this silhouette index, we will choose the partition in 4. We make this choice because the WSSPlot advised us the partition in 4.</p>


<hr>
<h2 id=\"output\">Output</h2>
<hr>
<h4>A partition is selected by clicking on the dendrogram at the desired height.</h4>
<p>You can change it at any tmie.</p>
<div id=\"dendrogram\" style=\"display:inline-block\">
  <img src=\"GS/dendro.png\">
  <img src=\"GS/dedroCut.png\">
</div><br>
<h3 id=\"map\">Map</h3>
<p><img src=\"GS/map.png\"></p><br>
<h3 id=\"silhouette\">Silhouette</h3>
<p><img src=\"GS/silhouette.png\"></p>
<p>This plot shows the silhouette index for each observation of the selected partition. The observations are sorted in descending order and by cluster.</p><br>
<h3 id=\"summary\">Additionnal informations</h3>
<p>Above the map you can see a small description of the cluster. You can find the average value of each variable.</p>
<p><img src=\"GS/Consol.png\"></p>
<p>Below the silhouette plot is a summary of the different values for each partition. You will find the dlim, the number of clusters, the WSS value, the average sil_widht and, if you are in the univariate case, the Moran index (MORAN, 1950) for each partition. The Moran index is a measure of spatial autocorrelation. It is calculated for each cluster of the partition and then the average is calculated to characterise the partition.</p>
<p><img src=\"GS/summary.png\"></p>


<hr>
<h2 id=\"references\">References</h2>
<hr>
<p>EVERITT, B.S., S. LANDAU et M. LEESE(2001). “Cluster Analysis”. In : London UK: Arnold.</p>
<p>MORAN, P. A. P. (1950). \"Notes on Continuous Stochastic Phenomena\". Biometrika. 37 (1): 17–23. <a href=\"https://doi.org/10.2307/2332142\" class=\"uri\">doi.org/10.2307/2332142</a>. JSTOR 2332142.</p>
<p>ROUSSEEUW, P.J. (1987). “Silhouettes: a graphical aid to the interpretation and validation of cluster analysis”. In : J. Comput. Appl. Math. 20, 53–65. <a href=\"DOI:https://doi.org/10.1016/0377-0427(87)90125-7\" class=\"uri\">DOI:https://doi.org/10.1016/0377-0427(87)90125-7</a>.</p>
<p>WOILLEZ, M., J.RIVOIRARD et P.PETITGAS(2009). “Notes on survey-based spatial indicators for monitoring fish populations”. In : Aquat. Living Resour.22, p. 155–164.</p>

                                      "
                                    )
                                    )
                           )
                           )

validation <- function(D1,D2,valid){
  #dist ?
  if(dim(D2)[1] == dim(D2)[2]){
    D2 = as.matrix(D2)
    if(sum(diag(D2)) == 0 && isSymmetric(D2)){
      D2 = as.dist(D2)
    }
  }

  if(class(D2)[1] != "dist"){
    #matrix ?
    if(!is.matrix(D2)){
      if(!is.data.frame(D2)){return("stratigraphic connection data must be a data.frame.")}
      #number of col  ?
      if(valid == 1){
        if(length(D2[1,])!=2){
          return(
            "Stratigraphic connection must have 2 colunms. Change data type parameter or try to inverse contingency table and stratigraphic connection."
          )}
      }else{
        if(length(D2[1,])!=3){
          return("Time range data must have 3 colunms. Change data type parameter or try to inverse contingency table and time range data.")}
      }
      if(valid == 1){
        D2 <- as.matrix(adjacency(D2))
      }else{
        if(!is.numeric(D2[,2]) || !is.numeric(D2[,3])){return("The upper and lower limit must be numeric.")}
        D2 <- overlap(D2)
      }
    }
    #diag = 0 ?
    if(sum(diag(D2)) != 0){
      return("D2: The diagonal must be equal to 0.")
    }
    #symetric ?
    if(!isSymmetric(D2)){
      return("D2 is not symmetric.")
    }
  }

  if(class(D1)[1] != "dist") {
    #matrix ?
    if(min(D1) < 0){
      return("First source is a contingency table and can not contain negative value.")
    }
    if(!is.matrix(D1)){

      D1 <- as.matrix(CAdist(D1,nCP="max",graph=FALSE))
    }
    #diag = 0 ?
    if(sum(diag(D1)) != 0){
      stop("D1: The diagonal must be equal to 0.")
    }
    #symetric ?
    if(!isSymmetric(D1)){
      stop("D1 is not symmetric.")
    }
  }

  D1 <- as.matrix(D1)
  D2 <- as.matrix(D2)

  if(!dim(D1)[1] == dim(D1)[2]){return("D1 not a square matrix.")}
  if(!dim(D2)[1] == dim(D2)[2]){return("D2 not a square matrix.")}
  if(!length(D1) == length(D2)){return("Contingency table and the second source have not the same size.")}
  #dist (positive matrix) ?
  if(min(D1) < 0){return("D1 is not positive.")}
  if(min(D2) < 0){return("D2 is not positive.")}

  return("")
}

generateClone <- function(D1,D2,i,j){
  D1bis <- rbind(D1,clone=D1[i,])
  D1bis <- cbind(D1bis,clone=c(D1[i,],0))

  D2bis <- rbind(D2,clone=D2[j,])
  D2bis <- cbind(D2bis,clone=c(D2[j,],0))

  return(list(D1bis,D2bis))
}

temp_cov <- function (df) {
  indice <- function(inf1,sup1,inf2,sup2){
    if(sup1 < inf1 || inf2 > sup2){
      return(99)
    }else{
      inf1 <- as.numeric(inf1)
      sup1 <- as.numeric(sup1)
      inf2 <- as.numeric(inf2)
      sup2 <- as.numeric(sup2)
      total_overlap_inf <- min(inf1,inf2)
      total_overlap_sup <- max(sup1,sup2)
      total_overlap <- total_overlap_sup - total_overlap_inf
      within_overlap_inf <- max(inf1,inf2)
      within_overlap_sup <- min(sup1,sup2)
      within_overlap <- within_overlap_sup - within_overlap_inf
      i <- within_overlap / total_overlap
      return(i)
    }
  }
  recouvrement_indice_matrix <- function(df){
    df <- as.data.frame(df)
    overlap_matrix <- matrix(nrow = length(df[,1]), ncol = length(df[,1]))
    for(i in 1:length(df[,1])){
      for(j in 1:length(df[,1])){
        if(i != j){
          ens1 <- df[i,]
          ens2 <- df[j,]
          ind <- indice(ens1[1,2],ens1[1,3],ens2[1,2],ens2[1,3])
          overlap_matrix[i,j] <- overlap_matrix[j,i] <- ind
        }
      }
    }
    diag(overlap_matrix) <- 1
    return(overlap_matrix)
  }

  O <- recouvrement_indice_matrix(df[,1:3])

  D <- O + 1
  D <- 2 - D
  D <- D / 2

  dimnames(D) <- list(df[,1],df[,1])

  return(D)
}

let <- function(alphabet) function(i) {
  base10toA <- function(n, A) {
    stopifnot(n >= 0L)
    N <- length(A)
    j <- n %/% N
    if (j == 0L) A[n + 1L] else paste0(Recall(j - 1L, A), A[n %% N + 1L])
  }
  vapply(i-1L, base10toA, character(1L), alphabet)
}

Int2Alpha <- function(int){
  first <- trunc(int)
  l1 <- let(LETTERS)(first)
  second <- arrondi((int - trunc(int))*10,2)
  l2 <- c()
  for(i in 1:length(second)){
    if(second[i] == 0){
      l2 <- c(l2,"")
    }else{
      f <- nchar(second[i])
      if(f == 1){
        l2 <- c(l2,as.character(second[i]))
      }
      if(f >= 3){
        new <- unlist(strsplit(as.character(second[i]),".",fixed=T))
        new <- paste0(new[1],new[2])
        l2 <- c(l2,new)
      }
    }
  }
  return(paste0(l1,l2))
}

make.chars <- function(length.out, case, n.char = NULL) {
  if(is.null(n.char))
    n.char <- ceiling(log(length.out, 26))
  m <- sapply(n.char:1, function(x) {
    rep(rep(1:26, each = 26^(x-1)) , length.out = length.out)
  })
  m.char <- switch(case,
                   'lower' = letters[m],
                   'upper' = LETTERS[m]
  )
  m.char <- LETTERS[m]
  dim(m.char) <- dim(m)

  apply(m.char, 1, function(x) paste(x, collapse = ""))
}


get.letters <- function(length.out, case = 'upper'){
  max.char <- ceiling(log(length.out, 26))
  grp <- rep(1:max.char, 26^(1:max.char))[1:length.out]
  unlist(lapply(unique(grp), function(n) make.chars(length(grp[grp == n]), case = case, n.char = n)))
}

Alpha2Int <- function(alpha){
  letter <- get.letters(800)
  first <- c()
  second <- c()
  for(i in 1:length(alpha)){
    first <- c(first,str_extract(alpha[i], "[aA-zZ]+"))
    tmp <- str_extract(alpha[i], "[0-9]+")
    if(is.na(tmp)){tmp <- ""}
    second <- c(second,tmp)
  }
  n <- nchar(max(second))
  second <- as.numeric(paste0("0.",second))

  second[is.na(second)] <- 0
  for(i in 1:length(alpha)){
    first[i] <- which(letter == first[i])
  }
  first <- as.numeric(first)
  return(first+second)
}

CAdist <- function(df, nCP = NULL, graph = TRUE){
  #===============================================================================
  # GENERATE DISTANCE MATRIX WITH CORRESPONDENCE ANALYSiS (CA) (Fr: Analyse Factorielle des Correspondances)
  #
  # Authors : A.COULON
  # Last update : 23 october 2018
  #
  # Arguments:
  # df	      The pottery contingence table
  # nCP       The number of principal component to keep
  # graph     Logical for print or not the plot of the CA
  #
  #===============================================================================

  if(!is.table(df)){
    if(is.matrix(df)){df <- as.data.frame(df)}
    if(!is.data.frame(df)){stop("df is not a data frame.")}
  }

  if(FALSE %in% (df == trunc(df))){stop("The data frame must contain integers.")}
  max <- min( nrow(df)-1,ncol(df)-1)

  if(nCP == "max"){nCP <- max}
  if(nCP > max){nCP <- max}

  df.CA <- CA(df, ncp = nCP, graph=graph)
  dist <- dist(df.CA$row$coord)
  return(as.matrix(dist))
}


EPM <- function(x) {
  x <- as.matrix(x)

  x[is.na(x)] <- 0
  ni. <- rep(1,nrow(x))
  nj. <- margin.table(x, margin = 2)
  n. <- sum(nj.)

  TabInde <- (nj.%*%t(ni.)/n.)
  TabInde <- t(TabInde)
  x <- prop.table(x, margin = 1)
  TabEcart <- x - TabInde
  TabEcart <- as.data.frame(TabEcart)
  return(TabEcart)
}

plot_EPPM <- function(x,show,permute,col_weight) {

  if(is.table(x)){
    x <- as.data.frame(x)
    x <- matrix(x[,3],
                ncol=sqrt(length(x[,2])),
                dimnames=list(unique(x[,1]),unique(x[,1])))
    x <- as.data.frame(x)
  }
  if(is.matrix(x)){x <- as.data.frame(x)}
  if(!is.data.frame(x)){stop("x is not a data frame.")}
  for (i in 1:length(x[1,])){
    if(!(sum(x[,i]) == round(sum(x[,i])))){stop("The data frame must contains integers.")}
  }
  .data <- c()
  Cluster <- c()
  labels <- factor(rownames(x),levels = rev(unique(rownames(x))))
  #permute type in x
  if(permute){

    if(show == "EPPM"){
      ecart <- as.data.frame(EPM(x))
      ecart[ecart < 0] <- 0
      i <- seq(nrow(ecart),1)
      barycenter <- colSums(ecart * i) / colSums(ecart)
      x <- x[,order(barycenter)]
    }else{
      i <- seq(nrow(x),1)
      barycenter <- colSums(x * i) / colSums(x)
      x <- x[,order(barycenter)]
    }

  }
  rowsum <- rowSums(x)
  #add Weight to x
  x <- cbind(x, Weight = rowSums(x) / sum(x) * rowSums(x))
  #data
  data <- as.data.frame(x)
  data <- mutate(data,ens = labels,rowsum = rowsum)
  data <- gather(data,key = "type", value = "frequency",-ens,-rowsum, factor_key = TRUE)
  data <- mutate(data,frequency = frequency / rowsum)
  #remove col rowsum
  data <- data[-2]
  data$frequency[is.na(data$frequency)] <- 0

  #EPPM
  ecart <- as.data.frame(EPM(x[-length(x[1,])]))
  save_ecart <- ecart
  ecart <- cbind(ecart,Weight=c(0))
  ecart[ecart < 0] <- 0
  ecart <- mutate(ecart,ens = labels)
  ecart <- gather(ecart,key = "type", value = "EPPM", -ens, factor_key = TRUE)
  ecart$EPPM[ecart$type == "Weight"] <- 0

  # Join data and EPPM
  data <- inner_join(data,ecart,by = c("ens", "type"))
  data <- mutate(data,frequency = frequency - EPPM)
  data <- gather(data,key = "legend", value = "frequency", -ens,
                 -type, factor_key = TRUE)
  #if insert hiatus in df replace NA by 0 (NA are generating by divising by 0 (rowsum of a hiatus is 0))
  data[is.na(data)] <- 0
  #center (div /2 et copy for one part - and the other +)
  data <- rbind(data,data)
  data <- mutate(data,frequency = frequency * c(rep(.5, nrow(data)/2), rep(-.5, nrow(data)/2)))

  #color for Weight
  tmp <- data$frequency[data$type == "Weight"]
  levels(data$legend) <- c(levels(data$legend),"1","2","3","4","5","6","7","8","9","10","Weight")
  data$legend[data$type == "Weight"] <- "Weight"

  breaks <- c("frequency","EPPM","Weight")

  #define the color
  freq <- c("frequency","frequencies","freq","fq","fr\u00E9quence","fr\u00E9quences","fr\u00E9q")
  EPPM <- c("EPPM","ecart","\u00E9cart","ecarts","\u00E9carts")
  if(show %in% freq){
    color <- c("frequency"="#bebebe","EPPM"="#bebebe","Weight"="#7f7f7f")
    #remove EPPM from legend
    breaks <- c("frequency","Weight")
    default_col <- "#bebebe"
  }else{
    if(show %in% EPPM){
      color <- c("frequency"=NA,"EPPM"="black","Weight"="#7f7f7f")
      #remove frequency from legend
      breaks <- c("EPPM","Weight")
      default_col <- NA
    }else{
      #default color grey: frequency, black: EPPM, my_palette: weight
      color <- c("frequency"="#bebebe","EPPM"="black","Weight"="#7f7f7f")
      default_col <- NA
    }
  }
  #ggplot
  p <- ggplot(data = data) +
    facet_grid( ~type, scales = "free", space = "free_x") +
    geom_col(aes(x = ens, y = frequency, fill = legend), width = 1) +
    scale_x_discrete(labels = rev(rownames(x))) +
    coord_flip() +
    scale_fill_manual(values = color,
                      aesthetics = "fill",
                      limits = breaks,
                      na.value = default_col
    ) +
    scale_y_continuous(breaks = c(-.1,.1),labels = c("","20% ")) +
    labs(
      title = "Seriograph",
      subtitle = "EPPM (Ecart Positif au Pourcentage Moyen) Positive Deviation at Average Percentage",
      caption = "Warnings: The frequencies are calculated independently for each element (row).
You can see the relative number of observations in the Weight column"
    ) +
    theme_grey(base_size = 15) +
    annotate("segment",x = 0, y = -.04, xend = 0, yend = .04, color = "white") +
    theme(legend.position = "bottom",
          panel.spacing = unit(.2, "lines"),
          strip.text.x = element_text(angle = 90),
          axis.text.x = element_text(hjust=1,margin = margin(t = -13)),
          axis.ticks.length = unit(.5, "cm"),
          axis.title.x = element_blank(),
          axis.title.y = element_blank(),
          axis.ticks.y = element_blank(),
          plot.caption = element_text(face = "italic", hjust = 0)
    )
  plot(p)
}

CA_var_data <- function(res, xax = 1, yax = 2, var_sup = TRUE, var_hide = "None",
                        var_lab_min_contrib = 0) {
  tmp_x <- res$vars %>%
    filter(Axis == xax) %>%
    select("Level", "Position", "Type", "Class", "Coord", "Contrib", "Cos2", "Count")
  tmp_y <- res$vars %>%
    filter(Axis == yax) %>%
    select("Level", "Position", "Type", "Class", "Coord", "Contrib", "Cos2", "Count")
  if (!var_sup) {
    tmp_x <- tmp_x %>% filter(Type == 'Active')
    tmp_y <- tmp_y %>% filter(Type == 'Active')
  }
  if (var_hide != "None") {
    tmp_x <- tmp_x %>% filter(Position != var_hide)
    tmp_y <- tmp_y %>% filter(Position != var_hide)
  }
  tmp <- tmp_x %>%
    left_join(tmp_y, by = c("Level", "Position", "Type", "Class", "Count")) %>%
    mutate(Contrib = Contrib.x + Contrib.y,
           Cos2 = Cos2.x + Cos2.y,
           tooltip = paste(paste0("<strong>", Level, "</strong><br />"),
                           paste0("<strong>",
                                  gettext("Position", domain = "R-explor"),
                                  ":</strong> ", Position, "<br />"),
                           paste0("<strong>Axis ",xax," :</strong> ", Coord.x, "<br />"),
                           paste0("<strong>Axis ", yax," :</strong> ", Coord.y, "<br />"),
                           ifelse(is.na(Cos2), "",
                                  paste0("<strong>",
                                         gettext("Squared cosinus", domain = "R-explor"),
                                         ":</strong> ", Cos2, "<br />")),
                           ifelse(is.na(Contrib), "",
                                  paste0("<strong>",
                                         gettext("Contribution:", domain = "R-explor"),
                                         "</strong> ", Contrib, "<br />")),
                           ifelse(is.na(Count), "",
                                  paste0("<strong>",
                                         gettext("Count:", domain = "R-explor"),
                                         "</strong> ", Count))),
           Lab = ifelse(Contrib >= as.numeric(var_lab_min_contrib) |
                          (is.na(Contrib) & as.numeric(var_lab_min_contrib) == 0), Level, ""))
  data.frame(tmp)
}

elbow_finder <- function(x_values, y_values) {
  # Max values to create line
  max_x_x <- max(x_values)
  max_x_y <- y_values[which.max(x_values)]
  max_y_y <- max(y_values)
  max_y_x <- x_values[which.max(y_values)]
  max_df <- data.frame(x = c(max_y_x, max_x_x), y = c(max_y_y, max_x_y))

  # Creating straight line between the max values
  fit <- lm(max_df$y ~ max_df$x)

  # Distance from point to line
  distances <- c()
  for(i in 1:length(x_values)) {
    distances <- c(distances,
                   abs( coef(fit)[2] * x_values[i] - y_values[i] + coef(fit)[1] ) / sqrt( coef(fit)[2]^2 + 1^2))
  }
  return(distances)
}

corCriterion <- function(tree,D1,D2) {
  d2 <- cophenetic(tree)
  res <- abs(cor(as.dist(D1),d2) - cor(as.dist(D2),d2))
  return(res)
}

server <- function(input, output, session) {
  #------------------------------------------------------#
  #  modalDialog wait message
  #------------------------------------------------------#
  waitModal <- function() {
    modalDialog(title="Initialisation", size = c("s"),
                div(style="height:25px;"),
                h4("Wait . . . ", style="margin-left:100px;"),
                hr(style="border-color: #222222;"),
                footer = NULL
    )
  }
  #------------------------------------------------------#
  #  modalDialog sort periods
  #------------------------------------------------------#
  sortPeriod <- function(failed = FALSE) {
    modalDialog(title="Sort clusters:",size=c("m"),
                div(style="height:15px;"),
                h3("Sort rows",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("Sort each row from the oldest to the newest!"),
                helpText("Note: Drag and drop clusters to sort them.
                         Drag unused row into the trash to remove them from the seriograph.
                         You can insert 'Hiatus' by picking it up and dropping it off where you want."),
                orderInput('seq2',span(icon("sort-amount-up",lib = "font-awesome"),'Sort the cluster'),
                           items = Int2Alpha(sort(unique(values[["cluster"]]))),
                           placeholder = 'There must be at least one item', connect = "rm",
                           item_class = 'success'),
                hr(style="border-color: #b2b2b2;"),
                column(4,
                       orderInput('hiatus', span(icon("plus",lib = "font-awesome"),'Add'), items = c("Hiatus"),
                                  connect = 'seq2', as_source = TRUE, item_class = 'warning', width = '70px')
                ),
                column(8,
                       orderInput('rm', span(icon("trash",lib = "font-awesome"),'Remove from seriograph'),
                                  items = NULL,connect = "seq2", placeholder = 'Drag the items here to delete them',
                                  item_class = 'danger')
                ),
                div(style="height:50px;"),
                if (failed)
                  div(tags$b("You have to select the sequence!", style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("tri", "OK")
                )
    )
  }
  #------------------------------------------------------#
  #  modalDialog subdivise
  #------------------------------------------------------#
  subModal1 <- function(failed = FALSE) {
    modalDialog(title="divide a cluster:",size=c("s"),
                div(style="height:15px;"),
                h3("Subdivide a cluster",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("Select one cluster to divide!"),
                selectInput("where","Cluster to divide:",
                            choices = as.list(sort(Int2Alpha((values[["div"]])))),
                            selected = 1,
                            multiple = FALSE),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("sub1", "Next")
                )
    )
  }
  #------------------------------------------------------#
  #  modalDialog subdivise2
  #------------------------------------------------------#
  subModal2 <- function(failed = FALSE) {#
    modalDialog(title="divide a cluster:",size=c("s"),
                div(style="height:15px;"),
                h3("Subdivide a cluster",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("How many sub-class do you want ?"),
                sliderInput("sk",label = "Number:" ,min = 2,
                            max = length(values[["cluster_ori"]][values[["cluster_ori"]] == Alpha2Int(input$where)])-1,
                            step = 1, value = 2),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("back", "Back"),
                  actionButton("sub2", "Finish")
                )
    )
  }
  #------------------------------------------------------#
  #  modalDialog import
  #------------------------------------------------------#
  import1Modal <- function(failed = FALSE) {
    modalDialog(title="Step 1:",size=c("l"),
                div(style="height:15px;"),
                h3("Step 1",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("Upload file contingency table"),
                # Input: Select a file ----
                fileInput("fileD1", span(icon("import", lib = "glyphicon"),"Choose CSV File"),
                          multiple = FALSE,
                          accept = c("text/csv",
                                     "text/comma-separated-values,text/plain",
                                     ".csv")
                ),
                actionButton(inputId = "toggle", span(icon("cogs",lib = "font-awesome"),"Settings")),
                fluidRow(shinydashboard::box(id="myBox",
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Checkbox if file has header ----
                                                      span(strong("Header")),
                                                      checkboxInput("headerD1", "Header (colnames)", TRUE),
                                                      checkboxInput("rownamesD1", "rownames", TRUE)
                                               ),
                                               column(width = 6,
                                                      # Input: Select quotes ----input$dec
                                                      radioButtons("quoteD1", "Quote",
                                                                   choices = c(None = "",
                                                                               "Double Quote" = '"',
                                                                               "Single Quote" = "'"),
                                                                   selected = '"')
                                               )
                                             ),
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Select separator ----
                                                      radioButtons("sepD1", "Separator",
                                                                   choices = c(Comma = ",",
                                                                               Semicolon = ";",
                                                                               Tab = "\t",
                                                                               Space = " "),
                                                                   selected = ";")
                                               ),
                                               column(width = 6,
                                                      radioButtons("decD1", "Decimal",
                                                                   choices = c("Dot" = '.', "Comma" = ","),
                                                                   selected = '.')
                                               )
                                             )
                )),
                tableOutput("contentsD1"),
                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.",
                             style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("import2", icon("arrow-right",lib = "font-awesome"))
                )
    )
  }

  #------------------------------------------------------#
  #  modalDialog import
  #------------------------------------------------------#
  import2Modal <- function(failed = FALSE) {
    modalDialog(title="Step 2:",size=c("l"),
                div(style="height:15px;"),
                h3("Step 2",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("Upload file temporal data"),
                radioButtons("datatype", "Type",
                             choices = c("Stratigraphic connection" = 1,
                                         "Time range data" = 2),
                             selected = 1),

                fileInput("fileD2", span(icon("import", lib = "glyphicon"),"Choose CSV File"),
                          multiple = FALSE,
                          accept = c("text/csv",
                                     "text/comma-separated-values,text/plain",
                                     ".csv")
                ),
                actionButton(inputId = "toggle", span(icon("cogs",lib = "font-awesome"),"Settings")),
                fluidRow(shinydashboard::box(id="myBox",
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Checkbox if file has header ----
                                                      span(strong("Header")),
                                                      checkboxInput("headerD2", "Header (colnames)", TRUE),
                                                      checkboxInput("rownamesD2", "rownames", FALSE)
                                               ),
                                               column(width = 6,
                                                      # Input: Select quotes ----input$dec
                                                      radioButtons("quoteD2", "Quote",
                                                                   choices = c(None = "",
                                                                               "Double Quote" = '"',
                                                                               "Single Quote" = "'"),
                                                                   selected = '"')
                                               )
                                             ),
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Select separator ----
                                                      radioButtons("sepD2", "Separator",
                                                                   choices = c(Comma = ",",
                                                                               Semicolon = ";",
                                                                               Tab = "\t",
                                                                               Space = " "),
                                                                   selected = ";")
                                               ),
                                               column(width = 6,
                                                      radioButtons("decD2", "Decimal",
                                                                   choices = c("Dot" = '.',
                                                                               "Comma" = ","),
                                                                   selected = '.')
                                               )
                                             )
                )),
                tableOutput("contentsD2"),
                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.",
                             style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("back1", icon("arrow-left",lib = "font-awesome")),
                  actionButton("import3", icon("arrow-right",lib = "font-awesome"))
                )
    )
  }

  #------------------------------------------------------#
  #  modalDialog import
  #------------------------------------------------------#
  import3Modal <- function(failed = FALSE) {
    modalDialog(title="Step 3:",size=c("l"),
                div(style="height:15px;"),
                h3("Step 3",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("Visualisation and confirm"),
                tableOutput("contentsD1"),

                tableOutput("contentsD2"),


                v <- validation(values[["D1"]],values[["D2"]],input$datatype),


                if(v == ""){
                  div(tags$b("Valid data!", style = "color: green;"))
                }else{
                  div(tags$b(paste("Invalid data!",v), style = "color: red;"))
                },

                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.",
                             style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("back2", icon("arrow-left",lib = "font-awesome")),
                  actionButton("Finishimport", "Finish")
                )
    )
  }

  #------------------------------------------------------#
  #  modalDialog select axis
  #------------------------------------------------------#
  pre_selectaxis <- function() {
    modalDialog(title="Select number of axis:",size=c("l"),
                div(style="height:15px;"),
                h3("Select number of axis:",style="text-align:center;"),hr(style="border-color: #222222;"),

                HTML('<p>Our first source of information is a contingency table. We need to construct a distance matrix from this table. We perform a correspondence analysis (CA) on the contingency table and then use the distances of the components (chi-square metric).</p>
                     <p>It is necessary to choose the number of axes (CA components) to be used to construct the distance matrix.</p>
                     <p>By examining the eigenvalues, it is possible to determine the number of principal axes to be considered. The eigenvalues correspond to the amount of information retained by each axis.</p>
                     <p>A heuristic method is often used: one constructs the graph of eigenvalues sorted in descending order and retains the eigenvalues (and associated principal components) preceding the first "elbow".</p>
                     <p>Another approach is based on maintaining an approximation quality of the original data, measured by the explained inertia rate (e.g. 85%). As many axes as necessary are chosen so that the sum of the corresponding eigenvalues exceeds the target inertia rate.</p>'),
                footer = tagList(
                  actionButton("nextCA", "OK")
                )
    )
  }
  selectaxis <- function() {
    modalDialog(title="Select number of axes:",size=c("l"),
                div(style="height:15px;"),
                h3("Select number of axis:",style="text-align:center;"),hr(style="border-color: #222222;"),

                plotlyOutput("axisCAplot"),
                sliderInput("axisNumber",label = "Number:" ,min = 1,
                            max = min( nrow(values[["data1"]])-1,ncol(values[["data1"]])-1),
                            step = 1, value = min( nrow(values[["data1"]])-1,ncol(values[["data1"]])-1)),

                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("aCA", "OK")
                )
    )
  }
  selectaxis2 <- function() {
    modalDialog(title="Select number of axis:",size=c("l"),
                div(style="height:15px;"),
                h3("Select number of axis:",style="text-align:center;"),hr(style="border-color: #222222;"),

                plotlyOutput("axisCAplot"),
                sliderInput("axisNumber",label = "Number:" ,min = 1,
                            max = min( nrow(values[["data1"]])-1,ncol(values[["data1"]])-1),
                            step = 1, value = min( nrow(values[["data1"]])-1,ncol(values[["data1"]])-1)),

                footer = tagList(
                  actionButton("aCA", "OK")
                )
    )
  }

  values <- reactiveValues()
  values[["nCP"]] <- "max"
  values[["cluster"]] <- c()
  values[["ADD"]] <- FALSE
  values[["hiatus"]] <- NULL
  values[["sortperiodtest"]] <- FALSE
  values[["D_alpha"]] <- NULL
  values[["valid"]] <- 1

  values[["alphaplot"]] <- NULL
  values[["dendrogram"]] <- NULL
  values[["seriograph"]] <- NULL
  values[["CA"]] <- NULL
  values[["truecolor"]] <- NULL
  values[["order"]] <- NULL
  values[["table_serio"]] <- NULL
  shinyjs::hide("toggle")
  shinyjs::show("nodata")

  observeEvent(input$axe1,{
    values[["axe1"]] <- input$axe1
  })

  observeEvent(input$axe2,{
    values[["axe2"]] <- input$axe2
  })

  observeEvent(input$width1,{
    values[["width1"]] <- input$width1
  })
  observeEvent(input$height1,{
    values[["height1"]] <- input$height1
  })

  observeEvent(input$width2,{
    values[["width2"]] <- input$width2
  })
  observeEvent(input$height2,{
    values[["height2"]] <- input$height2
  })

  observeEvent(input$width3,{
    values[["width3"]] <- input$width3
  })
  observeEvent(input$height3,{
    values[["height3"]] <- input$height3
  })

  observeEvent(input$width4,{
    values[["width4"]] <- input$width4
  })
  observeEvent(input$height4,{
    values[["height4"]] <- input$height4
  })

  observeEvent(input$width5,{
    values[["width5"]] <- input$width5
  })
  observeEvent(input$height5,{
    values[["height5"]] <- input$height5
  })

  observeEvent(input$width6,{
    values[["width6"]] <- input$width6
  })
  observeEvent(input$height4,{
    values[["height6"]] <- input$height6
  })

  observeEvent(input$method,{
    if(input$method == 1){values[["method"]] <- "ward.D"}
    if(input$method == 2){values[["method"]] <- "ward.D2"}
    if(input$method == 3){values[["method"]] <- "single"}
    if(input$method == 4){values[["method"]] <- "complete"}
    if(input$method == 5){values[["method"]] <- "average"}
    if(input$method == 6){values[["method"]] <- "mcquitty"}
    #if(input$method == 7){values[["method"]] <- "median"}
    #if(input$method == 8){values[["method"]] <- "centroid"}

    values[["cluster"]] <- values[["cluster_ori"]]
    values[["ADD"]] <- FALSE
    values[["div"]] <- as.numeric(names(table(values[["cluster_ori"]]))[table(values[["cluster_ori"]])>1])
    enable("subdivise")
    values[["hiatus"]] <- NULL
    values[["sortperiodtest"]] <- FALSE

    values[["step"]] <- NULL
    values[["which"]] <- NULL
    values[["where"]] <- NULL
    values[["order"]] <- NULL
    values[["firstClustSett"]] <- TRUE
  })


  #raw stratigraphic data (Network)
  values[["data2"]] <- values[["raw"]] <- data.frame(
    ens = c("AI09","AI08","AI07","AI06","AI05","AI04","AI03",
            "AI02","AI01","AO05","AO04","AO03","AO02","AO01","APQR03","APQR02","APQR01"),
    linked_at = c("AI08,AI06","AI07","AI04","AI05","AI01","AI03","AI02","","","AO04","AO03",
                  "AO02,AO01","","","APQR02","APQR01","")
  )
  #contingency table
  values[["data1"]] <- values[["cont"]] <- data.frame(
    Cat10 = c(1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
    Cat20 = c(4,8,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0),
    Cat30 = c(18,24,986,254,55,181,43,140,154,177,66,1,24,15,0,31,37),
    Cat40.50 = c(17,121,874,248,88,413,91,212,272,507,187,40,332,174,17,288,224),
    Cat60 = c(0,0,1,0,0,4,4,3,0,3,0,0,0,0,0,0,0),
    Cat70 = c(3,1,69,54,10,72,7,33,74,36,16,4,40,5,0,17,13),
    Cat80 = c(4,0,10,0,12,38,2,11,38,26,25,1,18,4,0,25,7),
    Cat100.101 = c(23,4,26,51,31,111,36,47,123,231,106,21,128,77,10,151,114),
    Cat102 = c(0,1,2,2,4,4,13,14,6,6,0,0,12,5,1,17,64),
    Cat110.111.113 = c(0,0,22,1,17,21,12,20,30,82,15,22,94,78,18,108,8),
    Cat120.121 = c(0,0,0,0,0,0,0,0,0,0,66,0,58,9,0,116,184),
    Cat122 = c(0,0,0,0,0,0,0,0,0,0,14,0,34,5,0,134,281),
    row.names = c("AI01","AI02","AI03","AI04","AO03","AI05","AO01","AI07","AI08",
                  "AO02","AI06","AO04","APQR01","APQR02","AO05","APQR03","AI09")
  )

  output$data1 <- renderTable(rownames = TRUE, bordered = TRUE,digits = 0,{
    values[["data1"]]
  })

  output$data2 <- renderTable(bordered = TRUE,{
    values[["data2"]]
  })

  output$data1f <- renderTable(rownames = TRUE, bordered = TRUE,digits = 0,{
    data.frame(
      Cat10 = c(1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
      Cat20 = c(4,8,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0),
      Cat30 = c(18,24,986,254,55,181,43,140,154,177,66,1,24,15,0,31,37),
      Cat40.50 = c(17,121,874,248,88,413,91,212,272,507,187,40,332,174,17,288,224),
      Cat60 = c(0,0,1,0,0,4,4,3,0,3,0,0,0,0,0,0,0),
      Cat70 = c(3,1,69,54,10,72,7,33,74,36,16,4,40,5,0,17,13),
      Cat80 = c(4,0,10,0,12,38,2,11,38,26,25,1,18,4,0,25,7),
      Cat100.101 = c(23,4,26,51,31,111,36,47,123,231,106,21,128,77,10,151,114),
      Cat102 = c(0,1,2,2,4,4,13,14,6,6,0,0,12,5,1,17,64),
      Cat110.111.113 = c(0,0,22,1,17,21,12,20,30,82,15,22,94,78,18,108,8),
      Cat120.121 = c(0,0,0,0,0,0,0,0,0,0,66,0,58,9,0,116,184),
      Cat122 = c(0,0,0,0,0,0,0,0,0,0,14,0,34,5,0,134,281),
      row.names = c("AI01","AI02","AI03","AI04","AO03","AI05","AO01","AI07","AI08",
                    "AO02","AI06","AO04","APQR01","APQR02","AO05","APQR03","AI09")
    )
  })

  output$data2f <- renderTable(bordered = TRUE,{
    data.frame(
      ens = c("AI09","AI08","AI07","AI06","AI05","AI04","AI03",
              "AI02","AI01","AO05","AO04","AO03","AO02","AO01","APQR03","APQR02","APQR01"),
      linked_at = c("AI08,AI06","AI07","AI04","AI05","AI01","AI03","AI02","","","AO04","AO03",
                    "AO02,AO01","","","APQR02","APQR01","")
    )
  })

  output$data3f <- renderTable(bordered = TRUE,digits=0,{
    data.frame(
      ens = c("AI09","AI08","AI07","AI06","AI05","AI04","AI03",
              "AI02","AI01","AO05","AO04","AO03","AO02","AO01","APQR03","APQR02","APQR01"),
      inf = c(400,420,500,590,600,550,400,750,700,750,900,800,700,800,850,1000,1050),
      sup = c(500,500,600,800,800,700,600,800,900,850,1100,950,800,900,900,1100,1100)
    )
  })

  output$axisCAplot <- renderPlotly({
    cont <- values[["data1"]]
    tmp <- CA(cont, graph=FALSE)

    x <- paste0(rownames(tmp$eig)," (",arrondi(tmp$eig[,3],1),"%)")
    Eigen_value <- tmp$eig[,1]
    df2 <- data.frame(x,Eigen_value)
    df2 <- within(df2,Principal_Component <- factor(x,levels = x))

    plot_ly(df2,
            x = ~Principal_Component,
            y = ~Eigen_value,
            type = "bar"
    )
  })

  output$timerange <- renderPlotly({
    data <- values[["data2"]]
    data <- data[order(data[,1]),]
    res <- timerange(data = data,cluster = values[["cluster"]])
    values[["order"]] <- res$order
    res$plot
  })


  output$selectAlpha <- renderPlot({
    if(values[["firstClustSett"]]){
      showModal(pre_selectaxis())
    }else{
      showModal(waitModal())
      shinyjs::show("toggle")
      shinyjs::hide("nodata")
      progress1 <- Progress$new(session, min = 0, max = 2)
      on.exit(progress1$close())
      progress1$set(message = 'Initialisation and calcul of the default alpha ..')

      raw <- values[["data2"]]
      cont <- values[["data1"]]

      # here
      #dist ?
      if(dim(raw)[1] == dim(raw)[2]){
        D2 = as.matrix(raw)
      }else{
        if(values[["valid"]] == 1){
          D2 <- adjacency(raw)
        }else{
          D2 <- overlap(raw)
        }
      }

      D1 <- CAdist(cont,nCP=values[["nCP"]],graph=TRUE)
      values[["CA"]] <- recordPlot()
      progress1$set(value = 1)
      sa <- hclustcompro_select_alpha(D1, D2, method = values[["method"]],resampling=FALSE, ite= 5)
      values[["alphaplot"]] <- sa$alpha.plot
      print(sa$alpha.plot)

      progress1$set(value = 2)

      updateSliderInput(session, "alpha", value = sa$alpha)
      updateSliderInput(session, "k", value = values[["nb_classes"]],min=2,
                        max = length(values[["raw"]][,1])-1,step = 1)

      removeModal()
    }
  })

  observeEvent(input$aCA,values[["firstClustSett"]] <- FALSE)

  output$dendrogramme <- renderPlot({
    raw <- values[["data2"]]
    cont <- values[["data1"]]
    #dist ?
    if(dim(raw)[1] == dim(raw)[2]){
      D2 = as.matrix(raw)
    }else{
      if(values[["valid"]] == 1){
        D2 <- adjacency(raw)
      }else{
        D2 <- overlap(raw)
      }
    }
    D1 <- CAdist(cont,nCP=values[["nCP"]],graph=FALSE)
    alpha <- input$alpha

    #create mixing matrix
    D1 <- as.dist(D1)
    D2 <- as.dist(D2)
    D1 <- as.matrix(D1)
    D2 <- as.matrix(D2)
    #sort labels
    D1.Tmp <- D1[sort(labels(D1)[[1]]),sort(labels(D1)[[1]])]
    D2.Tmp <- D2[sort(labels(D2)[[1]]),sort(labels(D2)[[1]])]
    D1 <- as.dist(D1.Tmp)
    D2 <- as.dist(D2.Tmp)
    #normalization
    if(max(D1) != 0){
      D1 <- D1 / max(D1)
    }
    if(max(D2) != 0){
      D2 <- D2 / max(D2)
    }
    #mixt matrix
    dist.mixt <- (alpha) * D1 + (1-alpha) * D2
    dist.mixt <- as.dist(dist.mixt)
    values[["D_alpha"]] <- dist.mixt

    #CAH
    tree <- hclust(dist.mixt, method = values[["method"]])
    values[["tree"]] <- tree

    #wss
    D <- as.matrix(dist.mixt)
    n <- length(D[,1])
    if(length(D[,1]) > 20){n <- 20}
    values[["n"]] <- n

    wss <- c()
    sil <- c()
    for ( i in 2:(n-1)){
      cutree <- cutree(tree,i)
      res <- cluster.stats(D,cutree)
      wss <- c(wss,res$within.cluster.ss)
      sil <- c(sil,res$avg.silwidth)
    }
    values[["wss_v"]] <- wss
    values[["sil_v"]] <- sil
    Within_Sum_of_Square <- rbind(seq(2,(n-1)),wss)
    color_wss <- elbow_finder(seq(2,(n-1)),wss)

    dataplot <- t(Within_Sum_of_Square)
    dataplot <- as.data.frame(dataplot)
    colnames(dataplot) <- c("Number_of_groups","Within_Sum_of_Square")
    q <- ggplot(dataplot,aes(x=Number_of_groups, y=Within_Sum_of_Square)) +
      geom_point(aes(color=color_wss),size=2) + scale_colour_gradientn(colours=c("brown1","chartreuse3")) +
      geom_line(linetype = 3) +
      scale_x_continuous(breaks = dataplot$Number_of_groups ,
                         labels = dataplot$Number_of_groups ) +
      ylim(0,max(wss)+1) +
      labs(title = "Within Sum of Square" , subtitle = "Partition evaluation") +
      theme(legend.position="none")
    values[["wss"]] <- q

    #silhouette
    Average_Sil_Width <- rbind(seq(2,(n-1)),sil)
    dataplot <- t(Average_Sil_Width)
    dataplot <- as.data.frame(dataplot)
    dataplot <- na.omit(dataplot)
    colnames(dataplot) <- c("Number_of_groups","Average_Sil_Width")
    p <- ggplot(dataplot,aes(x=Number_of_groups, y=Average_Sil_Width)) +
      geom_point(aes(color=Average_Sil_Width),size=2) +
      scale_colour_gradientn(colours=c("brown1","chartreuse3")) +
      geom_line(linetype = 3) + ylim(0,1) +
      scale_x_continuous(breaks = dataplot$Number_of_groups,
                         labels = dataplot$Number_of_groups) +
      labs(title = "Average Silhouette Width", subtitle = "Partition evaluation") +
      theme(legend.position="none")
    values[["silhouette"]] <- p

    #dendrogram
    title <- paste("Cluster Dendrogram with alpha =",alpha)
    subtitle <- paste("The mixing matrix is made of ",alpha * 100,"% of D1 and ",(1-alpha) * 100,"% of D2.")
    plot(tree,
         h=-1, main = title, sub = subtitle, xlab=paste0("dist.mixt, method: \"",values[["method"]],"\""))
    if(length(input$k) == 0){
      k <- 2
    }else{
      k <- input$k
    }
    cluster <- cutree(tree, k = k)
    order <- unique(cluster[tree$order])
    for(i in 1:length(order)){
      if(i != order[i] && i < order[i]){
        cluster[cluster == i] <- 0
        cluster[cluster == order[i]] <- i
        cluster[cluster == 0] <- order[i]
        order <- unique(cluster[tree$order])
      }
    }

    if(values[["ADD"]] == FALSE && values[["sortperiodtest"]] == FALSE){
      values[["clusterserio"]] <- values[["cluster"]] <- cluster
      values[["cluster_ori"]] <- cluster
      values[["div"]] <- as.numeric(names(table(values[["cluster_ori"]]))[table(values[["cluster_ori"]])>1])
    }
    rect.hclust(tree, k = k, border = rainbow_hcl(k, c = 80, l = 60, start = 0, end = 360*(k-1)/k))
    col <- rainbow_hcl(length(unique(cluster)), c = 80, l = 60, start = 0,
                       end = 360*(length(unique(cluster))-1)/(length(unique(cluster))))
    for(i in 1:length(unique(cluster))){
      x <- mean(which(cluster[tree$order] == sort(unique(cluster))[i]))
      mtext(paste0(LETTERS[i]), side = 1, line = .5, at = x, col = col[i], cex=1.5)
    }
    mtext("Cluster:",side = 1, line = .5, at = 0, col = "#767676", cex=1.2)
    mtext("Order:",side = 1, line = 1.5, at = 0, col = "#767676", cex=1.2)
    if(values[["sortperiodtest"]]){

    }
    #if subdivide
    if(values[["ADD"]]){
      #rectangle
      for(i in 1:length(values[["which"]])){
        #col <- rainbow_hcl(length(unique(values[["cluster"]])), c = 80, l = 60, start = 0,
        #            end = 360*(length(unique(values[["cluster"]]))-1)/(length(unique(values[["cluster"]]))))
        #index <- which(unique(values[["cluster"]][tree$order]) > values[["where"]][[i]])
        rect.hclust(tree,h = tree$height[values[["step"]][[i]]+1],which = values[["which"]][[i]],
                    border = values[["truecolor"]][values[["where"]][[i]]])#col[index])
      }
      #cluster white erase
      for(i in 1:length(unique(values[["cluster_ori"]]))){
        x <- mean(which(values[["cluster_ori"]][tree$order] == sort(unique(values[["cluster_ori"]]))[i]))
        mtext(Int2Alpha(sort(unique(values[["cluster_ori"]]))[i]), side = 1, line = .5, at = x, col = "white",
              cex=1.5)
      }
      #cluster col
      for(i in 1:length(unique(values[["cluster"]]))){
        x <- mean(which(values[["cluster"]][tree$order] == sort(unique(values[["cluster"]]))[i]))
        j <- trunc(sort(unique(values[["cluster"]]))[i])
        mtext(Int2Alpha(sort(unique(values[["cluster"]]))[i]), side = 1, line = .5, at = x,
              col = values[["truecolor"]][j], cex=1.5)
      }
      for(i in 1:length(unique(values[["clusterserio"]]))){
        #if not remove  period
        if(!nchar(sort(unique(values[["clusterserio"]]))[i]) == 5){
          x <- mean(which(values[["clusterserio"]][tree$order] == sort(unique(values[["clusterserio"]]))[i]))
          #y <- which(unique(values[["clusterserio"]][tree$order]) == sort(unique(values[["clusterserio"]]))[i])
          j <- trunc(sort(unique(values[["cluster"]]))[i])
          mtext(paste0(sort(unique(values[["clusterserio"]]))[i]), side = 1, line = 1.5, at = x,
                col = "#333333", cex=1.3)
          #if remove elt  period
        }else{
          x <- mean(which(values[["clusterserio"]][tree$order] == sort(unique(values[["clusterserio"]]))[i]))
          #y <- which(unique(values[["clusterserio"]][tree$order]) == sort(unique(values[["clusterserio"]]))[i])
          j <- trunc(sort(unique(values[["cluster"]]))[i])
          mtext("Rm", side = 1, line = 1.5, at = x, col = "#333333", cex=1.3)
        }
      }
      #not subdivide
    }else{
      for(i in 1:length(unique(values[["clusterserio"]]))){
        #if not remove  period
        if(!nchar(sort(unique(values[["clusterserio"]]))[i]) == 5){
          x <- mean(which(values[["clusterserio"]][tree$order] == sort(unique(values[["clusterserio"]]))[i]))
          #y <- which(unique(values[["clusterserio"]][tree$order]) == sort(unique(values[["clusterserio"]]))[i])
          j <- trunc(sort(unique(values[["cluster"]]))[i])
          mtext(paste0(sort(unique(values[["clusterserio"]]))[i]), side = 1, line = 1.5, at = x,
                col = "#333333", cex=1.3)
          #if remove elt  period
        }else{
          x <- mean(which(values[["clusterserio"]][tree$order] == sort(unique(values[["clusterserio"]]))[i]))
          #y <- which(unique(values[["clusterserio"]][tree$order]) == sort(unique(values[["clusterserio"]]))[i])
          j <- trunc(sort(unique(values[["cluster"]]))[i])
          mtext("Rm", side = 1, line = 1.5, at = x, col = "#333333", cex=1.3)
        }
      }
    }
    values[["dendrogram"]] <- recordPlot()
  })

  output$EPPM <- renderPlot({
    seq <- values[["clusterserio"]]
    hiatus <- values[["hiatus"]]
    cont <- values[["data1"]]
    cluster_ori <- values[["cluster_ori"]]
    cont <- cont[sort(labels(cont)[[1]]),]
    cont2 <- mutate(cont, Cluster = factor(seq))
    cont2 <- group_by(cont2, Cluster)
    cont2 <- summarise_all(cont2, sum)
    cont2 <- as.data.frame(cont2)
    rownames(cont2) <- as.character(unique(sort(seq)))
    cont2 <- cont2[,-1]
    remove <- which(nchar(rownames(cont2)) == 5)
    if(length(remove) >= 1){
      cont2 <- cont2[-remove,]
    }
    cont2 <- cont2[order(as.numeric(rownames(cont2)),decreasing = T),]
    rownames(cont2) <- paste0('Cluster.',Int2Alpha(as.numeric(unique(cluster_ori)))[order(unique(seq),decreasing = T)])
    #hiatus
    if(length(hiatus) > 0 && sum(is.na(hiatus))==0){
      #insert hiatus
      label <- "Hiatus"
      insert <- values[["hiatus"]]
      print(insert)
      insert <- sort(length(cont2[,1]) - insert)
      for (i in length(insert):1){
        if(insert[i] < length(cont2[,1]) && insert[i] >= 1 && insert[i] == trunc(insert[i])){
          cont2 <- rbind(cont2[1:insert[i],], label = rep(0,length(cont2[1,])),
                         cont2[(insert[i]+1):length(cont2[,1]),])
          if(length(insert)== 1){
            rownames(cont2)[insert[i]+1] <- label
          }else{
            rownames(cont2)[insert[i]+1] <- paste0(label,".",(length(insert)+1 - i))
          }
        }else{
          warning("Some positions are outside the range. Ex: ",insert[i])
        }
      }
    }
    show <- c("both","frequency","EPPM")[as.numeric(input$show)]
    values[["table_serio"]] <- cont2
    plot_EPPM(cont2, permute=input$permute, col_weight = input$col_weight, show = show)
    values[["seriograph"]] <- recordPlot()
  })

  output$CA <- renderScatterD3({

    updateNumericInput(session, "axe1", max = min( nrow(values[["data1"]])-1,ncol(values[["data1"]])-1))
    updateNumericInput(session, "axe2", max = min( nrow(values[["data1"]])-1,ncol(values[["data1"]])-1))

    res <- explor::prepare_results(CA(values[["data1"]], ncp = min( nrow(values[["data1"]])-1,ncol(values[["data1"]])-1)))
    xax = values[["axe1"]]
    yax = values[["axe2"]]
    var_sup = TRUE
    var_hide = "None"
    var_lab_min_contrib = 0
    point_size = 64
    col_var = "Position"
    symbol_var = NULL
    size_var = NULL
    size_range = c(10,300)
    zoom_callback = NULL
    in_explor = FALSE

    ## Settings changed if not run in explor
    html_id <- if(in_explor) "explor_var" else  NULL
    dom_id_svg_export <- if(in_explor) "explor-var-svg-export" else NULL
    dom_id_lasso_toggle <- if(in_explor) "explor-var-lasso-toggle" else NULL
    lasso <- if(in_explor) TRUE else FALSE
    lasso_callback <- if(in_explor) explor_multi_lasso_callback() else NULL
    zoom_callback <- if(in_explor) explor_multi_zoom_callback(type = "var") else NULL

    var_data <- CA_var_data(res, xax, yax, var_sup, var_hide, var_lab_min_contrib)

    scatterD3::scatterD3(
      x = var_data[, "Coord.x"],
      y = var_data[, "Coord.y"],
      xlab = names(res$axes)[res$axes == xax],
      ylab = names(res$axes)[res$axes == yax],
      lab = var_data[, "Lab"],
      point_size = point_size,
      point_opacity = 1,
      col_var = if (is.null(col_var)) NULL else var_data[,col_var],
      col_lab = col_var,
      symbol_var = if (is.null(symbol_var)) NULL else var_data[,symbol_var],
      symbol_lab = symbol_var,
      size_var = if (is.null(size_var)) NULL else var_data[,size_var],
      size_lab = size_var,
      size_range = if (is.null(size_var)) c(10,300) else c(30,400) * point_size / 32,
      tooltip_text = var_data[, "tooltip"],
      type_var = ifelse(var_data[, "Class"] == "Quantitative", "arrow", "point"),
      unit_circle = var_sup && "Quantitative" %in% var_data[,"Class"],
      key_var = paste(var_data[,"Position"], var_data[,"Level"], sep = "-"),
      fixed = TRUE,
      html_id = html_id,
      dom_id_svg_export = dom_id_svg_export,
      dom_id_lasso_toggle = dom_id_lasso_toggle,
      lasso = lasso,
      lasso_callback = lasso_callback,
      zoom_callback = zoom_callback
    )
  })

  output$WSS <- renderPlot({
    print(values[["wss"]])
  })

  output$silhouette <- renderPlot({
    print(values[["silhouette"]])
  })

  observeEvent(input$aCA,{
    values[["nCP"]] <- input$axisNumber
    removeModal()
  })
  observeEvent(input$nextCA,{
    removeModal()
    showModal(selectaxis2())
  })
  observeEvent(input$sort,{
    showModal(sortPeriod())
  })

  observeEvent(input$tri,{
    remove <- input$rm_order
    remove <- remove[2][remove[2] != "Hiatus"]
    order <- input$seq2_order

    val <- order[[ "text" ]][1]
    if(is.na(val) || ( length(order[2]) == 1 && val == "Hiatus" )){
      removeModal()
    }else{
      hiatus <- which(order[2] == "Hiatus")
      for (i in 1:length(hiatus)){
        hiatus[i] <- hiatus[i] - 1 * i
      }

      values[["hiatus"]] <- hiatus
      order <- order[2][order[2] != "Hiatus"]
      remove <- remove[!is.na(remove)]

      if(length(remove) >= 1){
        remove <- Alpha2Int(remove)
      }

      split <- Alpha2Int(order)

      if(length(split)==0){
        removeModal()
        showModal(sortPeriod(TRUE))
      }else{
        cluster <- values[["cluster"]]
        seq <- cluster

        if(length(remove) >= 1){
          for(i in 1:length(remove)){
            removeseq <- which(seq %in% remove[i])
            seq[removeseq] <- seq[removeseq] + as.numeric(paste0("0.00",i))
          }
        }
        c <- 0
        d <- 1
        pelt <- 0
        for (i in 1:length(split)) {
          elt <- as.numeric(split[i])
          felt <- as.numeric(split[i+1])
          if(elt != as.integer(elt)){
            if(as.integer(elt) != as.integer(pelt)){
              c <- c + 1
              d <- 1
            }
            seq[cluster == elt] <- as.numeric(paste0(c,".",d))
            d <- d + 1
          }else{
            c <- c + 1
            seq[cluster == elt] <- c
            d <- 1
          }
          pelt <- elt
        }
        values[["clusterserio"]] <- seq
        values[["sortperiodtest"]] <- TRUE
        removeModal()
      }
    }
  })

  observeEvent(input$subdivise,{
    showModal(subModal1())
  })

  observeEvent(input$sub1,{
    removeModal()
    showModal(subModal2())
  })

  observeEvent(input$sub2,{
    removeModal()
    ok <- TRUE
    cluster <- values[["cluster"]]
    tree <- values[["tree"]]
    where <- Alpha2Int(input$where)
    values[["div"]] <- values[["div"]][values[["div"]] != where]
    if(length(values[["div"]]) == 0){
      disable("subdivise")
    }
    sk <- input$sk
    step <- length(cluster)-1
    while(ok){
      clusters <- cutree(tree,h = tree$height[step])
      if(length(unique(clusters[cluster == where])) == sk){
        ok <- FALSE
      }else{
        step <- step - 1
      }
    }
    #reorder value
    order <- unique(clusters[tree$order])
    for(i in 1:length(order)){
      if(i != order[i] && i < order[i]){
        clusters[clusters == i] <- 0
        clusters[clusters == order[i]] <- i
        clusters[clusters == 0] <- order[i]
        order <- unique(clusters[tree$order])
      }
    }
    nb <- which(values[["cluster_ori"]] == Alpha2Int(input$where))
    nb <- clusters[nb]
    which <- sort(unique(nb))
    #rename value
    resume <- table(clusters[cluster == where])
    c <- 1
    for(i in as.numeric(names(resume))){
      clusters[clusters == i] <- c
      c <- c+1
    }
    c <- c-1
    clusters <- clusters / 10 ^nchar(c)
    for(i in 1:length(cluster)){
      ifelse(cluster[i] == where,
             cluster[i] <- where + clusters[i],
             cluster[i] <- cluster[i])
    }
    values[["clusterserio"]] <- values[["cluster"]] <- cluster
    values[["step"]][[length(values[["step"]])+1]] <- step
    values[["which"]][[length(values[["which"]])+1]] <- which
    values[["where"]][[length(values[["where"]])+1]] <- Alpha2Int(input$where)
    values[["cluster_ori"]] <- cluster
    values[["ADD"]] <- TRUE
  })

  observeEvent(input$alpha,{
    values[["cluster"]] <- values[["cluster_ori"]]
    values[["ADD"]] <- FALSE
    values[["div"]] <- as.numeric(names(table(values[["cluster_ori"]]))[table(values[["cluster_ori"]])>1])
    enable("subdivise")
    values[["hiatus"]] <- NULL
    values[["sortperiodtest"]] <- FALSE
    values[["order"]] <- NULL
  })

  observeEvent(input$k,{
    values[["cluster"]] <- values[["cluster_ori"]]
    values[["ADD"]] <- FALSE
    values[["div"]] <- as.numeric(names(table(values[["cluster_ori"]]))[table(values[["cluster_ori"]])>1])
    enable("subdivise")
    values[["hiatus"]] <- NULL
    values[["sortperiodtest"]] <- FALSE
    values[["truecolor"]] <- rainbow_hcl(input$k, c = 80, l = 60, start = 0,
                                         end = 360*(input$k-1)/input$k)
    values[["step"]] <- NULL
    values[["which"]] <- NULL
    values[["where"]] <- NULL
    values[["order"]] <- NULL
    values[["table_serio"]] <- NULL
  })

  observeEvent(input$back,{
    removeModal()
    showModal(subModal1())
  })

  observeEvent(input$reset,{
    values[["cluster"]] <- values[["cluster_ori"]]
    values[["ADD"]] <- FALSE
    values[["div"]] <- as.numeric(names(table(values[["cluster_ori"]]))[table(values[["cluster_ori"]])>1])
    enable("subdivise")
    values[["hiatus"]] <- NULL
    values[["sortperiodtest"]] <- FALSE
    values[["step"]] <- NULL
    values[["which"]] <- NULL
    values[["where"]] <- NULL
    values[["order"]] <- NULL
    values[["table_serio"]] <- NULL
  })

  output$contentsD1 <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.

    req(input$fileD1)
    inFile <- input$fileD1

    if(is.null(inFile))
      return(NULL)

    tryCatch({
      if(input$rownamesD1){
        df <- read.table(input$fileD1$datapath,
                         header = input$headerD1,
                         sep = input$sepD1,
                         dec = input$decD1,
                         quote = input$quoteD1,
                         row.names = 1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }else{
        df <- read.table(input$fileD1$datapath,
                         header = input$headerD1,
                         sep = input$sepD1,
                         dec = input$decD1,
                         quote = input$quoteD1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }

    },
    error = function(e) {
      # return a safeError if a parsing error occurs
      stop(safeError(e))
    })
    values[["D1"]] <- df
    return(head(df))
  },rownames=TRUE)

  output$contentsD2 <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.

    req(input$fileD2)
    inFile <- input$fileD2

    if(is.null(inFile))
      return(NULL)

    tryCatch({
      if(input$rownamesD2){
        df <- read.table(input$fileD2$datapath,
                         header = input$headerD2,
                         sep = input$sepD2,
                         dec = input$decD2,
                         quote = input$quoteD2,
                         row.names = 1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }else{
        df <- read.table(input$fileD2$datapath,
                         header = input$headerD2,
                         sep = input$sepD2,
                         dec = input$decD2,
                         quote = input$quoteD2,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }
    },
    error = function(e) {
      # return a safeError if a parsing error occurs
      stop(safeError(e))
    })
    if(input$datatype == 2){values[["timerange"]] <- df}
    values[["D2"]] <- df
    return(head(df))
  },rownames=TRUE)

  observeEvent(input$data,{
    showModal(import1Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$back1,{
    showModal(import1Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$back2,{
    showModal(import2Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$import1,{
    showModal(import1Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$import2,{
    showModal(import2Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$import3,{
    showModal(import3Modal())
  })

  observeEvent(input$Finishimport,{

    v <- validation(values[["D1"]],values[["D2"]],input$datatype)

    if(!v == ""){
      removeModal()
      showModal(import3Modal(TRUE))
    }else{
      removeModal()
      values[["nCP"]] <- "max"
      values[["cluster"]] <- values[["cluster_ori"]]
      values[["ADD"]] <- FALSE
      values[["div"]] <- as.numeric(names(table(values[["cluster_ori"]]))[table(values[["cluster_ori"]])>1])
      enable("subdivise")
      values[["data1"]] <- values[["D1"]]
      values[["data2"]] <- values[["D2"]]
      values[["D1"]] <- NULL
      values[["D2"]] <- NULL
      values[["PA"]] <- NULL
      values[["sortperiodtest"]] <- FALSE
      values[["clusterserio"]] <- c(1)
      values[["valid"]] <- input$datatype
      values[["firstClustSett"]] <- TRUE
      updateNumericInput(session, "axe1", value = 1)
      updateNumericInput(session, "axe2", value = 2)
    }

  })

  observeEvent(input$data,{
    shinyjs::disable("import2")
  })

  observeEvent(input$import2,{
    shinyjs::disable("import3")
  })

  observeEvent(input$fileD1,{
    shinyjs::enable("import2")
  })

  observeEvent(input$fileD2,{
    shinyjs::enable("import3")
  })

  observeEvent(input$toggle, {
    if(input$toggle %% 2 == 0){
      shinyjs::hide(id = "myBox")
    }else{
      shinyjs::show(id = "myBox")
    }
  })

  observeEvent(input$refresh, {
    session$reload()
  })

  observeEvent(input$axesCA, {
    showModal(selectaxis())
  })

  #EXPORT BUTTON
  output$alpha.pdf <- downloadHandler(
    filename = "alpha.pdf",
    content = function(file) {
      cairo_pdf(file,width=12)
      print(values[["alphaplot"]])
      dev.off()
    }
  )

  output$alpha.png <- downloadHandler(
    filename = "alpha.png",
    content = function(file) {
      png(file,
          width = values[["width1"]],
          height = values[["height1"]]
      )
      print(values[["alphaplot"]])
      dev.off()
    }
  )

  output$dendrogram.pdf <- downloadHandler(
    filename = "dendrogram.pdf",
    content = function(file) {
      cairo_pdf(file,width=12)
      print(values[["dendrogram"]])
      dev.off()
    }
  )

  output$dendrogram.png <- downloadHandler(
    filename = "dendrogram.png",
    content = function(file) {
      png(file,
          width = values[["width2"]],
          height = values[["height2"]]
      )
      print(values[["dendrogram"]])
      dev.off()
    }
  )

  output$seriograph.pdf <- downloadHandler(
    filename = "seriograph.pdf",
    content = function(file) {
      cairo_pdf(file,width=12)
      print(values[["seriograph"]])
      dev.off()
    }
  )

  output$seriograph.png <- downloadHandler(
    filename = "seriograph.png",
    content = function(file) {
      png(file,
          width = values[["width3"]],
          height = values[["height3"]]
      )
      print(values[["seriograph"]])
      dev.off()
    }
  )

  output$silplot.pdf <- downloadHandler(
    filename = "silplot.pdf",
    content = function(file) {
      cairo_pdf(file,width=12)
      print(values[["silplot"]])
      dev.off()
    }
  )

  output$silplot.png <- downloadHandler(
    filename = "silplot.png",
    content = function(file) {
      png(file,
          width = values[["width4"]],
          height = values[["height4"]]
      )
      print(values[["silplot"]])
      dev.off()
    }
  )

  output$wss.pdf <- downloadHandler(
    filename = "wss.pdf",
    content = function(file) {
      cairo_pdf(file,width=7)
      print(values[["wss"]])
      dev.off()
    }
  )

  output$wss.png <- downloadHandler(
    filename = "wss.png",
    content = function(file) {
      png(file,
          width = 500,
          height = 500
      )
      print(values[["wss"]])
      dev.off()
    }
  )

  output$silhouette.pdf <- downloadHandler(
    filename = "silhouette.pdf",
    content = function(file) {
      cairo_pdf(file,width=7)
      print(values[["silhouette"]])
      dev.off()
    }
  )

  output$silhouette.png <- downloadHandler(
    filename = "silhouette.png",
    content = function(file) {
      png(file,
          width = 500,
          height = 500
      )
      print(values[["silhouette"]])
      dev.off()
    }
  )

  output$table_serio.csv <- downloadHandler(
    filename = function() {
      paste("table_serio", ".csv", sep = "")
    },
    content = function(file) {
      write.csv2(values[["table_serio"]], file, row.names = TRUE)
    }
  )


  observeEvent(input$seriograph.pdf,{
    showNotification("wait for backup in progress",duration=15)
  })

  output$detail <- renderPlotly({
    raw <- values[["data2"]]
    cont <- values[["data1"]]
    if(values[["valid"]] == 1){
      D2 <- adjacency(raw)
    }else{
      D2 <- temp_cov(raw)
    }
    D1 <- CAdist(cont,nCP=values[["nCP"]],graph=FALSE)
    method <- values[["method"]]
    acc = 2


    METHODS <- c("ward.D", "single", "complete", "average", "mcquitty",
                 "median", "centroid", "ward.D2")
    if (method == "ward") {
      message("The \"ward\" method has been renamed to \"ward.D\"; note new avaible method \"ward.D2\"")
      method <- "ward.D"
    }
    i.meth <- pmatch(method, METHODS)
    if (is.na(i.meth))
      stop("invalid clustering method", paste("", method))
    if (i.meth == -1)
      stop("ambiguous clustering method", paste("", method))

    method <- METHODS[i.meth]
    if(!is.numeric(acc)) {stop("acc must be numeric!")}
    if(acc < 1) {
      acc <- 1;
      message("The minimal value allowed for acc is 1.")
    }

    if(class(D1)[1] == "perioclust_cl"){
      if(is.null(D1$D1) || is.null(D1$D2)){
        stop("We don't find dissmilarities in perioclust_cl object.")
      }else{
        D2 <- D1$D2
        D1 <- D1$D1
      }
    }else{
      if(is.null(D1) || is.null(D2)){
        stop("We don't find dissmilarities in D1 and D2 arguments.")
      }
      #test
      if(class(D1)[1] != "dist"){
        #matrix ?
        if(!is.matrix(D1)){
          D1 <- as.matrix(CAdist(cont,nCP=values[["nCP"]],graph=FALSE))
        }
        #diag = 0 ?
        if(sum(diag(D1)) != 0){
          stop("D1: in distance matrix the diagonal is equal to 0.")
        }
        #symetric ?
        if(!isSymmetric(D1)){
          stop("D1 is not symmetric.")
        }
      }
      if(class(D2)[1] != "dist"){
        #matrix ?
        if(!is.matrix(D2)){
          if(valid){
            D2 <- adjacency(raw)
          }else{
            D2 <- temp_cov(raw)
          }
        }
        #diag = 0 ?
        if(sum(diag(D2)) != 0){
          stop("D2: in distance matrix the diagonal is equal to 0.")
        }
        #symetric ?
        if(!isSymmetric(D2)){
          stop("D2 is not symmetric.")
        }
      }
      if(!length(D1[,1]) == length(D1[1,])){stop("D1 not a square matrix.")}
      if(!length(D2[,1]) == length(D2[1,])){stop("D2 not a square matrix.")}
      #dist (positive matrix) ?
      if(min(D1) < 0){stop("D1 not a matrix dissimilarity (positive).")}
      if(min(D2) < 0){stop("D2 not a matrix distance (positive).")}
      if(median(D2) != 0 && median(D2) != 1){
        if(median(D1) == 0 || median(D1) == 1){
          message("We switch the two matrices. Stratigraphic data must be in D1 and ceramic data in D2.")
          tmp <- D1
          D1 <- D2
          D2 <- tmp
        }
      }
      #normalization
      if(max(D1) != 0){
        D1 <- D1 / max(D1)
      }
      if(max(D2) != 0){
        D2 <- D2 / max(D2)
      }
      #create mixing matrix
      D1 <- as.dist(D1)
      D2 <- as.dist(D2)
      D1 <- as.matrix(D1)
      D2 <- as.matrix(D2)
      #sort labels
      D1.Tmp <- D1[sort(labels(D1)[[1]]),sort(labels(D1)[[1]])]
      D2.Tmp <- D2[sort(labels(D2)[[1]]),sort(labels(D2)[[1]])]
      D1 <- as.matrix(D1.Tmp)
      D2 <- as.matrix(D2.Tmp)
    }

    D1save <- as.matrix(D1)
    D2save <- as.matrix(D2)

    save_lines <- NULL
    alpha_seq <- seq(0,1,0.01)

    showModal(waitModal())

    withProgress(message = 'Resampling process', value = 0, {
      nb_elt <- length(D1save[,1])
      for(i in 1:nb_elt){
        # update progress bar
        incProgress(1/nb_elt, detail = paste("Doing part", i))
        Sys.sleep(0.1)
        df <- NULL
        for(j in 1:nb_elt){
          if(j != i ){
            list <- generateClone(D1save,D2save,i,j)
            D1 <- list[[1]]
            D2 <- list[[2]]
            dist.dend <- c()
            for(alpha in alpha_seq){
              Mdist <- (alpha)*D1 + (1-alpha)*D2
              mixt.dist <- as.dist(Mdist)
              tree <- fastcluster::hclust(mixt.dist,method=method)
              new <- corCriterion(tree,D1,D2)
              dist.dend <- c(dist.dend,new)
            }
            if(is.null(df)){
              df <- data.frame(alpha = alpha_seq, height = dist.dend)
            }else{
              df <- cbind(df,dist.dend)
            }
          }
        }
        mean <- unlist(lapply(as.data.frame(t(df[-1])),mean))
        if(is.null(save_lines)){
          save_lines <- cbind(df$alpha,mean)
        }else{
          save_lines <- cbind(save_lines,mean)
        }
      }
    })
    colnames(save_lines) <- c("alpha",rownames(D1)[1:nb_elt])
    rownames(save_lines) <- c()

    removeModal()

    p <- plot_ly(width = 760, height = 400)
    for (k in 2:length(colnames(save_lines))) {
      p <- add_lines(p,x=save_lines[,1],y=save_lines[,k],name=colnames(save_lines)[k])
    }
    p
  })

  observe({
    if(2 > 1){
      output$SilPlot <- renderPlot({
        cluster <- values[["cluster"]]
        cls <- sort(unique(cluster))
        for (i in length(cls):1){
          cluster[cluster == cls[i]] <- i
        }
        Dmatrix <- values[["D_alpha"]]
        names <- labels(Dmatrix)
        text <- c("")
        for (i in 1:length(names)){
          text <- paste0(text,i,":",names[i],"\n")
        }
        sil <- silhouette(cluster,Dmatrix)
        col <- rainbow_hcl(length(unique(trunc(values[["cluster"]]))), c = 80, l = 60, start = 0,
                           end = 360*(length(unique(trunc(values[["cluster"]])))-1)/(length(unique(trunc(values[["cluster"]])))))
        col <- col[trunc(sort(values[["cluster"]]))]
        par(fig=c(.2,1,0,1), new=TRUE)
        plot(sil, col = col)
        par(fig=c(0,.2,0,1), new=TRUE)
        plot.new()
        mtext(text,3,adj=0,padj=1.1)
        values[["silplot"]] <- recordPlot()
      })
    }
  })


  observeEvent(input$wiki, {
    updateTabsetPanel(session, "navbar", selected = "4")
  })
  observeEvent(input$over, {
    updateTabsetPanel(session, "tabsetperso", selected = "Overview")
  })
  observeEvent(input$clustsett, {
    updateTabsetPanel(session, "tabsetperso", selected = "Clustering settings")
  })
  observeEvent(input$resviz, {
    updateTabsetPanel(session, "tabsetperso", selected = "Results visualization")
  })

  ##dendrogramme click
  output$click_info <- renderPrint({
    if(length(input$k) == 0){
      cat("hcluscompro")
    }else{

      size <- c()

      cluster <- values[["cluster"]]
      nb <- length(unique(cluster))
      tot <- length(cluster)
      cat("The selected partition contains",nb,"clusters among",tot,"observations.\ncluster:\t")
      for (k in unique(values[["cluster"]])){
        size <- c(size,length(cluster[cluster == k]))
        cat(k,"\t")
      }
      cat("\nSize:\t\t")
      for(y in 1:nb){
        cat(size[y],"\t")
      }

    }
  })

  output$formula <- renderUI({
    withMathJax(paste0("$$ \\boldsymbol{D}_\\alpha=\\alpha \\boldsymbol{D}_1+(1-\\alpha) \\boldsymbol{D}_2 $$"))
  })


}

estimateDensityMulti <- function(data){
  cat("\nEstimation of multidimensionnal density ..")
  #Smooth cross-validation bandwidth selector
  H.scv <- Hscv(data)
  fhat <- kde(data, H.scv, eval.points=data)
  res <- fhat$estimate*100
  cat(". Done")
  return(res)
}

IW <- function(x)
{
  #===============================================================================
  # SOMMES DES ECARTS AUX CARRE
  #===============================================================================
  sum(scale(x, scale = FALSE)^2)
}

spatialpatches2 <- function (x, y, z, w, Lim.D = 100, A.li = 0, modproj = NA, mlong = NA, mlat = NA)
{
  #===============================================================================
  # IDENTIFICATION OF SPATIAL PATCHES
  # AND COUNT OF CENTERS OF GRAVITY OF SPATIAL PATCHES
  #
  # Routine from EU program Fisboat, DG-Fish, STREP n? 502572
  # Authors : P.Petitgas (Ifremer), M.Woillez and J.Rivoirard (Mines-ParisTech)
  # Last update : 01 march 2008
  #
  # Arguments:
  # x	      The x-coordinate of the first population
  #	        Can be a vector, a matrix or an xyZ list (-,|,[]).
  # y	      The y-coordinates of the first population
  # z	      The regionalised variable of the first population.
  #	        If missing, the results of 'cgi' will concern the samples only.
  # w	      Optional. A weight or an area of influence of the first population.
  #	        Set to 1 if missing
  # Lim.D   Select minimum distance from sample to patch centre: to
  #         identify patches (units are those of coordinates)
  # A.li    Visualisation of gravity centres for those patches with
  #         abundance > A.li (in %)
  # modproj	Optional. Indicates the type of projection to perform.
  # mlong   mean longitude in DEGREES of the data set to be transformed
  # mlat    mean latitude in DEGREES of the data set to be transformed
  #	        See 'dg2nm' for precisions
  #
  #===============================================================================

  patch.id <- function(xx,yy,zz,ww,dli,ali,modproj,mlong,mlat)
  {
    ############################################################################
    # inputs : xx , yy , ww & zz ranked by decreasing order of zz
    #          dli is a distance limit from the patch gravity centre to define
    #          the patch border
    #
    # the function starts from the richest zz value and considers each sample
    # in decreasing order of zz. It tests whether the current value is spatially
    # close enough to the gravity centre of previously formed patches. if not a
    # new patch is considered. and so on until the last value. Patches of nul
    # values are returned with centres as NA and code 0 and their areas are
    # summed.
    #
    # outputs : the patch number for each sample,
    #           the gravity center for each patch,
    #           the percent abundance and area for each patch
    ############################################################################

    g <- c(1,rep(0,(length(xx)-1)))
    for (j in 1:(length(xx)-1)) {
      xg <- tapply(ww[1:j]*zz[1:j]*xx[1:j],g[1:j],sum,na.rm=T) /
        tapply(ww[1:j]*zz[1:j],g[1:j],sum,na.rm=T)
      yg <- tapply(ww[1:j]*zz[1:j]*yy[1:j],g[1:j],sum,na.rm=T) /
        tapply(ww[1:j]*zz[1:j],g[1:j],sum,na.rm=T)
      d <- sqrt(((xg-x1[j+1]))^2 + (yg-y1[j+1])^2)
      o <- order(d)
      if (d[o[1]] < dli){
        g[j+1] <- o[1]
      }else{
        g[j+1] <- max(g[1:j], na.rm=T)+1
      }
    }
    #second passage des obs pour affiner centres de gravite
    for(i in 1:length(xx)) {
      xg <- tapply(ww*zz*xx,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
      yg <- tapply(ww*zz*yy,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
      d <- sqrt(((xg-x1[i]))^2+(yg-y1[i])^2)
      o <- order(d)
      g[i] <- o[1]
    }

    xg <- tapply(ww*zz*xx,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    yg <- tapply(ww*zz*yy,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    pb <- tapply(ww*zz,g,mean,na.rm=TRUE)
    inertiaTot <- sum(scale(zz, scale = FALSE)^2)
    InertiaWithin <- tapply(zz,g,IW)
    sel <- (tapply(ww*zz,g,sum,na.rm=TRUE)/sum(ww*zz,na.rm=TRUE))*100>=ali
    res <- max(unique(g)[sel],na.rm=TRUE)
    n <- sort(unique(g))
    selna <- is.na(xg)
    if (sum(selna) > 1) {
      n <- c(n[!selna],0)
      pb <- c(pb[!selna],0)
      pa <- c(pa[!selna],sum(pa[selna],na.rm=T))
      xg <- c(xg[!selna],NA)
      yg <- c(yg[!selna],NA)
      g[!is.element(g,n)] <- 0
    }
    return(list(n=g,mat=cbind(n=n,xg=round(xg,2),yg=round(yg,2),zg=round(pb*100,2)),
                InertiaWithin=InertiaWithin,
                InertiaWithintot=sum(InertiaWithin),
                InertiaBetween=inertiaTot - sum(InertiaWithin),
                inertiaTot=inertiaTot,
                nsp=res))
  }

  # prepare data
  Xsta <- x		#Xsta <- bid$x
  Ysta <- y		#Ysta <- bid$y
  Zsta <- z
  Wsta <- w/sum(w,na.rm=TRUE)

  # exclude NA values
  SEL <- is.na(Xsta)+is.na(Ysta)+is.na(Zsta)+is.na(Wsta)
  Xsta <- Xsta[!SEL]
  Ysta <- Ysta[!SEL]
  Zsta <- Zsta[!SEL]
  Wsta <- Wsta[!SEL]

  # order data by decreasing order of value z
  zz1 <- sort(Zsta,decreasing=TRUE,index.return=TRUE)
  w1 <- Wsta[zz1$ix]
  x1 <- Xsta[zz1$ix]
  y1 <- Ysta[zz1$ix]
  z1 <- zz1$x

  # identify patches around high values
  SP <- patch.id(x1,y1,z1,w1,Lim.D,A.li,modproj=modproj,mlong=mlong,mlat=mlat)

  # graphical visualisation : patch identification
  #shows data points, circles for data values, number of patch &
  #crosses for patch gravity centres with abundance > A.li of total
  o <- sort(zz1$ix,index.return=TRUE)
  SP$n <- SP$n[o$ix]
  SP
}

WSS <- function(hist,data)
{
  #====================================================================================
  # WSS        Return the within cluster sum of square for each partition.
  #
  # hist       The historique of the partitions
  # data       The geographical data and third variable Z
  #====================================================================================

  out <- list()
  n <- length(hist[1,])
  Itot <- calctot(data)
  for (i in 1:n){
    WSS <- c()
    #recup compo of one partition i
    compo <- hist[,i]

    #calculer Within-cluster Sum of Square
    X <- data [,1]
    Y <- data [,2]
    Z <- data [,3]
    XG <- tapply(X * Z,compo,sum)/tapply(Z,compo,sum)
    YG <- tapply(Y * Z,compo,sum)/tapply(Z,compo,sum)
    nbgrp <- length(XG)
    for(j in 1:nbgrp){
      subX <- X[compo==j]
      subY <- Y[compo==j]
      subZ <- Z[compo==j]
      DX <- subX - XG[j]
      DY <- subY - YG[j]
      D <- sqrt(DX^2 + DY^2)
      INERT <- sum(subZ * (D^2))/sum(Z)
      WSS <- c(WSS,INERT)
    }
    out <- c(out,list(WSS))
  }
  out
}

calctot <- function(data)
{
  X <- data[,1]
  Y <- data[,2]
  Z <- data[,3]
  XG <- sum(X * Z)/sum(Z)
  YG <- sum(Y * Z)/sum(Z)
  DX <- X - XG
  DY <- Y - YG
  D <- sqrt(DX^2 + DY^2)
  INERT <- sum(Z * (D^2))/sum(Z)
  INERT
}

dist_m <- function(data)
{
  #===================================================================================
  # DISTANCE MATRIX   Weighted Eucidiean distance
  #
  # data      data.frame or matrix with 3 colunms (X, Y, Z)
  #===================================================================================
  square <-function(x){return(x*x)}
  Dmatrix <- matrix(ncol=length(data[,1]), nrow=length(data[,1]))
  for(i in 1:length(data[,1])){
    for (j in 1:length(data[,1])){
      #Weighted Eucidiean distance
      distance_pond <- sqrt( ( square(data[i,1]-data[j,1]) + square(data[i,2]-data[j,2]) ) *
                               square(data[i,3]-data[j,3])
      )
      Dmatrix[i,j] <- distance_pond
      Dmatrix[j,i] <- distance_pond
    }
  }
  return(Dmatrix)
}

transfo_z <- function(z)
{
  #====================================================================================
  # Transfo Z  Confidence interval on Empirical Cumulative Distribution Function (ECDF)
  #            We only can use z data positive like density, count or frequencies.
  #
  # z         vector of values
  #====================================================================================
  cat("\nEstimation of univariate density ..")
  dx <- density(z)
  xnew <- z # find density value on a new point
  dx.approx <- approx(dx$x,dx$y,xout=xnew)
  pi <- dx.approx$y*100 # densite estimee * 100 en chaque point du jeu de donnees
  cat(". Done")
  return(pi)
}

addLegendCustom <- function(map, colors, labels, sizes, opacity = 0.5, title)
{
  #===================================================================================
  # Custom legend for diffrent sizes of point in final map
  #
  # Arguments:
  # map       The leaflet map object
  # colors    The colors of the points
  # labels    The labels of the points
  # sizes     The sizes signification of the points in the map
  # opacity   The opacity of the background
  #
  # You can change the title and the position of the legend directly in the code. (last line of the function)
  #===================================================================================

  colorAdditions <- paste0(colors, ";border-radius: 50%;margin-top:10%; width:", 2*sizes, "px; height:", 2*sizes, "px")
  labelAdditions <- paste0("<div style='display: inline-block;height: ",
                           2*sizes, "px;margin-top: 4px;line-height: 0px;'>", labels, "</div>")

  return(addLegend(map, colors = colorAdditions, labels = labelAdditions, opacity = opacity,
                   title = title, position = "topright"))
}

"ecartmatrice" <-
  function (historique = "matrix composition of groups for each dlim")
  {
    #===================================================================================
    # ECART MATRICE
    #
    # Author : A. Coulon (Universite de Nantes)
    # Last update : 22 march 2018
    #
    # Arguments:
    # historique   The composition of groups for each dlim and each initial elements
    #
    #===================================================================================

    historique <- as.matrix(historique)
    #variable securite
    OK <- 1

    if(length(historique[,1]) <= 1){
      cat("No Cluster")
    }
    ecart <- matrix(ncol = length(historique[,1]), nrow = length(historique[,1]))
    #On parcours le tableau pour tous les j on les compare a tous les i (compare tous les elt entre eux)
    for (j in 1:length(historique[,1])) {
      for (i in 1:length(historique[,1])) {
        if(i > j){
          v <- historique[i,]==historique[j,]
          somme <- sum(v %in% FALSE)
          ecart[i,j] <- somme
        }

      }
    }
    ecart <- replace(ecart,is.na(ecart),Inf)
    ecart
  }

IndiceMoran <- function(hist,data) {
  out <- list()
  X <- data[,1]
  Y <- data[,2]
  Z <- data[,3]

  n <- length(hist[1,])
  for (i in 1:n){
    #recup compo of one partition i
    compo <- hist[,i]

    nb <- length(unique(compo))

    for(j in 1:nb){
      subX <- X[compo==j]
      subY <- Y[compo==j]
      subZ <- Z[compo==j]

      if(length(subX) > 3){

        if(length(unique(subZ)) > 2 && length(unique(subX)) > 2 && length(unique(subY)) > 2){
          data.dists <- as.matrix(dist(cbind(subX, subY)))
          Moran <- Moran.I(subZ, data.dists)
        }else{
          Moran <- list(observed= NA, p.value = NA)
        }

      }else{
        Moran <- list(observed= NA, p.value = NA)
      }

      if(!exists("M.I")){
        M.I <- data.frame(grp=c(Moran$observed,Moran$p.value),row.names = c("Moran.I","p.value"))
      }else{
        M.I <- cbind(M.I,grp=c(Moran$observed,Moran$p.value))
      }
    }
    out <- c(out,list(M.I))
    rm(M.I)
  }
  out
}

load(file = "./data/datarcheo.RData")
load(file = "./data/datacancer.RData")

#----------------------------------------------------------------------------------------------#
###########################                - SERVER -                ###########################
#----------------------------------------------------------------------------------------------#
server <- function(input, output, session) {

  dlmap <- reactiveValues(map = 0)
  values <- reactiveValues()
  plot.dend <- reactiveValues(main=NULL, add=NULL, add2=NULL)
  values[["path"]] <- "wait.png"
  values[["multi"]] <- "wait"
  values[["lonlat"]] <- TRUE
  values[["silhouetteplot"]] <- NULL
  values[["labelall"]] <- NULL
  values[["isLabel"]] <- TRUE

  #------------------------------------------------------#
  # Download
  #------------------------------------------------------#
  # output$map.png <- downloadHandler(
  #   filename = "map.png",
  #   content = function(file) {
  #     showModal(waitModal())
  #     mapshot(dlmap$map, file = file)
  #     removeModal()
  #   }
  # )

  #------------------------------------------------------#
  #  modalDialog zdata ask
  #------------------------------------------------------#
  # Return the UI for a modal dialog with data selection input. If 'failed' is
  # TRUE, then display a message that the previous value was invalid.
  dataModal <- function(failed = FALSE, data = 1,multi = FALSE) {
    if(data == 3 && multi=="Univariate"){
      modalDialog(title="datatypemessage",size = c("s"),

                    h4("Data:",style="margin-left:10px;"),hr(style="border-color: #222222;"),
                    span("This classification requires geographical coordinates and a third variable: var."),
                    br(),
                    span("We use var to weight the distance and var must be a frequency or density and always positive."),
                    br(),br(),
                    radioButtons("z_select", "Choose data type of your var",
                                 choices = c(positive = "un",
                                             other = "de"),
                                 selected = "de"),

                  if (failed)
                    div(tags$b("You have to choose", style = "color: red;")),
                  footer = tagList(
                    modalButton("Cancel"),
                    actionButton("dismiss_modal","OK")
                  )
      )
    }else{
      modalDialog(title="datatypemessage",size = c("s"),
                  h4("Are you sure ? "),hr(style="border-color: #222222;"),
                  footer = tagList(
                    modalButton("Cancel"),
                    actionButton("dismiss_modal","OK")
                  )
      )
    }
  }
  #------------------------------------------------------#
  #  modalDialog cuttree message
  #------------------------------------------------------#
  # Return the UI for a modal dialog with data selection input. If 'failed' is
  # TRUE, then display a message that the previous value was invalid.
  cuttreeModal <- function(failed = FALSE) {
    modalDialog(title="cuttreemessage",size = c("l"),
                h4("Cut the tree:",style="margin-left:10px;"),hr(style="border-color: #222222;"),
                span("You can clic on the dendrogram "),img(src="tree.png",height = 23, width = 26),span(" in order to select the dlim where to cut the tree."),
                span("The map and the silhouette plot will be automatically updated."),

                if (failed)
                  div(tags$b("You have to choose", style = "color: red;")),
                footer = tagList(
                  modalButton("OK")
                )
    )
  }
  #------------------------------------------------------#
  #  modalDialog wait message
  #------------------------------------------------------#
  waitModal <- function(failed = FALSE) {
    modalDialog(title="waitmessage",size = c("s"),
                h4("Wait . . . ",style="margin-left:100px;"),hr(style="border-color: #222222;"),
                footer = NULL
    )
  }

  #------------------------------------------------------#
  #  modalDialog upload
  #------------------------------------------------------#
  uploadModal <- function(failed = FALSE) {
    modalDialog(title = "upload",size=c("l"),
                h4("Upload file"),
                # Input: Select a file ----
                fileInput("file1", span(icon("import", lib = "glyphicon"),"Choose CSV File"),
                          multiple = FALSE,
                          accept = c("text/csv",
                                     "text/comma-separated-values,text/plain",
                                     ".csv")
                ),

                # Horizontal line ----
                hr(style="border-color: #222222;"),
                actionButton(inputId = "toggle", span(icon("cogs",lib = "font-awesome"),"Settings")),
                fluidRow(id="myBox",
                  column(width = 5,
                         span(strong("Non-geographic variables.")),
                         sliderTextInput("UniMulti","Your case is:",choices=c("Univariate","Multivariate"),width = "100px"),
                         hr(style="border-color: #C6C0AA;"),
                         sliderInput("nb_var","How many (non-geographic) variables do you have?",min=2,max=6,value=2)
                  ),
                  column(width = 3,
                         # Input: Select Label ----
                         span(strong("Your data have:")),
                         checkboxInput("label", "Label", TRUE),
                         checkboxInput("lonlat", "Longitude\nLatitude", TRUE),
                         hr(style="border-color: #C6C0AA;"),
                         # Input: Select separator ----
                         radioButtons("sep", "Separator",
                                      choices = c(Comma = ",",
                                                  Semicolon = ";",
                                                  Tab = "\t",
                                                  Space = " "),
                                      selected = ";")
                  ),
                  column(width = 3,
                         # Input: Checkbox if file has header ----
                         span(strong("Header")),
                         checkboxInput("header", "Header", TRUE),
                         hr(style="border-color: #C6C0AA;"),
                         # Input: Select quotes ----input$dec
                         radioButtons("quote", "Quote",
                                      choices = c(None = "",
                                                  "Double Quote" = '"',
                                                  "Single Quote" = "'"),
                                      selected = '"'),
                         radioButtons("dec", "Decimal",
                                      choices = c("Dot" = '.',
                                                  "Comma" = ","),
                                      selected = '.')
                  )
                ),
                # Horizontal line ----
                hr(style="border-color: #C6C0AA;"),
                fluidRow(
                  column(width = 8
                  ),
                  column(width = 2,
                         h4("Match:")
                  ),
                  column(width = 1,
                         imageOutput("image2",width = "40px", height = "40px")
                  )
                ),
                h4("How your data must be organized"),
                hr(style="border-color: #222222;"),
                tableOutput("table"),
                # Output: Data file ----
                h4("Your data"),hr(style="border-color: #222222;"),
                tableOutput("contents"),
                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.", style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("okUL", "OK")
                )
    )
  }
  #------------------------------------------------------#
  #  Uploading File
  #------------------------------------------------------#
  output$image2 <- renderImage({

    if (values[["path"]] == "yes.jpg") {
      return(list(
        src = "www/yes.jpg",
        width="40px",
        contentType = "image/jpg",
        alt = "yes"
      ))
    }else{
      return(list(
        src = "www/no.jpg",
        width="40px",
        contentType = "image/jpg",
        alt = "no"
      ))
    }

  }, deleteFile = FALSE)

  observe({
    if(!is.null(input$UniMulti)){
      if(input$UniMulti == "Univariate"){
        shinyjs::hide("nb_var")
      }else{
        shinyjs::show("nb_var")
      }
    }
  })

  observeEvent(input$UL,{
    showModal(uploadModal())
    shinyjs::hide("myBox")
    shinyjs::hide("nb_var")
  })

  observe({
    if(!is.null(values[["test"]])){
      if(values[["test"]] == length(values[["DF"]][1,])){
        values[["path"]] <- "yes.jpg"
      }else{
        values[["path"]] <- "no.jpg"
      }
    }
  })

  observeEvent(input$okUL,{
    if(values[["test"]] == length(values[["DF"]][1,])){
      values[["nb_var"]] <- input$nb_var
      values[["multi"]] <- input$UniMulti
      values[["lonlat"]] <- input$lonlat

      data <- values[["DF"]]
      multi <- values[["multi"]]
      lonlat <- values[["lonlat"]]

      if(!is.numeric(data[,1])){

        showNotification("coordinates must be numeric! (Check the decimal character)",type = "warning", duration=15)
        showModal(uploadModal(failed = TRUE))
        shinyjs::hide("nb_var")
      }else{
        if(!is.numeric(data[,2])){

          showNotification("coordinates must be numeric! (Check the decimal character)",type = "warning", duration=15)
          showModal(uploadModal(failed = TRUE))
          shinyjs::hide("nb_var")
        }else{
          if(!is.numeric(data[,3])){

            showNotification("variable must be numeric! (Check the decimal character)",type = "warning", duration=15)
            showModal(uploadModal(failed = TRUE))
            shinyjs::hide("nb_var")
          }else{
            #change data select
            updateSelectInput(session, "select", selected = 3)
            removeModal()
          }

        }


      }



    }else{
      showModal(uploadModal(failed = TRUE))
      shinyjs::hide("nb_var")
    }
  })

  output$contents <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.
    req(input$file1)
    inFile <- input$file1

    if(is.null(inFile))
      return(NULL)

    tryCatch({
        df <- read.csv(input$file1$datapath,
                       header = input$header,
                       sep = input$sep,
                       dec = input$dec,
                       quote = input$quote)
      },
      error = function(e) {
        # return a safeError if a parsing error occurs
        stop(safeError(e))
    })

    if (!is.null(inFile)){
      DF <- df
      values[["DF"]] <- DF
    }

    return(head(df))

  })

  output$table <- renderTable(
    if(input$UniMulti == "Univariate"){
      if(input$label){
        values[["test"]] <- 4
        values[["isLabel"]] <- TRUE
        data.frame(coord.1="X",coord.2="Y",var="var",label_optional="label")
      }else{
        values[["test"]] <- 3
        values[["isLabel"]] <- FALSE
        data.frame(coord.1="X",coord.2="Y",var="var")
      }

    }else{
      if(input$nb_var == 2){
        if(input$label){
          values[["test"]] <- 5
          values[["isLabel"]] <- TRUE
          data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",label_optional="label")
        }else{
          values[["test"]] <- 4
          values[["isLabel"]] <- FALSE
          data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2")
        }
      }else{
        if(input$nb_var == 3){
          if(input$label){
            values[["test"]] <- 6
            values[["isLabel"]] <- TRUE
            data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",label_optional="label")
          }else{
            values[["test"]] <- 5
            values[["isLabel"]] <- FALSE
            data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3")
          }
        }else{
          if(input$nb_var == 4){
            if(input$label){
              values[["test"]] <- 7
              values[["isLabel"]] <- TRUE
              data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",label_optional="label")
            }else{
              values[["test"]] <- 6
              values[["isLabel"]] <- FALSE
              data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4")
            }
          }else{
            if(input$nb_var == 5){
              if(input$label){
                values[["test"]] <- 8
                values[["isLabel"]] <- TRUE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5",label_optional="label")
              }else{
                values[["test"]] <- 7
                values[["isLabel"]] <- FALSE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5")
              }
            }else{
              if(input$label){
                values[["test"]] <- 9
                values[["isLabel"]] <- TRUE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5",var.6="var6",label="label")
              }else{
                values[["test"]] <- 8
                values[["isLabel"]] <- FALSE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5",var.6="var6")
              }

            }

          }

        }

      }

    }
  )

  observe({
    if(!values[["isLabel"]]){
      shinyjs::hide(id = "Plabel")
    }else{
      shinyjs::show(id = "Plabel")
    }
  })

  observeEvent(input$toggle, {
    if(input$toggle %% 2 == 0){
      shinyjs::hide(id = "myBox")
    }else{
      shinyjs::show(id = "myBox")
    }
  })

  #------------------------------------------------------#
  #  MapClust
  #------------------------------------------------------#
  observe({
    if(input$select==3){
      if (is.null(values[["DF"]])){
        showNotification("Oops ! you forgot a little thing ! To plot your data, you need to follow a few things:
                       You must import your own data! Re Run after import!",type = "warning", duration = 100)
      }else{
        if(input$UniMulti == "Univariate"){
          if(input$label){
            values[["isLabel"]] <- TRUE
          }else{
            values[["isLabel"]] <- FALSE
          }

        }else{
          if(input$nb_var == 2){
            if(input$label){
              values[["isLabel"]] <- TRUE
            }else{
              values[["isLabel"]] <- FALSE
            }
          }else{
            if(input$nb_var == 3){
              if(input$label){
                values[["isLabel"]] <- TRUE
              }else{
                values[["isLabel"]] <- FALSE
              }
            }else{
              if(input$nb_var == 4){
                if(input$label){
                  values[["isLabel"]] <- TRUE
                }else{
                  values[["isLabel"]] <- FALSE
                }
              }else{
                if(input$nb_var == 5){
                  if(input$label){
                    values[["isLabel"]] <- TRUE
                  }else{
                    values[["isLabel"]] <- FALSE
                  }
                }else{
                  if(input$label){
                    values[["isLabel"]] <- TRUE
                  }else{
                    values[["isLabel"]] <- FALSE
                  }

                }

              }

            }

          }

        }
      }

    }else{
      values[["isLabel"]] <- TRUE
    }
  })

  ##select data and run calcul
  observeEvent(input$Run,{
    if (input$select==3 && is.null(values[["DF"]])){
      showNotification("Oops ! you forgot a little thing ! To plot your data, you need to follow a few things:
                       You must import your own data and after press again the run button!",type = "error", duration = 100)
      showModal(uploadModal())
    }else{
      if (input$select==3){showModal(dataModal(data=3,multi=input$UniMulti))}
      if (input$select==2){showModal(dataModal(data=2))}
      if (input$select==1){showModal(dataModal(data=1))}
    }
  })

  observeEvent(input$dismiss_modal,{
    removeModal()
    showModal(waitModal())

    if(input$select==1){
      data <- datarcheo
      ctrl <- "de"
      multi <- "Univariate"
      lonlat <- TRUE
    }
    if(input$select==2){
      data <- datacancer
      ctrl <- "un"
      multi <- "Univariate"
      lonlat <- TRUE
    }
    OK <- TRUE
    if(input$select==3){
      if (is.null(values[["DF"]])){
        showNotification("There is no upload data. We select datarcheo by default",type = "warning", duration = 100)
        data <- datarcheo
      }else{
        data <- values[["DF"]]
        multi <- values[["multi"]]
        lonlat <- values[["lonlat"]]

        if(!is.numeric(data[,1])){
          OK <- FALSE
          showNotification("coordinates must be numeric!",type = "warning", duration=15)
        }
        if(!is.numeric(data[,2])){
          OK <- FALSE
          showNotification("coordinates must be numeric!",type = "warning", duration=15)
        }

        if(multi == "Multivariate"){
          for(i in 3:(2+values[["nb_var"]])){
            if(!is.numeric(data[,i])){
              OK <- FALSE
              showNotification("variable must be numeric!",type = "warning", duration=15)
            }
          }
        }else{
          if(!is.numeric(data[,3])){
            OK <- FALSE
            showNotification("variable must be numeric!",type = "warning", duration=15)
          }
        }
      }
    }
    if(OK == TRUE){

      values[["multi2"]] <- multi
      if(input$Plabel==T){Plabel = T}
      if(input$Plabel==F){Plabel = F}
      if(input$extenddend==T){extenddend = T}
      if(input$extenddend==F){extenddend = F}
      if(input$select==3 && multi == "Univariate"){
        if(input$z_select=="un"){ctrl="un"}else{ctrl="de"}
      }
      values[["data"]] <- data

      values[["index"]] <- NULL
      binf = 0
      bsup = max(dist(data[,1:2]))
      pas = (max(dist(data[,1:2]))/20)
      plot.dend$add <- NULL
      plot.dend$add2 <- NULL

      progress1 <- Progress$new(session, min = 0, max = 2)
      on.exit(progress1$close())
      progress1$set(message = 'Initialisation and calcul of the distance matrix ..')
      progress1$set(value = 1)

      progress2 <- Progress$new(session,style = "notification", min = 0, max = round((bsup - binf) / pas,0))
      on.exit(progress2$close())
      progress2$set(message = 'Calculation in progress',
                    detail = 'This may take a while...')


      #Si Z negatif : reduction
      if(multi == "Multivariate"){
        Z <- try(Z <- estimateDensityMulti(data[,3:(values[["nb_var"]]+2)]))
        values[["mdata"]] <- data[,3:(values[["nb_var"]]+2)]
        if(!is.numeric(Z)){
          Z <- transfo_z(data[,3])
          values[["multi2"]] <- "Univariate"
          multi <- "Univariate"
          showNotification("Error in chol.default(S) :
    the leading minor of order 1 is not positive definite.",type="error")
          showNotification("There was a error during the kernel density estimation. We consider only the first variable for the rest of the process.",type="warning",duration=40)
          saveZ <- data[,3]
          data[,3] <- Z
        }else{
          saveZ <- rep(7,length(data[,3]))
        }

      }else{
        Z <- data[,3]
        if(!ctrl=="un"){
          Z <- transfo_z(Z)
        }else{
          if(min(Z)<0){
            Z <- transfo_z(Z)
            showNotification("The var data have been changed due to negative value in third column.",type="error")
          }
        }
        saveZ <- data[,3]
        data[,3] <- Z
      }


      #on enregistre dans l'element renvoyer a l'utilisateur les parametres utilises
      X <- data[,1]
      Y <- data[,2]
      L <- data[,length(data[1,])]
      data <- data.frame(X,Y,Z)
      xg <- c()
      groupes=1
      dlim=1
      xend=1
      yend=1

      res <- NULL
      values[["Size"]] <- saveZ
      values[["C"]] <- ifelse(saveZ<0,"2, 5","1")
      W = rep(1 , length(Y))


      if (bsup <= binf) {
        showNotification("The bsup value must be higher than binf",type = "error", duration = 10)
        stop("The bsup value must be higher than binf")
      }

      label <- as.character(L)

      id <- c(1:length(data[,1]))

      if(values[["isLabel"]]){
        values[["labelall"]] <- label
      }else{
        values[["labelall"]] <- id
        label <- as.character(id)
      }


      data=cbind(data,id)
      data=as.data.frame(data)

      if( bsup > max(dist(data[,1:2])) ) {
        cat("The maximal value of dlim is: ",round(max(dist(data[,1:2])),2))
        cat("bsup have been changed in order to not run high dlim (refers to data)")
        bsup <- max(dist(data[,1:2]))
      }

      #remise a zero des objects
      names <- c()
      #initialisation matrice historique des groupes
      historique <- matrix(ncol=0, nrow=length(X))

      cat("\nInitialisation and calcul of the distance matrix ..")
      DiMatrix <- dist_m(data)
      progress1$set(value = 2)
      cat(". Done\n")
      cat("Running Progress:\n")
      values[["DiMatrix"]] <- DiMatrix
      #creation barre de progression(pb)
      pb <- txtProgressBar( min = 0, max = round((bsup - binf) / pas,0) , style = 3 )
      progress1$close()

      pre_k=0
      k = 1
      pass =0
      grp=c(1)
      data= cbind(data,grp)
      dlim <- bsup
      #running spatialpatches (WOILLEZ et al., 2009)
      Pch <- spatialpatches2(X , Y , Z , w = rep(1 , length(Y)) , Lim.D = dlim)
      grp1 <- Pch$n
      names <- c(round(dlim,2))
      historique <- cbind(historique,grp1)

      data$grp=grp

      if (!(length(unique(data$grp)) <= 1)) {
        showNotification("The maximal value of dlim is not enough to create one patch at the top of the tree.
                         The groups may be merged too early. Try higher bsup.",type = "message", duration = 10)
      }

      while ( dlim >= binf) {

        #update progress bar
        Sys.sleep(0.1)
        setTxtProgressBar(pb, k)
        progress2$set(value = k)

        #if k have not been increment
        if(k == pre_k){
          dlim <- nextdlim
        }else{
          dlim = bsup - k * pas
          #save value of k for this loop. need for chack next loop time
          pre_k <- k
        }

        if (round(dlim,2) > 0) {

          nb_grp=length(unique(data$grp))
          if(nb_grp>1){
            #More than one patch so we separate the data
            grp_tmp <- c()

            for (a in 1:nb_grp){
              # a take the different value corresponding to  the numero of cluster

              # eval permit to run the text into (in order to run, in the loop, data1 and data2 and not dataa)
              # parse permit to pass text to expression in eval(...)
              # Separtion of data into sub dataset (data1, data2, ...) order by the cluster vector Pch$n
              eval(parse(text=paste0("data",a,"=data[data$grp==",a,",]")))

              # Run spatialpatche for each of the new dataset (data1, data2, ...)
              # if number of element i is sup to 2
              if(eval(parse(text=paste0("length(data",a,"[,1])>3")))){
                eval(parse(text=paste0("Pch",a," <- spatialpatches2(data",a,"[,1],data",a,"[,2],data",a,"[,3],w = rep(1 , length(data",a,"[,2])),Lim.D = dlim)")))
              }else{
                eval(parse(text=paste0("Pch",a,"<-NULL")))
                eval(parse(text=paste0("Pch",a,"$n<-rep(1,length(data",a,"[,1]))")))
              }

              #recuperation of the cluster vector for each sub dataset (data1, data2, ...)
              eval(parse(text=paste0("data",a,"$grp=Pch",a,"$n")))
              #On ajoute a en dizaine au nombre pour ne pas generer de doublons entre les differents a
              eval(parse(text=paste0("data",a,"$grp=as.numeric(paste(",a,",data",a,"$grp,sep=''))")))

              #Si 3 groupes.
              if( eval(parse(text=paste0("length(unique(data",a,"$grp)) == 3"))) ){
                eval(parse(text=paste0("XX <- data",a,"[,1]")))
                eval(parse(text=paste0("YY <- data",a,"[,2]")))
                eval(parse(text=paste0("ZZ <- data",a,"[,3]")))

                eval(parse(text=paste0("WW <- rep(1 , length(data",a,"[,2]))")))
                eval(parse(text=paste0("xg <- tapply(WW*ZZ*XX,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                eval(parse(text=paste0("yg <- tapply(WW*ZZ*YY,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                diff1_2 <- abs(xg[1]-xg[2])
                diff1_3 <- abs(xg[1]-xg[3])
                diff2_3 <- abs(xg[2]-xg[3])
                if(diff2_3 > diff1_2 && diff2_3 > diff1_3){
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"2,",a,"2,",a,"1)")))
                }else{
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"1,",a,"1,",a,"2)")))
                }
                nextdlim <- dlim - (pas/3)
                #pass take value 1 in order to re rsun one time the loop witjhout change k and replace dlim by nextdlim.
                #This nextdlim permit to separate in two times (one dlim and one nextdlim near to dlim) the 3 patchs.
                pass <- 1
              }
              #Si plus de 3 groupes.
              if( eval(parse(text=paste0("length(unique(data",a,"$grp)) >= 4"))) ){
                eval(parse(text=paste0("XX <- data",a,"[,1]")))
                eval(parse(text=paste0("YY <- data",a,"[,2]")))
                eval(parse(text=paste0("ZZ <- data",a,"[,3]")))

                eval(parse(text=paste0("WW <- rep(1 , length(data",a,"[,2]))")))
                eval(parse(text=paste0("xg <- tapply(WW*ZZ*XX,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                eval(parse(text=paste0("yg <- tapply(WW*ZZ*YY,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                diff1_2 <- abs(xg[1]-xg[2])
                diff1_3 <- abs(xg[1]-xg[3])
                diff2_3 <- abs(xg[2]-xg[3])
                if(diff2_3 > diff1_2 && diff2_3 > diff1_3){
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"2,",a,"2,",a,"1)")))
                }else{
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"1,",a,"1,",a,"2)")))
                }
              }
              #fusion of the cluster vectors
              eval(parse(text=paste0( "grp_tmp <- rbind(grp_tmp,data",a,")" )))
            }

            grp_tmp <- grp_tmp[order(grp_tmp$id,decreasing=F), ]

            #change the cluster names (11, 21, ...) into new (1, 2, ...) more simple
            namegrp<-c()
            namegrp<-as.factor(grp_tmp$grp)
            levels(namegrp) <- c(1:length(levels(namegrp)))
            data$grp<-as.numeric(namegrp)

          }else{
            # only one patch in all so we continu on all the data
            Pch <- spatialpatches2(X , Y , Z , w = rep(1 , length(Y)) , Lim.D = dlim)
            grp <- Pch$n
            data$grp=grp
          }

          if(round(dlim,1) > 0){
            j=length(historique[1,])
            #compare last and new compo
            v <- data$grp==historique[,j]
            somme <- sum(v %in% FALSE)
            if(somme != 0){
              #if data$grp is different than the last compo hist
              #add the dlim to name and the hist
              names <- c(names , round(dlim,2))
              historique <- cbind(historique , data$grp)
            }else{
              if(round(dlim,2) < names[length(names)]){
                #IF no difference : replace the last dlim name by the new
                names[length(names)] <- round(dlim,2)
              }
            }
          }
        }
        if(pass == 1){
          #if pass egal to 1 don't increment K and re run one time the while loop. pass egal 0 for next time of loop
          pass <- 0
        }else{
          k = k + 1
        }
      }#end while

      dlim <- as.numeric(colnames(historique)[length(colnames(historique))])

      historique <- cbind(grp1,historique)
      names <- c(round(bsup,2),names)
      historique <- as.matrix(historique)
      colnames(historique) <- names
      res$dlim <- names


      eval_class <- "test"
      output$log <- renderPrint({
        eval_class
      })

      #silhouette
      Average_Sil_Width <- c()
      for(i in 1:length(historique[1,])){
        cluster <- historique[,i]
        if(length(unique(cluster))>1){
          sil <- silhouette(cluster,DiMatrix)
          avg <- mean(sil[(length(cluster)*2):(length(cluster)*3)])
          Average_Sil_Width <- c(Average_Sil_Width,avg)
        }
      }
      number <- apply(historique,2,unique)
      number <- lapply(number,length)

      Number_of_groups <- as.vector(unlist(number))[number>1]

      Average_Sil_Width <- rbind(Number_of_groups,Average_Sil_Width)
      values[["sil_width"]] <- Average_Sil_Width

      historique <- cbind(id,historique)
      hist<-c()

      cpy_historique <- historique
      cpy_historique<-cbind(label,cpy_historique)

      #On trie l'historique par la colonne du dlim minimal
      historique<-historique[order(historique[,length(historique[1,])],decreasing=F), ]
      cpy_historique<-cpy_historique[order(as.numeric(cpy_historique[,length(cpy_historique[1,])]),decreasing=F), ]


      label_ini_grp <- c()
      label_tmp <- c()
      label_avant <- c()
      label_grp <- c()

      if(extenddend){

        for(b in 2:(length(cpy_historique[,1]))){
          c=b-1
          if(!(FALSE %in% (historique[b,2:length(historique[1,])]==historique[c,2:length(historique[1,])]))){
            #Doublon
            label_tmp <- as.character(cpy_historique[b,1])
            label_avant <- as.character(cpy_historique[c,1])
            if(is.null(label_grp)){
              label_grp <- c(label_avant,label_tmp)
            }else{
              label_grp <- c(label_grp,label_avant,label_tmp)
            }
          }else{
            #Pas doublon
            hist<-rbind(hist,historique[c,])
            label_grp <- c(label_grp,as.character(cpy_historique[c,1]))
            label_grp <- paste(unique(label_grp), collapse = ".")
            label_ini_grp<-c(label_ini_grp,label_grp)
            label_grp <- c()
          }
        }

        #add the last elt
        hist<-rbind(hist,historique[length(historique[,1]),])
        label_grp <- paste(unique(label_grp), collapse = ".")
        label_ini_grp<-c(label_ini_grp,label_grp)

      }else{
        #----------------
        for(b in 2:(length(cpy_historique[,1]))){
          c=b-1
          label_grp <- c(label_grp,as.character(cpy_historique[c,1]))
          label_ini_grp<-c(label_ini_grp,label_grp)
          label_grp <- c()
        }
        hist <- historique
        #add last elt
        label_grp <- unique(as.character(cpy_historique[length(historique[,1]),1]))
        label_ini_grp <- c(label_ini_grp,label_grp)
        #----------------
      }


      #We cut the label colunms (Not use here because we work on the entire cluster the label and id of
      # each members of one cluster are still save in data)
      hist<-as.matrix(hist)
      hist<-hist[,2:length(historique[1,])]

      ecart=ecartmatrice(hist)
      values[["hist"]] <- hist
      #if un seul groupe pas de div gerer ce cas

      ##Les segments
      #creation colonne x
      x <- c()
      for (b in 1:length(hist[,1])) {
        x <- c(x , b)
      }
      hist<-as.matrix(hist)
      #creation colonne y
      y <- rep(binf , length(hist[,1]))

      #creation d'une matrice ou on stockera les coordonnees pour les segments du drendrogramme
      coord <- cbind(x , y)

      #initialisation des segments
      segment <- c()

      #boucle se repetant (nombre de grp initiaux -1) fois 3 pour ecrire les segments
      #Pour 6 grps il faut (6-1)*3=15 segments. n grp il faut (n-1)*3 segments.
      for (a in 1:(length(hist[,1])-1)) {

        #on recherche min (premi?re fusion)
        mini <- min(ecart)
        #On recherche les indices
        index_vect <- which.min(ecart)
        index <- arrayInd(index_vect,c(length(ecart[,1]),length(ecart[,1])))
        grp1 <- index[1]
        grp2 <- index[2]

        #on recupere dlim dans l'objet res$dlim qui contient tous les dlim utilises. length(..)-mini pour
        # partir de la fin et remonter. mini est le nombre de dlim a passer vers la gauche.

        if(length(res$dlim)-mini!=0){
          dlim_tmp <- res$dlim[length(res$dlim)-mini]

          #on construit 3 segments (deux qui monte et un qui fusionne branche horizontal)

          # segments verticaux
          # x = l'abscisse du tab coord pour grp correspondant  y = coord[grp_,2]    xend = x    yend = dlim_tmp
          ######################################################################################################
          segment <- rbind(segment , c(coord[grp1,1] ,     coord[grp1,2] ,  coord[grp1,1] ,         dlim_tmp))
          segment <- rbind(segment , c(coord[grp2,1] ,     coord[grp2,2] ,  coord[grp2,1] ,         dlim_tmp))

          # segment horizontal
          ##################     x = coord[grp1,1]     y = dlim_tmp     xend = coord[grp2,1]    yend = dlim_tmp
          ######################################################################################################
          segment <- rbind(segment,c(coord[grp1,1] ,       dlim_tmp ,       coord[grp2,1] ,         dlim_tmp))


          ## on sauvegarde la hauteur (y / dlim) a laquelle les grps sont rendus
          #MAJ valeur de y (dlim) grp1
          coord[grp1,2] <- dlim_tmp
          #MAJ ligne grp1, valeur en X du nouveau grp (fusion) moyenne des deux
          coord[grp1,1] <- (coord[grp1,1]+coord[grp2,1])/2
          #on marque la ligne du grp2 en mettant x=0, pour ne plus la prendre en compte dans prochaine
          # fusion puisque ce grp a fusionne avec grp1
          coord[grp2,1] <- 0
          #on elimine la ligne/colonne du grp2 dans les valeurs (historique grp)
          ecart[,grp2] <- Inf
          ecart[grp2,] <- Inf
        }
      }
      groupes=1
      dlim=1
      xend=1
      yend=1
      #on rajoute un dernier segment pour la racine de l'arbre
      segment <- rbind(segment , c(max(coord[,1]) , max(coord[,2]) , max(coord[,1]) , (max(res$dlim)+pas)))
      segment <- data.frame(segment)
      #on prepare tableau segment
      names(segment) <- c("groupes" , "dlim" , "xend" , "yend")
      row.names(segment) <- NULL


      #on creer un tableau pour les noms (implant au bout des branches y=0)
      label <- matrix(ncol = 3 , nrow = length(hist[,1]))
      for (a in 1:length(hist[,1])) {
        label[a,1] <- a
        label[a,2] <- 0
        label[a,3] <- label_ini_grp[a]
      }
      label <- data.frame(label)

      values[["label"]] <- label
      values[["dlim"]]<-res$dlim

      #dendrogramme affiche avec indication pour le clic de cut.
      plot.dend$main <- ggplot(segment) +
        geom_segment(data = segment,aes(x = groupes , y = dlim , xend = xend , yend = yend )) +
        ylim(0, max(res$dlim) + pas)
      #Title subtitle caption
      plot.dend$main <- plot.dend$main + labs(title = "Hierarchical Clustering" , subtitle = "Spatialpatches")
      #axes x
      plot.dend$main <- plot.dend$main + scale_x_continuous(breaks = c())

      plot.dend$main <- plot.dend$main + theme(
        panel.grid = element_line(colour = "grey")
      )

      historique<-historique[order(historique[,1],decreasing=F), ]
      #WSS
      Sum_of_Square <- WSS(historique[,-1],data)
      Within_Sum_of_Square <- c()
      for (i in Sum_of_Square) {
        if(length(i)>1){
          Within_Sum_of_Square <- c(Within_Sum_of_Square,sum(i))
        }
      }

      WSS2 <- c()
      for (i in Sum_of_Square) {
        WSS2 <- c(WSS2,sum(i))
      }
      values[["WSS2"]] <- WSS2

      Within_Sum_of_Square <- rbind(Number_of_groups,Within_Sum_of_Square)
      values[["WSS"]] <- Within_Sum_of_Square

      if(multi == "Univariate"){
        Moran <- IndiceMoran(historique[,-1],data)
        values[["Moran"]] <- Moran
      }


      values[["historique"]] <- historique
      values[["data"]] <- data

      output$distPlot <- renderPlot({
        values[["dendplot"]] <- plot.dend$main + plot.dend$add + plot.dend$add2
        plot.dend$main + plot.dend$add + plot.dend$add2
      })

      output$avgsilPlot <- renderPlot({
        #plot eval sil
        dataplot <- t(values[["sil_width"]])
        dataplot <- as.data.frame(dataplot)
        colnames(dataplot) <- c("number_of_groups","Average_Sil_Width")
        minisil <- min(dataplot$Average_Sil_Width)
        if(minisil > 0 ){
          minisil <- 0
        }else{
          minisil <- -1
        }
        plot <- ggplot(data=dataplot, aes(x=number_of_groups, y=Average_Sil_Width))
        plot <- plot + geom_line(linetype = 3)
        plot <- plot +   geom_point(aes(color=Average_Sil_Width), size=3) + scale_colour_gradientn(colours=c("brown1","chartreuse3"))
        plot <- plot +   scale_x_continuous(breaks = dataplot$number_of_groups,
                                            labels = dataplot$number_of_groups )
        plot <- plot +   labs(title = "Average Silhouette Width" , subtitle = "Partition evaluation")
        plot <- plot + ylim(minisil,1)
        plot <- plot + theme(
          legend.position="bottom",
          plot.background = element_rect(
            colour = "#222222",
            linewidth = 1
          )
        )
        values[["avesilplot"]] <- plot
        plot
      })

      output$WSSPlot <- renderPlot({
        #plot WSS
        dataplot <- t(values[["WSS"]])
        dataplot <- as.data.frame(dataplot)
        colnames(dataplot) <- c("number_of_groups","Within_Sum_of_Square")
        maxwws <- max(dataplot$Within_Sum_of_Square)
        plot <- ggplot(data=dataplot, aes(x=number_of_groups, y=Within_Sum_of_Square))
        plot <- plot + geom_line(linetype = 3)
        plot <- plot + geom_point(aes(color=Within_Sum_of_Square), size=3) + scale_colour_gradientn(colours=c("chartreuse3","brown1"))
        plot <- plot + scale_x_continuous(breaks = dataplot$number_of_groups ,
                                          labels = dataplot$number_of_groups )
        plot <- plot + labs(title = "Within Sum of Square" , subtitle = "Partition evaluation")
        plot <- plot + theme(
          legend.position="bottom",
          plot.background = element_rect(
            colour = "#222222",
            linewidth = 1
          )
        )
        values[["WSSplot"]] <- plot
        plot
      })
      removeModal()
      showModal(cuttreeModal())
      #TODO : add download log graph map and all data
    }else{
      removeModal()
    }

  })

  ##dendrogramme click
  output$click_info <- renderPrint({

    if(is.null(values[["index"]])){
      cat("Hierarchical Divise Clustering")
    }else{
      multi <- values[["multi2"]]
      cat(multi, " ")
      nb_var <- values[["nb_var"]]
      cat(nb_var)

      size <- c()
      data <- values[["data"]]
      if(multi=="Univariate"){
        Z <- values[["Size"]]
      }else{
        Z <-values[["mdata"]]
      }
      cluster <- values[["historique"]][,values[["index"]]+1]
      nb <- length(unique(cluster))
      tot <- length(cluster)
      X_mean <- tapply(data[,1],cluster,mean)
      X_sd <- tapply(data[,1],cluster,sd)
      Y_mean <- tapply(data[,2],cluster,mean)
      Y_sd <- tapply(data[,2],cluster,sd)
      cat("The selected partition contains",nb,"clusters among",tot,"observations.\ncluster:\t")
      for (k in 1:nb){
        size <- c(size,length(cluster[cluster == k]))
        cat(k,"\t")
      }
      cat("\nSize:\t\t")
      for(y in 1:nb){
        cat(size[y],"\t")
      }
      cat("\nX_mean:\t\t")
      for(y in 1:nb){
        cat(round(X_mean[y],2),"\t")
      }
      cat("\nX_sd:\t\t")
      for(y in 1:nb){
        cat(round(X_sd[y],2),"\t")
      }
      cat("\nY_mean:\t\t")
      for(y in 1:nb){
        cat(round(Y_mean[y],2),"\t")
      }
      cat("\nY_sd:\t\t")
      for(y in 1:nb){
        cat(round(Y_sd[y],2),"\t")
      }
      if(multi == "Univariate"){
        cat("\nvar_mean:\t")
        var_mean <- tapply(Z,cluster,mean)
        for(y in 1:nb){
          cat(round(var_mean[y],2),"\t")
        }
        cat("\nvar_sd:\t\t")
        var_sd <- tapply(Z,cluster,sd)
        for(y in 1:nb){
          cat(round(var_sd[y],2),"\t")
        }
      }else{
        for(i in 1:nb_var){
          var_mean <- tapply(Z[,i],cluster,mean)
          cat("\nvar",i,"_mean:\t",sep="")
          for(y in 1:nb){
            cat(round(var_mean[y],2),"\t")
          }
        }
      }
    }
  })

  output$click_info2 <- renderPrint({

    multi <- values[["multi2"]]

    if(is.null(values[["index"]])){
      cat("Summary")
    }else{
      cat("> summary()")
      cat("\n  -----------------------------------\n  dlim\t  n\tWSS\tsil\tMoran\n  -----------------------------------\n")
      j <- 1
      for (i in 1:length(values[["dlim"]])){
        dlim <- values[["dlim"]][i]
        cluster <- values[["historique"]][,i+1]
        number_grp <- length(unique(cluster))
        if(number_grp == 1){
          sil <- NA
        }else{
          sil <- values[["sil_width"]][2,j]
          j <- j+1
        }
        WSS <- values[["WSS2"]][i]
        if(multi == "Univariate"){
          Moran <- mean(na.omit(unlist(values[["Moran"]][[i]][1,])))
        }
        cat(" ",dlim,"\t")
        cat(" ",number_grp,"\t")
        cat(round(WSS,2),"\t")
        cat(round(sil,2),"\t")
        if(multi == "Univariate"){
          cat(round(Moran,2),"\t")
        }
        cat("\n")
      }
      cat("  -----------------------------------\n")
    }
  })



  observeEvent(input$Plabel,{
    if(input$Plabel){
      label <- values[["label"]]
      if(!is.null(values[["index"]])){
        type <- values[["hist"]][,values[["index"]]]
      }else {
        type <- rep(1,length(values[["hist"]][,1]))
      }
      couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
                   "#222222", "darkmagenta", "mediumpurple4",
                   "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
                   "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
                   "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
      pal <- colorFactor(couleur,as.factor(type))
      col <- pal(as.factor(type))
      if(!is.null(plot.dend$add)){
        plot.dend$add2 <- geom_text(data = label, aes(x = as.numeric(paste(label[,1])) ,
                                                      y = as.numeric(paste(label[,2])) ,
                                                      label = as.character(label[,3])) ,
                                    fontface = "bold" , nudge_x = -0.12 , colour = col ,
                                    hjust = 0 , vjust = 0.25 , size = 4 , angle = 90)
      }else{
        plot.dend$add2 <- geom_text(data = label, aes(x = as.numeric(paste(label[,1])) ,
                                                      y = as.numeric(paste(label[,2])) ,
                                                      label = as.character(label[,3])) ,
                                    fontface = "bold" , nudge_x = -0.12 , colour = "royalBlue4" ,
                                    hjust = 0 , vjust = 0.25 , size = 4 , angle = 90)
      }
    }else{
      plot.dend$add2 <- NULL
    }

  })


  observeEvent(input$plot_click,{
    diff <- values[["dlim"]]-input$plot_click$y
    #which.max (<=0) return index of the first value <= 0
    if((TRUE %in% (diff <= 0))){
      index <- which.max(diff <= 0)
    }else{
      index <- which.min(diff)
    }
    values[["cutdlim"]] <- values[["dlim"]][index]
    values[["index"]] <- index
    showNotification(paste("cutdlim : ",values[["cutdlim"]]),type = "message",duration = 10)

    plot.dend$add
    plot.dend$add2
    plot.dend$add <- geom_hline(yintercept = values[["cutdlim"]] + 0.02 , col = "dimgrey" , lty = 2 ,lwd = 0.5)
    if (input$Plabel) {
      if(!is.null(values[["index"]])){
        type <- values[["hist"]][,values[["index"]]]
      }else {
        type <- rep(1,length(values[["hist"]][,1]))
      }
      couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
                   "#222222", "darkmagenta", "mediumpurple4",
                   "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
                   "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
                   "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
      pal <- colorFactor(couleur,as.factor(type))
      col <- pal(as.factor(type))
      label <- values[["label"]]
      plot.dend$add2 <- geom_text(data = label, aes(x = as.numeric(paste(label[,1])) ,
                                                    y = as.numeric(paste(label[,2])) ,
                                                    label = as.character(label[,3])) ,
                                                   fontface = "bold" , nudge_x = -0.12 , colour = col ,
                                                   hjust = 0 , vjust = 0.25 , size = 4 , angle = 90)
    }else{
      plot.dend$add2 <- NULL
    }
  })

  ##leaflet map
  output$mymap <- renderLeaflet({
    #leaflet
    if(input$select == 3){
      lonlat <- values[["lonlat"]]
    }else{
      lonlat <- TRUE
    }
    if(lonlat){
      esri <- structure(c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                          "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                          "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                          "Esri.WorldGrayCanvas"),
                        names = c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                                   "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                                   "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                                   "Esri.WorldGrayCanvas"))
      m <- leaflet()
      for (provider in esri) {
        m <- m %>% addProviderTiles(provider, group = provider)
      }
      m <- m %>%
        addLayersControl(baseGroups = names(esri),
                         options = layersControlOptions(collapsed = TRUE),
                         position = "topright"
        ) %>%
        addMiniMap(tiles = esri[[1]], toggleDisplay = TRUE,minimized = TRUE,
                   position = "bottomright"
        ) %>%
        htmlwidgets::onRender("
                              function(el, x) {
                                var myMap = this;
                                myMap.on('baselayerchange',
                                function (e) {
                                  myMap.minimap.changeLayer(L.tileLayer.provider(e.name));
                                })
                              }"
        )
    }else{
      m <- leaflet()
    }
    dlmap$map <- m
  })

  observe({
    if(input$select == 3){
      lonlat <- values[["lonlat"]]
    }else{
      lonlat <- TRUE
    }
    multi <- values[["multi2"]]
    couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
                 "#222222", "darkmagenta", "mediumpurple4",
                 "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
                 "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
                 "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
    if(is.null(values[["data"]])){
      data <- datarcheo
    }else{
      data <- values[["data"]]
    }
    if(!is.null(values[["index"]])){
      type <- values[["historique"]][,values[["index"]]+1]
    }else {
      type <- rep(1,240)
    }
    Size <- values[["Size"]]
    if(!is.null(Size)){
      sign = FALSE
      if(min(Size)<0){sign = TRUE}
      C <- values[["C"]]
      data[,3] <- as.numeric(Size)

      if(multi == "Univariate"){
        Size <- 10*(0.1+(abs(data[,3])/max(abs(data[,3]))))+5
      }

      pal <- colorFactor(couleur,as.factor(type))
      col <- pal(as.factor(type))
      col2 <- data.frame(col,type)
      col2 <- col2[order(col2[,2],decreasing=F), ]
      values[["col"]] <- as.character(col2[,1])
      leafletProxy("mymap", data = data) %>%
        clearMarkers() %>% clearControls
        if(input$Plabel){
          leafletProxy("mymap", data = data) %>% addCircleMarkers(lng = data[,1], lat =data[,2], popup = ~htmlEscape(values[["labelall"]]), radius = Size, fillColor =  col,
                                      fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=C
          )
        }else{
          leafletProxy("mymap", data = data) %>% addCircleMarkers(lng = data[,1], lat = data[,2], radius = Size, fillColor = col,
                                      fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=C
          )
        }
        leafletProxy("mymap", data = data) %>%
        fitBounds(lat1 = max(data[,2]), lng1 = max(data[,1]), lat2 = min(data[,2]), lng2 = min(data[,1])
        ) %>%
        addLegend("bottomleft", pal = pal, values = as.factor(type),
                  title = "Cluster",
                  opacity = 1
        )

      if(multi == "Univariate"){
        leafletProxy("mymap", data = data) %>%
          addLegendCustom(colors = c("black", "black", "black"),
                          labels = c(round(min(abs(data[,3])),2),
                                     round(min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2  ,2),
                                     round(max(abs(data[,3])),2)),
                          sizes = c(10*(0.1+( min(abs(data[,3])) /max(abs(data[,3]))))+5,
                                    10*(0.1+((min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2) /max(abs(data[,3]))))+5,
                                    10*(0.1+( max(abs(data[,3])) /max(abs(data[,3]))))+5),
                          title = "Size"
          )
      }
    }
    if(!is.null(Size)){
      if(sign){
        leafletProxy("mymap", data = data) %>%
          addLegendCustom(colors = c("white", "white"), labels = c("line: positive","dotted-line: negative"),
                          sizes = 1, title="Sign"
          )
      }
    }
    if(!is.null(Size)){
      if(lonlat){
        esri <- structure(c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                            "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                            "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                            "Esri.WorldGrayCanvas"),
                          names = c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                                     "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                                     "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                                     "Esri.WorldGrayCanvas"))
        m = leaflet()
        for (provider in esri) {
          m <- m %>% addProviderTiles(provider, group = provider)
        }
        m = m %>%
          addLayersControl(baseGroups = names(esri),
                           options = layersControlOptions(collapsed = TRUE),
                           position = "topright"
          ) %>%
          addMiniMap(tiles = esri[[1]], toggleDisplay = TRUE,minimized = TRUE,
                     position = "bottomright"
          ) %>%
          htmlwidgets::onRender("
                                function(el, x) {
                                var myMap = this;
                                myMap.on('baselayerchange',
                                function (e) {
                                myMap.minimap.changeLayer(L.tileLayer.provider(e.name));
                                })
                                }")
      }else{
        m <- leaflet()
      }

      m <- m %>%
      addCircleMarkers(lng = data[,1], lat = data[,2], radius = Size, fillColor =  col,
                       fill = T, fillOpacity = 0.8, opacity=1, stroke = T,weight = 1.5, color="black", dashArray=C
      ) %>%
      fitBounds(lat1 = max(data[,2]), lng1 = max(data[,1]), lat2 = min(data[,2]), lng2 = min(data[,1])
      ) %>%
      addLegend("bottomleft", pal = pal, values = as.factor(type),
                title = "Cluster",
                opacity = 1
      )
      if(multi == "Univariate"){
        m <- m %>%
          addLegendCustom(colors = c("black", "black", "black"),
                          labels = c(round(min(abs(data[,3])),2),
                                     round(min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2  ,2),
                                     round(max(abs(data[,3])),2)),
                          sizes = c(10*(0.1+( min(abs(data[,3])) /max(abs(data[,3]))))+5,
                                    10*(0.1+((min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2) /max(abs(data[,3]))))+5,
                                    10*(0.1+( max(abs(data[,3])) /max(abs(data[,3]))))+5),
                          title = "Size"
          )
      }
      if(sign){
        m <- m %>%
          addLegendCustom(colors = c("white", "white"), labels = c("line: positive","dotted-line: negative"),
                          sizes = 1, title="Sign"
          )
      }
      dlmap$map <- m
    }

  })

  ##silhouette plot
  observe({
    if(!is.null(values[["index"]])){
      type <- values[["historique"]][,values[["index"]]+1]
      DiMatrix = values[["DiMatrix"]]
      if(length(as.numeric(type)) != dim(DiMatrix)[1]){
        output$silPlot <- renderPlot({},height = 1)
      }else{
        if(length(unique(type)) == 1){
          showNotification("You have to create more than one patch. The \"Silhouette plot\" can't be calculate for one cluster.", type="warning", duration=10)
          output$silPlot <- renderPlot({},height = 1)
        }else{
          output$silPlot <- renderPlot({
            cluster = type
            DiMatrix = values[["DiMatrix"]]
            sil <- silhouette(cluster,DiMatrix)
            plot(sil,col=values[["col"]])
            values[["silhouetteplot"]] <- sil
          },height = function(){2.5*length(type)})
        }
      }
    }else{
      output$silPlot <- renderPlot({},height = 1)
    }
  })



  #EXPORT BUTTON
  output$all.pdf <- downloadHandler(
    filename = "mapclust.pdf",
    content = function(file) {
      cairo_pdf(file,onefile=T)
      print(values[["dendplot"]])
      print(values[["avesilplot"]])
      print(values[["WSSplot"]])
      if(!is.null(values[["silhouetteplot"]])){
        plot(values[["silhouetteplot"]],col=values[["col"]])
      }
      dev.off()
    }
  )

  }

library(SPARTAAS)
suppressMessages(library(shinyjs))
library(shinyjqui)
suppressMessages(library(shinydashboard))
library(colorspace)
suppressMessages(library(dplyr))
library(tidyr)
library(ggplot2)
library(shinyWidgets)
library(FactoMineR)
library(scatterD3)
library(dplyr)
library(fpc)
suppressMessages(library(plotly))
library(cluster) #for silhouette
library(stringr) #sub_extract
library(shinythemes)

ui <- fluidPage(useShinyjs(),withMathJax(),style="padding-top: 150px;",theme = shinytheme("flatly"),
    tags$style(HTML('#sort,#data,#subdivise,#tri,#sub1,#sub2,
         #import2,#import3,#import3,#Finishimport,#back1,#back2{color:white;background-color:#337ab7;border-color:#337ab7}
         #back{color:white;background-color:#f0ad4e}
         #tabsetperso{background-color:#e5e5e5;}
         #help{text-align;right;}
         #tabsetperso{margin-bottom:20px;}
         .btn-default {
                    background-color:#5da3a8;
                    border-color:#5da3a8;
         }
                  .dropdown-menu > li > a {
                    background-color:#fff;
                    border-color:#dce4ec;
         }
    ')),
    tags$head(
      tags$link(rel = "icon", type = "image/gif", href = "https://spartaas.gitpages.huma-num.fr/r-package/img/lambda.png"),
      tags$title("hclustcompro")
    ),
  #header panel
  absolutePanel(class = "panel panel-default",
    style="z-index: 2000;padding: 8px; background: #ecf0f1; opacity: 1;border-bottom: 1px solid #2c3e50;",
    top = 0, left = 0, right = 0,
    fixed = TRUE,
    HTML("<h2>hclustcompro: <b>h</b>ierarchical <b>clust</b>ering by <b>compro</b>mise</h2>"),
    div(span(strong("SPARTAAS | hclustcompro")))
  ),
  #footer
  absolutePanel(style="z-index: 2000;padding: 0px; border-bottom: 0px solid #CCC; background: #fff;opacity: 1;",
                bottom = 0, left = 0,
                fixed = TRUE,
                div(span(strong("SPARTAAS")),span("[Bellanger,Coulon,Husi]",style ="font-size:14px;"))
  ),
  #go top panel
  absolutePanel(class = "panel-hover",
    style="opacity:0.8;z-index: 1000;padding: 8px; border-bottom: 1px solid #CCC;
    background: #FFFFEE;width:30px;margin-right:10px;",
    bottom = 20, right = 0,
    fixed = TRUE,
    HTML("<a href=\"#top\"><span class=\"glyphicon glyphicon-chevron-up\"></span></a>")
  ),
  #-----------------------------------#
  #    NavBar
  #-----------------------------------#
  navbarPage("hclustcompro",id="navbar",
    tabPanel(value = "1",span(icon("home", lib = "glyphicon"),strong("Home")),style="max-width:1200px;",
             br(),
             sidebarLayout(
               #-----------------------------------#
               sidebarPanel(style="",
                            h1("R Package:"),
                            br(),
                            HTML("<p>This method is part of the <a target='_blank' href='https://spartaas.gitpages.huma-num.fr/r-package/index.html'>SPARTAAS</a> package.</p>
                                                        <p>If you are interested you can install our R package available on the <a href='https://cran.r-project.org/package=SPARTAAS'>CRAN</a>.</p>

                                                        ")

               ),
               #-----------------------------------#
               mainPanel(
                 HTML("
                                              <h3>Introduction</h3>
                                              <p>hclustcompro is a hierarchical clustering that takes into account two sources of information associated with the same objects. This method, called CAH by compromise (hclustcompro), allows a compromise between the hierarchies obtained from each source considered separately. A convex combination of the dissimilarities associated with each of the sources is used to modify the dissimilarity measure in the classical CAH algorithm. The choice of the mixing parameter is the key point of the method. We propose an objective function to be minimised based on the absolute difference of the correlations between the initial dissimilarities and the kinetic distances, as well as a resampling procedure to ensure the robustness of the choice of the mixing parameter.</p>
                                              ")
               )
             )
    ),
    tabPanel(value = "2",span(icon("chart-area", lib = "font-awesome"),strong("hclustcompro")),style="max-width:1200px;",
              tabsetPanel(id = "tabsetperso",type="tabs",
                      tabPanel(id="tab11","Overview",
                               sidebarPanel(width = 5,
                                            h4("Import your data:"),actionButton("data",
                                                                                 icon("import", lib = "glyphicon")),
                                            h4("Refresh app:"),actionButton("refresh",
                                                                            icon("refresh", lib = "glyphicon")),
                                            hr(style="border-color: #222222;"),
                                            h4("guide lines"),
                                            HTML("<p>You have three subtabs to run hclustcompro:</p>"),
                                            actionLink(inputId = "over", label = "Overview (here)"),
                                            HTML("<ul>
                                         <li>Import your data <br>(you can skip this step and
                                         test hclustcompro with the default dataset)</li>
                                         </ul>"),
                                            actionLink(inputId = "clustsett", label = "Clustering settings"),
                                          HTML("<ul>
                                         <li>Select the correct number of axes in the correspondence analysis</li>
                                         <li>Select the alpha value</li>
                                         <li>Select the number of groups <br>The optimal number of groups button helps you with two evaluation scores (e.g. WSS: within sum of squares and average silhouette score)</li>
                                         </ul>"),
                                          actionLink(inputId = "resviz", label = "Results visualization"),
                                          HTML("<ul>
                                         <li>Sort the clusters on the seriograph in chronological order (Sort clusters button)</li>
                                         </ul><br><br>
                                              ")

                               ),
                               mainPanel(style = 'max-width:1000px;',width = 7,
                                         h4("The data"),
                                         HTML("<p>hclustcompro uses two types of input data. The first is a contingency table (counting ceramics or other furniture). The second can be either stratigraphic relationships or dates (time range) for the same individuals as the first source.</p>
                                              <p>See example below.</p>"),
                                         actionLink(inputId = "wiki", label = "Go to wiki for more detail"),
                                         h4("First data source (contingency table)"),
                                         hr(style="border-color: #222222;"),
                                         tableOutput("data1f"),
                                         h4("Second data source (Stratigraphic connection or Time range data)"),
                                         hr(style="border-color: #222222;"),
                                         h5("Stratigraphic connection:"),
                                         tableOutput("data2f"),
                                         h5("Timerange data:"),
                                         tableOutput("data3f")
                                )
                      ),
             tabPanel(id="tab12","Clustering settings",
               sidebarPanel(
                            h4("Change number of axes in correspondence analysis:"),actionButton("axesCA", "Change"),
                            hr(style="border-color: #c2c2c2;"),
                            column(7,
                                   h4("Result of correspondence analysis:"),
                                   dropdownButton(
                                     h3("Correspondence analysis"),
                                     numericInput("axe1", "Axe X", 1, min = 1, max = NA, step = 1),
                                     numericInput("axe2", "Axe Y", 2, min = 1, max = NA, step = 1),
                                     scatterD3Output("CA"),
                                     icon = icon("info"), width = "600px", size = "sm",
                                     right=FALSE
                                   )
                            ),
                            div(style="height:1260px;"),
                            selectInput("method", label = "Aggregation method:",
                                        choices = list(
                                          "ward.D2" = 2,
                                          "complete" = 4,
                                          "single" = 3,
                                          "average (= UPGMA)" = 5,
                                          "mcquitty (= WPGMA)" = 6 #,"median (= WPGMC)" = 7,"centroid (= UPGMC)" = 8
                                        ), selected = 2),




                            column(7,
                                   actionButton("subdivise",span(icon("scissors",lib="font-awesome"),"Subdivide"))
                            ),
                            column(5,
                                   actionButton("reset",span(icon("rotate-left",lib="font-awesome"),"Reset"))
                            ),div(style="height:40px;")


                            ),
               mainPanel(

                         ####################################
                         ##        SelectAlpha PLOT        ##
                         ####################################
                         br(),br(),
                         h3("Select Alpha"),
                         hr(style="border-color: #222222;"),
                         column(11,
                                dropdownButton(
                                  downloadButton("alpha.pdf","Save as pdf"),
                                  hr(style="border-color: #222222;"),
                                  downloadButton("alpha.png","save as png"),
                                  numericInput("width1", "width", 900, min = 0, max = NA, step = 1, width = NULL),
                                  numericInput("height1", "height", 500, min = 0, max = NA, step = 1, width = NULL),
                                  circle = TRUE, status = "primary",
                                  icon = icon("floppy-disk", lib = "glyphicon"), width = "150px",
                                  size = "sm",
                                  tooltip = tooltipOptions(title = "Download")
                                )
                         ),
                         column(1,
                                # Help
                                dropdownButton(
                                  h3("Select Alpha"),
                                  hr(style="border-color: #222222;"),
                                  p("A criterion for the choice of alpha IN [0;1] must be determined by balancing the weights between the two sources of information in the final classification."),
                                  p("The CorCrit_alpha criterion represents the difference in absolute value between two cophenetic correlations (cophenetic correlation is defined as the correlation between two distance matrices. It is calculated by considering the half distance matrices as vectors. It measures how faithfully a dendrogram preserves the pairwise distances between the original unmodeled data points). The first correlation is related to the comparison between D1 and ultrametric distances from the clustering with alpha fixed, while the second compares D2 and ultrametric distances from the clustering with alpha fixed. As a compromise between the information provided by D1 and D2, we then decided to estimate alpha with hat(alpha) as the minimum."),
                                  circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                  size = "sm",right=TRUE,
                                  tooltip = tooltipOptions(title = "Help", placement = "top")
                                )
                                # Help
                         ),br(),br(),

                         jqui_resizable(plotOutput("selectAlpha")),br(),


                         sliderInput("alpha", label = "Alpha", min = 0, max = 1, step = 0.01, value = 0),


                         br(),
                         uiOutput("formula"),
                         #HTML("<p><img style=\"width:200px;\" src=\"GS/formule.png\"></p>"),


                         ####################################
                         #            NB GROUPE             #
                         ####################################
                         br(),br(),
                         h3("Select the number of groups in the partition"),
                         hr(style="border-color: #222222;"),
                         column(width=6,
                                column(10,
                                       dropdownButton(
                                         downloadButton("silhouette.pdf","Save as pdf"),
                                         hr(style="border-color: #222222;"),
                                         downloadButton("silhouette.png","save as png"),
                                         circle = TRUE, status = "primary",
                                         icon = icon("floppy-disk", lib = "glyphicon"), width = "150px",
                                         size = "sm",
                                         tooltip = tooltipOptions(title = "Download")
                                       ),
                                ),
                                column(1,
                                       # Help
                                       dropdownButton(
                                         h3("Average silhouette widths"),
                                         hr(style="border-color: #222222;"),
                                         p("This graph shows the average silhouette width of each partition (Rousseeuw 1987). The silhouette width is a bounded index between -1 and 1, calculated for each observation. The closer the value is to 1, the better the observation is classified. We look for the average value for a partition that is closest to 1."),
                                         circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                         size = "sm",right=TRUE,
                                         tooltip = tooltipOptions(title = "Help", placement = "top")
                                       )
                                       # Help
                                ),

                                plotOutput("silhouette")
                         ),

                         column(width=6,
                                column(10,
                                      dropdownButton(
                                        downloadButton("wss.pdf","Save as pdf"),
                                        hr(style="border-color: #222222;"),
                                        downloadButton("wss.png","save as png"),
                                        circle = TRUE, status = "primary",
                                        icon = icon("floppy-disk", lib = "glyphicon"), width = "150px",
                                        size = "sm",
                                        tooltip = tooltipOptions(title = "Download")
                                      )
                                ),
                                column(1,
                                       # Help
                                       dropdownButton(
                                         h3("WSS"),
                                         hr(style="border-color: #222222;"),
                                         p("This is the plot of the within group sum of squares against the number of clusters. The within group sum of squares decreases as the number of clusters increases. In this plot, the best partition is when adding one or more clusters doesn't decrease the WSS value. It's called the elbow method."),
                                         circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                         size = "sm",right=TRUE,
                                         tooltip = tooltipOptions(title = "Help", placement = "top")
                                       )
                                       # Help
                                ),
                                plotOutput("WSS")
                         ),div(style="height:500px;"),

                         sliderInput("k",label ="Number of class",min = 2,max = 20,step = 1, value = 2),


                         ####################################
                         #         Dendrogram PLOT          #
                         ####################################
                         h3("Dendrogram"),
                         hr(style="border-color: #222222;"),
                         verbatimTextOutput("click_info"),
                         column(11,
                                dropdownButton(
                                  downloadButton("dendrogram.pdf","Save as pdf"),
                                  hr(style="border-color: #222222;"),
                                  downloadButton("dendrogram.png","save as png"),
                                  numericInput("width2", "width", 900, min = 0, max = NA, step = 1, width = NULL),
                                  numericInput("height2", "height", 500, min = 0, max = NA, step = 1, width = NULL),
                                  circle = TRUE, status = "primary", icon = icon("floppy-disk", lib = "glyphicon"),
                                  width = "150px",
                                  size = "sm",
                                  tooltip = tooltipOptions(title = "Download")
                                )
                         ),
                         column(1
                         ),br(),br(),
                         shinyjqui::jqui_resizable(plotOutput("dendrogramme")),br(),br(),



                         hr(style="border-color: #c2c2c2;"),
                         br(),
                         column(11,
                                dropdownButton(
                                  downloadButton("silplot.pdf","Save as pdf"),
                                  hr(style="border-color: #222222;"),
                                  downloadButton("silplot.png","save as png"),
                                  numericInput("width4", "width", 900, min = 0, max = NA, step = 1, width = NULL),
                                  numericInput("height4", "height", 500, min = 0, max = NA, step = 1, width = NULL),
                                  circle = TRUE, status = "primary", icon = icon("floppy-disk",
                                                                      lib = "glyphicon"), width = "150px",
                                  size = "sm",
                                  tooltip = tooltipOptions(title = "Download")
                                )
                         ),
                         column(1,
                                # Help
                                dropdownButton(
                                  h3("Silhouette plot"),
                                  hr(style="border-color: #222222;"),
                                  p("Silhouette refers to a method of interpretation and validation of
                                    consistency within clusters of data."),
                                  p("The silhouette value is a measure of how similar an object is to
                                    its own cluster (cohesion) compared to other clusters (separation).
                                    The silhouette ranges from −1 to +1, where a high value indicates
                                    that the object is well matched to its own cluster and poorly matched
                                    to neighboring clusters. If most objects have a high value, then the
                                    clustering configuration is appropriate. If many points have a low or
                                    negative value, then the clustering configuration may have too many or
                                    too few clusters. "),
                                  circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                  size = "sm",right=TRUE,
                                  tooltip = tooltipOptions(title = "Help", placement = "top")
                                )
                                # Help
                         ),br(),br(),
                         jqui_resizable(plotOutput("SilPlot"))
                         )
             ),
             tabPanel(id="tab13","Results visualization",
                      div(id="nodata",
                          HTML("<br><p><strong>Warnings:</strong> You must go to the clustering settings tab to set up the clustering first.</p>")
                          ),
                      div(id="toggle",
                          sidebarPanel(style = 'max-width:300px;',
                                       div(style="height:200px;"),
                                       prettyToggle(
                                         inputId = "permute",
                                         label_on = "Seriation",
                                         label_off = "Seriation",
                                         icon_on = icon("check"),
                                         icon_off = icon("trash"),
                                         value = TRUE
                                       ),
                                       selectInput("show", label = "Show", choices = list("both" = 1,
                                                                                          "frequencies" = 2,
                                                                                          "EPPM" = 3),
                                                   selected = 1),
                                       actionButton("sort",span(icon("sort",lib="font-awesome"),"Sort cluster"))
                          ),
                          mainPanel(style = 'max-width:1000px;',
                                    ####################################
                                    ##        Seriograph PLOT         ##
                                    ####################################
                                    br(),br(),
                                    h3("Seriograph"),
                                    hr(style="border-color: #222222;"),
                                    column(11,
                                           dropdownButton(
                                             downloadButton("seriograph.pdf","Save as pdf"),
                                             hr(style="border-color: #222222;"),
                                             downloadButton("seriograph.png","save as png"),
                                             numericInput("width3", "width", 900, min = 0, max = NA,
                                                          step = 1, width = NULL),
                                             numericInput("height3", "height", 500, min = 0, max = NA,
                                                          step = 1, width = NULL),
                                             circle = TRUE, status = "primary", icon = icon("floppy-disk",
                                                                    lib = "glyphicon"), width = "150px",
                                             size = "sm",
                                             tooltip = tooltipOptions(title = "Download")
                                           )
                                    ),
                                    column(1,
                                           # Help
                                           dropdownButton(
                                             h3("The seriograph (Desachy 2004)"),
                                             hr(style="border-color: #222222;"),
                                             p("This tool makes it possible to highlight artisanal
                                               evolutions over time as well as to understand commercial
                                               relations thanks to imported ceramics. The percentages of
                                               each ceramic category are displayed. The percentages are
                                               calculated independently for each class. The percentage display
                                               allows you to compare the different classes but does not provide
                                               information on the differences in terms of enrolment. To fill
                                               this gap,
                                               the proportion of each class of their workforce is displayed on
                                               the seriograph (weight column). We have generalized this
                                               representation
                                               for other contingency data with hclustcompro object."),
                                             circle = TRUE, status = "danger", icon = icon("question"),
                                             width = "500px",
                                             size = "sm",right=TRUE,
                                             tooltip = tooltipOptions(title = "Help", placement = "top")
                                           )
                                           # Help
                                    ),br(),br(),

                                    jqui_resizable(plotOutput("EPPM")),
                                    downloadButton("table_serio.csv", "Download the contingency table"),
                                    div(style="height:20px;"),
                                    ####################################
                                    ##         timerange PLOT         ##
                                    ####################################
                                    conditionalPanel(
                                      condition = "1 == 0",
                                      checkboxInput("test","test",value = TRUE)
                                    ),
                                    conditionalPanel(
                                      condition = "input.datatype == 2 && input.test",
                                      br(),br(),
                                      h3("Timerange plot"),
                                      hr(style="border-color: #222222;"),
                                      column(11

                                      ),
                                      column(1,
                                             # Help
                                             dropdownButton(
                                               h3("Timerange clust"),
                                               hr(style="border-color: #222222;"),
                                               p("Each observation has a timerange estimation. We display the
                                                 timeranges grouping by cluster"),
                                               circle = TRUE, status = "danger", icon = icon("question"),
                                               width = "300px",
                                               size = "sm",right=TRUE,
                                               tooltip = tooltipOptions(title = "Help", placement = "top")
                                             )
                                             # Help
                                      ),br(),br(),
                                      jqui_resizable(plotlyOutput("timerange")),
                                      div(style="height:20px;"),

                                    )

                          )
                      )
             )
          )
    ),
    tabPanel(value = "3",span(icon("envelope", lib = "glyphicon"),strong("Contact")),style="max-width:1200px;",
      HTML("<style>
        .carte{
        border-left-width: 4px;
        border-left-style: solid;
        border-color: #2ac0a2;
        padding-left: 10px;
        }
        </style>
        <h3>Authors:</h3>
        <hr>
        <div class=\"carte\"><h4>L. Bellanger</h4><h5>mail: &lt;lise.bellanger@univ-nantes.fr&gt;</h5></div>
        <div class=\"carte\"><h4>P. Husi</h4><h5>mail: &lt;philippe.husi@univ-tours.fr&gt;</h5></div>
        <div class=\"carte\"><h4>A. Coulon</h4></div>
        <h3>Maintainer:</h3>
        <hr>
        <div class=\"carte\"><h4>A. Coulon</h4><h5>mail: &lt;arthur-coulon@outlook.fr&gt;</h5></div>
        ")
    ),
    #-----------------------------------#
    #    Wiki
    #-----------------------------------#
    tabPanel(value = "4",span(icon("question-sign", lib = "glyphicon"),strong("Wiki")),style="max-width:1200px;",
             HTML("<style>body {text-align: justify}</style><h1>Get started with the hclustcompro application</h1>
                                      <h2>Table of content</h2>
                                      <div id=\"TOC\">
                                      <ul>
                                      <li><a href=\"#introduction\">Introduction</a>
                                        <ul>
                                          <li><a href=\"#hclustcompro\">hclustcompro</a></li>
                                          <li><a href=\"#CAdist\">Construction of D1</a></li>
                                          <li><a href=\"#adjacency\">Construction of D2 for spatial data</a></li>
                                          <li><a href=\"#overlap\">Construction of D2 for temporal data</a></li>
                                          <li><a href=\"#corcrit\">Correlation Criterion</a></li>
                                        </ul>
                                      </li>
                                      <li><a href=\"#data1\">The inputs</a>
                                        <ul>
                                          <li><a href=\"#alpha\">Alpha</a></li>
                                          <li><a href=\"#k\">Number of class</a></li>
                                          <li><a href=\"#aggregation\">Aggregation method</a></li>
                                          <li><a href=\"#subdivide\">Subdivide</a></li>
                                          <li><a href=\"#seriation\">Seriation</a></li>
                                          <li><a href=\"#weight\">Weight color indicator</a></li>
                                          <li><a href=\"#show1\">Vizualisation</a></li>
                                          <li><a href=\"#sort1\">Sort clusters</a></li>
                                          <li><a href=\"#import\">Import your data</a></li>
                                          <li><a href=\"#csv\">CSV Format and write.table</a></li>
                                        </ul>
                                      </li>
                                      <li><a href=\"#outputs\">The outputs</a>
                                        <ul>
                                          <li><a href=\"#eval\">Evaluation Plot</a>
                                            <ul>
                                              <li><a href=\"#wssplot\">Within Sum of Square Plot</a></li>
                                              <li><a href=\"#avesilplot\">Average Silhouette Plot</a></li>
                                            </ul>
                                          </li>
                                          <li><a href=\"#selectalpha\">Select alpha</a></li>
                                          <li><a href=\"#dendrogram\">Dendrogram</a></li>
                                          <li><a href=\"#seriograph\">Seriograph</a></li>
                                          <li><a href=\"#timerangeclust\">Timerange</a></li>

                                          <li><a href=\"#CAp\">Correspondences analysis</a></li>

                                        </ul>
                                      </li>

                                      <li><a href=\"#references\">References</a></li>
                                      </ul>
                                      </div>

<hr>
<h2 id=\"introduction\">Introduction</h2>
<hr>
<p>Hierarchical clustering method by compromise. The method uses two sources of information.</p>

<h3 id=\"hclustcompro\">hclustcompro</h3>
<p>The merging of the two data sources is done by a parameter (alpha) that allows to weight each source.</p>
<p>$$ \\boldsymbol{D}_\\alpha=\\alpha \\boldsymbol{D}_1+(1-\\alpha) \\boldsymbol{D}_2 $$</p>

<p>The first is a contingency table. The rows must be individuals (site, ...) and the columns must be categories (type, ...). The second concerns the same individuals and can be stratigraphic connection type or time range data. The stratigraphic link object is a data frame with two columns. The first column contains the network elements (the same number as the number of lines in D1) and the second column contains a list of all the other elements connected to it. The list is a string of element names separated by commas.
</p><p><img src=\"GS/network.png\"></p>
<p>The data frame for time range data contains the same first column. The second column contains the lower time limit and the third column contains the upper time limit.</p>
<p><img src=\"GS/timerange.png\"></p>

<h3 id=\"CAdist\">Construction of D1</h3>
<p>Our first source of information is a contingency table. We need to construct a distance matrix from this table. We perform a correspondence analysis (CA) on the contingency table and then use the distances of the components (chi-square metric).</p>

<p>You must select the number of axes (CA components) to be used to construct the distance matrix.</p>

<p>By examining the eigenvalues, it is possible to determine the number of principal axes to be considered. The eigenvalues correspond to the amount of information retained by each axis.</p>

<p>A heuristic method is often used: one constructs the graph of eigenvalues sorted in descending order, retaining the eigenvalues (and associated principal components) that precede the first 'elbow'.</p>

<p>Another approach is based on maintaining an approximation quality of the original data, measured by the explained inertia rate (e.g. 85%). As many axes as necessary are chosen so that the sum of the corresponding eigenvalues exceeds the target inertia rate.</p>

<p><img src=\"GS/ncp.png\"></p>
<p><img src=\"GS/ncpslider.png\"></p>

<h3 id=\"adjacency\">Construction of D2 for stratigraphic connection</h3>
<p>From this data set, we construct an adjacency matrix. Based on this matrix, we generate a dissimilarity matrix. The matrix contains only 0 or 1, 1 if there is no relationship and 0 if there is a relationship.</p>


<h3 id=\"overlap\">Construction of D2 for time range data</h3>
<p>The overlap index is the ratio between the overlap or separation and the cumulative extend of both individuals. We define the cumulative extend of both as: the minimum of the lower limits of the pair of individuals and the maximum of the upper limits. We define the overlap or separation as: the maximum of the lower limits and the minimum of the upper limits.</p>
<p>From this ratio, we construct a dissimilarity by setting the value between 0 and 1, ensuring that the closer the value is to 1, the further apart the time ranges are.</p>
<p><img style=\"width:500px;\" src=\"GS/overlap.png\"></p>

<h3 id=\"corcrit\">Correlation Criterion</h3>
<p>A criterion for choosing alpha IN [0;1] must be determined by balancing the weights between the two
sources of information in the final classification. To obtain alpha, we define the following criterion:
</p><p>$$\\operatorname{CorCrit}_\\alpha=\\left|\\operatorname{Cor}\\left(\\mathbf{D}_\\alpha^{\\text {coph }}, \\mathbf{D}_1\\right)-\\operatorname{Cor}\\left(\\mathbf{D}_\\alpha^{\\text {coph }}, \\mathbf{D}_2\\right)\\right|
$$</p>
The CorCrit_alpha criterion in (1) represents the difference in absolute value between two cophenetic correlations (cophenetic correlation is defined as the correlation between two distance matrices. It is calculated by considering the half distance matrices as vectors. It measures how faithfully a dendrogram preserves the pairwise distances between the original unmodeled data points). The first correlation is related to the comparison between D1 and ultrametric distances from the clustering with alpha fixed, while the second compares D2 and ultrametric distances from the clustering with alpha fixed. Then, in order to compromise between the information provided by D1 and D2, we decided to estimate alpha with hat(alpha) such that:</p>
<p>$$
\\hat{\\alpha}=\\min _\\alpha \\text { CorCrit }{ }_\\alpha
$$</p>



<hr>
<h2 id=\"data1\">The inputs</h2>
<hr>
<h3 id=\"alpha\">Alpha</h3>
<p>The slider input allows you to select the value of the mixing parameter. You select the weight to be given to the first dissimilarity matrix (alpha) and the second (1-alpha). By default, the value is the one previously defined when estimating using the correlation criterion.</p>
<p><img src=\"GS/alpha.png\"></p>

<h3 id=\"k\">Number of class</h3>
<p>The slider input defines the number of clusters you want. To help you decide, you can view the WSS plot and silhouette information.</p>
<p><img src=\"GS/k.png\"></p>

<h3 id=\"aggregation\">Aggregation method</h3>
<p>The hierarchical clustering can use several aggregation methods when constructing the dendrogram. You can choose any of the methods, except the ones with the risk of inversion (if you know what you are doing, you can use the package version for this).</p>
<p><img src=\"GS/aggregation.png\"></p>

<h4>Ward.D2</h2>
<p>Ward's method is a criterion used in hierarchical cluster analysis. Ward's minimum variance method is a special case of the objective function approach originally proposed by Joe H. Ward, Jr. Ward proposed a general agglomerative hierarchical clustering procedure where the criterion for selecting the pair of clusters to merge at each step is based on the optimal value of an objective function.</p>

<h4>complete</h2>
<p>At each step, the two clusters separated by the shortest distance are combined. In complete-linkage clustering, the link between two clusters contains all element pairs, and the distance between clusters is equal to the distance between the two elements (one in each cluster) that are farthest apart. The shortest of these links remaining at each step causes the two clusters whose elements are involved to merge.</p>

<h4>single</h2>
<p>The distance between two clusters is determined by a single pair of elements, namely the two elements (one in each cluster) that are closest to each other. The shortest of these links remaining at any step causes the two clusters whose elements are involved to merge.</p>

<h4>average (UPGMA)</h2>
<p>UPGMA (unweighted pair group method with arithmetic mean) is a simple agglomerative (bottom-up) hierarchical clustering method. It is generally credited to Sokal and Michener. The UPGMA method is similar to its weighted variant, the WPGMA method. Note that the term 'unweighted' indicates that all distances contribute equally to each mean computed, and does not refer to the mathematics by which it is obtained.
<br><br>
At each step, the two closest clusters are combined to form a higher level cluster. The distance between any two clusters A and B, of size (i.e. cardinality) | A | and | B | respectively, is taken to be the average of all distances d ( x , y ) between pairs of objects x in A and y in B, i.e. the average distance between the elements of each cluster.</p>

<h4>mcquitty (WPGMA)</h2>
<p>WPGMA (Weighted Pair Group Method with Arithmetic Mean) is a simple agglomerative (bottom-up)
hierarchical clustering method, generally attributed to Sokal and Michener.
The WPGMA method is similar to its unweighted variant, the UPGMA method.
<br><br>
At each step, the closest two clusters, say A and B, are combined into a higher-level cluster A ∪ B. Then,
its distance to another cluster C, d ( A U B , C ), is simply the arithmetic mean of the distances d ( A , C )
and d ( B , C ).</p>

<h3 id=\"subdivide\">Subdivide</h3>
<p>You can subdivide a cluster by clicking on the button, selecting the cluster you want to subdivide and selecting the number of subclusters. You can reset the dendrogram using the reset button next to the dendrogram.</p>
<p><img src=\"GS/subdivide.png\"></p>

<h3 id=\"seriation\">Seriation</h3>
<p>This input allows you to enable or disable the seriation of the columns. The matrix permutation uses an algorithm called 'reciprocal averages'. Each row is assigned a rank from 1 to n, the number of rows. For each column, a barycentre is calculated by weighting according to the row rank. Finally, the columns are reorganised by sorting them according to their barycentre.</p>
<p><img src=\"GS/seriation.png\"></p>


<h3 id=\"show1\">Visualization</h3>
<p>This input allows you to select the element to be plotted. There are tree options: plot the positive deviation from the average percentage (EPPM in French), plot the frequency or plot both. The average percentage is calculated for each category (columns) on the total number of accounts (all classes combined). From the average percentage, we obtain for each category and for each row the difference between the percentage of the category in the class and the average percentage. The EPPM corresponds to the notion of independence deviation (between rows and columns, between categories and time classes) in a chi-square test approach. Although this approach is fundamental in statistical analysis, the independence deviations here are purely indicative and are not associated with a p_value that could determine the significance of the deviations.</p>
<p><img src=\"GS/show.png\"></p>

<h3 id=\"sort1\">Sort clusters</h3>
<p>The rows are initially in the order in which they appear on the dendrogram. It is possible to rearrange the classes in time. In the interface you can drag and drop the class (green) to change the order. You can add a gap between two clusters and also remove a cluster by dragging and dropping it in the remove section.</p>
<p><img src=\"GS/sort.png\"></p>

<h3 id=\"import\">Import your data</h3>
<p>You can import your data. There are two steps. You need to upload a first csv file for the first source of information.
The data must be a contingency table.</p>
<p><img src=\"GS/import1.png\"></p>
<p>The second step is to upload the second source. You can choose between two types of data (stratigraphic link or time range data). The stratigraphic link object is a data frame with two columns. The first column contains the elements (the same number as the number of rows in the contingency table) and the second column contains a list of all the other elements connected to it. The list is a string consisting of the names (those in the first column) of the elements separated by a comma.</p>
<p><img src=\"GS/network_ui.png\"></p><p>The data frame for time range data contains the same first column. The second column contains the lower time limit and the third column contains the upper time limit.</p>
<p><img src=\"GS/timerange_ui.png\"></p>

<p><img src=\"GS/import3.png\"></p>
<p>The settings allow you to import different data frame organisation (header, column separator, ...).</p>
<h4 id=\"header\">Header</h4>
<p>Yes or no option. Do you have headers on your columns?</p>
<h4 id=\"rownames\">Rownames</h4>
<p>Yes or no option. Do you have row names on your rows?</p>
<h4 id=\"separator\">Separator</h4>
<p>Select the character you want to use to separate the columns.</p>
<h4 id=\"quote\">Quote</h4>
<p>Select the quotation marks to use on strings.</p>
<h4 id=\"dec\">Decimal</h4>
<p>Select the character to use to indicate the decimal point.</p>

<h4 id=\"csv\">CSV Format and write.table</h4>
<p>It is a data.frame with columns separated by semicolons \";\".</p>
<p>The input format for importing data is the .csv format, but also supports the .txt format as a .csv file.</p>
<p>In R, you can export your data frame to a csv file using write.csv2 or write.table. In a csv you can choose a character to separate the columns. In the same way, you can define the character to indicate the decimal point.</p>
<code>
write.table(data,file=\"path/to/name_file.csv\",sep=\";\",dec=\".\",row.names=FALSE,quote=FALSE)
</code>
<p>In Excel you can save in csv format in order to import your data frame.</p>
<p>The import interface allows you to set these values using the 'header', 'decimal', 'separator' and 'quote' options.</p>


<hr>
<h2 id=\"output\">The outputs</h2>
<hr>

<h3 id=\"eval\">Evaluation Plot</h2>

<p>It is essential to be able to evaluate the different partitions of the hierarchy in order to identify the most relevant one(s). In our case, the number k of clusters of the partition to be retained is based on several indicators calculated for different values of K: the total within-class sum of squares (WSS) and the global average of the silhouette widths.</p>
<p>When running the hclustcompro method, you have to make a choice. You must cut the dendrogram. This operation selects the partition. To compare all the possibilities, you can see the evaluation plots (WSSPlot and AveSilPlot). These two plots evaluate the relative quality of the partition.</p>
<h3 id=\"wssplot\">Within Sum of Square Plot (WSSPlot)</h3>
<p>This is the plot of the within group sum of squares against the number of clusters. The within group sum of squares decreases as the number of clusters increases. In this plot, the best partition is when adding one or more clusters doesn't decrease the WSS value. It's called the elbow method.</p>
<h4>Example:</h4>
<p><img src=\"GS/WSS.png\"></p>
<p>On this graph, we start by looking at the value for the lowest number of classes: 2. If I add a third class, we see that the WSS value will decrease (from 2.4 to 1.8). If I add another class, this value decreases again (from 1.8 to 1.4). We are looking for the moment when it is not interesting to add a class. In this case, we can keep a partition with 8 or 9 classes.</p>
<h3 id=\"avesilplot\">Average silhouette Plot</h3>
<p>This graph shows the average silhouette width of each partition (ROUSSEEUW 1987). The silhouette width is a bounded index between -1 and 1, calculated for each observation. The closer the value is to 1, the better the observation is classified. We look for the average value for a partition that is closest to 1.</p>
<h4>Example:</h4>
<p><img src=\"GS/AveSil.png\"></p>
<p>On this graph we look for the maximum value. The best evaluation corresponds to the division into 9 classes. Looking at the second best partition, we identify the one with 10 classes.</p>

<h3 id=\"selectalpha\">Select alpha</h3>
<p>This plot shows the correlation criterion for each alpha.</p>
<p><img src=\"GS/selectalpha.png\"></p>

<h3 id=\"dendrogram\">Dendrogram</h3>
<p>The dendrogram is the result of the clustering process. You can see the clusters and sub-clusters.</p>
<p><img src=\"GS/dendrogram.png\"></p>

<h3 id=\"seriograph\">Seriograph</h3>
<p><p>We have chosen the serigraph (DESACHY 2004). This tool makes it possible to highlight the evolution of ceramics over time as well as to understand the commercial relations thanks to the imported ceramics. The percentages of each category of ceramics per set are displayed. The percentages are calculated independently for each set (row). The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).</p>
<p>We have generalized this representation for other contingency data with hclustcompro object.</p>
<p><img src=\"GS/seriograph.png\"></p>
<p>In order to facilitate the exploitation of the data tables, we propose here a computerised graphic processing tool (EPPM serigraph - for Ecart Positif aux Pourcentages Moyens - positive deviation from the average percentage), which does not require specialised statistical skills and is adapted to the case of stratified sites, where the study of the evolution of artefacts can be based on the relative chronology provided by the excavation.</p>
<p>The treatment consists firstly of transforming this table of counts into a table of percentages, the total number in each set (each row) being reduced to 100; these are the proportions, or frequencies, of the types in the sets are thus compared.</p>
<p>The display of positive deviations from the mean percentages (EPPM) shows in black on a lighter background the percentage shares that are higher than the mean percentage of the variable, so as to highlight the most significant part of the values in the table.This display is simply adapted to the seriograph: when a percentage is greater than the average percentage of the type, the excess share (called here EPPM: positive deviation from the average percentage) is shown in black, centred around the axis of the type, on the grey background of the percentage bar.</p>
<p>The table is then transformed into a graphic matrix where these percentages are expressed, for each type, by horizontal bars centred on the column axis. When the rows are ordered chronologically, the silhouette formed by the superposition of these frequency bars bars makes it possible to visualise the evolution over time of the type concerned.</p>
<p>The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).</p>
<p>The processing technique applies to sets whose chronological order is not known; the lines of the graph are to be reorganised so as to obtain boat-shaped silhouettes following the hypothesis of a chronological evolution corresponding to the seriation model.</p>


<h3 id=\"timerangeclust\">Timerange plot</h3>
<p>Show the different time ranges of observations for each cluster (you will need to import time range data).</p>
<p><img src=\"GS/timerangeclust.png\"></p>


<h3 id=\"CAp\">Correspondences analysis</h3>
<p>You can see the different planes of correspondence analysis used to construct the first dissimilarity matrix D1.</p>
<p><img src=\"GS/CA.png\"></p>


<hr>
<h2 id=\"references\">References</h2>
<hr>
<p><br>
  Rousseeuw, P.J. (1987). “Silhouettes: a graphical aid to the interpretation and validation of cluster analysis”.
  In : J. Comput. Appl. Math. 20, 53–65. <a href=\"https://doi.org/10.1016/0377-0427(87)90125-7\" target=\"_blank\">⟨doi:10.1016/0377-0427(87)90125-7⟩</a>.
  <br><br>Ward, J. H., Jr. (1963). \"Hierarchical Grouping to Optimize an Objective Function\".
  Journal of the American Statistical Association, 58, 236–244.
  <br><br>Sokal R. and Michener C. (1958). \"A statistical method for evaluating systematic
  relationships\". University of Kansas Science Bulletin. 38: 1409–1438.
  <br><br>Desachy B. (2004). \"Le sériographe EPPM : un outil informatisé de sériation graphique pour tableaux de comptages. In: Revue archéologique de Picardie, n°3-4, 2004. Céramiques domestiques et terres cuites architecturales. Actes des journées d'étude d'Amiens (2001-2002-2003) pp. 39-56 <a href=\"https://doi.org/10.3406/pica.2004.2396\" target=\"_blank\">⟨doi:10.3406/pica.2004.2396⟩</a>.</p>
<br><br><br><br>



")
    )
  )
)

library(SPARTAAS)
library(shiny)
suppressMessages(library(shinyjs))
library(shinyjqui)
suppressMessages(library(shinydashboard))
library(colorspace)
suppressMessages(library(dplyr))
library(tidyr)
library(ggplot2)
library(shinyWidgets)
library(FactoMineR)
library(scatterD3)
library(dplyr)
library(fpc)
suppressMessages(library(plotly))
library(cluster) #for silhouette
library(stringr) #sub_extract
library(shinythemes)

ui <- fluidPage(useShinyjs(),withMathJax(),style="padding-top: 150px;",theme = shinytheme("flatly"),
                tags$style(HTML('#sort,#data,#subdivise,#tri,#sub1,#sub2,
         #import2,#import3,#import3,#Finishimport,#back1,#back2{color:white;background-color:#337ab7;border-color:#337ab7}
         #back{color:white;background-color:#f0ad4e}
         #tabsetperso{background-color:#e5e5e5;}
         #help{text-align;right;}
         #tabsetperso{margin-bottom:20px;}
         .btn-default {
                    background-color:#5da3a8;
                    border-color:#5da3a8;
         }
                  .dropdown-menu > li > a {
                    background-color:#fff;
                    border-color:#dce4ec;
         }
    ')),
                tags$head(
                  tags$link(rel = "icon", type = "image/gif", href = "https://spartaas.gitpages.huma-num.fr/r-package/img/lambda.png"),
                  tags$title("hclustcompro")
                ),
                #header panel
                absolutePanel(class = "panel panel-default",
                              style="z-index: 2000;padding: 8px; background: #ecf0f1; opacity: 1;border-bottom: 1px solid #2c3e50;",
                              top = 0, left = 0, right = 0,
                              fixed = TRUE,
                              HTML("<h2>hclustcompro: <b>h</b>ierarchical <b>clust</b>ering by <b>compro</b>mise</h2>"),
                              div(span(strong("SPARTAAS | hclustcompro")))
                ),
                #footer
                absolutePanel(style="z-index: 2000;padding: 0px; border-bottom: 0px solid #CCC; background: #fff;opacity: 1;",
                              bottom = 0, left = 0,
                              fixed = TRUE,
                              div(span(strong("SPARTAAS")),span("[Bellanger,Coulon,Husi]",style ="font-size:14px;"))
                ),
                #go top panel
                absolutePanel(class = "panel-hover",
                              style="opacity:0.8;z-index: 1000;padding: 8px; border-bottom: 1px solid #CCC;
    background: #FFFFEE;width:30px;margin-right:10px;",
                              bottom = 20, right = 0,
                              fixed = TRUE,
                              HTML("<a href=\"#top\"><span class=\"glyphicon glyphicon-chevron-up\"></span></a>")
                ),
                #-----------------------------------#
                #    NavBar
                #-----------------------------------#
                navbarPage("hclustcompro",id="navbar",
                           tabPanel(value = "1",span(icon("home", lib = "glyphicon"),strong("Home")),style="max-width:1200px;",
                                    br(),
                                    sidebarLayout(
                                      #-----------------------------------#
                                      sidebarPanel(style="",
                                                   h1("R Package:"),
                                                   br(),
                                                   HTML("<p>This method is part of the <a target='_blank' href='https://spartaas.gitpages.huma-num.fr/r-package/index.html'>SPARTAAS</a> package.</p>
                                                        <p>If you are interested you can install our R package available on the <a href='https://cran.r-project.org/package=SPARTAAS'>CRAN</a>.</p>

                                                        ")

                                      ),
                                      #-----------------------------------#
                                      mainPanel(
                                        HTML("
                                              <h3>Introduction</h3>
                                              <p>hclustcompro is a hierarchical clustering that takes into account two sources of information associated with the same objects. This method, called CAH by compromise (hclustcompro), allows a compromise between the hierarchies obtained from each source considered separately. A convex combination of the dissimilarities associated with each of the sources is used to modify the dissimilarity measure in the classical CAH algorithm. The choice of the mixing parameter is the key point of the method. We propose an objective function to be minimised based on the absolute difference of the correlations between the initial dissimilarities and the kinetic distances, as well as a resampling procedure to ensure the robustness of the choice of the mixing parameter.</p>
                                              ")
                                      )
                                    )
                           ),
                           tabPanel(value = "2",span(icon("chart-area", lib = "font-awesome"),strong("hclustcompro")),style="max-width:1200px;",
                                    tabsetPanel(id = "tabsetperso",type="tabs",
                                                tabPanel(id="tab11","Overview",
                                                         sidebarPanel(width = 5,
                                                                      h4("Import your data:"),actionButton("data",
                                                                                                           icon("import", lib = "glyphicon")),
                                                                      h4("Refresh app:"),actionButton("refresh",
                                                                                                      icon("refresh", lib = "glyphicon")),
                                                                      hr(style="border-color: #222222;"),
                                                                      h4("guide lines"),
                                                                      HTML("<p>You have three subtabs to run hclustcompro:</p>"),
                                                                      actionLink(inputId = "over", label = "Overview (here)"),
                                                                      HTML("<ul>
                                         <li>Import your data <br>(you can skip this step and
                                         test hclustcompro with the default dataset)</li>
                                         </ul>"),
                                                                      actionLink(inputId = "clustsett", label = "Clustering settings"),
                                                                      HTML("<ul>
                                         <li>Select the correct number of axes in the correspondence analysis</li>
                                         <li>Select the alpha value</li>
                                         <li>Select the number of groups <br>The optimal number of groups button helps you with two evaluation scores (e.g. WSS: within sum of squares and average silhouette score)</li>
                                         </ul>"),
                                                                      actionLink(inputId = "resviz", label = "Results visualization"),
                                                                      HTML("<ul>
                                         <li>Sort the clusters on the seriograph in chronological order (Sort clusters button)</li>
                                         </ul><br><br>
                                              ")

                                                         ),
                                                         mainPanel(style = 'max-width:1000px;',width = 7,
                                                                   h4("The data"),
                                                                   HTML("<p>hclustcompro uses two types of input data. The first is a contingency table (counting ceramics or other furniture). The second can be either stratigraphic relationships or dates (time range) for the same individuals as the first source.</p>
                                              <p>See example below.</p>"),
                                                                   actionLink(inputId = "wiki", label = "Go to wiki for more detail"),
                                                                   h4("First data source (contingency table)"),
                                                                   hr(style="border-color: #222222;"),
                                                                   tableOutput("data1f"),
                                                                   h4("Second data source (Stratigraphic connection or Time range data)"),
                                                                   hr(style="border-color: #222222;"),
                                                                   h5("Stratigraphic connection:"),
                                                                   tableOutput("data2f"),
                                                                   h5("Timerange data:"),
                                                                   tableOutput("data3f")
                                                         )
                                                ),
                                                tabPanel(id="tab12","Clustering settings",
                                                         sidebarPanel(
                                                           h4("Change number of axes in correspondence analysis:"),actionButton("axesCA", "Change"),
                                                           hr(style="border-color: #c2c2c2;"),
                                                           column(7,
                                                                  h4("Result of correspondence analysis:"),
                                                                  dropdownButton(
                                                                    h3("Correspondence analysis"),
                                                                    numericInput("axe1", "Axe X", 1, min = 1, max = NA, step = 1),
                                                                    numericInput("axe2", "Axe Y", 2, min = 1, max = NA, step = 1),
                                                                    scatterD3Output("CA"),
                                                                    icon = icon("info"), width = "600px", size = "sm",
                                                                    right=FALSE
                                                                  )
                                                           ),
                                                           div(style="height:1260px;"),
                                                           selectInput("method", label = "Aggregation method:",
                                                                       choices = list(
                                                                         "ward.D2" = 2,
                                                                         "complete" = 4,
                                                                         "single" = 3,
                                                                         "average (= UPGMA)" = 5,
                                                                         "mcquitty (= WPGMA)" = 6 #,"median (= WPGMC)" = 7,"centroid (= UPGMC)" = 8
                                                                       ), selected = 2),




                                                           column(7,
                                                                  actionButton("subdivise",span(icon("scissors",lib="font-awesome"),"Subdivide"))
                                                           ),
                                                           column(5,
                                                                  actionButton("reset",span(icon("rotate-left",lib="font-awesome"),"Reset"))
                                                           ),div(style="height:40px;")


                                                         ),
                                                         mainPanel(

                                                           ####################################
                                                           ##        SelectAlpha PLOT        ##
                                                           ####################################
                                                           br(),br(),
                                                           h3("Select Alpha"),
                                                           hr(style="border-color: #222222;"),
                                                           column(11,
                                                                  dropdownButton(
                                                                    downloadButton("alpha.pdf","Save as pdf"),
                                                                    hr(style="border-color: #222222;"),
                                                                    downloadButton("alpha.png","save as png"),
                                                                    numericInput("width1", "width", 900, min = 0, max = NA, step = 1, width = NULL),
                                                                    numericInput("height1", "height", 500, min = 0, max = NA, step = 1, width = NULL),
                                                                    circle = TRUE, status = "primary",
                                                                    icon = icon("floppy-disk", lib = "glyphicon"), width = "150px",
                                                                    size = "sm",
                                                                    tooltip = tooltipOptions(title = "Download")
                                                                  )
                                                           ),
                                                           column(1,
                                                                  # Help
                                                                  dropdownButton(
                                                                    h3("Select Alpha"),
                                                                    hr(style="border-color: #222222;"),
                                                                    p("A criterion for the choice of alpha IN [0;1] must be determined by balancing the weights between the two sources of information in the final classification."),
                                                                    p("The CorCrit_alpha criterion represents the difference in absolute value between two cophenetic correlations (cophenetic correlation is defined as the correlation between two distance matrices. It is calculated by considering the half distance matrices as vectors. It measures how faithfully a dendrogram preserves the pairwise distances between the original unmodeled data points). The first correlation is related to the comparison between D1 and ultrametric distances from the clustering with alpha fixed, while the second compares D2 and ultrametric distances from the clustering with alpha fixed. As a compromise between the information provided by D1 and D2, we then decided to estimate alpha with hat(alpha) as the minimum."),
                                                                    circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                                                    size = "sm",right=TRUE,
                                                                    tooltip = tooltipOptions(title = "Help", placement = "top")
                                                                  )
                                                                  # Help
                                                           ),br(),br(),

                                                           jqui_resizable(plotOutput("selectAlpha")),br(),


                                                           sliderInput("alpha", label = "Alpha", min = 0, max = 1, step = 0.01, value = 0),


                                                           br(),
                                                           uiOutput("formula"),
                                                           #HTML("<p><img style=\"width:200px;\" src=\"GS/formule.png\"></p>"),


                                                           ####################################
                                                           #            NB GROUPE             #
                                                           ####################################
                                                           br(),br(),
                                                           h3("Select the number of groups in the partition"),
                                                           hr(style="border-color: #222222;"),
                                                           column(width=6,
                                                                  column(10,
                                                                         dropdownButton(
                                                                           downloadButton("silhouette.pdf","Save as pdf"),
                                                                           hr(style="border-color: #222222;"),
                                                                           downloadButton("silhouette.png","save as png"),
                                                                           circle = TRUE, status = "primary",
                                                                           icon = icon("floppy-disk", lib = "glyphicon"), width = "150px",
                                                                           size = "sm",
                                                                           tooltip = tooltipOptions(title = "Download")
                                                                         ),
                                                                  ),
                                                                  column(1,
                                                                         # Help
                                                                         dropdownButton(
                                                                           h3("Average silhouette widths"),
                                                                           hr(style="border-color: #222222;"),
                                                                           p("This graph shows the average silhouette width of each partition (Rousseeuw 1987). The silhouette width is a bounded index between -1 and 1, calculated for each observation. The closer the value is to 1, the better the observation is classified. We look for the average value for a partition that is closest to 1."),
                                                                           circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                                                           size = "sm",right=TRUE,
                                                                           tooltip = tooltipOptions(title = "Help", placement = "top")
                                                                         )
                                                                         # Help
                                                                  ),

                                                                  plotOutput("silhouette")
                                                           ),

                                                           column(width=6,
                                                                  column(10,
                                                                         dropdownButton(
                                                                           downloadButton("wss.pdf","Save as pdf"),
                                                                           hr(style="border-color: #222222;"),
                                                                           downloadButton("wss.png","save as png"),
                                                                           circle = TRUE, status = "primary",
                                                                           icon = icon("floppy-disk", lib = "glyphicon"), width = "150px",
                                                                           size = "sm",
                                                                           tooltip = tooltipOptions(title = "Download")
                                                                         )
                                                                  ),
                                                                  column(1,
                                                                         # Help
                                                                         dropdownButton(
                                                                           h3("WSS"),
                                                                           hr(style="border-color: #222222;"),
                                                                           p("This is the plot of the within group sum of squares against the number of clusters. The within group sum of squares decreases as the number of clusters increases. In this plot, the best partition is when adding one or more clusters doesn't decrease the WSS value. It's called the elbow method."),
                                                                           circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                                                           size = "sm",right=TRUE,
                                                                           tooltip = tooltipOptions(title = "Help", placement = "top")
                                                                         )
                                                                         # Help
                                                                  ),
                                                                  plotOutput("WSS")
                                                           ),div(style="height:500px;"),

                                                           sliderInput("k",label ="Number of class",min = 2,max = 20,step = 1, value = 2),


                                                           ####################################
                                                           #         Dendrogram PLOT          #
                                                           ####################################
                                                           h3("Dendrogram"),
                                                           hr(style="border-color: #222222;"),
                                                           verbatimTextOutput("click_info"),
                                                           column(11,
                                                                  dropdownButton(
                                                                    downloadButton("dendrogram.pdf","Save as pdf"),
                                                                    hr(style="border-color: #222222;"),
                                                                    downloadButton("dendrogram.png","save as png"),
                                                                    numericInput("width2", "width", 900, min = 0, max = NA, step = 1, width = NULL),
                                                                    numericInput("height2", "height", 500, min = 0, max = NA, step = 1, width = NULL),
                                                                    circle = TRUE, status = "primary", icon = icon("floppy-disk", lib = "glyphicon"),
                                                                    width = "150px",
                                                                    size = "sm",
                                                                    tooltip = tooltipOptions(title = "Download")
                                                                  )
                                                           ),
                                                           column(1
                                                           ),br(),br(),
                                                           shinyjqui::jqui_resizable(plotOutput("dendrogramme")),br(),br(),



                                                           hr(style="border-color: #c2c2c2;"),
                                                           br(),
                                                           column(11,
                                                                  dropdownButton(
                                                                    downloadButton("silplot.pdf","Save as pdf"),
                                                                    hr(style="border-color: #222222;"),
                                                                    downloadButton("silplot.png","save as png"),
                                                                    numericInput("width4", "width", 900, min = 0, max = NA, step = 1, width = NULL),
                                                                    numericInput("height4", "height", 500, min = 0, max = NA, step = 1, width = NULL),
                                                                    circle = TRUE, status = "primary", icon = icon("floppy-disk",
                                                                                                                   lib = "glyphicon"), width = "150px",
                                                                    size = "sm",
                                                                    tooltip = tooltipOptions(title = "Download")
                                                                  )
                                                           ),
                                                           column(1,
                                                                  # Help
                                                                  dropdownButton(
                                                                    h3("Silhouette plot"),
                                                                    hr(style="border-color: #222222;"),
                                                                    p("Silhouette refers to a method of interpretation and validation of
                                    consistency within clusters of data."),
                                                                    p("The silhouette value is a measure of how similar an object is to
                                    its own cluster (cohesion) compared to other clusters (separation).
                                    The silhouette ranges from −1 to +1, where a high value indicates
                                    that the object is well matched to its own cluster and poorly matched
                                    to neighboring clusters. If most objects have a high value, then the
                                    clustering configuration is appropriate. If many points have a low or
                                    negative value, then the clustering configuration may have too many or
                                    too few clusters. "),
                                                                    circle = TRUE, status = "danger", icon = icon("question"), width = "500px",
                                                                    size = "sm",right=TRUE,
                                                                    tooltip = tooltipOptions(title = "Help", placement = "top")
                                                                  )
                                                                  # Help
                                                           ),br(),br(),
                                                           jqui_resizable(plotOutput("SilPlot"))
                                                         )
                                                ),
                                                tabPanel(id="tab13","Results visualization",
                                                         div(id="nodata",
                                                             HTML("<br><p><strong>Warnings:</strong> You must go to the clustering settings tab to set up the clustering first.</p>")
                                                         ),
                                                         div(id="toggle",
                                                             sidebarPanel(style = 'max-width:300px;',
                                                                          div(style="height:200px;"),
                                                                          prettyToggle(
                                                                            inputId = "permute",
                                                                            label_on = "Seriation",
                                                                            label_off = "Seriation",
                                                                            icon_on = icon("check"),
                                                                            icon_off = icon("trash"),
                                                                            value = TRUE
                                                                          ),
                                                                          selectInput("show", label = "Show", choices = list("both" = 1,
                                                                                                                             "frequencies" = 2,
                                                                                                                             "EPPM" = 3),
                                                                                      selected = 1),
                                                                          actionButton("sort",span(icon("sort",lib="font-awesome"),"Sort cluster"))
                                                             ),
                                                             mainPanel(style = 'max-width:1000px;',
                                                                       ####################################
                                                                       ##        Seriograph PLOT         ##
                                                                       ####################################
                                                                       br(),br(),
                                                                       h3("Seriograph"),
                                                                       hr(style="border-color: #222222;"),
                                                                       column(11,
                                                                              dropdownButton(
                                                                                downloadButton("seriograph.pdf","Save as pdf"),
                                                                                hr(style="border-color: #222222;"),
                                                                                downloadButton("seriograph.png","save as png"),
                                                                                numericInput("width3", "width", 900, min = 0, max = NA,
                                                                                             step = 1, width = NULL),
                                                                                numericInput("height3", "height", 500, min = 0, max = NA,
                                                                                             step = 1, width = NULL),
                                                                                circle = TRUE, status = "primary", icon = icon("floppy-disk",
                                                                                                                               lib = "glyphicon"), width = "150px",
                                                                                size = "sm",
                                                                                tooltip = tooltipOptions(title = "Download")
                                                                              )
                                                                       ),
                                                                       column(1,
                                                                              # Help
                                                                              dropdownButton(
                                                                                h3("The seriograph (Desachy 2004)"),
                                                                                hr(style="border-color: #222222;"),
                                                                                p("This tool makes it possible to highlight artisanal
                                               evolutions over time as well as to understand commercial
                                               relations thanks to imported ceramics. The percentages of
                                               each ceramic category are displayed. The percentages are
                                               calculated independently for each class. The percentage display
                                               allows you to compare the different classes but does not provide
                                               information on the differences in terms of enrolment. To fill
                                               this gap,
                                               the proportion of each class of their workforce is displayed on
                                               the seriograph (weight column). We have generalized this
                                               representation
                                               for other contingency data with hclustcompro object."),
                                                                                circle = TRUE, status = "danger", icon = icon("question"),
                                                                                width = "500px",
                                                                                size = "sm",right=TRUE,
                                                                                tooltip = tooltipOptions(title = "Help", placement = "top")
                                                                              )
                                                                              # Help
                                                                       ),br(),br(),

                                                                       jqui_resizable(plotOutput("EPPM")),
                                                                       downloadButton("table_serio.csv", "Download the contingency table"),
                                                                       div(style="height:20px;"),
                                                                       ####################################
                                                                       ##         timerange PLOT         ##
                                                                       ####################################
                                                                       conditionalPanel(
                                                                         condition = "1 == 0",
                                                                         checkboxInput("test","test",value = TRUE)
                                                                       ),
                                                                       conditionalPanel(
                                                                         condition = "input.datatype == 2 && input.test",
                                                                         br(),br(),
                                                                         h3("Timerange plot"),
                                                                         hr(style="border-color: #222222;"),
                                                                         column(11

                                                                         ),
                                                                         column(1,
                                                                                # Help
                                                                                dropdownButton(
                                                                                  h3("Timerange clust"),
                                                                                  hr(style="border-color: #222222;"),
                                                                                  p("Each observation has a timerange estimation. We display the
                                                 timeranges grouping by cluster"),
                                                                                  circle = TRUE, status = "danger", icon = icon("question"),
                                                                                  width = "300px",
                                                                                  size = "sm",right=TRUE,
                                                                                  tooltip = tooltipOptions(title = "Help", placement = "top")
                                                                                )
                                                                                # Help
                                                                         ),br(),br(),
                                                                         jqui_resizable(plotlyOutput("timerange")),
                                                                         div(style="height:20px;"),

                                                                       )

                                                             )
                                                         )
                                                )
                                    )
                           ),
                           tabPanel(value = "3",span(icon("envelope", lib = "glyphicon"),strong("Contact")),style="max-width:1200px;",
                                    HTML("<style>
        .carte{
        border-left-width: 4px;
        border-left-style: solid;
        border-color: #2ac0a2;
        padding-left: 10px;
        }
        </style>
        <h3>Authors:</h3>
        <hr>
        <div class=\"carte\"><h4>L. Bellanger</h4><h5>mail: &lt;lise.bellanger@univ-nantes.fr&gt;</h5></div>
        <div class=\"carte\"><h4>P. Husi</h4><h5>mail: &lt;philippe.husi@univ-tours.fr&gt;</h5></div>
        <div class=\"carte\"><h4>A. Coulon</h4></div>
        <h3>Maintainer:</h3>
        <hr>
        <div class=\"carte\"><h4>A. Coulon</h4><h5>mail: &lt;arthur-coulon@outlook.fr&gt;</h5></div>
        ")
                           ),
                           #-----------------------------------#
                           #    Wiki
                           #-----------------------------------#
                           tabPanel(value = "4",span(icon("question-sign", lib = "glyphicon"),strong("Wiki")),style="max-width:1200px;",
                                    HTML("<style>body {text-align: justify}</style><h1>Get started with the hclustcompro application</h1>
                                      <h2>Table of content</h2>
                                      <div id=\"TOC\">
                                      <ul>
                                      <li><a href=\"#introduction\">Introduction</a>
                                        <ul>
                                          <li><a href=\"#hclustcompro\">hclustcompro</a></li>
                                          <li><a href=\"#CAdist\">Construction of D1</a></li>
                                          <li><a href=\"#adjacency\">Construction of D2 for spatial data</a></li>
                                          <li><a href=\"#overlap\">Construction of D2 for temporal data</a></li>
                                          <li><a href=\"#corcrit\">Correlation Criterion</a></li>
                                        </ul>
                                      </li>
                                      <li><a href=\"#data1\">The inputs</a>
                                        <ul>
                                          <li><a href=\"#alpha\">Alpha</a></li>
                                          <li><a href=\"#k\">Number of class</a></li>
                                          <li><a href=\"#aggregation\">Aggregation method</a></li>
                                          <li><a href=\"#subdivide\">Subdivide</a></li>
                                          <li><a href=\"#seriation\">Seriation</a></li>
                                          <li><a href=\"#weight\">Weight color indicator</a></li>
                                          <li><a href=\"#show1\">Vizualisation</a></li>
                                          <li><a href=\"#sort1\">Sort clusters</a></li>
                                          <li><a href=\"#import\">Import your data</a></li>
                                          <li><a href=\"#csv\">CSV Format and write.table</a></li>
                                        </ul>
                                      </li>
                                      <li><a href=\"#outputs\">The outputs</a>
                                        <ul>
                                          <li><a href=\"#eval\">Evaluation Plot</a>
                                            <ul>
                                              <li><a href=\"#wssplot\">Within Sum of Square Plot</a></li>
                                              <li><a href=\"#avesilplot\">Average Silhouette Plot</a></li>
                                            </ul>
                                          </li>
                                          <li><a href=\"#selectalpha\">Select alpha</a></li>
                                          <li><a href=\"#dendrogram\">Dendrogram</a></li>
                                          <li><a href=\"#seriograph\">Seriograph</a></li>
                                          <li><a href=\"#timerangeclust\">Timerange</a></li>

                                          <li><a href=\"#CAp\">Correspondences analysis</a></li>

                                        </ul>
                                      </li>

                                      <li><a href=\"#references\">References</a></li>
                                      </ul>
                                      </div>

<hr>
<h2 id=\"introduction\">Introduction</h2>
<hr>
<p>Hierarchical clustering method by compromise. The method uses two sources of information.</p>

<h3 id=\"hclustcompro\">hclustcompro</h3>
<p>The merging of the two data sources is done by a parameter (alpha) that allows to weight each source.</p>
<p>$$ \\boldsymbol{D}_\\alpha=\\alpha \\boldsymbol{D}_1+(1-\\alpha) \\boldsymbol{D}_2 $$</p>

<p>The first is a contingency table. The rows must be individuals (site, ...) and the columns must be categories (type, ...). The second concerns the same individuals and can be stratigraphic connection type or time range data. The stratigraphic link object is a data frame with two columns. The first column contains the network elements (the same number as the number of lines in D1) and the second column contains a list of all the other elements connected to it. The list is a string of element names separated by commas.
</p><p><img src=\"GS/network.png\"></p>
<p>The data frame for time range data contains the same first column. The second column contains the lower time limit and the third column contains the upper time limit.</p>
<p><img src=\"GS/timerange.png\"></p>

<h3 id=\"CAdist\">Construction of D1</h3>
<p>Our first source of information is a contingency table. We need to construct a distance matrix from this table. We perform a correspondence analysis (CA) on the contingency table and then use the distances of the components (chi-square metric).</p>

<p>You must select the number of axes (CA components) to be used to construct the distance matrix.</p>

<p>By examining the eigenvalues, it is possible to determine the number of principal axes to be considered. The eigenvalues correspond to the amount of information retained by each axis.</p>

<p>A heuristic method is often used: one constructs the graph of eigenvalues sorted in descending order, retaining the eigenvalues (and associated principal components) that precede the first 'elbow'.</p>

<p>Another approach is based on maintaining an approximation quality of the original data, measured by the explained inertia rate (e.g. 85%). As many axes as necessary are chosen so that the sum of the corresponding eigenvalues exceeds the target inertia rate.</p>

<p><img src=\"GS/ncp.png\"></p>
<p><img src=\"GS/ncpslider.png\"></p>

<h3 id=\"adjacency\">Construction of D2 for stratigraphic connection</h3>
<p>From this data set, we construct an adjacency matrix. Based on this matrix, we generate a dissimilarity matrix. The matrix contains only 0 or 1, 1 if there is no relationship and 0 if there is a relationship.</p>


<h3 id=\"overlap\">Construction of D2 for time range data</h3>
<p>The overlap index is the ratio between the overlap or separation and the cumulative extend of both individuals. We define the cumulative extend of both as: the minimum of the lower limits of the pair of individuals and the maximum of the upper limits. We define the overlap or separation as: the maximum of the lower limits and the minimum of the upper limits.</p>
<p>From this ratio, we construct a dissimilarity by setting the value between 0 and 1, ensuring that the closer the value is to 1, the further apart the time ranges are.</p>
<p><img style=\"width:500px;\" src=\"GS/overlap.png\"></p>

<h3 id=\"corcrit\">Correlation Criterion</h3>
<p>A criterion for choosing alpha IN [0;1] must be determined by balancing the weights between the two
sources of information in the final classification. To obtain alpha, we define the following criterion:
</p><p>$$\\operatorname{CorCrit}_\\alpha=\\left|\\operatorname{Cor}\\left(\\mathbf{D}_\\alpha^{\\text {coph }}, \\mathbf{D}_1\\right)-\\operatorname{Cor}\\left(\\mathbf{D}_\\alpha^{\\text {coph }}, \\mathbf{D}_2\\right)\\right|
$$</p>
The CorCrit_alpha criterion in (1) represents the difference in absolute value between two cophenetic correlations (cophenetic correlation is defined as the correlation between two distance matrices. It is calculated by considering the half distance matrices as vectors. It measures how faithfully a dendrogram preserves the pairwise distances between the original unmodeled data points). The first correlation is related to the comparison between D1 and ultrametric distances from the clustering with alpha fixed, while the second compares D2 and ultrametric distances from the clustering with alpha fixed. Then, in order to compromise between the information provided by D1 and D2, we decided to estimate alpha with hat(alpha) such that:</p>
<p>$$
\\hat{\\alpha}=\\min _\\alpha \\text { CorCrit }{ }_\\alpha
$$</p>



<hr>
<h2 id=\"data1\">The inputs</h2>
<hr>
<h3 id=\"alpha\">Alpha</h3>
<p>The slider input allows you to select the value of the mixing parameter. You select the weight to be given to the first dissimilarity matrix (alpha) and the second (1-alpha). By default, the value is the one previously defined when estimating using the correlation criterion.</p>
<p><img src=\"GS/alpha.png\"></p>

<h3 id=\"k\">Number of class</h3>
<p>The slider input defines the number of clusters you want. To help you decide, you can view the WSS plot and silhouette information.</p>
<p><img src=\"GS/k.png\"></p>

<h3 id=\"aggregation\">Aggregation method</h3>
<p>The hierarchical clustering can use several aggregation methods when constructing the dendrogram. You can choose any of the methods, except the ones with the risk of inversion (if you know what you are doing, you can use the package version for this).</p>
<p><img src=\"GS/aggregation.png\"></p>

<h4>Ward.D2</h2>
<p>Ward's method is a criterion used in hierarchical cluster analysis. Ward's minimum variance method is a special case of the objective function approach originally proposed by Joe H. Ward, Jr. Ward proposed a general agglomerative hierarchical clustering procedure where the criterion for selecting the pair of clusters to merge at each step is based on the optimal value of an objective function.</p>

<h4>complete</h2>
<p>At each step, the two clusters separated by the shortest distance are combined. In complete-linkage clustering, the link between two clusters contains all element pairs, and the distance between clusters is equal to the distance between the two elements (one in each cluster) that are farthest apart. The shortest of these links remaining at each step causes the two clusters whose elements are involved to merge.</p>

<h4>single</h2>
<p>The distance between two clusters is determined by a single pair of elements, namely the two elements (one in each cluster) that are closest to each other. The shortest of these links remaining at any step causes the two clusters whose elements are involved to merge.</p>

<h4>average (UPGMA)</h2>
<p>UPGMA (unweighted pair group method with arithmetic mean) is a simple agglomerative (bottom-up) hierarchical clustering method. It is generally credited to Sokal and Michener. The UPGMA method is similar to its weighted variant, the WPGMA method. Note that the term 'unweighted' indicates that all distances contribute equally to each mean computed, and does not refer to the mathematics by which it is obtained.
<br><br>
At each step, the two closest clusters are combined to form a higher level cluster. The distance between any two clusters A and B, of size (i.e. cardinality) | A | and | B | respectively, is taken to be the average of all distances d ( x , y ) between pairs of objects x in A and y in B, i.e. the average distance between the elements of each cluster.</p>

<h4>mcquitty (WPGMA)</h2>
<p>WPGMA (Weighted Pair Group Method with Arithmetic Mean) is a simple agglomerative (bottom-up)
hierarchical clustering method, generally attributed to Sokal and Michener.
The WPGMA method is similar to its unweighted variant, the UPGMA method.
<br><br>
At each step, the closest two clusters, say A and B, are combined into a higher-level cluster A ∪ B. Then,
its distance to another cluster C, d ( A U B , C ), is simply the arithmetic mean of the distances d ( A , C )
and d ( B , C ).</p>

<h3 id=\"subdivide\">Subdivide</h3>
<p>You can subdivide a cluster by clicking on the button, selecting the cluster you want to subdivide and selecting the number of subclusters. You can reset the dendrogram using the reset button next to the dendrogram.</p>
<p><img src=\"GS/subdivide.png\"></p>

<h3 id=\"seriation\">Seriation</h3>
<p>This input allows you to enable or disable the seriation of the columns. The matrix permutation uses an algorithm called 'reciprocal averages'. Each row is assigned a rank from 1 to n, the number of rows. For each column, a barycentre is calculated by weighting according to the row rank. Finally, the columns are reorganised by sorting them according to their barycentre.</p>
<p><img src=\"GS/seriation.png\"></p>


<h3 id=\"show1\">Visualization</h3>
<p>This input allows you to select the element to be plotted. There are tree options: plot the positive deviation from the average percentage (EPPM in French), plot the frequency or plot both. The average percentage is calculated for each category (columns) on the total number of accounts (all classes combined). From the average percentage, we obtain for each category and for each row the difference between the percentage of the category in the class and the average percentage. The EPPM corresponds to the notion of independence deviation (between rows and columns, between categories and time classes) in a chi-square test approach. Although this approach is fundamental in statistical analysis, the independence deviations here are purely indicative and are not associated with a p_value that could determine the significance of the deviations.</p>
<p><img src=\"GS/show.png\"></p>

<h3 id=\"sort1\">Sort clusters</h3>
<p>The rows are initially in the order in which they appear on the dendrogram. It is possible to rearrange the classes in time. In the interface you can drag and drop the class (green) to change the order. You can add a gap between two clusters and also remove a cluster by dragging and dropping it in the remove section.</p>
<p><img src=\"GS/sort.png\"></p>

<h3 id=\"import\">Import your data</h3>
<p>You can import your data. There are two steps. You need to upload a first csv file for the first source of information.
The data must be a contingency table.</p>
<p><img src=\"GS/import1.png\"></p>
<p>The second step is to upload the second source. You can choose between two types of data (stratigraphic link or time range data). The stratigraphic link object is a data frame with two columns. The first column contains the elements (the same number as the number of rows in the contingency table) and the second column contains a list of all the other elements connected to it. The list is a string consisting of the names (those in the first column) of the elements separated by a comma.</p>
<p><img src=\"GS/network_ui.png\"></p><p>The data frame for time range data contains the same first column. The second column contains the lower time limit and the third column contains the upper time limit.</p>
<p><img src=\"GS/timerange_ui.png\"></p>

<p><img src=\"GS/import3.png\"></p>
<p>The settings allow you to import different data frame organisation (header, column separator, ...).</p>
<h4 id=\"header\">Header</h4>
<p>Yes or no option. Do you have headers on your columns?</p>
<h4 id=\"rownames\">Rownames</h4>
<p>Yes or no option. Do you have row names on your rows?</p>
<h4 id=\"separator\">Separator</h4>
<p>Select the character you want to use to separate the columns.</p>
<h4 id=\"quote\">Quote</h4>
<p>Select the quotation marks to use on strings.</p>
<h4 id=\"dec\">Decimal</h4>
<p>Select the character to use to indicate the decimal point.</p>

<h4 id=\"csv\">CSV Format and write.table</h4>
<p>It is a data.frame with columns separated by semicolons \";\".</p>
<p>The input format for importing data is the .csv format, but also supports the .txt format as a .csv file.</p>
<p>In R, you can export your data frame to a csv file using write.csv2 or write.table. In a csv you can choose a character to separate the columns. In the same way, you can define the character to indicate the decimal point.</p>
<code>
write.table(data,file=\"path/to/name_file.csv\",sep=\";\",dec=\".\",row.names=FALSE,quote=FALSE)
</code>
<p>In Excel you can save in csv format in order to import your data frame.</p>
<p>The import interface allows you to set these values using the 'header', 'decimal', 'separator' and 'quote' options.</p>


<hr>
<h2 id=\"output\">The outputs</h2>
<hr>

<h3 id=\"eval\">Evaluation Plot</h2>

<p>It is essential to be able to evaluate the different partitions of the hierarchy in order to identify the most relevant one(s). In our case, the number k of clusters of the partition to be retained is based on several indicators calculated for different values of K: the total within-class sum of squares (WSS) and the global average of the silhouette widths.</p>
<p>When running the hclustcompro method, you have to make a choice. You must cut the dendrogram. This operation selects the partition. To compare all the possibilities, you can see the evaluation plots (WSSPlot and AveSilPlot). These two plots evaluate the relative quality of the partition.</p>
<h3 id=\"wssplot\">Within Sum of Square Plot (WSSPlot)</h3>
<p>This is the plot of the within group sum of squares against the number of clusters. The within group sum of squares decreases as the number of clusters increases. In this plot, the best partition is when adding one or more clusters doesn't decrease the WSS value. It's called the elbow method.</p>
<h4>Example:</h4>
<p><img src=\"GS/WSS.png\"></p>
<p>On this graph, we start by looking at the value for the lowest number of classes: 2. If I add a third class, we see that the WSS value will decrease (from 2.4 to 1.8). If I add another class, this value decreases again (from 1.8 to 1.4). We are looking for the moment when it is not interesting to add a class. In this case, we can keep a partition with 8 or 9 classes.</p>
<h3 id=\"avesilplot\">Average silhouette Plot</h3>
<p>This graph shows the average silhouette width of each partition (ROUSSEEUW 1987). The silhouette width is a bounded index between -1 and 1, calculated for each observation. The closer the value is to 1, the better the observation is classified. We look for the average value for a partition that is closest to 1.</p>
<h4>Example:</h4>
<p><img src=\"GS/AveSil.png\"></p>
<p>On this graph we look for the maximum value. The best evaluation corresponds to the division into 9 classes. Looking at the second best partition, we identify the one with 10 classes.</p>

<h3 id=\"selectalpha\">Select alpha</h3>
<p>This plot shows the correlation criterion for each alpha.</p>
<p><img src=\"GS/selectalpha.png\"></p>

<h3 id=\"dendrogram\">Dendrogram</h3>
<p>The dendrogram is the result of the clustering process. You can see the clusters and sub-clusters.</p>
<p><img src=\"GS/dendrogram.png\"></p>

<h3 id=\"seriograph\">Seriograph</h3>
<p><p>We have chosen the serigraph (DESACHY 2004). This tool makes it possible to highlight the evolution of ceramics over time as well as to understand the commercial relations thanks to the imported ceramics. The percentages of each category of ceramics per set are displayed. The percentages are calculated independently for each set (row). The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).</p>
<p>We have generalized this representation for other contingency data with hclustcompro object.</p>
<p><img src=\"GS/seriograph.png\"></p>
<p>In order to facilitate the exploitation of the data tables, we propose here a computerised graphic processing tool (EPPM serigraph - for Ecart Positif aux Pourcentages Moyens - positive deviation from the average percentage), which does not require specialised statistical skills and is adapted to the case of stratified sites, where the study of the evolution of artefacts can be based on the relative chronology provided by the excavation.</p>
<p>The treatment consists firstly of transforming this table of counts into a table of percentages, the total number in each set (each row) being reduced to 100; these are the proportions, or frequencies, of the types in the sets are thus compared.</p>
<p>The display of positive deviations from the mean percentages (EPPM) shows in black on a lighter background the percentage shares that are higher than the mean percentage of the variable, so as to highlight the most significant part of the values in the table.This display is simply adapted to the seriograph: when a percentage is greater than the average percentage of the type, the excess share (called here EPPM: positive deviation from the average percentage) is shown in black, centred around the axis of the type, on the grey background of the percentage bar.</p>
<p>The table is then transformed into a graphic matrix where these percentages are expressed, for each type, by horizontal bars centred on the column axis. When the rows are ordered chronologically, the silhouette formed by the superposition of these frequency bars bars makes it possible to visualise the evolution over time of the type concerned.</p>
<p>The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).</p>
<p>The processing technique applies to sets whose chronological order is not known; the lines of the graph are to be reorganised so as to obtain boat-shaped silhouettes following the hypothesis of a chronological evolution corresponding to the seriation model.</p>


<h3 id=\"timerangeclust\">Timerange plot</h3>
<p>Show the different time ranges of observations for each cluster (you will need to import time range data).</p>
<p><img src=\"GS/timerangeclust.png\"></p>


<h3 id=\"CAp\">Correspondences analysis</h3>
<p>You can see the different planes of correspondence analysis used to construct the first dissimilarity matrix D1.</p>
<p><img src=\"GS/CA.png\"></p>


<hr>
<h2 id=\"references\">References</h2>
<hr>
<p><br>
  Rousseeuw, P.J. (1987). “Silhouettes: a graphical aid to the interpretation and validation of cluster analysis”.
  In : J. Comput. Appl. Math. 20, 53–65. <a href=\"https://doi.org/10.1016/0377-0427(87)90125-7\" target=\"_blank\">⟨doi:10.1016/0377-0427(87)90125-7⟩</a>.
  <br><br>Ward, J. H., Jr. (1963). \"Hierarchical Grouping to Optimize an Objective Function\".
  Journal of the American Statistical Association, 58, 236–244.
  <br><br>Sokal R. and Michener C. (1958). \"A statistical method for evaluating systematic
  relationships\". University of Kansas Science Bulletin. 38: 1409–1438.
  <br><br>Desachy B. (2004). \"Le sériographe EPPM : un outil informatisé de sériation graphique pour tableaux de comptages. In: Revue archéologique de Picardie, n°3-4, 2004. Céramiques domestiques et terres cuites architecturales. Actes des journées d'étude d'Amiens (2001-2002-2003) pp. 39-56 <a href=\"https://doi.org/10.3406/pica.2004.2396\" target=\"_blank\">⟨doi:10.3406/pica.2004.2396⟩</a>.</p>
<br><br><br><br>



")
                           )
                )
)

EPM <- function(x) {
  x <- as.matrix(x)

  x[is.na(x)] <- 0
  ni. <- rep(1,nrow(x))
  nj. <- margin.table(x, margin = 2)
  n. <- sum(nj.)

  TabInde <- (nj.%*%t(ni.)/n.)
  TabInde <- t(TabInde)
  x <- prop.table(x, margin = 1)
  TabEcart <- x - TabInde
  TabEcart <- as.data.frame(TabEcart)
  return(TabEcart)
}

plot_EPPM <- function(x,show,permute) {
  if(is.table(x)){
    x <- as.data.frame(x)
    x <- matrix(x[,3],
                ncol=sqrt(length(x[,2])),
                dimnames=list(unique(x[,1]),unique(x[,1])))
    x <- as.data.frame(x)
  }
  if(is.matrix(x)){x <- as.data.frame(x)}
  if(!is.data.frame(x)){stop("x is not a data frame.")}
  for (i in 1:length(x[1,])){
    if(!(sum(x[,i]) == round(sum(x[,i])))){stop("The data frame must contains integers.")}
  }
  .data <- c()
  Cluster <- c()
  labels <- factor(rownames(x),levels = rev(unique(rownames(x))))
  #permute type in x
  if(permute){

    if(show == "EPPM"){
      ecart <- as.data.frame(EPM(x))
      ecart[ecart < 0] <- 0
      i <- seq(nrow(ecart),1)
      barycenter <- colSums(ecart * i) / colSums(ecart)
      x <- x[,order(barycenter)]
    }else{
      i <- seq(nrow(x),1)
      barycenter <- colSums(x * i) / colSums(x)
      x <- x[,order(barycenter)]
    }

  }
  rowsum <- rowSums(x)
  #add Weight to x
  x <- cbind(x, Weight = rowSums(x) / sum(x) * rowSums(x))
  #data
  data <- as.data.frame(x)
  data <- mutate(data,ens = labels,rowsum = rowsum)
  data <- gather(data,key = "type", value = "frequency",-ens,-rowsum, factor_key = TRUE)
  data <- mutate(data,frequency = frequency / rowsum)
  #remove col rowsum
  data <- data[-2]
  data$frequency[is.na(data$frequency)] <- 0

  #EPPM
  ecart <- as.data.frame(EPM(x[-length(x[1,])]))
  save_ecart <- ecart
  ecart <- cbind(ecart,Weight=c(0))
  ecart[ecart < 0] <- 0
  ecart <- mutate(ecart,ens = labels)
  ecart <- gather(ecart,key = "type", value = "EPPM", -ens, factor_key = TRUE)
  ecart$EPPM[ecart$type == "Weight"] <- 0

  # Join data and EPPM
  data <- inner_join(data,ecart,by = c("ens", "type"))
  data <- mutate(data,frequency = frequency - EPPM)
  data <- gather(data,key = "legend", value = "frequency", -ens,
                 -type, factor_key = TRUE)
  #if insert hiatus in df replace NA by 0 (NA are generating by divising by 0 (rowsum of a hiatus is 0))
  data[is.na(data)] <- 0
  #center (div /2 et copy for one part - and the other +)
  data <- rbind(data,data)
  data <- mutate(data,frequency = frequency * c(rep(.5, nrow(data)/2), rep(-.5, nrow(data)/2)))

  #color for Weight
  tmp <- data$frequency[data$type == "Weight"]
  levels(data$legend) <- c(levels(data$legend),"1","2","3","4","5","6","7","8","9","10","Weight")
  data$legend[data$type == "Weight"] <- "Weight"

  breaks <- c("frequency","EPPM","Weight")

  #define the color
  freq <- c("frequency","frequencies","freq","fq","fr\u00E9quence","fr\u00E9quences","fr\u00E9q")
  EPPM <- c("EPPM","ecart","\u00E9cart","ecarts","\u00E9carts")
  if(show %in% freq){
    color <- c("frequency"="#bebebe","EPPM"="#bebebe","Weight"="#7f7f7f")
    #remove EPPM from legend
    breaks <- c("frequency","Weight")
    default_col <- "#bebebe"
  }else{
    if(show %in% EPPM){
      color <- c("frequency"=NA,"EPPM"="black","Weight"="#7f7f7f")
      #remove frequency from legend
      breaks <- c("EPPM","Weight")
      default_col <- NA
    }else{
      #default color grey: frequency, black: EPPM, my_palette: weight
      color <- c("frequency"="#bebebe","EPPM"="black","Weight"="#7f7f7f")
      default_col <- NA
    }
  }
  #ggplot
  p <- ggplot(data = data) +
    facet_grid( ~type, scales = "free", space = "free_x") +
    geom_col(aes(x = ens, y = frequency, fill = legend), width = 1) +
    scale_x_discrete(labels = rev(rownames(x))) +
    coord_flip() +
    scale_fill_manual(values = color,
                      aesthetics = "fill",
                      limits = breaks,
                      na.value = default_col
    ) +
    scale_y_continuous(breaks = c(-.1,.1),labels = c("","20% ")) +
    labs(
      title = "Seriograph",
      subtitle = "EPPM (Ecart Positif au Pourcentage Moyen) Positive Deviation at Average Percentage",
      caption = "Warnings: The frequencies are calculated independently for each element (row).
You can see the relative number of observations in the Weight column"
    ) +
    theme_grey(base_size = 15) +
    annotate("segment",x = 0, y = -.04, xend = 0, yend = .04,color = "white") +
    theme(legend.position = "bottom",
          panel.spacing = unit(.2, "lines"),
          strip.text.x = element_text(angle = 90),
          axis.text.x = element_text(hjust=1,margin = margin(t = -13)),
          axis.ticks.length = unit(.5, "cm"),
          axis.title.x = element_blank(),
          axis.title.y = element_blank(),
          axis.ticks.y = element_blank(),
          plot.caption = element_text(face = "italic", hjust = 0)
    )
  plot(p)
}

server <- function(input, output, session) {
  #------------------------------------------------------#
  #  modalDialog import
  #------------------------------------------------------#
  import1Modal <- function(failed = FALSE) {
    modalDialog(title="Step 1:",size=c("l"),
                div(style="height:15px;"),
                h3("Step 1",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("Upload file contingency table"),
                # Input: Select a file ----
                fileInput("fileD1", span(icon("import", lib = "glyphicon"),"Choose CSV File"),
                          multiple = FALSE,
                          accept = c("text/csv",
                                     "text/comma-separated-values,text/plain",
                                     ".csv")
                ),
                actionButton(inputId = "toggle", span(icon("cogs",lib = "font-awesome"),"Settings")),
                fluidRow(shinydashboard::box(id="myBox",
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Checkbox if file has header ----
                                                      span(strong("Header")),
                                                      checkboxInput("headerD1", "Header (colnames)", TRUE),
                                                      checkboxInput("rownamesD1", "rownames", TRUE)
                                               ),
                                               column(width = 6,
                                                      # Input: Select quotes ----input$dec
                                                      radioButtons("quoteD1", "Quote",
                                                                   choices = c(None = "",
                                                                               "Double Quote" = '"',
                                                                               "Single Quote" = "'"),
                                                                   selected = '"')
                                               )
                                             ),
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Select separator ----
                                                      radioButtons("sepD1", "Separator",
                                                                   choices = c(Comma = ",",
                                                                               Semicolon = ";",
                                                                               Tab = "\t",
                                                                               Space = " "),
                                                                   selected = ";")
                                               ),
                                               column(width = 6,
                                                      radioButtons("decD1", "Decimal",
                                                                   choices = c("Dot" = '.', "Comma" = ","),
                                                                   selected = '.')
                                               )
                                             )
                )),
                tableOutput("contentsD1"),
                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.",
                             style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("Finishimport", "Finish")
                )
    )
  }
  #------------------------------------------------------#
  #  modalDialog sort periods
  #------------------------------------------------------#
  sortPeriod <- function(failed = FALSE) {
    modalDialog(title="Sort clusters:",size=c("m"),
                div(style="height:15px;"),
                h3("Sort rows",style="text-align:center;"),hr(style="border-color: #222222;"),
                h4("Sort each row from the oldest to the newest!"),
                helpText("Note: Drag and drop rows to sort them.
                         Drag unused row into the trash to remove them from the seriograph.
                         You can insert 'Hiatus' by picking it up and dropping it off where you want."),
                orderInput('seq2',span(icon("sort-amount-up",lib = "font-awesome"),'Sort the row'),
                           items = labels(values[["data1"]])[[1]] [ values[["new_order"]] ] ,
                           placeholder = 'There must be at least one item',
                           connect = "rm",
                           item_class = 'success'),
                hr(style="border-color: #b2b2b2;"),
                column(4,
                       orderInput('hiatus', span(icon("plus",lib = "font-awesome"),'Add'), items = c("Hiatus"),
                                  connect = 'seq2', as_source = TRUE, item_class = 'warning', width = '70px')
                ),
                column(8,
                       orderInput('rm', span(icon("trash",lib = "font-awesome"),'Remove from seriograph'),
                                  items = values[["rm"]],
                                  connect = "seq2",
                                  placeholder = 'Drag the items here to delete them',
                                  item_class = 'danger')
                ),
                div(style="height:50px;"),
                if (failed)
                  div(tags$b("You have to select the sequence!", style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("tri", "OK")
                )
    )
  }

  values <- reactiveValues()
  values[["hiatus"]] <- NULL
  values[["rm"]] <- NULL
  values[["seriograph"]] <- NULL
  shinyjs::hide("toggle")
  #contingency table
  df = data.frame(
    Cat10 = c(1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
    Cat20 = c(4,8,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0),
    Cat30 = c(18,24,986,254,55,181,43,140,154,177,66,1,24,15,0,31,37),
    Cat40.50 = c(17,121,874,248,88,413,91,212,272,507,187,40,332,174,17,288,224),
    Cat60 = c(0,0,1,0,0,4,4,3,0,3,0,0,0,0,0,0,0),
    Cat70 = c(3,1,69,54,10,72,7,33,74,36,16,4,40,5,0,17,13),
    Cat80 = c(4,0,10,0,12,38,2,11,38,26,25,1,18,4,0,25,7),
    Cat100.101 = c(23,4,26,51,31,111,36,47,123,231,106,21,128,77,10,151,114),
    Cat102 = c(0,1,2,2,4,4,13,14,6,6,0,0,12,5,1,17,64),
    Cat110.111.113 = c(0,0,22,1,17,21,12,20,30,82,15,22,94,78,18,108,8),
    Cat120.121 = c(0,0,0,0,0,0,0,0,0,0,66,0,58,9,0,116,184),
    Cat122 = c(0,0,0,0,0,0,0,0,0,0,14,0,34,5,0,134,281),
    row.names = c("AI01","AI02","AI03","AI04","AO03","AI05","AO01","AI07","AI08",
                  "AO02","AI06","AO04","APQR01","APQR02","AO05","APQR03","AI09")
  )
  values[["data1"]] <- df
  values[["new_order"]] <- 1:17

  #url param
  o=observe({
    query <- parseQueryString(session$clientData$url_search)

    for (i in 1:(length(reactiveValuesToList(input)))) {
      nameval = names(reactiveValuesToList(input)[i])
      valuetoupdate = query[[nameval]]

      if (!is.null(query[[nameval]])) {
        if (is.na(as.numeric(valuetoupdate))) {
          #updateTextInput(session, nameval, value = valuetoupdate)
          if(nameval == 'data'){
            data1 = read.csv(valuetoupdate,header =TRUE, row.names=1, sep=",")
            if(ncol(data1)==0){
              data1 = read.csv(valuetoupdate,header =TRUE, row.names=1, sep=";")
            }
            values[["data1"]] = data1
            values[["new_order"]] <- 1:length(labels(data1)[[1]])
          }
        }
        else {
          #updateTextInput(session, nameval, value = as.numeric(valuetoupdate))
        }
      }

    }
    o$destroy()

  })

  observeEvent(input$width,{
    values[["width"]] <- input$width
  })
  observeEvent(input$height,{
    values[["height"]] <- input$height
  })



  output$EPPM <- renderPlot({
    cont <- values[["data1"]]
    show <- c("both","frequency","EPPM")[as.numeric(input$show)]

    hiatus <- values[["hiatus"]]
    new_order <- values[["new_order"]]

    #reorder and rm
    if(length(new_order) > 0){
      cont <- cont[new_order,]
    }

    #hiatus
    if(length(hiatus) > 0 && sum(is.na(hiatus)) == 0){
      #insert hiatus
      label <- "Hiatus"
      insert <- values[["hiatus"]]
      insert <- sort(insert)
      for (i in length(insert):1){
        if(insert[i] < length(cont[,1]) && insert[i] >= 1 && insert[i] == trunc(insert[i])){
          cont <- rbind(cont[1:insert[i],], label = rep(0,length(cont[1,])),
                         cont[(insert[i]+1):length(cont[,1]),])
          if(length(insert)== 1){
            row.names(cont)[insert[i]+1] <- label
          }else{
            row.names(cont)[insert[i]+1] <- paste0(label,".",(length(insert)+1 - i))
          }
        }else{
          warning("Some positions are outside the range. Ex: ",insert[i])
        }
      }
    }

    plot_EPPM(cont, permute=input$permute, show = show)
    values[["seriograph"]] <- recordPlot()
  })

  observeEvent(input$tri,{
    order <- input$seq2

    values[["rm"]] <- input$rm

    if(is.na(order)[1] || ( length(order) == 1 && order[1] == "Hiatus" )){
      removeModal()
    }else{
      hiatus <- which(order == "Hiatus")
      for (i in 1:length(hiatus)){
        hiatus[i] <- hiatus[i] - 1 * i
      }

      values[["hiatus"]] <- hiatus

      order <- order[order != "Hiatus"]

      new_order <- rep(NA,length(order))
      for(i in 1:length(order)){
        new_order[i] <- which(labels(values[["data1"]])[[1]] == order[i])
      }
      values[["new_order"]] <- new_order
    }
    removeModal()
  })



  observeEvent(input$data,{
    showModal(import1Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$Finishimport,{
    values[["data1"]] <- values[["D1"]]
    values[["new_order"]] <- labels(values[["D1"]])[[1]]
    removeModal()
  })

  observeEvent(input$sort,{
    showModal(sortPeriod())
  })

  output$contentsD1 <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.

    req(input$fileD1)
    inFile <- input$fileD1

    if(is.null(inFile))
      return(NULL)

    tryCatch({
      if(input$rownamesD1){
        df <- read.table(input$fileD1$datapath,
                         header = input$headerD1,
                         sep = input$sepD1,
                         dec = input$decD1,
                         quote = input$quoteD1,
                         row.names = 1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }else{
        df <- read.table(input$fileD1$datapath,
                         header = input$headerD1,
                         sep = input$sepD1,
                         dec = input$decD1,
                         quote = input$quoteD1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }

    },
    error = function(e) {
      # return a safeError if a parsing error occurs
      stop(safeError(e))
    })
    values[["D1"]] <- df
    return(head(df))
  },rownames=TRUE)


  #EXPORT BUTTON
  output$seriograph.pdf <- downloadHandler(
    filename = "seriograph.pdf",
    content = function(file) {
      cairo_pdf(file,width=12)
      print(values[["seriograph"]])
      dev.off()
    }
  )

  output$seriograph.png <- downloadHandler(
    filename = "seriograph.png",
    content = function(file) {
      png(file,
          width = values[["width"]],
          height = values[["height"]]
      )
      print(values[["seriograph"]])
      dev.off()
    }
  )


  observeEvent(input$toggle, {
    if(input$toggle %% 2 == 0){
      shinyjs::hide(id = "myBox")
    }else{
      shinyjs::show(id = "myBox")
    }
  })



}

library(SPARTAAS)
suppressMessages(library(shinyjs))
library(shinyjqui)
suppressMessages(library(shinydashboard))
library(colorspace)
suppressMessages(library(dplyr))
library(tidyr)
library(ggplot2)
library(shinyWidgets)
library(ade4)
library(shinythemes)
library(shinycssloaders)
library(plotly)
library(scatterD3)
#library(caret)


ui <- fluidPage(useShinyjs(),withMathJax(),style="padding-top: 150px;",theme = shinytheme("flatly"),
    tags$style(HTML('#sort,#data,#subdivise,#tri,#sub1,#sub2,
         #import2,#import3,#import3,#Finishimport,#back1,#back2{color:white;background-color:#337ab7;border-color:#337ab7}
         #back{color:white;background-color:#f0ad4e}
         #tabsetperso{background-color:#e5e5e5;}
         #help{text-align;right;}
         #tabsetperso{margin-bottom:20px;}
         .btn-default {
           background-color:#5da3a8;
           border-color:#5da3a8;
         }
         .dropdown-menu > li > a {
           background-color:#fff;
           border-color:#dce4ec;
         }
         #cerardat_plot_ref > div {display:flex;flex-wrap:wrap;width:94vw;}
         #cerardat_plot_ref > div > div.shiny-plot-output {width:450px !important;}
         #cerardat_plot_sup > div {display:flex;flex-wrap:wrap;width:94vw;}
         #cerardat_plot_sup > div > div.shiny-plot-output {width:450px !important;}
         #lmplot > div {display:flex;flex-wrap:wrap;width:65vw;}
         #lmplot > div > div.shiny-plot-output {width:450px !important;}
         .disabled{color:grey}
         .disabled:hover{color:grey;background-color:#e5e5e5 !important}
    ')),
    tags$head(
      tags$link(rel = "icon", type = "image/gif", href = "https://spartaas.gitpages.huma-num.fr/r-package/img/lambda.png"),
      tags$title("cerardat")
    ),
  #header panel
  absolutePanel(class = "panel panel-default",
    style="z-index: 2000;padding: 8px; background: #ecf0f1; opacity: 1;border-bottom: 1px solid #2c3e50;",
    top = 0, left = 0, right = 0,
    fixed = TRUE,
    h2("cerardat: ceramic estimation date"),
    div(span(strong("SPARTAAS | cerardat")))
  ),
  #footer
  absolutePanel(style="z-index: 2000;padding: 0px; border-bottom: 0px solid #CCC; background: #fff;opacity: 1;",
                bottom = 0, left = 0,
                fixed = TRUE,
                div(span(strong("SPARTAAS")),span("[Bellanger,Coulon,Husi]",style ="font-size:14px;"))
  ),
  #go top panel
  absolutePanel(class = "panel-hover",
    style="opacity:0.8;z-index: 1000;padding: 8px; border-bottom: 1px solid #CCC;
    background: #FFFFEE;width:30px;margin-right:10px;",
    bottom = 20, right = 0,
    fixed = TRUE,
    HTML("<a href=\"#top\"><span class=\"glyphicon glyphicon-chevron-up\"></span></a>")
  ),
  #-----------------------------------#
  #      NavBar
  #-----------------------------------#
  navbarPage("cerardat",id="navbar",
    tabPanel(value = "1",span(icon("home", lib = "glyphicon"),strong("Home")),style="max-width:1200px;",
             br(),
             sidebarLayout(
               #-----------------------------------#
               sidebarPanel(style="",
                            h1("R Package:"),
                            br(),
                            HTML("<p>This method is part of the <a target='_blank' href='https://spartaas.gitpages.huma-num.fr/r-package/index.html'>SPARTAAS</a> package.</p>
                                                        <p>If you are interested you can install our R package available on the <a target='_blank' href='https://cran.r-project.org/package=SPARTAAS'>CRAN</a>.</p>

                                                        ")
               ),
               #-----------------------------------#
               mainPanel(
                 HTML("
                      <h3>Introduction</h3>
                      <p>Analyzing chronological patterns is one of the major issues in archaeology. How can the date of a specific
context be estimated? Is it possible to identify residual and intrusive material in it at the same time?
Numerous statistical methodological approaches have been developed and implemented to estimate
dates but have less often addressed the issue of socio-economic area or the functional interpretation of
contexts. This article deals with the construction and analysis of two different probability estimate
density curves of context dates using pottery. By contrasting the two curves we can define the boundaries
of the socio-economic area and make a chrono-functional interpretation of a context. This statistical
tool allows the archaeologist to visualize and analyze chronological patterns easily.</p>
<p>Dating an archaeological context is always delicate. It is
important to keep in mind what is to be dated, either a one-off
event occurring at a specific moment, or a process built up over
time. This study proposes datings based on the statistical modelling
of pottery data, in contrast to “traditional” datings which are
frequently based on an intuitive comparison of pottery assemblages.
Of course, this does not in any way deny the importance of
the pottery expert’s knowledge in the dating process; on the
contrary, it enables this knowledge to be better integrated through
a systemic procedure, which is essential when there is a very large
corpus of data to be analyzed.</p>
<p>The methodology is based on a statistical and visual approach
using two estimated density curves to date each archaeological
context. Two steps were required in the statistical procedure, each
leading to the construction of a density curve. The first enabled us
to estimate a date corresponding to the <i>terminus post quem</i> of the
context, a cursor reflecting an event dated in calendar time.
The second step, based on the results of the first,
allows the chronological profile of the context to be estimated,
giving a picture that is closer to archaeological time, in other words
the rate of accumulation.</p>
                      ")
               )
             )
    ),
    tabPanel(value = "2",span(icon("chart-area",lib="font-awesome"),strong("cerardat")),style="max-width:1200px;",
              tabsetPanel(id = "tabsetperso",type="tabs",
                      tabPanel(id="tab11","Overview",
                               sidebarPanel(width = 5,
                                            fluidRow(
                                              h4("Import your data:"),
                                              column(10,
                                                     actionButton("data",icon("import", lib = "glyphicon"))
                                              ),
                                              column(1,
                                                     # Help
                                                     dropdownButton(
                                                       h3("data format"),
                                                       hr(style="border-color: #222222;"),
                                                       HTML("<p>The data are in the form of two datasets.</p>
                                                       <p>1/ Reference data: count table plus a first column with dates, if known.</p>
                                                       <p>2/ Supplementary data: count table (same number of columns) plus a column for dates, if known.</p>
                                                       <p>Example:</p>
                                                       <img style='width:280px;' src='data_exemple.png' alt='data'>"),
                                                       circle = TRUE, status = "danger", icon = icon("question"), width = "300px",
                                                       size = "sm",right=FALSE,
                                                       tooltip = tooltipOptions(title = "Help", placement = "top")
                                                     )
                                                     # Help
                                              )
                                            ),



                                            div(style="height:20px;"),
                                            HTML("
<p>In settings:</p>
<ul>
  <li>You can check the results of the Correspondance Analysis</li>
  <li><b>You must choose the number of components</b> to keep in the linear model</li>
  <li>You can compare the predicted dates with the actual dates and define the confidence interval you want.</li>
</ul>
<p>And then you can see the results in the result and visualization tabs.</p>
                                                 ")
                               ),
                               mainPanel(style = 'max-width:1000px;',width = 7,
                                         HTML("
<p>For each archaeological context, we used pottery to construct
and analyze two different probability density curves of estimated
dates, one based on the dating of events in calendar time (hereafter
called “event time”), and one based on duration/time-span (hereafter
called “accumulation time”). The chain of reasoning for a given
archaeological context can be summarized as follows:</p>
<ul>
  <li>Obtain two density curves representing the dating of an
archaeological context.<ul>
    <li>Step 1: The first is a Gaussian curve derived from a linear
regression model. This allows the mean date of issue of
a coin to be estimated from the pottery profile. From an
archaeological point of view, this estimation of a terminus
post quem, with all the biases involved in the use of coins for
dating, provides a fundamental basis for chronology building
in calendar time;</li>
    <li>Step 2: The second curve is a mixture of Gaussians derived
from the previous model. The date of an archaeological
context is estimated using the weighted average of the
estimated dates of the pottery fabrics found within it.
Assuming that the fabrics are statistically independent, the
associated probability density of the date can be estimated
as a weighted sum of Gaussian distributions. From an
archaeological point of view, this dating estimation provides
a closer representation of accumulation time recorded in the
soil (Olivier, 2001; Wirtz and Olivier, 2003). At best,
depending on the quality of the archaeological context, it
can be interpreted as a formation process reflecting the
duration or succession of events on the scale of archaeological
time, and at worst, as imprecise dating due to
contamination of the context by residual or intrusive
material.</li>
  </ul></li>
  <li>Compare the density curves, in order to:<ul>
    <li>Validate the method from a chronological perspective,
exploring the duration or intensity of occupation for each
context;</li>
    <li>Identify the boundaries of socio-economic entities within
the broader area (supplemantary data), using a chronological model based on the reference site;</li>
    <li>Obtain a clearer understanding of chrono-functional issues
through a better interpretation of the type of archaeological context.</li>
  </ul></li>
</ul>

                                              ")
                                )
                      ),
             tabPanel(id="tab12","Settings",
                      tabsetPanel(id = "tabsetperso1",type="tabs",
                                  tabPanel(id="set1","Correspondance Analysis (CA)",
                                           sidebarPanel(
                                              HTML("
<p>A correspondence analysis (CA) was carried out to summarize
the information in the reference corpus data; the data matrix
consisted of the archaeological contexts and fabrics quantified
using MINVC whether or not they had
been dated with coins.</p>
<p>In order to estimate a date for the context, it is essential to refer
to objects dated by another source, in this instance, nonresidual
coins. These contexts were selected on a very strict
basis for their chrono-stratigraphic reliability, level of domestic
occupation, or enclosures with long urban stratigraphic
sequences, thereby minimizing any bias linked to disparity
between the date of the coin and that of the context.</p>
<p>estimating the date of an event recorded in
the ground (an archaeological context for the archaeologist) based
on the pottery assemblage of which it is comprised is achieved
by fitting a regression model linking a known date in calendar time,
in this instance the date of issue of a coin, to its pottery profile.</p>
<p>We used the results of the first step and the properties of the CA
to obtain an estimated value of the date of each fabric. We could
then define the archaeological time shown as dateAc, in other
words the accumulation time of a context, as the weighted sum of
fabric datings; the weights are the proportions of MINVC of each
fabric in the context.</p>

                                                   ")
                                           ),
                                           mainPanel(

                                             ####################################
                                             ##        Select Component        ##
                                             ####################################
                                             br(),br(),
                                             h3("Correspondence analysis"),
                                             hr(style="border-color: #222222;"),
                                             h4("Eigenvalues"),

                                             withSpinner(jqui_resizable(plotlyOutput("eigenvalue")),color='#18bc9c'),br(),

                                             numericInput("axe1", "Axe X", 1, min = 1, max = NA, step = 1),
                                             numericInput("axe2", "Axe Y", 2, min = 1, max = NA, step = 1),
                                             withSpinner(jqui_resizable(scatterD3Output("CA")),color='#18bc9c'),

                                             br()
                                           )
                                           ),
                                  tabPanel(id="set2","Linear model (lm)",
                                           sidebarPanel(
                                             verbatimTextOutput("cerar"),
                                             sliderInput("k", label = "Number of Components", min = 1, max = 20, step = 1, value = 2),
                                             HTML("
<p>We then kept only the first factorial axes, accounting for a part of the total
variance. In this way, becomes an incomplete
reconstitution of the data. This principle is used in many factor
analysis techniques, providing away of reducing the number of
explanatory variables in the linear regression model(the other components
may reasonably be ignored!) (Matzner-Løber, 2010).</p>
                                                  "),
                                             hr(style = "border-top: 1px solid #acacac;"),

                                             h4("statistical testing of residual hypotheses"),
                                             span("normality of residuals:"),
                                             verbatimTextOutput("SW"),
                                             span(textOutput("sw"), style="color:red"),
                                             span("no autocorrelation of residuals:"),
                                             verbatimTextOutput("DW"),
                                             span(textOutput("dw"), style="color:red"),
                                             span("heteroskedasticity of the variance of the residuals:"),
                                             verbatimTextOutput("BP"),
                                             span(textOutput("bp"), style="color:red")

                                           ),
                                           mainPanel(
                                             h4("Number of components to be kept in the linear model"),
                                             selectInput("quid", "display:",
                                                         c("both" = "both",
                                                           "PRESS" = "PRESS",
                                                           "MSE" = "MSE")),
                                             withSpinner(jqui_resizable(plotOutput("RMSE")),color='#18bc9c'),br(),
                                             withSpinner(jqui_resizable(plotOutput("R_sq")),color='#18bc9c'),br(),

                                             br(),
                                             uiOutput("formula_dateEv"),
                                             br(),
                                             verbatimTextOutput("lm"),
                                             uiOutput("lmplot")
                                           )
                                  ),
                                  tabPanel(id="set2","Control",
                                           sidebarPanel(
                                             numericInput("confidence", "Confidence:", 0.95, min = 0.00, max = 0.99, step= 0.01)#,
                                             #verbatimTextOutput("checkbox"),
                                             #checkboxGroupInput("col.refbox", "Choose which data are the reference:",
                                             #                   choiceNames = list(""),
                                             #                   choiceValues = list(0)
                                             #)
                                           ),
                                           mainPanel(
                                             h4("Comparison of estimated dates (black) with actual dates in ref data (red)"),
                                             HTML("<p>It is the dates that are used to build the model. It is advisable to choose reliable sets. You can see the effect of adding or removing components in the model on the date estimates.</p>"),
                                             withSpinner(jqui_resizable(plotlyOutput("evaldateref")),color='#18bc9c'),
                                             h4("Comparison of estimated dates (black) with actual dates in sup data (red)"),
                                             HTML("<p>These dates are not used in the construction of the model. It is advisable to add sets that are either far from the study area or whose pottery quality is not as reliable as the one above. These dates can also be used to check if the model is working.</p>"),
                                             withSpinner(jqui_resizable(plotlyOutput("evaldatesup")),color='#18bc9c'),

                                             br()
                                           )
                                  )
                      )

             ),
             tabPanel(id="tab13","Results",
                      tabsetPanel(id = "tabsetperso2",type="tabs",
                                  tabPanel(id="datesite","Dating archaeological contexts (dateEV and dateAC)",
                                           tableOutput('tablesite')
                                  ),
                                  tabPanel(id="dategt","Dating technical group",
                                           tableOutput('tableGT')
                                  )
                      )
             ),
             tabPanel(id="tab14","Visualization",
                      downloadButton("downloadData","Download plots as png"),br(),
                      tabsetPanel(id = "tabsetperso2",type="tabs",
                        tabPanel(id="col.ref","ref data",
                                 ####################################
                                 ##         cerardat plot          ##
                                 ####################################
                                 uiOutput("cerardat_plot_ref")
                                 ),
                        tabPanel(id="col.sub","sup data",
                                 uiOutput("cerardat_plot_sup")
                        )
                      )
             )
          )
    ),
    tabPanel(value = "3",span(icon("envelope", lib = "glyphicon"),strong("Contact")),style="max-width:1200px;",
      HTML("<style>
        .carte{
        border-left-width: 4px;
        border-left-style: solid;
        border-color: #2ac0a2;
        padding-left: 10px;
        }
        </style>
        <h3>Authors:</h3>
        <hr>
        <div class=\"carte\"><h4>L. Bellanger</h4><h5>mail: &lt;lise.bellanger@univ-nantes.fr&gt;</h5></div>
        <div class=\"carte\"><h4>P. Husi</h4><h5>mail: &lt;philippe.husi@univ-tours.fr&gt;</h5></div>
        <div class=\"carte\"><h4>A. Coulon</h4></div>
        <h3>Maintainer:</h3>
        <hr>
        <div class=\"carte\"><h4>A. Coulon</h4><h5>mail: &lt;arthur-coulon@outlook.fr&gt;</h5></div>
        ")
    )
  )
)

load(file = "./data/datacerardat.RData")

press <- function(linear.model) {
  # calculate the predictive residuals
  pr <- residuals(linear.model)/(1-lm.influence(linear.model)$hat)
  # calculate the PRESS
  PRESS <- mean(pr^2)
  return(PRESS)
}

log10Tck <- function(side, type){
  lim <- switch(side,
                x = par('usr')[1:2],
                y = par('usr')[3:4],
                stop("side argument must be 'x' or 'y'"))
  at <- floor(lim[1]) : ceiling(lim[2])
  return(switch(type,
                minor = outer(1:9, 10^(min(at):max(at))),
                major = 10^at,
                stop("type argument must be 'major' or 'minor'")
  ))
}

CA_var_data <- function(res, xax = 1, yax = 2, var_sup = TRUE, var_hide = "None",
                        var_lab_min_contrib = 0) {
  tmp_x <- res$vars %>%
    filter(Axis == xax) %>%
    select("Level", "Position", "Type", "Class", "Coord", "Contrib", "Cos2", "Count")
  tmp_y <- res$vars %>%
    filter(Axis == yax) %>%
    select("Level", "Position", "Type", "Class", "Coord", "Contrib", "Cos2", "Count")
  if (!var_sup) {
    tmp_x <- tmp_x %>% filter(Type == 'Active')
    tmp_y <- tmp_y %>% filter(Type == 'Active')
  }
  if (var_hide != "None") {
    tmp_x <- tmp_x %>% filter(Position != var_hide)
    tmp_y <- tmp_y %>% filter(Position != var_hide)
  }
  tmp <- tmp_x %>%
    left_join(tmp_y, by = c("Level", "Position", "Type", "Class", "Count")) %>%
    mutate(Contrib = Contrib.x + Contrib.y,
           Cos2 = Cos2.x + Cos2.y,
           tooltip = paste(paste0("<strong>", Level, "</strong><br />"),
                           paste0("<strong>",
                                  gettext("Position", domain = "R-explor"),
                                  ":</strong> ", Position, "<br />"),
                           paste0("<strong>Axis ",xax," :</strong> ", Coord.x, "<br />"),
                           paste0("<strong>Axis ", yax," :</strong> ", Coord.y, "<br />"),
                           ifelse(is.na(Cos2), "",
                                  paste0("<strong>",
                                         gettext("Squared cosinus", domain = "R-explor"),
                                         ":</strong> ", Cos2, "<br />")),
                           ifelse(is.na(Contrib), "",
                                  paste0("<strong>",
                                         gettext("Contribution:", domain = "R-explor"),
                                         "</strong> ", Contrib, "<br />")),
                           ifelse(is.na(Count), "",
                                  paste0("<strong>",
                                         gettext("Count:", domain = "R-explor"),
                                         "</strong> ", Count))),
           Lab = ifelse(Contrib >= as.numeric(var_lab_min_contrib) |
                          (is.na(Contrib) & as.numeric(var_lab_min_contrib) == 0), Level, ""))
  data.frame(tmp)
}

server <- function(input, output, session) {
  #------------------------------------------------------#
  #  modalDialog wait message
  #------------------------------------------------------#
  waitModal <- function() {
    modalDialog(title="Initialisation", size = c("s"),
                div(style="height:25px;"),
                h4("Calculation in progress"),
                span('This may take a while...'),
                hr(style="border-color: #222222;"),
                footer = NULL
    )
  }
  #------------------------------------------------------#
  #  modalDialog import
  #------------------------------------------------------#
  import1Modal <- function(failed = FALSE) {
    modalDialog(title="Reference dataset:",size=c("l"),
                div(style="height:15px;"),
                h3("Reference dataset",style="text-align:center;"),hr(style="border-color: #222222;"),
                p("reference data: count table plus a first column giving dates if known."),
                h4("Upload file"),
                # Input: Select a file ----
                fileInput("fileD1", span(icon("import", lib = "glyphicon"),"Choose CSV File"),
                          multiple = FALSE,
                          accept = c("text/csv",
                                     "text/comma-separated-values,text/plain",
                                     ".csv")
                ),
                actionButton(inputId = "toggle", span(icon("gears",lib = "font-awesome"),"Settings")),
                fluidRow(shinydashboard::box(id="myBox",
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Checkbox if file has header ----
                                                      span(strong("Header")),
                                                      checkboxInput("headerD1", "Header (colnames)", TRUE),
                                                      checkboxInput("rownamesD1", "rownames", TRUE)
                                               ),
                                               column(width = 6,
                                                      # Input: Select quotes ----input$dec
                                                      radioButtons("quoteD1", "Quote",
                                                                   choices = c(None = "",
                                                                               "Double Quote" = '"',
                                                                               "Single Quote" = "'"),
                                                                   selected = '"')
                                               )
                                             ),
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Select separator ----
                                                      radioButtons("sepD1", "Separator",
                                                                   choices = c(Comma = ",",
                                                                               Semicolon = ";",
                                                                               Tab = "\t",
                                                                               Space = " "),
                                                                   selected = ";")
                                               ),
                                               column(width = 6,
                                                      radioButtons("decD1", "Decimal",
                                                                   choices = c("Dot" = '.', "Comma" = ","),
                                                                   selected = '.')
                                               )
                                             )
                )),
                tableOutput("contentsD1"),
                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.",
                             style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("import2", "Next")
                )
    )
  }

  import2Modal <- function(failed = FALSE) {
    modalDialog(title="Supplementary dataset:",size=c("l"),
                div(style="height:15px;"),
                h3("Supplementary dataset",style="text-align:center;"),hr(style="border-color: #222222;"),
                p("Supplementary data: count table (same number of columns) plus a column for dates if known."),
                h4("Upload file"),
                # Input: Select a file ----
                fileInput("fileD2", span(icon("import", lib = "glyphicon"),"Choose CSV File"),
                          multiple = FALSE,
                          accept = c("text/csv",
                                     "text/comma-separated-values,text/plain",
                                     ".csv")
                ),
                actionButton(inputId = "toggle", span(icon("gears",lib = "font-awesome"),"Settings")),
                fluidRow(shinydashboard::box(id="myBox",
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Checkbox if file has header ----
                                                      span(strong("Header")),
                                                      checkboxInput("headerD2", "Header (colnames)", TRUE),
                                                      checkboxInput("rownamesD2", "rownames", TRUE)
                                               ),
                                               column(width = 6,
                                                      # Input: Select quotes ----input$dec
                                                      radioButtons("quoteD2", "Quote",
                                                                   choices = c(None = "",
                                                                               "Double Quote" = '"',
                                                                               "Single Quote" = "'"),
                                                                   selected = '"')
                                               )
                                             ),
                                             fluidRow(
                                               column(width = 6,
                                                      # Input: Select separator ----
                                                      radioButtons("sepD2", "Separator",
                                                                   choices = c(Comma = ",",
                                                                               Semicolon = ";",
                                                                               Tab = "\t",
                                                                               Space = " "),
                                                                   selected = ";")
                                               ),
                                               column(width = 6,
                                                      radioButtons("decD2", "Decimal",
                                                                   choices = c("Dot" = '.', "Comma" = ","),
                                                                   selected = '.')
                                               )
                                             )
                )),
                tableOutput("contentsD2"),
                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.",
                             style = "color: red;")),
                footer = tagList(
                  actionButton("import1", "Back"),
                  actionButton("Finishimport", "Finish")
                )
    )
  }

  values <- reactiveValues()

  #raw data
  values[["date"]] = datacerardat$date
  values[["row.sup"]] = datacerardat$row.sup
  values[["df"]] = datacerardat$df
  values[["row.ref"]] = NULL
  values[["cerar_res"]] = NULL
  values[["D1"]] = NULL
  values[["D2"]] = NULL


  shinyjs::disable(selector='a[data-value="Results"]')
  shinyjs::disable(selector='a[data-value="Visualization"]')

  observeEvent(input$data,{
    showModal(import1Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$import1,{
    showModal(import1Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$import2,{
    showModal(import2Modal())
    shinyjs::hide(id = "myBox")
  })

  observeEvent(input$toggle, {
    if(input$toggle %% 2 == 0){
      shinyjs::hide(id = "myBox")
    }else{
      shinyjs::show(id = "myBox")
    }
  })

  observeEvent(input$Finishimport,{
    values[["date"]] = c(values[["D1"]][,1],values[["D2"]][,1])
    values[["row.sup"]] = (length(values[["D1"]][,1])+1):(length(values[["D1"]][,1])+length(values[["D2"]][,1]))
    values[["df"]] = rbind(values[["D1"]][,-c(1)],values[["D2"]][,-c(1)])
    values[["row.ref"]] = NULL
    values[["cerar_res"]] = NULL
    shinyjs::disable(selector='a[data-value="Results"]')
    shinyjs::disable(selector='a[data-value="Visualization"]')

    removeModal()
  })

  output$contentsD1 <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.

    req(input$fileD1)
    inFile <- input$fileD1

    if(is.null(inFile)){
      return(NULL)
    }

    tryCatch({
      if(input$rownamesD1){
        df <- read.table(input$fileD1$datapath,
                         header = input$headerD1,
                         sep = input$sepD1,
                         dec = input$decD1,
                         quote = input$quoteD1,
                         row.names = 1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }else{
        df <- read.table(input$fileD1$datapath,
                         header = input$headerD1,
                         sep = input$sepD1,
                         dec = input$decD1,
                         quote = input$quoteD1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }

    },
    error = function(e) {
      # return a safeError if a parsing error occurs
      stop(safeError(e))
    })
    values[["D1"]] <- df
    return(head(df))
  },rownames=TRUE)

  output$contentsD2 <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.

    req(input$fileD2)
    inFile <- input$fileD2

    if(is.null(inFile))
      return(NULL)

    tryCatch({
      if(input$rownamesD2){
        df <- read.table(input$fileD2$datapath,
                         header = input$headerD2,
                         sep = input$sepD2,
                         dec = input$decD2,
                         quote = input$quoteD2,
                         row.names = 1,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }else{
        df <- read.table(input$fileD2$datapath,
                         header = input$headerD2,
                         sep = input$sepD2,
                         dec = input$decD2,
                         quote = input$quoteD2,
                         check.names = TRUE,
                         stringsAsFactors=FALSE)
      }
    },
    error = function(e) {
      # return a safeError if a parsing error occurs
      stop(safeError(e))
    })
    values[["D2"]] <- df
    return(head(df))
  },rownames=TRUE)

  output$eigenvalue <- renderPlotly({
    date = values[["date"]]
    df = values[["df"]]
    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))

    #nettoyage des GT ie GT<5
    GT_rm_ref = which(colSums(df[row.ref,])<5)
    #?????????????????
    #GT_rm_sup = which(colSums(df[-c(GT_rm_ref),row.sup])<5)

    #data reference
    if(sum(GT_rm_ref)==0){
      data_ref = df[row.ref,]
    }else{
      data_ref = df[row.ref,-c(GT_rm_ref)]
    }

    DOF = min(ncol(data_ref)-1,nrow(data_ref)-1)
    ca.NMI.init <- ade4::dudi.coa(data_ref,scannf=FALSE,nf=DOF)

    inertia = ade4::inertia.dudi(ca.NMI.init, row.inertia=T, col.inertia=T)
    data_df = data.frame(
      Dim = 1:DOF,
      Inertia = inertia$tot.inertia[,1] / sum(inertia$tot.inertia[,1]),
      Cumul = inertia$tot.inertia[,3]
    )
    p = ggplot(data_df, aes(x=Dim, y=Inertia,text = paste("Percentage:",arrondi(Inertia,4)*100,"%<br>Cumul:",arrondi(Cumul,2),"%"))) +
      geom_bar(stat="identity",fill="steelblue")+
      theme_minimal()
    p
  })

  output$RMSE <- renderPlot({
    df = values[["df"]]
    date = values[["date"]]
    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))

    #vector for ref data (every not in row.sup)

    #nettoyage des GT ie GT<5
    GT_rm_ref = which(colSums(df[row.ref,])<5)
    #?????????????????
    #GT_rm_sup = which(colSums(df[-c(GT_rm_ref),row.sup])<5)

    #data reference
    if(sum(GT_rm_ref)==0){
      data_ref = df[row.ref,]
    }else{
      data_ref = df[row.ref,-c(GT_rm_ref)]
    }

    #DOF Degrees of freedom
    DOF = min(ncol(data_ref)-1,nrow(data_ref)-1)

    #Correspondance analysis
    ca.NMI <- ade4::dudi.coa(data_ref,scannf=FALSE,nf=DOF)

    #df with coord ens (li) de reference + date monnaie
    DATA_REF = cbind(ca.NMI$li, date=date[row.ref])
    max = min(length(DATA_REF[!is.na(DATA_REF$date),1])-2,DOF)


    #specify the cross-validation method
    formula <- "date ~"
    #ctrl <- caret::trainControl(method = "LOOCV")
    MSE <- c()
    PRESS <- c()
    R_sq <- c()

    for(i in 1:max){
      if(i == 1){
        formula <- paste0(formula,' Axis',i)
      }else{
        formula <- paste0(formula,' + Axis',i)
      }

      lm = lm(as.formula(formula),data = DATA_REF)
      MSE <- c(MSE,mean(lm$residuals^2))
      PRESS <- c(PRESS,press(lm))

    }
    PRESS_max=max(PRESS[is.finite(PRESS)])

    if(input$quid == "both"){
      plot(x=1:max,y=PRESS,log="y",yaxt='n',type="o",pch=1,ylab="PRediction Error Sum Of Squares (PRESS)",
           xlab="Number of component in lm()",ylim = c(min(MSE),PRESS_max))
      points(x = which(PRESS == min(PRESS)), y=min(PRESS),pch=16,cex=1.1)
      axis(2, at=log10Tck('y','major'), tck= 0.02, labels=format(log10Tck('y','major'), scientific = T))
      axis(2, at=log10Tck('y','minor'), tck= 0.01, labels=NA)
      abline(v=which(PRESS == min(PRESS)),lty=3,col="black")
      points(x=1:max,y=MSE,type="o",pch=1,ylab="",xlab="",col="#cc525b")
      mtext("Mean Squared Error (MSE)",side=4,col="#cc525b",line=.5)
    }
    if(input$quid == "PRESS"){
      plot(x=1:max,y=PRESS,log="y",yaxt='n',type="o",pch=1,ylab="PRediction Error Sum Of Squares (PRESS)",
           xlab="Number of component in lm()")
      points(x = which(PRESS == min(PRESS)), y=min(PRESS),pch=16,cex=1.1)
      axis(2, at=log10Tck('y','major'), tck= 0.02, labels=format(log10Tck('y','major'), scientific = T))
      axis(2, at=log10Tck('y','minor'), tck= 0.01, labels=NA)
      abline(v=which(PRESS == min(PRESS)),lty=3,col="black")
    }
    if(input$quid == "MSE"){
      plot(x=1:max,y=MSE,type="o",pch=1,ylab="Mean Squared Error (MSE)",col="#cc525b",
           xlab="Number of component in lm()",ylim = c(0,max(MSE)))
    }



    shinyjs::enable(selector='a[data-value="Results"]')
    shinyjs::enable(selector='a[data-value="Visualization"]')

  })

  output$R_sq <- renderPlot({
    df = values[["df"]]
    date = values[["date"]]
    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))

    #vector for ref data (every not in row.sup)

    #nettoyage des GT ie GT<5
    GT_rm_ref = which(colSums(df[row.ref,])<5)
    #?????????????????
    #GT_rm_sup = which(colSums(df[-c(GT_rm_ref),row.sup])<5)

    #data reference
    if(sum(GT_rm_ref)==0){
      data_ref = df[row.ref,]
    }else{
      data_ref = df[row.ref,-c(GT_rm_ref)]
    }

    #DOF Degrees of freedom
    DOF = min(ncol(data_ref)-1,nrow(data_ref)-1)

    #Correspondance analysis
    ca.NMI <- ade4::dudi.coa(data_ref,scannf=FALSE,nf=DOF)

    #df with coord ens (col) de reference + date monnaie
    DATA_REF = cbind(ca.NMI$li, date=date[row.ref])
    max = min(length(DATA_REF[!is.na(DATA_REF$date),1])-2,DOF)


    #specify the cross-validation method
    formula <- "date ~"
    #ctrl <- caret::trainControl(method = "LOOCV")
    MSE <- c()
    MSE_sd <- c()
    PRESS <- c()
    R_sq <- c()

    for(i in 1:max){
      if(i == 1)
        formula <- paste0(formula,' Axis',i)
      else
        formula <- paste0(formula,' + Axis',i)

      #model <- caret::train(as.formula(formula), data = DATA_REF, method = "lm", trControl = ctrl, na.action=na.omit)
      #RMSE <- c(RMSE,model$results$RMSE)
      #R_sq <- c(R_sq,model$results$Rsquared)

      lm = lm(as.formula(formula),data = DATA_REF)
      R_sq <- c(R_sq,summary(lm)$adj.r.squared)

    }
    min_R = min(R_sq)
    if(min(R_sq) > 0)min_R= 0
    if(min(R_sq) > 0.25)min_R= 0.25
    if(min(R_sq) > 0.5)min_R= 0.5
    if(min(R_sq) > 0.75)min_R= 0.75

    plot(x=1:max,y=R_sq,type="o",pch=1,main="Adjusted R²",ylab="Adj.R²",
           xlab="Number of component in lm()",ylim = c(min_R,1))

  })


  #output$checkbox <- renderText({
  #  row.ref = which(!(1:length(values[["df"]][1,]) %in% values[["row.sup"]]))
  #  date = values[["date"]]
  #
  #  updateCheckboxGroupInput(session, "row.refbox",
  #                           choiceNames = as.list(colnames(values[["df"]])),
  #                           choiceValues = as.list(1:ncol(values[["df"]])),
  #                           selected = row.ref
  #  )
  #  ""
  #})

  output$cerar <- renderText({
    date = values[["date"]]
    df = values[["df"]]
    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))

    #todo: while convergence

    #check rowsum ens
    if(sum(rowSums(df)<5)!=0){
      warning(paste0("The sums of rows ",capture.output(cat(row.names(df)[which(rowSums(df)<5)]))," are less than 5. They were suppressed from the analysis."))
      #retire index de la col dans row.sup
      tmp.row.status = data.frame(
        index = c(row.ref,row.sup),
        status = c(rep("ref",length(row.ref)),rep("sup",length(row.sup)))
      )

      tmp.row.status = tmp.row.status[order(tmp.row.status$index),]
      #new row.sup
      tmp.row.status = tmp.row.status[!rowSums(df)<5,]
      #retire la col de date
      date = date[!rowSums(df)<5]
      #retire la col de df
      df = df[!rowSums(df)<5,]

      row.ref = which(tmp.row.status$status == "ref")
      row.sup = which(tmp.row.status$status == "sup")

      values[["df"]] = df
      values[["date"]] = date
      values[["row.sup"]] = row.sup
    }

    values[["cerar_res"]] = cerardat(df, row.sup, date, nf = input$k, graph=F, confidence = input$confidence)
    ""
  })

  output$lm <- renderPrint({
    summary(values[["cerar_res"]]$lm)
  })

  output$SW <- renderPrint({
    print(values[["cerar_res"]]$Shapiro_Wilks)
  })

  output$DW <- renderPrint({
    print(values[["cerar_res"]]$Durbin_Watson)
  })

  output$BP <- renderPrint({
    print(values[["cerar_res"]]$Breusch_Pagan)
  })

  output$sw <- renderText({
    if(values[["cerar_res"]]$Shapiro_Wilks$p.value < 0.05){
      "Warning: The Shapiro-Wilks test indicates a problem with the normality of the residuals."
    }
  })

  output$dw <- renderText({
    if(values[["cerar_res"]]$Durbin_Watson$p.value < 0.05){
      "Warning: The Durbin-Watson test indicates a first order autocorrelation problem."
    }
  })

  output$bp <- renderText({
    if(values[["cerar_res"]]$Breusch_Pagan$p.value < 0.05){
      "Warning: The Breusch-Pagan test indicates a heteroskedasticity problem."
    }
  })

  output$lmplot <- renderUI({
    if(is.null(values[["cerar_res"]])){
      cerar_res = cerardat(df, row.sup, date, nf = input$k, graph=F, confidence = input$confidence)
    }else{
      cerar_res = values[["cerar_res"]]
    }

    plot_output_list <- lapply(c(1:4), function(i) {
      name <- paste("lm_plot", i, sep="")
      output[[name]] <- renderPlot({
        plot(cerar_res$lm,which = i,ask=FALSE)
      },width = 450,height = 450)
    })
    shinycssloaders::withSpinner(do.call(tagList,plot_output_list),color='#18bc9c')

  })


  output$evaldateref <- renderPlotly({
    date = values[["date"]]
    df = values[["df"]]

    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))

    if(is.null(values[["cerar_res"]])){
      cerar_res = cerardat(df, row.sup, date, nf = input$k, graph=F, confidence = input$confidence)
    }else{
      cerar_res = values[["cerar_res"]]
    }


    tmp = cerar_res$prediction[row.ref,]
    tmp = tmp[!is.na(tmp$date),]


    df = data.frame(
      names = rownames(tmp),
      date = tmp$date,
      lwr = tmp$lower_Ev,
      upr = tmp$upper_Ev,
      fit = tmp$Fit_dateEv
    )

    #generation du graph
    graph = suppressWarnings(ggplot(df) +
      geom_point(aes(x = names, y = date, text=paste("lower:",lwr,"<br>upper:",upr)),colour="red",shape=3, size=2) +
      geom_errorbar( aes(x=names, ymin=lwr, ymax=upr), width=.15, colour="#3f0b18", alpha=0.9, linewidth=1)+
      ylab("Date (year)") + xlab("Site") + ggtitle("") +
      theme_classic() +
      scale_y_continuous(breaks = scales::pretty_breaks(n = 10)) +
      theme(
        panel.grid.major.x = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                          linewidth = .1,
                                          linetype = 2),
        panel.grid.major.y = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                          linewidth = .1,
                                          linetype = 2),
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)
      ))
    graph

  })

  output$evaldatesup <- renderPlotly({
    date = values[["date"]]
    df = values[["df"]]
    row.sup = values[["row.sup"]]

    if(is.null(values[["cerar_res"]])){
      cerar_res = cerardat(df, row.sup, date, nf = input$k, graph=F, confidence = input$confidence)
    }else{
      cerar_res = values[["cerar_res"]]
    }


    tmp = cerar_res$prediction[row.sup,]
    tmp = tmp[!is.na(tmp$date),]

    if(dim(tmp)[1] == 0){
      graph = ggplot() +
        geom_blank() +
        theme_classic()
      graph
    }else{
      df = data.frame(
        names = rownames(tmp),
        date = tmp$date,
        lwr = tmp$lower_Ev,
        upr = tmp$upper_Ev,
        fit = tmp$Fit_dateEv
      )

      #generation du graph
      graph = suppressWarnings(ggplot(df) +
                                 geom_point(aes(x = names, y = date, text=paste("lower:",lwr,"<br>upper:",upr)),colour="red",shape=3, size=2) +
                                 geom_errorbar( aes(x=names, ymin=lwr, ymax=upr), width=.15, colour="#3f0b18", alpha=0.9, linewidth=1)+
                                 ylab("Date (year)") + xlab("Site") + ggtitle("") +
                                 theme_classic() +
                                 scale_y_continuous(breaks = scales::pretty_breaks(n = 10)) +
                                 theme(
                                   panel.grid.major.x = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                                                     linewidth = .1,
                                                                     linetype = 2),
                                   panel.grid.major.y = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                                                     linewidth = .1,
                                                                     linetype = 2),
                                   axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)
                                 ))
      graph
    }



  })

  observeEvent(input$axe1,{
    values[["axe1"]] <- input$axe1
  })

  observeEvent(input$axe2,{
    values[["axe2"]] <- input$axe2
  })

  observeEvent(input$confidence,{
    df = values[["df"]]
    date = values[["date"]]
    row.sup = values[["row.sup"]]

    values[["cerar_res"]] = cerardat(df, row.sup, date, nf = input$k, graph=F, confidence = input$confidence)
  })



  output$CA <- renderScatterD3({
    df = values[["df"]]
    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))

    #nettoyage des GT ie GT<5
    GT_rm_ref = which(colSums(df[row.ref,])<5)

    #?????????????????
    #GT_rm_sup = which(colSums(df[-c(GT_rm_ref),row.sup])<5)

    #data reference
    if(sum(GT_rm_ref)==0){
      data_ref = df[row.ref,]
    }else{
      data_ref = df[row.ref,-c(GT_rm_ref)]
    }
    DOF = min(ncol(data_ref)-1,nrow(data_ref)-1)

    updateNumericInput(session, "axe1", max = DOF)
    updateNumericInput(session, "axe2", max = DOF)

    res <- explor::prepare_results(dudi.coa(data_ref,nf=DOF,scannf = FALSE))

    xax = values[["axe1"]]
    yax = values[["axe2"]]
    var_sup = TRUE
    var_hide = "None"
    var_lab_min_contrib = 0
    point_size = 64
    col_var = "Position"
    symbol_var = NULL
    size_var = NULL
    size_range = c(10,300)
    zoom_callback = NULL
    in_explor = FALSE

    ## Settings changed if not run in explor
    html_id <- if(in_explor) "explor_var" else  NULL
    dom_id_svg_export <- if(in_explor) "explor-var-svg-export" else NULL
    dom_id_lasso_toggle <- if(in_explor) "explor-var-lasso-toggle" else NULL
    lasso <- if(in_explor) TRUE else FALSE
    lasso_callback <- if(in_explor) explor_multi_lasso_callback() else NULL
    zoom_callback <- if(in_explor) explor_multi_zoom_callback(type = "var") else NULL

    var_data <- CA_var_data(res, xax, yax)

    scatterD3::scatterD3(
      x = var_data[, "Coord.x"],
      y = var_data[, "Coord.y"],
      xlab = names(res$axes)[res$axes == xax],
      ylab = names(res$axes)[res$axes == yax],
      lab = var_data[, "Lab"],
      point_size = point_size,
      point_opacity = 1,
      col_var = if (is.null(col_var)) NULL else var_data[,col_var],
      col_lab = col_var,
      symbol_var = if (is.null(symbol_var)) NULL else var_data[,symbol_var],
      symbol_lab = symbol_var,
      size_var = if (is.null(size_var)) NULL else var_data[,size_var],
      size_lab = size_var,
      size_range = if (is.null(size_var)) c(10,300) else c(30,400) * point_size / 32,
      tooltip_text = var_data[, "tooltip"],
      type_var = ifelse(var_data[, "Class"] == "Quantitative", "arrow", "point"),
      unit_circle = var_sup && "Quantitative" %in% var_data[,"Class"],
      key_var = paste(var_data[,"Position"], var_data[,"Level"], sep = "-"),
      fixed = TRUE,
      html_id = html_id,
      dom_id_svg_export = dom_id_svg_export,
      dom_id_lasso_toggle = dom_id_lasso_toggle,
      lasso = lasso,
      lasso_callback = lasso_callback,
      zoom_callback = zoom_callback
    )
  })


  output$cerardat_plot_ref <- renderUI({
    df = values[["df"]]

    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))


    progress <- Progress$new(session, min=1, max=length(row.ref))
    on.exit(progress$close())

    progress$set(message = 'Calculation in progress',
                 detail = 'This may take a while...')

    if(is.null(values[["cerar_res"]])){
      cerar_res = cerardat(df, row.sup, date, nf = input$k, graph=F, confidence = input$confidence)
    }else{
      cerar_res = values[["cerar_res"]]
    }

    GT_date_sd = arrondi(cbind(
      Pred=data.frame(cerar_res$predict_obj_col$fit)[,1],
      data.frame(E_T=cerar_res$predict_obj_col$se.fit)
    ),1)

    ecart = max(cerar_res$date_gt)-min(cerar_res$date_gt)
    xlim = c(min(cerar_res$date_gt-ecart*0.07),max(cerar_res$date_gt+ecart*0.07))

    tmp_ = c()
    for(i in 1:ncol(cerar_res$cont_gt) )
    {
      date_accumulation <- nor1mix::norMix(mu = GT_date_sd[,1], w = unlist(as.vector(cerar_res$cont_gt[i,])), sigma= GT_date_sd[,2])
      date_accumulation_density = nor1mix::dnorMixL(date_accumulation, xlim=xlim)

      tmp_ = c(tmp_,max(date_accumulation_density$y))
      progress$set(value = i)
    }
    ylim=c(0,max(tmp_))

    plot_output_list <- lapply(row.ref, function(i) {
      name <- paste("cera_plot", i, sep="")
      output[[name]] <- renderPlot({
        plot(cerar_res,which=i,xlim=xlim,ylim=ylim)
      },width = 450,height = 450)
    })
    shinycssloaders::withSpinner(do.call(tagList,plot_output_list),color='#18bc9c')

  })

  output$cerardat_plot_sup <- renderUI({
    df = values[["df"]]

    row.sup = values[["row.sup"]]
    row.ref = which(!(1:length(df[,1]) %in% row.sup))

    progress <- Progress$new(session, min=1, max=length(row.sup))
    on.exit(progress$close())

    progress$set(message = 'Calculation in progress',
                 detail = 'This may take a while...')

    if(is.null(values[["cerar_res"]])){
      cerar_res = cerardat(df, row.sup, date, nf = input$k, graph=F, confidence = input$confidence)
    }else{
      cerar_res = values[["cerar_res"]]
    }

    GT_date_sd = arrondi(cbind(
      Pred=data.frame(cerar_res$predict_obj_col$fit)[,1],
      data.frame(E_T=cerar_res$predict_obj_col$se.fit)
    ),1)

    ecart = max(cerar_res$date_gt)-min(cerar_res$date_gt)
    xlim = c(min(cerar_res$date_gt-ecart*0.07),max(cerar_res$date_gt+ecart*0.07))

    tmp_ = c()
    for(i in 1:ncol(cerar_res$cont_gt) )
    {
      date_accumulation <- nor1mix::norMix(mu = GT_date_sd[,1], w = unlist(as.vector(cerar_res$cont_gt[i,])), sigma= GT_date_sd[,2])
      date_accumulation_density = nor1mix::dnorMixL(date_accumulation, xlim=xlim)

      tmp_ = c(tmp_,max(date_accumulation_density$y))
      progress$set(value = i)
    }
    ylim=c(0,max(tmp_))

    plot_output_list <- lapply(row.sup, function(i) {
      name <- paste("cera_plot", i, sep="")
      output[[name]] <- renderPlot({
        plot(cerar_res,which=i,xlim=xlim,ylim=ylim)
      },width = 450,height = 450)
    })
    shinycssloaders::withSpinner(do.call(tagList,plot_output_list),color='#18bc9c')

  })


  #EXPORT BUTTON
  output$downloadData <- downloadHandler(
    filename = "output.pdf",
    content = function(file) {
      cairo_pdf(file,onefile=T)
      for (i in 1:ncol(values[["df"]])) {
        plot(values[["cerar_res"]],which=i)
      }
      dev.off()
    }
  )

  output$tablesite <- renderTable({
    if(is.null(values[["cerar_res"]])){
      df=data.frame()
    }else{
      df=values[["cerar_res"]]$prediction
    }
  },digits = 0, rownames = T, bordered = T, striped = T, hover = T)

  output$tableGT <- renderTable({
    if(is.null(values[["cerar_res"]])){
      df=data.frame()
    }else{
      df=values[["cerar_res"]]$date_gt
    }
  },digits = 0, rownames = T, bordered = T, striped = T, hover = T)


  output$formula_dateEv <- renderUI({
    withMathJax(paste0("Linear model: $$ \\text { dateEv }_i=\\beta_0 + \\sum_{k=1}^K \\beta_k\\left(F^k\\right)_i+\\varepsilon_i \\quad \\forall i=1, \\ldots, I $$ for K the number of component kept in the model and I the number of observations with a date."))
  })

  observeEvent(input$data,{
    shinyjs::disable("import2")
  })

  observeEvent(input$import2,{
    shinyjs::disable("Finishimport")
  })

  observeEvent(input$fileD1,{
    shinyjs::enable("import2")
  })

  observeEvent(input$fileD2,{
    shinyjs::enable("Finishimport")
  })




}

library(SPARTAAS)
suppressMessages(library(shinyjs))
library(shinyjqui)
suppressMessages(library(shinydashboard))
library(colorspace)
suppressMessages(library(dplyr))
library(tidyr)
library(ggplot2)
library(shinyWidgets)
library(shinythemes)

ui <- fluidPage(useShinyjs(),style="padding-top: 150px;",theme = shinytheme("flatly"),
                tags$style(HTML('#sort,#data,#subdivise,#tri,#sub1,#sub2,
         #import2,#import3,#import3,#Finishimport,#back1,#back2{color:white;background-color:#337ab7}
         #back{color:white;background-color:#f0ad4e}
         #tabsetperso{background-color:#e5e5e5;}
         #help{text-align;right;}
         #tabsetperso{margin-bottom:20px;}
         .dropdown-menu > li > a {
                    background-color:#fff;
                    border-color:#dce4ec;
         }
                  .btn-default {
                  border-color:#337ab7;
                  }
    ')),

                tags$head(
                  tags$link(rel = "icon", type = "image/gif", href = "https://spartaas.gitpages.huma-num.fr/r-package/img/lambda.png"),
                  tags$title("seriograph")
                ),
                #header panel
                absolutePanel(class = "panel panel-default",
                              style="z-index: 2000;padding: 8px; background: #ecf0f1; opacity: 1;border-bottom: 1px solid #2c3e50;",
                              top = 0, left = 0, right = 0,
                              fixed = TRUE,
                              h2("seriograph: a graphic seriation tool for count tables"),
                              div(span(strong("SPARTAAS | seriograph")))
                ),
                #footer
                absolutePanel(style="z-index: 2000;padding: 0px; border-bottom: 0px solid #CCC; background: #fff;opacity: 1;",
                              bottom = 0, left = 0,
                              fixed = TRUE,
                              div(span(strong("SPARTAAS")),span("[Bellanger,Coulon,Husi]",style ="font-size:14px;"))
                ),
                #go top panel
                absolutePanel(class = "panel-hover",
                              style="opacity:0.8;z-index: 1000;padding: 8px; border-bottom: 1px solid #CCC;
    background: #FFFFEE;width:30px;margin-right:10px;",
                              bottom = 20, right = 0,
                              fixed = TRUE,
                              HTML("<a href=\"#top\"><span class=\"glyphicon glyphicon-chevron-up\"></span></a>")
                ),
                #-----------------------------------#
                #    NavBar
                #-----------------------------------#
                navbarPage("seriograph",id="navbar",
                           tabPanel(value = "1",span(icon("home", lib = "glyphicon"),strong("Home")),style="max-width:1200px;",
                                    br(),
                                    sidebarLayout(
                                      #-----------------------------------#
                                        sidebarPanel(style="",
                                                     h1("R Package:"),
                                                     br(),
                                                     HTML("<p>This method is part of the <a href='https://spartaas.gitpages.huma-num.fr/r-package/index.html' target='_blank'>SPARTAAS</a> package.</p>
                                                        <p>If you are interested you can install our R package available on the <a href='https://cran.r-project.org/package=SPARTAAS' target='_blank'>CRAN</a>.</p>
                                                        <p>There is also a macro version in LibreOffice Calc. You can find it <a href='https://abp.hypotheses.org/le-programme-bassin-parisien/les-projets/les-projets-associes-au-programme/outils-danalyse-graphique-des-donnees' target='_blank'>here</a>.</p>
                                                        <hr style=\"border-color: #222222;\">
                                                        <p><b>Desachy B.</b> (2004) Le sériographe EPPM : un outil informatisé de sériation graphique pour tableaux de comptages. In: Revue archéologique de Picardie, n°3-4, 2004. Céramiques domestiques et terres cuites architecturales. Actes des journées d'étude d'Amiens (2001-2002-2003) pp. 39-56 <a href=\"https://doi.org/10.3406/pica.2004.2396\" target=\"_blank\">⟨doi:10.3406/pica.2004.2396⟩</a>.</p>")

                                        ),
                                      #-----------------------------------#
                                      mainPanel(
                                        HTML("
                                              <h3>Introduction</h3>
                                              <p>In order to facilitate the exploitation of the data tables, we propose here a computerised graphic processing tool (EPPM serigraph - for Ecart Positif aux Pourcentages Moyens - positive deviation from the average percentage), which does not require specialised statistical skills and is adapted to the case of stratified sites, where the study of the evolution of artefacts can be based on the relative chronology provided by the excavation.</p>
                                              <p>The treatment consists firstly of transforming this table of counts into a table of percentages, the total number in each set (each row) being reduced to 100; these are the proportions, or frequencies, of the types in the sets are thus compared.</p>
                                              <p>The display of positive deviations from the mean percentages (EPPM) shows in black on a lighter background the percentage shares that are higher than the mean percentage of the variable, so as to highlight the most significant part of the values in the table.This display is simply adapted to the seriograph: when a percentage is greater than the average percentage of the type, the excess share (called here EPPM: positive deviation from the average percentage) is shown in black, centred around the axis of the type, on the grey background of the percentage bar.</p>
                                              <p>The table is then transformed into a graphic matrix where these percentages are expressed, for each type, by horizontal bars centred on the column axis. When the rows are ordered chronologically, the silhouette formed by the superposition of these frequency bars bars makes it possible to visualise the evolution over time of the type concerned.</p>
                                              <p>The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).</p>
                                              <p>The processing technique applies to sets whose chronological order is not known; the lines of the graph are to be reorganised so as to obtain boat-shaped silhouettes following the hypothesis of a chronological evolution corresponding to the seriation model.</p>

                                              ")
                                      )
                                    )
                           ),
                           tabPanel(value = "2",span(icon("chart-area", lib = "font-awesome"),strong("seriograph")),style="max-width:1200px;",
                                    sidebarPanel(style = 'max-width:300px;',
                                                 h4("Import your data:"),
                                                 column(10,
                                                        actionButton("data",
                                                                                      icon("import", lib = "glyphicon"))
                                                 ),
                                                 column(1,
                                                        # Help
                                                        dropdownButton(
                                                          h3("data format"),
                                                          hr(style="border-color: #222222;"),
                                                          HTML("<p>The data is in the form of a count table with the contexts in rows and the categories (GT) in columns.</p>
                                                       <p>Example:</p>
                                                       <img style='width:280px;' src='GS/exemple.png' alt='data'>"),
                                                          circle = TRUE, status = "danger", icon = icon("question"), width = "300px",
                                                          size = "sm",right=FALSE,
                                                          tooltip = tooltipOptions(title = "Help", placement = "top")
                                                        )
                                                        # Help
                                                 ),
                                                 div(style="height:200px;"),
                                                 prettyToggle(
                                                   inputId = "permute",
                                                   label_on = "Seriation",
                                                   label_off = "Seriation",
                                                   icon_on = icon("check"),
                                                   icon_off = icon("times"),
                                                   value = TRUE
                                                 ),
                                                 selectInput("show", label = "Show", choices = list("both" = 1,
                                                                                                    "frequencies" = 2,
                                                                                                    "EPPM" = 3),
                                                             selected = 1),
                                                 actionButton("sort",span(icon("sort",lib="font-awesome"),"Sort rows"))
                                    ),
                                    mainPanel(style = 'max-width:1000px;',
                                      ####################################
                                      ##        Seriograph PLOT         ##
                                      ####################################
                                      br(),br(),
                                      h3("Seriograph"),
                                      hr(style="border-color: #222222;"),
                                      column(11,
                                             dropdownButton(
                                               downloadButton("seriograph.pdf","Save as pdf"),
                                               hr(style="border-color: #222222;"),
                                               downloadButton("seriograph.png","save as png"),
                                               numericInput("width", "width", 900, min = 0, max = NA,
                                                            step = 1, width = NULL),
                                               numericInput("height", "height", 500, min = 0, max = NA,
                                                            step = 1, width = NULL),
                                               circle = TRUE, status = "primary", icon = icon("floppy-disk",
                                                                                              lib = "glyphicon"), width = "150px",
                                               size = "sm",
                                               tooltip = tooltipOptions(title = "Download")
                                             )
                                      ),
                                      column(1,
                                             # Help
                                             dropdownButton(
                                               h3("The seriograph (B. DESACHY)"),
                                               hr(style="border-color: #222222;"),
                                               p("This tool makes it possible to highlight artisanal
                                               evolutions over time as well as to understand commercial
                                               relations thanks to imported ceramics. The percentages of
                                               each ceramic category are displayed. The percentages are
                                               calculated independently for each class. The percentage display
                                               allows you to compare the different classes but does not provide
                                               information on the differences in terms of enrolment. To fill
                                               this gap,
                                               the proportion of each class of their workforce is displayed on
                                               the seriograph (weight column). We have generalized this
                                               representation
                                               for other contingency data with hclustcompro object.
                                               The seriograph can be used in a strictly deductive way, by blocking the order of the rows (archaeological contexts) when their succession is known, and by reordering only the columns, in order to examine the chronological behaviour of the variables, and thus to see in particular what does not obey a serial evolution: cyclical phenomena, or the problems of intrusion and residuality."),
                                               circle = TRUE, status = "danger", icon = icon("question"),
                                               width = "500px",
                                               size = "sm",right=TRUE,
                                               tooltip = tooltipOptions(title = "Help", placement = "top")
                                             )
                                             # Help
                                      ),br(),br(),

                                      jqui_resizable(plotOutput("EPPM")),
                                      div(style="height:20px;"),
                                    )

                           ),
                           tabPanel(value = "3",span(icon("envelope", lib = "glyphicon"),strong("Contact")),style="max-width:1200px;",
                                    HTML("<style>
        .carte{
        border-left-width: 4px;
        border-left-style: solid;
        border-color: #2ac0a2;
        padding-left: 10px;
        }
        </style>
        <h3>Authors:</h3>
        <hr>
        <div class=\"carte\"><h4>L. Bellanger</h4><h5>mail: &lt;lise.bellanger@univ-nantes.fr&gt;</h5></div>
        <div class=\"carte\"><h4>P. Husi</h4><h5>mail: &lt;philippe.husi@univ-tours.fr&gt;</h5></div>
        <div class=\"carte\"><h4>A. Coulon</h4></div>
        <h3>Maintainer:</h3>
        <hr>
        <div class=\"carte\"><h4>A. Coulon</h4><h5>mail: &lt;arthur-coulon@outlook.fr&gt;</h5></div>
        ")
                           ),
                           #-----------------------------------#
                           #    Wiki
                           #-----------------------------------#
                           tabPanel(value = "4",span(icon("question-sign", lib = "glyphicon"),strong("Wiki")),style="max-width:1200px;",
                                    HTML("<style>body {text-align: justify}</style><h1>Get started with the hclustcompro application</h1>
                                      <h2>Table of content</h2>
                                      <div id=\"TOC\">
                                      <ul>

                                      <li><a href=\"#output\">The outputs</a>
                                        <ul>
                                          <li><a href=\"#seriograph\">Seriograph</a></li>
                                        </ul>
                                      </li>

                                      <li><a href=\"#data1\">The inputs</a>
                                        <ul>
                                          <li><a href=\"#seriation\">Seriation</a></li>
                                          <li><a href=\"#weight\">Weight color indicator</a></li>
                                          <li><a href=\"#show1\">Vizualisation</a></li>
                                          <li><a href=\"#sort1\">Sort periods</a></li>
                                          <li><a href=\"#import\">Import your data</a></li>
                                          <li><a href=\"#csv\">CSV Format and write.table</a></li>
                                        </ul>
                                      </li>


                                      <li><a href=\"#references\">References</a></li>


                                      </ul>
                                      </div>

<hr>
<h2 id=\"output\">The outputs</h2>
<hr>
<h3 id=\"seriograph\">Seriograph</h3>
<p>We have chosen the serigraph (Desachy 2004). This tool makes it possible to highlight the evolution of ceramics over time as well as to understand the commercial relations thanks to the imported ceramics. The percentages of each category of ceramics per set are displayed. The percentages are calculated independently for each set (row). The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).</p>
<p>The seriograph can be used in a strictly deductive way, by blocking the order of the rows (archaeological contexts) when their succession is known, and by reordering only the columns, in order to examine the chronological behaviour of the variables, and thus to see in particular what does not obey a serial evolution: cyclical phenomena, or the problems of intrusion and residuality.</p>

<p><img src=\"GS/seriograph.png\"></p>

<p>In order to facilitate the exploitation of the data tables, we propose here a computerised graphic processing tool (EPPM serigraph - for Ecart Positif aux Pourcentages Moyens - positive deviation from the average percentage), which does not require specialised statistical skills and is adapted to the case of stratified sites, where the study of the evolution of artefacts can be based on the relative chronology provided by the excavation.</p>
<p>The treatment consists firstly of transforming this table of counts into a table of percentages, the total number in each set (each row) being reduced to 100; these are the proportions, or frequencies, of the types in the sets are thus compared.</p>
<p>The display of positive deviations from the mean percentages (EPPM) shows in black on a lighter background the percentage shares that are higher than the mean percentage of the variable, so as to highlight the most significant part of the values in the table.This display is simply adapted to the seriograph: when a percentage is greater than the average percentage of the type, the excess share (called here EPPM: positive deviation from the average percentage) is shown in black, centred around the axis of the type, on the grey background of the percentage bar.</p>
<p>The table is then transformed into a graphic matrix where these percentages are expressed, for each type, by horizontal bars centred on the column axis. When the rows are ordered chronologically, the silhouette formed by the superposition of these frequency bars bars makes it possible to visualise the evolution over time of the type concerned.</p>
<p>The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).</p>
<p>The processing technique applies to sets whose chronological order is not known; the lines of the graph are to be reorganised so as to obtain boat-shaped silhouettes following the hypothesis of a chronological evolution corresponding to the seriation model.</p>



<hr>
<h2 id=\"data1\">The inputs</h2>
<hr>
<h3 id=\"seriation\">Seriation</h3>
<p>This input allows you to enable or disable the seriation of the columns. The matrix permutation uses an algorithm called 'reciprocal averages'. Each row is assigned a rank from 1 to n, the number of rows. For each column, a barycentre is calculated by weighting according to the row rank. Finally, the columns are reorganised by sorting them according to their barycentre.</p>
<p><img src=\"GS/seriation.png\"></p>


<h3 id=\"show1\">Visualization</h3>
<p>This input allows you to select the element to be plotted. There are tree options: plot the positive deviation from the average percentage (EPPM in French), plot the frequency or plot both. The average percentage is calculated for each category (columns) on the total number of accounts (all classes combined). From the average percentage, we obtain for each category and for each row the difference between the percentage of the category in the class and the average percentage. The EPPM corresponds to the notion of independence deviation (between rows and columns, between categories and time classes) in a chi-square test approach. Although this approach is fundamental in statistical analysis, the independence deviations here are purely indicative and are not associated with a p_value that could determine the significance of the deviations.</p>
<p><img src=\"GS/show.png\"></p>

<h3 id=\"sort1\">Sort periods</h3>
<p>The rows are initially in the order of the data table. It is possible to reorder the rows in a temporal way. In the interface you can drag and drop the row (green) to change the order.</p>
<p><img src=\"GS/sort.png\"></p>

<h3 id=\"import\">Import your data</h3>
<p>You can import your data. You will need to upload a csv for the contingency table.</p>
<p>The data is in the form of a count table with the contexts in rows and the categories (GT) in columns.</p>
<img src='GS/exemple.png'>
<p><img src=\"GS/import1.png\"></p>

<p>The settings allow you to import different data frame organisation (header, column separator, ...).</p>
<h4 id=\"header\">Header</h4>
<p>Yes or no option. Do you have headers on your columns?</p>
<h4 id=\"rownames\">Rownames</h4>
<p>Yes or no option. Do you have row names on your rows?</p>
<h4 id=\"separator\">Separator</h4>
<p>Select the character you want to use to separate the columns.</p>
<h4 id=\"quote\">Quote</h4>
<p>Select the quotation marks to use on strings.</p>
<h4 id=\"dec\">Decimal</h4>
<p>Select the character to use to indicate the decimal point.</p>

<h4 id=\"csv\">CSV Format and write.table</h4>
<p>It is a data.frame with colunms separated by semicolons \";\".</p>
<p>The input format for importing data is the .csv format, but also supports the .txt format as a .csv file.</p>
<p>In R, you can export your data frame to a csv file using write.csv2 or write.table. In a csv you can choose a character to separate the columns. In the same way, you can define the character to indicate the decimal point.</p>
<code>
write.table(data,file=\"path/to/name_file.csv\",sep=\";\",dec=\".\",row.names=FALSE,quote=FALSE)
</code>
<p>In Excel you can save in csv format in order to import your data frame.</p>
<p>The import interface allows you to set these values using the 'header', 'decimal', 'separator' and 'quote' options.</p>




<hr>
<h2 id=\"references\">References</h2>
<hr>
<p><br>
  <br><br>Desachy B. (2004). Le sériographe EPPM : un outil informatisé de sériation graphique pour tableaux de comptages. In: Revue archéologique de Picardie, n°3-4, 2004. Céramiques domestiques et terres cuites architecturales. Actes des journées d'étude d'Amiens (2001-2002-2003) pp. 39-56 <a href=\"https://doi.org/10.3406/pica.2004.2396\" target=\"_blank\">⟨doi:10.3406/pica.2004.2396⟩</a>.
</p>
<br><br><br><br>


")
                           )
                )
)

generateClone <- function(D1,D2,i,j){
  D1bis <- rbind(D1,clone=D1[i,])
  D1bis <- cbind(D1bis,clone=c(D1[i,],0))

  D2bis <- rbind(D2,clone=D2[j,])
  D2bis <- cbind(D2bis,clone=c(D2[j,],0))

  return(list(D1bis,D2bis))
}

corCriterion <- function(tree,D1,D2) {
  d2 <- cophenetic(tree)
  res <- abs(cor(as.dist(D1),d2) - cor(as.dist(D2),d2))
  #res2 <- list()
  #res2$un <- cor(as.dist(D1),d2)
  #res2$de <- cor(as.dist(D2),d2)
  #return(res2)
  return(res)
}

search <- function(x){
  return(which(x == min(x)))
}

fusion <- function(c1, c2) {
  c3 <- c()
  for (i in 1:length(c2)) {
    c3[i] <- max(c1[i], c2[i])
  }
  names(c3) <- names(c2)
  return(c3)
}

sub_div <- function(tree,cluster,where,sk) {
  ok <- TRUE
  step <- length(cluster)-1
  while(ok){
    clusters <- cutree(tree,h = tree$height[step])
    if(length(unique(clusters[cluster == where])) == sk){
      ok <- FALSE
    }else{
      step <- step - 1
    }
  }
  #reorder value
  order <- unique(clusters[tree$order])
  for(i in 1:length(order)){
    if(i != order[i] && i < order[i]){
      clusters[clusters == i] <- 0
      clusters[clusters == order[i]] <- i
      clusters[clusters == 0] <- order[i]
      order <- unique(clusters[tree$order])
    }
  }
  #rename value
  resume <- table(clusters[cluster == where])
  c <- 1
  for(i in as.numeric(names(resume))){
    clusters[clusters == i] <- c
    c <- c+1
  }
  c <- c-1
  clusters <- clusters / 10 ^nchar(c)
  for(i in 1:length(cluster)){
    ifelse(cluster[i] == where,
           cluster[i] <- where + clusters[i],
           cluster[i] <- cluster[i])
  }
  return(list(cluster = cluster,step = step))
}

EPM <- function(x) {
  x <- as.matrix(x)

  x[is.na(x)] <- 0
  ni. <- rep(1,nrow(x))
  nj. <- margin.table(x, margin = 2)
  n. <- sum(nj.)

  TabInde <- (nj.%*%t(ni.)/n.)
  TabInde <- t(TabInde)
  x <- prop.table(x, margin = 1)
  TabEcart <- x - TabInde
  TabEcart <- as.data.frame(TabEcart)

  return(TabEcart)
}

plot_EPPM <- function(x,show,permute) {
  if(is.table(x)){
    x <- as.data.frame(x)
    x <- matrix(x[,3],
                ncol=sqrt(length(x[,2])),
                dimnames=list(unique(x[,1]),unique(x[,1])))
    x <- as.data.frame(x)
  }
  if(is.matrix(x)){x <- as.data.frame(x)}
  if(!is.data.frame(x)){stop("x is not a data frame.")}
  for (i in 1:length(x[1,])){
    if(!(sum(x[,i]) == arrondi(sum(x[,i])))){stop("The data frame must contains integers.")}
  }
  ens = 0
  .data <- c()
  Cluster <- c()
  labels <- factor(rownames(x),levels = rev(unique(rownames(x))))
  #permute type in x
  if(permute){

    if(show == "EPPM"){
      ecart <- as.data.frame(EPM(x))
      ecart[ecart < 0] <- 0
      i <- seq(nrow(ecart),1)
      barycenter <- colSums(ecart * i) / colSums(ecart)
      x <- x[,order(barycenter)]
    }else{
      i <- seq(nrow(x),1)
      barycenter <- colSums(x * i) / colSums(x)
      x <- x[,order(barycenter)]
    }

  }
  rowsum <- rowSums(x)
  #add Weight to x
  x <- cbind(x, Weight = rowSums(x) / sum(x) * rowSums(x))
  #data
  data <- as.data.frame(x)
  data <- mutate(data,ens = labels,rowsum = rowsum)
  data <- gather(data,key = "type", value = "frequency",-.data$ens,-.data$rowsum, factor_key = TRUE)
  data <- mutate(data,frequency = .data$frequency / .data$rowsum)
  #remove col rowsum
  data <- data[-2]
  data$frequency[is.na(data$frequency)] <- 0

  #EPPM
  ecart <- as.data.frame(EPM(x[-length(x[1,])]))
  save_ecart <- ecart
  ecart <- cbind(ecart,Weight=c(0))
  ecart[ecart < 0] <- 0
  ecart <- mutate(ecart,ens = labels)
  ecart <- gather(ecart,key = "type", value = "EPPM", -.data$ens, factor_key = TRUE)
  ecart$EPPM[ecart$type == "Weight"] <- 0

  # Join data and EPPM
  data <- inner_join(data,ecart,by = c("ens", "type"))
  data <- mutate(data,frequency = .data$frequency - .data$EPPM)
  data <- gather(data,key = "legend", value = "frequency", -.data$ens,
                 -.data$type, factor_key = TRUE)
  #if insert hiatus in df replace NA by 0 (NA are generating by divising by 0 (rowsum of a hiatus is 0))
  data[is.na(data)] <- 0
  #center (div /2 et copy for one part - and the other +)
  data <- rbind(data,data)
  data <- mutate(data,frequency = .data$frequency * c(rep(.5, nrow(data)/2), rep(-.5, nrow(data)/2)))

  #color for Weight
  tmp <- data$frequency[data$type == "Weight"]
  levels(data$legend) <- c(levels(data$legend),"1","2","3","4","5","6","7","8","9","10","Weight")
  data$legend[data$type == "Weight"] <- "Weight"

  breaks <- c("frequency","EPPM","Weight")

  #define the color
  if(show == "frequency"){
    color <- c("frequency"="#bebebe","EPPM"="#bebebe","Weight"="#7f7f7f")
    #remove EPPM from legend
    breaks <- c("frequency","Weight")
    default_col <- "#bebebe"
  }else{
    if(show == "EPPM"){
      color <- c("frequency"=NA,"EPPM"="black","Weight"="#7f7f7f")
      #remove frequency from legend
      breaks <- c("EPPM","Weight")
      default_col <- NA
    }else{
      #default color grey: frequency, black: EPPM, my_palette: weight
      color <- c("frequency"="#bebebe","EPPM"="black","Weight"="#7f7f7f")
      default_col <- NA
    }
  }
  contingency <- x[,1:(length(x[1,])-1)]
  contingency <- x[,-length(x[1,])]
  frequency <- contingency
  rowsum <- rowSums(contingency)
  for (k in 1:length(contingency[,1])){
    frequency[k,] <- frequency[k,] / rowsum[k]
  }

  #cat("\nContingency table:\n")
  #print(contingency)


  #ggplot
  p <- ggplot(data = data) +
    facet_grid( ~type, scales = "free", space = "free_x") +
    geom_col(aes(x = ens, y = frequency, fill = legend), width = 1) +
    scale_x_discrete(labels = rev(rownames(x))) +
    coord_flip() +
    scale_fill_manual(values = color,
                      aesthetics = "fill",
                      limits = breaks,
                      na.value = default_col
    ) +
    scale_y_continuous(breaks = c(-.1,.1),labels = c("","20% ")) +
    labs(
      title = "Seriograph",
      subtitle = "EPPM (Ecart Positif au Pourcentage Moyen) Positive Deviation from the Average Percentage",
      caption = "Warnings: The frequencies are calculated independently for each element (row).
You can see the relative number of observations in the Weight column"
    ) +
    annotate("segment",x = 0, y = -.04, xend = 0, yend = .04,linetype = "blank") +
    theme(legend.position = "bottom",
          panel.spacing = unit(.2, "lines"),
          strip.text.x = element_text(angle = 90),
          axis.text.x = element_text(hjust=1,margin = margin(t = -13)),
          axis.ticks.length=unit(.5, "cm"),
          axis.title.x = element_blank(),
          axis.title.y = element_blank(),
          axis.ticks.y = element_blank(),
          plot.caption = element_text(face = "italic", hjust = 0)

    )
  plot(p)

  save_positive_ecart <- save_ecart
  save_positive_ecart[save_positive_ecart < 0] <- 0
  return(list(
      seriograph = p,
      contingency = contingency,
      frequency = frequency,
      EPPM = save_positive_ecart,
      EPM = save_ecart
    )
  )
}

overlap_plot <- function(df,add=NULL,density = NULL, color = NULL, reorder_color = FALSE){
  if(!is.null(color)){
    change_color <- TRUE
    save_color <- as.character(color)
  }else{
    change_color <- FALSE
  }

  #fonction pour faire des lignes
  line <- function(x0, x1, y0, y1, color = "#666666",width = 2){
    l = {}
    for(i in 1:length(x0)){
      l[[i]] = list(type = "line", y0 = y0[i], y1 = y1[i],
                    x0 = x0[i], x1 = x1[i], line = list(color = color, width = width))
    }
    return(l)
  }

  Lower_bound <- NULL
  Upper_bound <- NULL
  Observations <- NULL
  lower_median <- NULL

  Cluster <- df[,4]

  if(is.numeric(Cluster)){
    if(TRUE %in% (Cluster != trunc(Cluster))){

      class_subdivide <- which(Cluster != trunc(Cluster))
      nb_cust <- as.vector(table(trunc(unique(Cluster[class_subdivide]))))
      class_subdivide <- unique(trunc(Cluster[Cluster != trunc(Cluster)]))

      Cluster. <- Cluster
      Cluster <- trunc(Cluster)

      df[,4] <- trunc(Cluster)
      df <- cbind(df,Cluster.)

      names(df) = c("Observations", "Lower_bound", "Upper_bound", "Cluster", "Cluster.")
      df$Observations <- as.factor(df$Observations)
      df$Cluster = as.factor(df$Cluster)
      df$Cluster. = as.factor(df$Cluster.)
      if(!is.null(density)){
        df <- cbind(df,density)
      }
      if(change_color){
        df <- cbind(df,save_color)
      }
      if(!is.null(add)){
        if(dim(add)[2] == 1){
          n <- names(add)
          eval(parse(
            text=paste0("add <- data.frame(",n,
                        " = add[order(df$Cluster,df$Cluster.,df$Lower_bound,df$Upper_bound),])")))
        }else{
          add = as.data.frame(add)[order(df$Cluster,df$Cluster.,df$Lower_bound,df$Upper_bound),]
        }
      }
      df = df[order(df$Cluster,df$Cluster.,df$Lower_bound,df$Upper_bound),]
      #add = add[order(df$Cluster,df$Cluster.,df$Lower_bound,df$Upper_bound),]

      ### on permute les groupes en fonction de la médiane basse des groupes
      order = df  %>% group_by(Cluster) %>%
        summarise(median = median(Lower_bound)) %>%
        arrange(median)

      if(!is.null(add)){add <- cbind(add,Cluster=df$Cluster)}

      df_fin = {}
      add_fin = {}
      for(i in order$Cluster){
        add_order = {}
        df_order = {}
        new_tab = filter(df,Cluster == i)
        if(!is.null(add)){ new_add = filter(add,Cluster == i)}
        if(!new_tab$Cluster[1] == new_tab$Cluster.[1]){
          order_y = new_tab %>% group_by(Cluster.) %>%
            summarise(median = median(Lower_bound)) %>% arrange(median)
          for(i in order_y$Cluster.){
            if(!is.null(add)){ add_order = rbind(add_order,new_add[new_tab$Cluster. == i,]) }
            df_order = rbind(df_order,new_tab[new_tab$Cluster. == i,])
          }
        }else{
          if(!is.null(add)){add_order = new_add[new_tab$Cluster == i,]}
          df_order = new_tab[new_tab$Cluster == i,]
        }
        if(!is.null(add)){add_fin = rbind(add_fin,add_order)}
        df_fin = rbind(df_fin,df_order)
      }
      if(!is.null(add)){
        if(dim(add)[2] == 2){
          add <- data.frame(add_fin)
          index <- which(names(add) == "Cluster")
          print(n)
          eval(parse(text = paste0("add <- data.frame(",n," = add[,-index])")))
        }else{
          add = add_fin
          index <- which(names(add) == "Cluster")
          add <- data.frame(add[,-index])
        }
      }
      df = df_fin

      order = factor(unique(df_fin$Cluster.), levels = sort(unique(df_fin$Cluster.)))

      df$Cluster = factor(df$Cluster,levels = unique(df$Cluster))
      df$Cluster. = factor(df$Cluster.,levels = unique(df$Cluster.))

      ### on récupère les premières et dernières observation de chaque groupe
      intersection = df %>% group_by(Cluster) %>%
        summarise(First_observations = first(Observations), Last_observations = last(Observations))
      ### On récupère la position du changement de groupe
      intersection_place = which(df$Observations %in% intersection$First_observations)[-1] - 1.5


      ### on récupère les premières et dernières observation de chaque groupe et sous groupes
      intersection_SG = df %>% group_by(Cluster.) %>%
        summarise(First_observations = first(Observations), Last_observations = last(Observations))


      ### On calcule un l'intervalle de confiance pour les dates des groupes et sous groupes
      intervalle_inf = intervalle_sup = {}
      for(i in unique(df$Cluster.)){
        intervalle_inf[i] = arrondi(quantile(df[df$Cluster. == i,]$Lower_bound,.10),0)
        intervalle_sup[i] = arrondi(quantile(df[df$Cluster. == i,]$Upper_bound,.90),0)
      }

      intersection_SG$borne_inf = intervalle_inf
      intersection_SG$borne_sup = intervalle_sup


      ### On récupère la position du changement de groupe et sous groupe
      intersection_SG_place = which(df$Observations %in% intersection_SG$First_observations)[-1] - 1.5
      intersection_SG_x_min = c(-.5,intersection_SG_place)
      intersection_SG_x_max = c(intersection_SG_place, length(levels(df$Observation))-.5)

      intersection_SG$x_min = intersection_SG_x_min
      intersection_SG$x_max = intersection_SG_x_max

      #df$Cluster = factor(df$Cluster,levels = unique(df$Cluster))
      #df$Cluster. = factor(df$Cluster.,levels = unique(df$Cluster.))
      df$Observations = factor(df$Observations,levels = unique(df$Observations))


      if(reorder_color){
        ## Re numerotation (ancien au recent)
        order_tmp <- as.numeric(levels(df$Cluster.))
        Cluster_tmp <- as.numeric(as.character(df$Cluster.))

        new <- c()
        count <- 1
        for (i in 1:length(order_tmp)){
          if(order_tmp[i] == trunc(order_tmp[i])){
            new <- c(new,count)
            count <- count + 1
          }
          if(order_tmp[i] != trunc(order_tmp[i])){
            t <- order_tmp[i]
            if(i != length(order_tmp)){
              tun <- order_tmp[i+1]
            }else{
              tun <- 0
            }
            if(trunc(t) == trunc(tun)){
              new <- c(new,count+order_tmp[i]-trunc(t))
            }else{
              new <- c(new,count+order_tmp[i]-trunc(t))
              count <- count + 1
            }
          }
        }
        new <- sort(new)
        Cluster_tmp <- Cluster_tmp * 10
        for(i in 1:length(new)){
          Cluster_tmp[Cluster_tmp == 10*order_tmp[i]] <- new[i]
        }
        df$Cluster. <- as.factor(Cluster_tmp)
        df$Cluster <- as.factor(trunc(Cluster_tmp))

        class_subdivide <- which(Cluster_tmp != trunc(Cluster_tmp))
        nb_cust <- as.vector(table(trunc(unique(Cluster_tmp[class_subdivide]))))
        class_subdivide <- unique(trunc(Cluster_tmp[Cluster_tmp != trunc(Cluster_tmp)]))

        ### On définit les couleurs
        if(change_color){
          color <- rainbow_hcl(length(unique(df$save_color)),
                               c = 80, l = 60, start = 0,
                               end = 360*(length(unique(df$save_color))-1)/(length(unique(df$save_color))))
        }else{
          color <- rainbow_hcl(length(unique(df$Cluster)),
                               c = 80, l = 60, start = 0,
                               end = 360*(length(unique(df$Cluster))-1)/(length(unique(df$Cluster))))
          new_color = {}
          for(i in 1:length(color)){
            if(i %in% class_subdivide){
              a = which(class_subdivide == i)
              color_i = rep(color[i],nb_cust[a])
            }else{
              color_i = color[i]
            }
            new_color = c(new_color,color_i)
          }
          color <- new_color
        }
      }else{
        Cluster_tmp <- as.numeric(as.character(df$Cluster.))

        class_subdivide <- which(Cluster_tmp != trunc(Cluster_tmp))
        nb_cust <- as.vector(table(trunc(unique(Cluster_tmp[class_subdivide]))))
        class_subdivide <- unique(trunc(Cluster_tmp[Cluster_tmp != trunc(Cluster_tmp)]))

        #### On définit les couleurs
        if(change_color){
          color <- rainbow_hcl(length(unique(df$save_color)),
                               c = 80, l = 60, start = 0,
                               end = 360*(length(unique(df$save_color))-1)/(length(unique(df$save_color))))
        }else{
          color <- rainbow_hcl(length(unique(df$Cluster)),
                               c = 80, l = 60, start = 0,
                               end = 360*(length(unique(df$Cluster))-1)/(length(unique(df$Cluster))))

          new_color = {}
          for(i in 1:length(color)){
            if(i %in% class_subdivide){
              a = which(class_subdivide == i)
              color_i = rep(color[i],nb_cust[a])
            }else{
              color_i = color[i]
            }
            new_color = c(new_color,color_i)
          }

          color <- new_color[as.numeric(order)]
        }
      }

      #graphique
      #fonction pour faire des lignes verticales
      vline <- function(x = 0, color = "#666666", dash = "", width = 2){
        l = {}
        for(i in 1:length(x)){
          l[[i]] = list(type = "line", y0 = 0, y1 = 1, yref = "paper",
                        x0 = x[i], x1 = x[i], line = list(color = color, width = width, dash = dash))
        }
        return(l)
      }
      a = vline(c(-.5,intersection_place,length(levels(df$Observation))-.5))
      b = vline(intersection_SG_place, dash = 'dot', width = 1)

      c = line(x0 = intersection_SG$x_min, x1 = intersection_SG$x_max, y0 = intersection_SG$borne_inf,
               y1 = intersection_SG$borne_inf)
      d = line(x0 = intersection_SG$x_min, x1 = intersection_SG$x_max, y0 = intersection_SG$borne_sup,
               y1 = intersection_SG$borne_sup)


      #on définit les axes
      ay <- list(
        title = "Years",
        ticks = "outside",
        dtick = 200)

      if(!is.null(add)){
        #add_text
        col_add_text <- NULL

        for(j in 1:dim(add)[1]){
          add_text <- NULL
          for(i in 1:dim(add)[2]){
            if(dim(add)[2] == 1){
              add_text <- paste0("</br> ",names(add)[i]," : ", as.character(add[,i][j]))
            }else{
              add_text <- paste0(add_text,"</br> ",names(add)[i]," : ", as.character(add[,i][j]))
            }
          }
          col_add_text <- c(col_add_text,add_text)
        }

        df$add_text <- col_add_text
      }else{
        df$add_text <- ""
      }

      if(change_color){
        save_color <- df$save_color
        name <- as.character(df$save_color)
      }else{
        save_color <- df$Cluster.
        name <- Int2Alpha(as.numeric(as.character(save_color)))
      }

      #on affiche le plot
      p = plot_ly(df) %>%
        add_segments(x = df$Observations, xend = df$Observations,
                     y = df$Lower_bound, yend = df$Lower_bound + ((df$Upper_bound - df$Lower_bound)/2),
                     showlegend = T, color = save_color, colors = color,
                     name=paste("Cluster", name),
                     legendgroup = paste("Cluster", name),
                     hoverinfo = 'text',
                     text = paste("</br> Observation :", df$Observations,
                                  "</br> Lower bound :", df$Lower_bound,
                                  "</br> Upper bound :", df$Upper_bound,
                                  "</br> Cluster :", Int2Alpha(as.numeric(as.character(df$Cluster.))),
                                  df$add_text
                     )) %>%
        add_segments(x = df$Observations, xend = df$Observations,
                     y =  df$Lower_bound + ((df$Upper_bound - df$Lower_bound)/2), yend = df$Upper_bound,
                     showlegend = F, color = save_color, colors = color,
                     legendgroup = paste("Cluster", name),
                     hoverinfo = 'text',
                     text = paste("</br> Observation :", df$Observations,
                                  "</br> Lower bound :", df$Lower_bound,
                                  "</br> Upper bound :", df$Upper_bound,
                                  "</br> Cluster :", Int2Alpha(as.numeric(as.character(df$Cluster.))),
                                  df$add_text
                     )) %>%
        add_markers(x = intersection_SG$First_observations, y = intersection_SG$borne_inf,
                    color = I('#666666'), marker = list(size = 0.01),
                    hoverinfo = "text", showlegend = F,
                    text = paste("</br> Interval : [", intersection_SG$borne_inf,
                                 ", ", intersection_SG$borne_sup, "]", sep = "")) %>%
        add_markers(x = intersection_SG$First_observations, y = intersection_SG$borne_sup,
                    color = I('#666666'), marker = list(size = 0.01),
                    hoverinfo = "text", showlegend = F,
                    text = paste("</br> Interval : [", intersection_SG$borne_inf,
                                 ", ", intersection_SG$borne_sup, "]", sep = "")) %>%
        add_markers(x = intersection_SG$Last_observations, y = intersection_SG$borne_inf,
                    color = I('#666666'), marker = list(size = 0.01),
                    hoverinfo = "text", showlegend = F,
                    text = paste("</br> Interval : [", intersection_SG$borne_inf,
                                 ", ", intersection_SG$borne_sup, "]", sep = "")) %>%
        add_markers(x = intersection_SG$Last_observations, y = intersection_SG$borne_sup,
                    color = I('#666666'), marker = list(size = 0.01),
                    hoverinfo = "text", showlegend = F,
                    text = paste("</br> Interval : [", intersection_SG$borne_inf,
                                 ", ", intersection_SG$borne_sup, "]", sep = ""))

      p <- p %>%
        layout(shapes = c(a,b,c,d),
               title = "Timerange",
               yaxis = list(title = "Years",zeroline = FALSE),
               xaxis = list(title = "Observations")
        )
      plot <- p
      if(!is.null(density)){
        save_dens <- df$density
        df$density <- df$density / sum(df$density)

        line <- function(x0, x1, y0, y1, color = "#666666",width = 2){
          l = {}
          for(i in 1:length(x0)){
            l[[i]] = list(type = "line", y0 = y0[i], y1 = y1[i], yref = "y2",
                          x0 = x0[i], x1 = x1[i], line = list(color = color, width = width))
          }
          return(l)
        }

        l <- line(x0 = 0:(length(df$density)-2),
                  y0 = df$density[1:(length(df$density)-1)],
                  x1 = 1:(length(df$density)-1),
                  y1 = df$density[2:length(df$density)],
                  color = "#E16A86",width = 2.5)

        q <- plot_ly(df) %>%
          add_markers(x = df$Observation, y = df$density, showlegend = F,
                      hoverinfo = 'text', color = I("#E16A86"), marker = list(size = 2),
                      hovertext = paste("</br> Observation :", df$Observations,
                                        "</br> Proportion :", arrondi(df$density,3),
                                        "</br> Variable :", arrondi(save_dens,3),
                                        df$add_text)
          ) %>%
          layout(shapes=c(a,b,l))


        plot <- subplot(
          p,q,nrows = 2, heights = c(0.7,0.3),shareX = TRUE, shareY = FALSE,titleY = TRUE
        ) %>%
          layout(title = "Periodization of observations by clusters",
                 yaxis = list(title="Years",zeroline = FALSE),
                 yaxis2 = list(title="Density"),
                 xaxis = list(title = "Observations")
          )
        density <- df$density
        names(density) <- df$Observation
      }

      df$Cluster <- df$Cluster.
      order = df %>% group_by(Cluster) %>%
        summarise(lower_median = median(Lower_bound),upper_median = median(Upper_bound)) %>%
        arrange(lower_median)
      #order$Cluster <- new
      return(structure(list(order = order, plot = plot, density = density, reorder_cluster = save_color),
                       class = c("temp_obj", "list")))
    }
  }

  names(df) = c("Observations", "Lower_bound", "Upper_bound", "Cluster")
  df$Cluster = as.factor(df$Cluster)
  if(!is.null(density)){
    df <- cbind(df,density)
  }
  if(change_color){
    df <- cbind(df,save_color)
  }
  if(!is.null(add)){
    if(dim(add)[2] == 1){
      n <- names(add)
      eval(parse(text=paste0("add <- data.frame(",n," = add[order(df$Lower_bound, df$Upper_bound),])")))
    }else{
      add = as.data.frame(add)[order(df$Lower_bound, df$Upper_bound),]

    }
  }

  df = df[order(df$Lower_bound, df$Upper_bound),]

  ### on ordonne les groupes en fonction de la médiane basse des groupes
  order = df  %>% group_by(Cluster) %>%
    summarise(lower_median = median(Lower_bound),upper_median = median(Upper_bound)) %>% arrange(lower_median)

  df_order = {}
  add_order = {}
  for(i in order$Cluster){
    if(!is.null(add)){
      if(dim(add)[2] == 1){
        add_order = c(add_order, as.character(add[df$Cluster == i,]))
      }else{
        add_order = rbind(add_order, add[df$Cluster == i,])
      }
    }
    df_order = rbind(df_order, df[df$Cluster == i,])
  }
  if(!is.null(add)){
    if(dim(add)[2] == 1){
      eval(parse(text=paste0("add <- data.frame(",n," = add_order)")))
    }else{
      add = as.data.frame(add_order)
    }
  }
  df = df_order
  df$Observations = factor(df$Observations, unique(df$Observations))


  ### on récupère les premières et dernières observations de chaque groupe
  intersection = df %>% group_by(Cluster) %>% summarise(First_observations = first(Observations),
                                                        Last_observations = last(Observations))

  ### On ordonne
  intersection_order = {}
  for(i in order$Cluster){intersection_order = rbind(intersection_order,
                                                     intersection[intersection$Cluster == i,])}
  intersection = intersection_order
  intersection$Cluster = factor(intersection$Cluster, levels = intersection$Cluster)

  ### On calcule un intervalle de confiance pour les dates
  intervalle_inf = intervalle_sup = {}
  for(i in unique(df$Cluster)){
    intervalle_inf[i] = arrondi(quantile(df[df$Cluster == i,]$Lower_bound,.10),0)
    intervalle_sup[i] = arrondi(quantile(df[df$Cluster == i,]$Upper_bound,.90),0)
  }

  intersection$borne_inf = intervalle_inf
  intersection$borne_sup = intervalle_sup

  ### On récupère la position du changement de groupe
  intersection_place = which(df$Observations %in% intersection$First_observations)[-1] - 1.5
  intersection_x_min = c(-.5,intersection_place)
  intersection_x_max = c(intersection_place, length(levels(df$Observation))-.5)

  nb_cluster <- length(unique(df$Cluster))
  if(nb_cluster > 1){
    intersection$x_min = intersection_x_min
    intersection$x_max = intersection_x_max
  }else{
    intersection$x_min = c(-.5)
    intersection$x_max = c(length(levels(df$Observation))-.5)
  }

  #### On définit les couleurs pour le graph
  if(change_color){
    color <- rainbow_hcl(length(unique(df$save_color)),
                         c = 80, l = 60, start = 0,
                         end = 360*(length(unique(df$save_color))-1)/(length(unique(df$save_color))))
  }else{
    color <- rainbow_hcl(length(unique(df$Cluster)),
                         c = 80, l = 60, start = 0,
                         end = 360*(length(unique(df$Cluster))-1)/(length(unique(df$Cluster))))
  }

  #graphique
  #fonction pour faire des lignes verticales
  vlinebis <- function(x = 0, color = "#666666"){
    l = {}
    for(i in 1:length(x)){
      l[[i]] = list(type = "line", y0 = 0, y1 = 1, yref = "paper",
                    x0 = x[i], x1 = x[i], line = list(color = color))
    }
    return(l)
  }

  #on définit les axes
  ay <- list(
    title = "Years",
    ticks = "outside",
    dtick = 200)

  #add_text
  if(!is.null(add)){
    col_add_text <- NULL
    for(j in 1:dim(add)[1]){
      add_text <- NULL
      for(i in 1:dim(add)[2]){
        if(dim(add)[2] == 1){
          add_text <- paste0("</br> ",names(add)[i]," : ", as.character(add[,i][j]))
        }else{
          add_text <- paste0(add_text,"</br> ",names(add)[i]," : ", as.character(add[,i][j]))
        }
      }
      col_add_text <- c(col_add_text,add_text)
    }
    df$add_text <- col_add_text
  }else{
    df$add_text <- ""
  }

  if(reorder_color){
    #re order
    order_tmp <- as.numeric(unique(df$Cluster))
    Cluster_tmp <- as.numeric(df$Cluster)
    for(i in 1:length(order_tmp)){
      if(i != order_tmp[i] && i < order_tmp[i]){
        Cluster_tmp[Cluster_tmp == i] <- 0
        Cluster_tmp[Cluster_tmp == order_tmp[i]] <- i
        Cluster_tmp[Cluster_tmp == 0] <- order_tmp[i]
        order_tmp <- unique(Cluster_tmp)
      }
    }
    df$Cluster <- as.factor(Cluster_tmp)
    order$Cluster <- as.factor(unique(Cluster_tmp))
  }


  if(change_color){
    save_color <- df$save_color
    name <- as.character(df$save_color)
  }else{
    save_color <- df$Cluster
    name <- Int2Alpha(as.numeric(as.character(df$Cluster)))
  }

  #on affiche le plot
  if(nb_cluster > 1){

    a = vlinebis(c(-.5,intersection_place,length(levels(df$Observation))-.5))
    b = line(x0 = intersection$x_min, x1 = intersection$x_max,
             y0 = intersection$borne_inf, y1 = intersection$borne_inf)
    c = line(x0 = intersection$x_min, x1 = intersection$x_max,
             y0 = intersection$borne_sup, y1 = intersection$borne_sup)

    Cl <- save_color
    p = plot_ly(df) %>%
      add_segments(x = df$Observations, xend = df$Observations,
                   y = df$Lower_bound, yend = df$Lower_bound + ((df$Upper_bound - df$Lower_bound)/2),
                   showlegend = T, color = save_color, colors = color, name=paste("Cluster", name),
                   legendgroup = paste("Cluster", name),
                   hoverinfo = 'text',
                   text = paste("</br> Observation :", df$Observations,
                                "</br> Lower bound :", df$Lower_bound,
                                "</br> Upper bound :", df$Upper_bound,
                                "</br> Cluster :", Int2Alpha(as.numeric(as.character(df$Cluster))),
                                df$add_text
                   )) %>%
      add_segments(x = df$Observations, xend = df$Observations,
                   y = df$Lower_bound + ((df$Upper_bound - df$Lower_bound)/2), yend = df$Upper_bound,
                   showlegend = F, color = save_color, colors = color,
                   legendgroup = paste("Cluster", name),
                   hoverinfo = 'text',
                   text = paste("</br> Observation :", df$Observations,
                                "</br> Lower bound :", df$Lower_bound,
                                "</br> Upper bound :", df$Upper_bound,
                                "</br> Cluster :", Int2Alpha(as.numeric(as.character(df$Cluster))),
                                df$add_text
                   )) %>%
      add_markers(x = intersection$First_observations, y = intersection$borne_inf,
                  color = I('#666666'), marker = list(size = 0.01),
                  hoverinfo = "text", showlegend = F,
                  text = paste("</br> Interval : [", intersection$borne_inf,
                               ", ", intersection$borne_sup, "]", sep = "")) %>%
      add_markers(x = intersection$First_observations, y = intersection$borne_sup,
                  color = I('#666666'), marker = list(size = 0.01),
                  hoverinfo = "text", showlegend = F,
                  text = paste("</br> Interval : [", intersection$borne_inf,
                               ", ", intersection$borne_sup, "]", sep = "")) %>%
      add_markers(x = intersection$Last_observations, y = intersection$borne_inf,
                  color = I('#666666'), marker = list(size = 0.01),
                  hoverinfo = "text", showlegend = F,
                  text = paste("</br> Interval : [", intersection$borne_inf,
                               ", ", intersection$borne_sup, "]", sep = "")) %>%
      add_markers(x = intersection$Last_observations, y = intersection$borne_sup,
                  color = I('#666666'), marker = list(size = 0.01),
                  hoverinfo = "text", showlegend = F,
                  text = paste("</br> Interval : [", intersection$borne_inf,
                               ", ", intersection$borne_sup, "]", sep = "")) %>%
      layout(shapes = c(a,b,c))
  }else{

    a = vlinebis(c(-.5,length(levels(df$Observation))-.5))

    Cl <- NULL
    p = plot_ly(df) %>%
      add_segments(x = df$Observations, xend = df$Observations,
                   y = df$Lower_bound, yend = df$Lower_bound + ((df$Upper_bound - df$Lower_bound)/2),
                   showlegend = T, color = save_color, colors = color, name=paste("Cluster", name),
                   legendgroup = paste("Cluster", name),
                   hoverinfo = 'text',
                   text = paste("</br> Observation :", df$Observations,
                                "</br> Lower bound :", df$Lower_bound,
                                "</br> Upper bound :", df$Upper_bound,
                                df$add_text
                   )) %>%
      add_segments(x = df$Observations, xend = df$Observations,
                   y = df$Lower_bound + ((df$Upper_bound - df$Lower_bound)/2), yend = df$Upper_bound,
                   showlegend = F, color = save_color, colors = color,
                   legendgroup = paste("Cluster", name),
                   hoverinfo = 'text',
                   text = paste("</br> Observation :", df$Observations,
                                "</br> Lower bound :", df$Lower_bound,
                                "</br> Upper bound :", df$Upper_bound,
                                df$add_text
                   )) %>%
      layout(shapes = c(a))
  }

  plot <- p
  if(!is.null(density)){
    save_dens <- df$density
    df$density <- df$density / sum(df$density)

    line <- function(x0, x1, y0, y1, color = "#666666",width = 2){
      l = {}
      for(i in 1:length(x0)){
        l[[i]] = list(type = "line", y0 = y0[i], y1 = y1[i], yref = "y2",
                      x0 = x0[i], x1 = x1[i], line = list(color = color, width = width))
      }
      return(l)
    }

    l <- line(x0 = 0:(length(df$density)-2),
              y0 = df$density[1:(length(df$density)-1)],
              x1 = 1:(length(df$density)-1),
              y1 = df$density[2:length(df$density)],
              color = "#E16A86",width = 2.5)

    q <- plot_ly(df) %>%
      add_markers(x = df$Observation, y = df$density, showlegend = F,
                  hoverinfo = 'text', color = I("#E16A86"), marker = list(size = 2),
                  hovertext = paste("</br> Observation :", df$Observations,
                                    "</br> Proportion :", arrondi(df$density,3),
                                    "</br> Variable :", arrondi(save_dens,3),
                                    df$add_text)
                  ) %>%
      layout(shapes = na.omit(c(a,l)))

    plot <- subplot(
      p,q, nrows = 2, heights = c(0.7,0.3), shareX = TRUE, shareY = FALSE
      ) %>%
      layout(title = "Timerange",
             yaxis = list(title = "Years",zeroline = FALSE),
             yaxis2 = list(title="Density"),
             xaxis = list(title = "Observations")
      )
    density <- df$density
    names(density) <- df$Observation
  }

  return(structure(list(order = order, density = density, plot = plot, reorder_cluster = Cl),
                   class = c("temp_obj", "list")))
}

arrondi<-function (x, acc = 0)
{
  .local <- function(x, acc) {
    x <- x * (10^acc)
    ifelse(abs(x%%1 - 0.5) < .Machine$double.eps^0.5, ceiling(x)/(10^acc),
           round(x)/(10^acc))
  }
  if (is.data.frame(x))
    return(data.frame(lapply(x, .local, acc)))
  .local(x, acc)
}


Int2Alpha <- function(int){
  #====================================#
  # convert integer into letter
  let <- function(alphabet) function(i) {
    base10toA <- function(n, A) {
      stopifnot(n >= 0L)
      N <- length(A)
      j <- n %/% N
      if (j == 0L) A[n + 1L] else paste0(Recall(j - 1L, A), A[n %% N + 1L])
    }
    vapply(i-1L, base10toA, character(1L), alphabet)
  }

  first <- trunc(int)
  l1 <- let(LETTERS)(first)
  second <- arrondi((int - trunc(int))*10,2)
  l2 <- c()
  for(i in 1:length(second)){
    if(second[i] == 0){
      l2 <- c(l2,"")
    }else{
      f <- nchar(second[i])
      if(f == 1){
        l2 <- c(l2,as.character(second[i]))
      }
      if(f >= 3){
        new <- unlist(strsplit(as.character(second[i]),".",fixed=T))
        new <- paste0(new[1],new[2])
        l2 <- c(l2,new)
      }
    }
  }
  return(paste0(l1,l2))
}

Alpha2Int <- function(alpha){
  #====================================#
  # convert letter into integer
  get.letters <- function(length.out, case = 'upper'){
    make.chars <- function(length.out, case, n.char = NULL) {
      if(is.null(n.char))
        n.char <- ceiling(log(length.out, 26))
      m <- sapply(n.char:1, function(x) {
        rep(rep(1:26, each = 26^(x-1)) , length.out = length.out)
      })
      m.char <- switch(case,
                       'lower' = letters[m],
                       'upper' = LETTERS[m]
      )
      m.char <- LETTERS[m]
      dim(m.char) <- dim(m)

      apply(m.char, 1, function(x) paste(x, collapse = ""))
    }
    max.char <- ceiling(log(length.out, 26))
    grp <- rep(1:max.char, 26^(1:max.char))[1:length.out]
    unlist(lapply(unique(grp), function(n) make.chars(length(grp[grp == n]), case = case, n.char = n)))
  }

  alpha <- toupper(alpha)
  letter <- get.letters(800)
  first <- c()
  second <- c()
  for(i in 1:length(alpha)){
    first <- c(first,str_extract(alpha[i], "[A-Z]+"))
    tmp <- str_extract(alpha[i], "[0-9]+")
    if(is.na(tmp)){tmp <- ""}
    second <- c(second,tmp)
  }
  n <- nchar(max(second))
  second <- as.numeric(paste0("0.",second))

  second[is.na(second)] <- 0
  for(i in 1:length(alpha)){
    first[i] <- which(letter == first[i])
  }
  first <- as.numeric(first)
  return(first+second)
}

elbow_finder <- function(x_values, y_values) {
  # Max values to create line
  max_x_x <- max(x_values)
  max_x_y <- y_values[which.max(x_values)]
  max_y_y <- max(y_values)
  max_y_x <- x_values[which.max(y_values)]
  max_df <- data.frame(x = c(max_y_x, max_x_x), y = c(max_y_y, max_x_y))

  # Creating straight line (between max and min)
  fit <- lm(max_df$y ~ max_df$x)

  # Distance from point to line
  distances <- c()
  for(i in 1:length(x_values)) {
    distances <- c(
      distances,
      abs( coef(fit)[2] * x_values[i] - y_values[i] + coef(fit)[1] ) / sqrt( coef(fit)[2]^2 + 1^2 )
    )
  }

  return(distances)
}

convert <- function(x){
  res <- c()
  for(i in 1:length(x)){
    split <- strsplit(as.character(x[i]), split = "")
    alpha <- split[[1]][length(split[[1]])]
    if(split[[1]][1] == "-"){stop("We can't convert ",as.character(x[i]),
                                  " to date. Enter the date as a number.")}
    if(alpha == "a"){unit <- "00"}
    if(alpha == "b"){unit <- "25"}
    if(alpha == "c"){unit <- "50"}
    if(alpha == "d"){unit <- "75"}
    if(alpha != "a" && alpha != "b" && alpha != "c" && alpha != "d"){unit <- "00"}
    if(length(split[[1]]) == 3){
      res <- c(res,as.numeric(paste0(split[[1]][1],split[[1]][2],unit)))
    }else{
      if(alpha != "a" && alpha != "b" && alpha != "c" && alpha != "d"){
        if(length(split[[1]]) == 2){
          res <- c(res,as.numeric(paste0(split[[1]][1],split[[1]][2],unit)))
        }
        if(length(split[[1]]) == 1){
          res <- c(res,as.numeric(paste0(split[[1]][1],unit)))
        }
      }else{
        res <- c(res,as.numeric(paste0(split[[1]][1],unit)))
      }
    }
  }
  return(res-100)
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage("This version changes the hclustcompro function behavior (updated from 1.2.4).\nYou can find the old version under the name perioclust or adapt your script to the new format of hclustcompro!")
}

estimateDensityMulti <- function(data){
  cat("\nEstimation of multidimensionnal density ..")
  #Smooth cross-validation bandwidth selector
  H.scv <- Hscv(data)
  fhat <- kde(data, H.scv, eval.points=data)
  res <- fhat$estimate*100
  cat(". Done")
  return(res)
}

IW <- function(x)
{
  #===============================================================================
  # SOMMES DES ECARTS AUX CARRE
  #===============================================================================
  sum(scale(x, scale = FALSE)^2)
}

spatialpatches2 <- function (x, y, z, w, Lim.D = 100, A.li = 0, modproj = NA, mlong = NA, mlat = NA)
{
  #===============================================================================
  # IDENTIFICATION OF SPATIAL PATCHES
  # AND COUNT OF CENTERS OF GRAVITY OF SPATIAL PATCHES
  #
  # Routine from EU program Fisboat, DG-Fish, STREP n? 502572
  # Authors : P.Petitgas (Ifremer), M.Woillez and J.Rivoirard (Mines-ParisTech)
  # Last update : 01 march 2008
  #
  # Arguments:
  # x	      The x-coordinate of the first population
  #	        Can be a vector, a matrix or an xyZ list (-,|,[]).
  # y	      The y-coordinates of the first population
  # z	      The regionalised variable of the first population.
  #	        If missing, the results of 'cgi' will concern the samples only.
  # w	      Optional. A weight or an area of influence of the first population.
  #	        Set to 1 if missing
  # Lim.D   Select minimum distance from sample to patch centre: to
  #         identify patches (units are those of coordinates)
  # A.li    Visualisation of gravity centres for those patches with
  #         abundance > A.li (in %)
  # modproj	Optional. Indicates the type of projection to perform.
  # mlong   mean longitude in DEGREES of the data set to be transformed
  # mlat    mean latitude in DEGREES of the data set to be transformed
  #	        See 'dg2nm' for precisions
  #
  #===============================================================================

  patch.id <- function(xx,yy,zz,ww,dli,ali,modproj,mlong,mlat)
  {
    ############################################################################
    # inputs : xx , yy , ww & zz ranked by decreasing order of zz
    #          dli is a distance limit from the patch gravity centre to define
    #          the patch border
    #
    # the function starts from the richest zz value and considers each sample
    # in decreasing order of zz. It tests whether the current value is spatially
    # close enough to the gravity centre of previously formed patches. if not a
    # new patch is considered. and so on until the last value. Patches of nul
    # values are returned with centres as NA and code 0 and their areas are
    # summed.
    #
    # outputs : the patch number for each sample,
    #           the gravity center for each patch,
    #           the percent abundance and area for each patch
    ############################################################################

    g <- c(1,rep(0,(length(xx)-1)))
    for (j in 1:(length(xx)-1)) {
      xg <- tapply(ww[1:j]*zz[1:j]*xx[1:j],g[1:j],sum,na.rm=T) /
        tapply(ww[1:j]*zz[1:j],g[1:j],sum,na.rm=T)
      yg <- tapply(ww[1:j]*zz[1:j]*yy[1:j],g[1:j],sum,na.rm=T) /
        tapply(ww[1:j]*zz[1:j],g[1:j],sum,na.rm=T)
      d <- sqrt(((xg-x1[j+1]))^2 + (yg-y1[j+1])^2)
      o <- order(d)
      if (d[o[1]] < dli){
        g[j+1] <- o[1]
      }else{
        g[j+1] <- max(g[1:j], na.rm=T)+1
      }
    }
    #second passage des obs pour affiner centres de gravite
    for(i in 1:length(xx)) {
      xg <- tapply(ww*zz*xx,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
      yg <- tapply(ww*zz*yy,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
      d <- sqrt(((xg-x1[i]))^2+(yg-y1[i])^2)
      o <- order(d)
      g[i] <- o[1]
    }

    xg <- tapply(ww*zz*xx,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    yg <- tapply(ww*zz*yy,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    pb <- tapply(ww*zz,g,mean,na.rm=TRUE)
    inertiaTot <- sum(scale(zz, scale = FALSE)^2)
    InertiaWithin <- tapply(zz,g,IW)
    sel <- (tapply(ww*zz,g,sum,na.rm=TRUE)/sum(ww*zz,na.rm=TRUE))*100>=ali
    res <- max(unique(g)[sel],na.rm=TRUE)
    n <- sort(unique(g))
    selna <- is.na(xg)
    if (sum(selna) > 1) {
      n <- c(n[!selna],0)
      pb <- c(pb[!selna],0)
      pa <- c(pa[!selna],sum(pa[selna],na.rm=T))
      xg <- c(xg[!selna],NA)
      yg <- c(yg[!selna],NA)
      g[!is.element(g,n)] <- 0
    }
    return(list(n=g,mat=cbind(n=n,xg=round(xg,2),yg=round(yg,2),zg=round(pb*100,2)),
                InertiaWithin=InertiaWithin,
                InertiaWithintot=sum(InertiaWithin),
                InertiaBetween=inertiaTot - sum(InertiaWithin),
                inertiaTot=inertiaTot,
                nsp=res))
  }

  # prepare data
  Xsta <- x		#Xsta <- bid$x
  Ysta <- y		#Ysta <- bid$y
  Zsta <- z
  Wsta <- w/sum(w,na.rm=TRUE)

  # exclude NA values
  SEL <- is.na(Xsta)+is.na(Ysta)+is.na(Zsta)+is.na(Wsta)
  Xsta <- Xsta[!SEL]
  Ysta <- Ysta[!SEL]
  Zsta <- Zsta[!SEL]
  Wsta <- Wsta[!SEL]

  # order data by decreasing order of value z
  zz1 <- sort(Zsta,decreasing=TRUE,index.return=TRUE)
  w1 <- Wsta[zz1$ix]
  x1 <- Xsta[zz1$ix]
  y1 <- Ysta[zz1$ix]
  z1 <- zz1$x

  # identify patches around high values
  SP <- patch.id(x1,y1,z1,w1,Lim.D,A.li,modproj=modproj,mlong=mlong,mlat=mlat)

  # graphical visualisation : patch identification
  #shows data points, circles for data values, number of patch &
  #crosses for patch gravity centres with abundance > A.li of total
  o <- sort(zz1$ix,index.return=TRUE)
  SP$n <- SP$n[o$ix]
  SP
}

WSS <- function(hist,data)
{
  #====================================================================================
  # WSS        Return the within cluster sum of square for each partition.
  #
  # hist       The historique of the partitions
  # data       The geographical data and third variable Z
  #====================================================================================

  out <- list()
  n <- length(hist[1,])
  Itot <- calctot(data)
  for (i in 1:n){
    WSS <- c()
    #recup compo of one partition i
    compo <- hist[,i]

    #calculer Within-cluster Sum of Square
    X <- data [,1]
    Y <- data [,2]
    Z <- data [,3]
    XG <- tapply(X * Z,compo,sum)/tapply(Z,compo,sum)
    YG <- tapply(Y * Z,compo,sum)/tapply(Z,compo,sum)
    nbgrp <- length(XG)
    for(j in 1:nbgrp){
      subX <- X[compo==j]
      subY <- Y[compo==j]
      subZ <- Z[compo==j]
      DX <- subX - XG[j]
      DY <- subY - YG[j]
      D <- sqrt(DX^2 + DY^2)
      INERT <- sum(subZ * (D^2))/sum(Z)
      WSS <- c(WSS,INERT)
    }
    out <- c(out,list(WSS))
  }
  out
}

calctot <- function(data)
{
  X <- data[,1]
  Y <- data[,2]
  Z <- data[,3]
  XG <- sum(X * Z)/sum(Z)
  YG <- sum(Y * Z)/sum(Z)
  DX <- X - XG
  DY <- Y - YG
  D <- sqrt(DX^2 + DY^2)
  INERT <- sum(Z * (D^2))/sum(Z)
  INERT
}

dist_m <- function(data)
{
  #===================================================================================
  # DISTANCE MATRIX   Weighted Eucidiean distance
  #
  # data      data.frame or matrix with 3 colunms (X, Y, Z)
  #===================================================================================
  square <-function(x){return(x*x)}
  Dmatrix <- matrix(ncol=length(data[,1]), nrow=length(data[,1]))
  for(i in 1:length(data[,1])){
    for (j in 1:length(data[,1])){
      #Weighted Eucidiean distance
      distance_pond <- sqrt( ( square(data[i,1]-data[j,1]) + square(data[i,2]-data[j,2]) ) *
                               square(data[i,3]-data[j,3])
      )
      Dmatrix[i,j] <- distance_pond
      Dmatrix[j,i] <- distance_pond
    }
  }
  return(Dmatrix)
}

transfo_z <- function(z)
{
  #====================================================================================
  # Transfo Z  Confidence interval on Empirical Cumulative Distribution Function (ECDF)
  #            We only can use z data positive like density, count or frequencies.
  #
  # z         vector of values
  #====================================================================================
  cat("\nEstimation of univariate density ..")
  dx <- density(z)
  xnew <- z # find density value on a new point
  dx.approx <- approx(dx$x,dx$y,xout=xnew)
  pi <- dx.approx$y*100 # densite estimee * 100 en chaque point du jeu de donnees
  cat(". Done")
  return(pi)
}

addLegendCustom <- function(map, colors, labels, sizes, opacity = 0.5, title)
{
  #===================================================================================
  # Custom legend for diffrent sizes of point in final map
  #
  # Arguments:
  # map       The leaflet map object
  # colors    The colors of the points
  # labels    The labels of the points
  # sizes     The sizes signification of the points in the map
  # opacity   The opacity of the background
  #
  # You can change the title and the position of the legend directly in the code. (last line of the function)
  #===================================================================================

  colorAdditions <- paste0(colors, ";border-radius: 50%;margin-top:10%; width:", 2*sizes, "px; height:", 2*sizes, "px")
  labelAdditions <- paste0("<div style='display: inline-block;height: ",
                           2*sizes, "px;margin-top: 4px;line-height: 0px;'>", labels, "</div>")

  return(addLegend(map, colors = colorAdditions, labels = labelAdditions, opacity = opacity,
                   title = title, position = "topright"))
}

"ecartmatrice" <-
  function (historique = "matrix composition of groups for each dlim")
  {
    #===================================================================================
    # ECART MATRICE
    #
    # Author : A. Coulon (Universite de Nantes)
    # Last update : 22 march 2018
    #
    # Arguments:
    # historique   The composition of groups for each dlim and each initial elements
    #
    #===================================================================================

    historique <- as.matrix(historique)
    #variable securite
    OK <- 1

    if(length(historique[,1]) <= 1){
      cat("No Cluster")
    }
    ecart <- matrix(ncol = length(historique[,1]), nrow = length(historique[,1]))
    #On parcours le tableau pour tous les j on les compare a tous les i (compare tous les elt entre eux)
    for (j in 1:length(historique[,1])) {
      for (i in 1:length(historique[,1])) {
        if(i > j){
          v <- historique[i,]==historique[j,]
          somme <- sum(v %in% FALSE)
          ecart[i,j] <- somme
        }

      }
    }
    ecart <- replace(ecart,is.na(ecart),Inf)
    ecart
  }

IndiceMoran <- function(hist,data) {
  out <- list()
  X <- data[,1]
  Y <- data[,2]
  Z <- data[,3]

  n <- length(hist[1,])
  for (i in 1:n){
    #recup compo of one partition i
    compo <- hist[,i]

    nb <- length(unique(compo))

    for(j in 1:nb){
      subX <- X[compo==j]
      subY <- Y[compo==j]
      subZ <- Z[compo==j]

      if(length(subX) > 3){

        if(length(unique(subZ)) > 2 && length(unique(subX)) > 2 && length(unique(subY)) > 2){
          data.dists <- as.matrix(dist(cbind(subX, subY)))
          Moran <- Moran.I(subZ, data.dists)
        }else{
          Moran <- list(observed= NA, p.value = NA)
        }

      }else{
        Moran <- list(observed= NA, p.value = NA)
      }

      if(!exists("M.I")){
        M.I <- data.frame(grp=c(Moran$observed,Moran$p.value),row.names = c("Moran.I","p.value"))
      }else{
        M.I <- cbind(M.I,grp=c(Moran$observed,Moran$p.value))
      }
    }
    out <- c(out,list(M.I))
    rm(M.I)
  }
  out
}

load(file = "./data/datarcheo.RData")
load(file = "./data/datacancer.RData")

#----------------------------------------------------------------------------------------------#
###########################                - SERVER -                ###########################
#----------------------------------------------------------------------------------------------#
server <- function(input, output, session) {

  dlmap <- reactiveValues(map = 0)
  values <- reactiveValues()
  plot.dend <- reactiveValues(main=NULL, add=NULL, add2=NULL)
  values[["path"]] <- "wait.png"
  values[["multi"]] <- "wait"
  values[["lonlat"]] <- TRUE
  values[["silhouetteplot"]] <- NULL
  values[["labelall"]] <- NULL
  values[["isLabel"]] <- TRUE

  #------------------------------------------------------#
  # Download
  #------------------------------------------------------#
  # output$map.png <- downloadHandler(
  #   filename = "map.png",
  #   content = function(file) {
  #     showModal(waitModal())
  #     mapshot(dlmap$map, file = file)
  #     removeModal()
  #   }
  # )

  #------------------------------------------------------#
  #  modalDialog zdata ask
  #------------------------------------------------------#
  # Return the UI for a modal dialog with data selection input. If 'failed' is
  # TRUE, then display a message that the previous value was invalid.
  dataModal <- function(failed = FALSE, data = 1,multi = FALSE) {
    if(data == 3 && multi=="Univariate"){
      modalDialog(title="datatypemessage",size = c("s"),

                    h4("Data:",style="margin-left:10px;"),hr(style="border-color: #222222;"),
                    span("This classification requires geographical coordinates and a third variable: var."),
                    br(),
                    span("We use var to weight the distance and var must be a frequency or density and always positive."),
                    br(),br(),
                    radioButtons("z_select", "Choose data type of your var",
                                 choices = c(positive = "un",
                                             other = "de"),
                                 selected = "de"),

                  if (failed)
                    div(tags$b("You have to choose", style = "color: red;")),
                  footer = tagList(
                    modalButton("Cancel"),
                    actionButton("dismiss_modal","OK")
                  )
      )
    }else{
      modalDialog(title="datatypemessage",size = c("s"),
                  h4("Are you sure ? "),hr(style="border-color: #222222;"),
                  footer = tagList(
                    modalButton("Cancel"),
                    actionButton("dismiss_modal","OK")
                  )
      )
    }
  }
  #------------------------------------------------------#
  #  modalDialog cuttree message
  #------------------------------------------------------#
  # Return the UI for a modal dialog with data selection input. If 'failed' is
  # TRUE, then display a message that the previous value was invalid.
  cuttreeModal <- function(failed = FALSE) {
    modalDialog(title="cuttreemessage",size = c("l"),
                h4("Cut the tree:",style="margin-left:10px;"),hr(style="border-color: #222222;"),
                span("You can clic on the dendrogram "),img(src="tree.png",height = 23, width = 26),span(" in order to select the dlim where to cut the tree."),
                span("The map and the silhouette plot will be automatically updated."),

                if (failed)
                  div(tags$b("You have to choose", style = "color: red;")),
                footer = tagList(
                  modalButton("OK")
                )
    )
  }
  #------------------------------------------------------#
  #  modalDialog wait message
  #------------------------------------------------------#
  waitModal <- function(failed = FALSE) {
    modalDialog(title="waitmessage",size = c("s"),
                h4("Wait . . . ",style="margin-left:100px;"),hr(style="border-color: #222222;"),
                footer = NULL
    )
  }

  #------------------------------------------------------#
  #  modalDialog upload
  #------------------------------------------------------#
  uploadModal <- function(failed = FALSE) {
    modalDialog(title = "upload",size=c("l"),
                h4("Upload file"),
                # Input: Select a file ----
                fileInput("file1", span(icon("import", lib = "glyphicon"),"Choose CSV File"),
                          multiple = FALSE,
                          accept = c("text/csv",
                                     "text/comma-separated-values,text/plain",
                                     ".csv")
                ),

                # Horizontal line ----
                hr(style="border-color: #222222;"),
                actionButton(inputId = "toggle", span(icon("cogs",lib = "font-awesome"),"Settings")),
                fluidRow(id="myBox",
                  column(width = 5,
                         span(strong("Non-geographic variables.")),
                         sliderTextInput("UniMulti","Your case is:",choices=c("Univariate","Multivariate"),width = "100px"),
                         hr(style="border-color: #C6C0AA;"),
                         sliderInput("nb_var","How many (non-geographic) variables do you have?",min=2,max=6,value=2)
                  ),
                  column(width = 3,
                         # Input: Select Label ----
                         span(strong("Your data have:")),
                         checkboxInput("label", "Label", TRUE),
                         checkboxInput("lonlat", "Longitude\nLatitude", TRUE),
                         hr(style="border-color: #C6C0AA;"),
                         # Input: Select separator ----
                         radioButtons("sep", "Separator",
                                      choices = c(Comma = ",",
                                                  Semicolon = ";",
                                                  Tab = "\t",
                                                  Space = " "),
                                      selected = ";")
                  ),
                  column(width = 3,
                         # Input: Checkbox if file has header ----
                         span(strong("Header")),
                         checkboxInput("header", "Header", TRUE),
                         hr(style="border-color: #C6C0AA;"),
                         # Input: Select quotes ----input$dec
                         radioButtons("quote", "Quote",
                                      choices = c(None = "",
                                                  "Double Quote" = '"',
                                                  "Single Quote" = "'"),
                                      selected = '"'),
                         radioButtons("dec", "Decimal",
                                      choices = c("Dot" = '.',
                                                  "Comma" = ","),
                                      selected = '.')
                  )
                ),
                # Horizontal line ----
                hr(style="border-color: #C6C0AA;"),
                fluidRow(
                  column(width = 8
                  ),
                  column(width = 2,
                         h4("Match:")
                  ),
                  column(width = 1,
                         imageOutput("image2",width = "40px", height = "40px")
                  )
                ),
                h4("How your data must be organized"),
                hr(style="border-color: #222222;"),
                tableOutput("table"),
                # Output: Data file ----
                h4("Your data"),hr(style="border-color: #222222;"),
                tableOutput("contents"),
                if (failed)
                  div(tags$b("Your data must be match with the example above ! Change parameter or file.", style = "color: red;")),
                footer = tagList(
                  modalButton("Cancel"),
                  actionButton("okUL", "OK")
                )
    )
  }
  #------------------------------------------------------#
  #  Uploading File
  #------------------------------------------------------#
  output$image2 <- renderImage({

    if (values[["path"]] == "yes.jpg") {
      return(list(
        src = "www/yes.jpg",
        width="40px",
        contentType = "image/jpg",
        alt = "yes"
      ))
    }else{
      return(list(
        src = "www/no.jpg",
        width="40px",
        contentType = "image/jpg",
        alt = "no"
      ))
    }

  }, deleteFile = FALSE)

  observe({
    if(!is.null(input$UniMulti)){
      if(input$UniMulti == "Univariate"){
        shinyjs::hide("nb_var")
      }else{
        shinyjs::show("nb_var")
      }
    }
  })

  observeEvent(input$UL,{
    showModal(uploadModal())
    shinyjs::hide("myBox")
    shinyjs::hide("nb_var")
  })

  observe({
    if(!is.null(values[["test"]])){
      if(values[["test"]] == length(values[["DF"]][1,])){
        values[["path"]] <- "yes.jpg"
      }else{
        values[["path"]] <- "no.jpg"
      }
    }
  })

  observeEvent(input$okUL,{
    if(values[["test"]] == length(values[["DF"]][1,])){
      values[["nb_var"]] <- input$nb_var
      values[["multi"]] <- input$UniMulti
      values[["lonlat"]] <- input$lonlat

      data <- values[["DF"]]
      multi <- values[["multi"]]
      lonlat <- values[["lonlat"]]

      if(!is.numeric(data[,1])){

        showNotification("coordinates must be numeric! (Check the decimal character)",type = "warning", duration=15)
        showModal(uploadModal(failed = TRUE))
        shinyjs::hide("nb_var")
      }else{
        if(!is.numeric(data[,2])){

          showNotification("coordinates must be numeric! (Check the decimal character)",type = "warning", duration=15)
          showModal(uploadModal(failed = TRUE))
          shinyjs::hide("nb_var")
        }else{
          if(!is.numeric(data[,3])){

            showNotification("variable must be numeric! (Check the decimal character)",type = "warning", duration=15)
            showModal(uploadModal(failed = TRUE))
            shinyjs::hide("nb_var")
          }else{
            #change data select
            updateSelectInput(session, "select", selected = 3)
            removeModal()
          }

        }


      }



    }else{
      showModal(uploadModal(failed = TRUE))
      shinyjs::hide("nb_var")
    }
  })

  output$contents <- renderTable({

    # input$file1 will be NULL initially. After the user selects
    # and uploads a file, head of that data file by default,
    # or all rows if selected, will be shown.
    req(input$file1)
    inFile <- input$file1

    if(is.null(inFile))
      return(NULL)

    tryCatch({
        df <- read.csv(input$file1$datapath,
                       header = input$header,
                       sep = input$sep,
                       dec = input$dec,
                       quote = input$quote)
      },
      error = function(e) {
        # return a safeError if a parsing error occurs
        stop(safeError(e))
    })

    if (!is.null(inFile)){
      DF <- df
      values[["DF"]] <- DF
    }

    return(head(df))

  })

  output$table <- renderTable(
    if(input$UniMulti == "Univariate"){
      if(input$label){
        values[["test"]] <- 4
        values[["isLabel"]] <- TRUE
        data.frame(coord.1="X",coord.2="Y",var="var",label_optional="label")
      }else{
        values[["test"]] <- 3
        values[["isLabel"]] <- FALSE
        data.frame(coord.1="X",coord.2="Y",var="var")
      }

    }else{
      if(input$nb_var == 2){
        if(input$label){
          values[["test"]] <- 5
          values[["isLabel"]] <- TRUE
          data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",label_optional="label")
        }else{
          values[["test"]] <- 4
          values[["isLabel"]] <- FALSE
          data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2")
        }
      }else{
        if(input$nb_var == 3){
          if(input$label){
            values[["test"]] <- 6
            values[["isLabel"]] <- TRUE
            data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",label_optional="label")
          }else{
            values[["test"]] <- 5
            values[["isLabel"]] <- FALSE
            data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3")
          }
        }else{
          if(input$nb_var == 4){
            if(input$label){
              values[["test"]] <- 7
              values[["isLabel"]] <- TRUE
              data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",label_optional="label")
            }else{
              values[["test"]] <- 6
              values[["isLabel"]] <- FALSE
              data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4")
            }
          }else{
            if(input$nb_var == 5){
              if(input$label){
                values[["test"]] <- 8
                values[["isLabel"]] <- TRUE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5",label_optional="label")
              }else{
                values[["test"]] <- 7
                values[["isLabel"]] <- FALSE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5")
              }
            }else{
              if(input$label){
                values[["test"]] <- 9
                values[["isLabel"]] <- TRUE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5",var.6="var6",label="label")
              }else{
                values[["test"]] <- 8
                values[["isLabel"]] <- FALSE
                data.frame(coord.1="X",coord.2="Y",var.1="var1",var.2="var2",var.3="var3",var.4="var4",var.5="var5",var.6="var6")
              }

            }

          }

        }

      }

    }
  )

  observe({
    if(!values[["isLabel"]]){
      shinyjs::hide(id = "Plabel")
    }else{
      shinyjs::show(id = "Plabel")
    }
  })

  observeEvent(input$toggle, {
    if(input$toggle %% 2 == 0){
      shinyjs::hide(id = "myBox")
    }else{
      shinyjs::show(id = "myBox")
    }
  })

  #------------------------------------------------------#
  #  MapClust
  #------------------------------------------------------#
  observe({
    if(input$select==3){
      if (is.null(values[["DF"]])){
        showNotification("Oops ! you forgot a little thing ! To plot your data, you need to follow a few things:
                       You must import your own data! Re Run after import!",type = "warning", duration = 100)
      }else{
        if(input$UniMulti == "Univariate"){
          if(input$label){
            values[["isLabel"]] <- TRUE
          }else{
            values[["isLabel"]] <- FALSE
          }

        }else{
          if(input$nb_var == 2){
            if(input$label){
              values[["isLabel"]] <- TRUE
            }else{
              values[["isLabel"]] <- FALSE
            }
          }else{
            if(input$nb_var == 3){
              if(input$label){
                values[["isLabel"]] <- TRUE
              }else{
                values[["isLabel"]] <- FALSE
              }
            }else{
              if(input$nb_var == 4){
                if(input$label){
                  values[["isLabel"]] <- TRUE
                }else{
                  values[["isLabel"]] <- FALSE
                }
              }else{
                if(input$nb_var == 5){
                  if(input$label){
                    values[["isLabel"]] <- TRUE
                  }else{
                    values[["isLabel"]] <- FALSE
                  }
                }else{
                  if(input$label){
                    values[["isLabel"]] <- TRUE
                  }else{
                    values[["isLabel"]] <- FALSE
                  }

                }

              }

            }

          }

        }
      }

    }else{
      values[["isLabel"]] <- TRUE
    }
  })

  ##select data and run calcul
  observeEvent(input$Run,{
    if (input$select==3 && is.null(values[["DF"]])){
      showNotification("Oops ! you forgot a little thing ! To plot your data, you need to follow a few things:
                       You must import your own data and after press again the run button!",type = "error", duration = 100)
      showModal(uploadModal())
    }else{
      if (input$select==3){showModal(dataModal(data=3,multi=input$UniMulti))}
      if (input$select==2){showModal(dataModal(data=2))}
      if (input$select==1){showModal(dataModal(data=1))}
    }
  })

  observeEvent(input$dismiss_modal,{
    removeModal()
    showModal(waitModal())

    if(input$select==1){
      data <- datarcheo
      ctrl <- "de"
      multi <- "Univariate"
      lonlat <- TRUE
    }
    if(input$select==2){
      data <- datacancer
      ctrl <- "un"
      multi <- "Univariate"
      lonlat <- TRUE
    }
    OK <- TRUE
    if(input$select==3){
      if (is.null(values[["DF"]])){
        showNotification("There is no upload data. We select datarcheo by default",type = "warning", duration = 100)
        data <- datarcheo
      }else{
        data <- values[["DF"]]
        multi <- values[["multi"]]
        lonlat <- values[["lonlat"]]

        if(!is.numeric(data[,1])){
          OK <- FALSE
          showNotification("coordinates must be numeric!",type = "warning", duration=15)
        }
        if(!is.numeric(data[,2])){
          OK <- FALSE
          showNotification("coordinates must be numeric!",type = "warning", duration=15)
        }

        if(multi == "Multivariate"){
          for(i in 3:(2+values[["nb_var"]])){
            if(!is.numeric(data[,i])){
              OK <- FALSE
              showNotification("variable must be numeric!",type = "warning", duration=15)
            }
          }
        }else{
          if(!is.numeric(data[,3])){
            OK <- FALSE
            showNotification("variable must be numeric!",type = "warning", duration=15)
          }
        }
      }
    }
    if(OK == TRUE){

      values[["multi2"]] <- multi
      if(input$Plabel==T){Plabel = T}
      if(input$Plabel==F){Plabel = F}
      if(input$extenddend==T){extenddend = T}
      if(input$extenddend==F){extenddend = F}
      if(input$select==3 && multi == "Univariate"){
        if(input$z_select=="un"){ctrl="un"}else{ctrl="de"}
      }
      values[["data"]] <- data

      values[["index"]] <- NULL
      binf = 0
      bsup = max(dist(data[,1:2]))
      pas = (max(dist(data[,1:2]))/20)
      plot.dend$add <- NULL
      plot.dend$add2 <- NULL

      progress1 <- Progress$new(session, min = 0, max = 2)
      on.exit(progress1$close())
      progress1$set(message = 'Initialisation and calcul of the distance matrix ..')
      progress1$set(value = 1)

      progress2 <- Progress$new(session,style = "notification", min = 0, max = round((bsup - binf) / pas,0))
      on.exit(progress2$close())
      progress2$set(message = 'Calculation in progress',
                    detail = 'This may take a while...')


      #Si Z negatif : reduction
      if(multi == "Multivariate"){
        Z <- try(Z <- estimateDensityMulti(data[,3:(values[["nb_var"]]+2)]))
        values[["mdata"]] <- data[,3:(values[["nb_var"]]+2)]
        if(!is.numeric(Z)){
          Z <- transfo_z(data[,3])
          values[["multi2"]] <- "Univariate"
          multi <- "Univariate"
          showNotification("Error in chol.default(S) :
    the leading minor of order 1 is not positive definite.",type="error")
          showNotification("There was a error during the kernel density estimation. We consider only the first variable for the rest of the process.",type="warning",duration=40)
          saveZ <- data[,3]
          data[,3] <- Z
        }else{
          saveZ <- rep(7,length(data[,3]))
        }

      }else{
        Z <- data[,3]
        if(!ctrl=="un"){
          Z <- transfo_z(Z)
        }else{
          if(min(Z)<0){
            Z <- transfo_z(Z)
            showNotification("The var data have been changed due to negative value in third column.",type="error")
          }
        }
        saveZ <- data[,3]
        data[,3] <- Z
      }


      #on enregistre dans l'element renvoyer a l'utilisateur les parametres utilises
      X <- data[,1]
      Y <- data[,2]
      L <- data[,length(data[1,])]
      data <- data.frame(X,Y,Z)
      xg <- c()
      groupes=1
      dlim=1
      xend=1
      yend=1

      res <- NULL
      values[["Size"]] <- saveZ
      values[["C"]] <- ifelse(saveZ<0,"2, 5","1")
      W = rep(1 , length(Y))


      if (bsup <= binf) {
        showNotification("The bsup value must be higher than binf",type = "error", duration = 10)
        stop("The bsup value must be higher than binf")
      }

      label <- as.character(L)

      id <- c(1:length(data[,1]))

      if(values[["isLabel"]]){
        values[["labelall"]] <- label
      }else{
        values[["labelall"]] <- id
        label <- as.character(id)
      }


      data=cbind(data,id)
      data=as.data.frame(data)

      if( bsup > max(dist(data[,1:2])) ) {
        cat("The maximal value of dlim is: ",round(max(dist(data[,1:2])),2))
        cat("bsup have been changed in order to not run high dlim (refers to data)")
        bsup <- max(dist(data[,1:2]))
      }

      #remise a zero des objects
      names <- c()
      #initialisation matrice historique des groupes
      historique <- matrix(ncol=0, nrow=length(X))

      cat("\nInitialisation and calcul of the distance matrix ..")
      DiMatrix <- dist_m(data)
      progress1$set(value = 2)
      cat(". Done\n")
      cat("Running Progress:\n")
      values[["DiMatrix"]] <- DiMatrix
      #creation barre de progression(pb)
      pb <- txtProgressBar( min = 0, max = round((bsup - binf) / pas,0) , style = 3 )
      progress1$close()

      pre_k=0
      k = 1
      pass =0
      grp=c(1)
      data= cbind(data,grp)
      dlim <- bsup
      #running spatialpatches (WOILLEZ et al., 2009)
      Pch <- spatialpatches2(X , Y , Z , w = rep(1 , length(Y)) , Lim.D = dlim)
      grp1 <- Pch$n
      names <- c(round(dlim,2))
      historique <- cbind(historique,grp1)

      data$grp=grp

      if (!(length(unique(data$grp)) <= 1)) {
        showNotification("The maximal value of dlim is not enough to create one patch at the top of the tree.
                         The groups may be merged too early. Try higher bsup.",type = "message", duration = 10)
      }

      while ( dlim >= binf) {

        #update progress bar
        Sys.sleep(0.1)
        setTxtProgressBar(pb, k)
        progress2$set(value = k)

        #if k have not been increment
        if(k == pre_k){
          dlim <- nextdlim
        }else{
          dlim = bsup - k * pas
          #save value of k for this loop. need for chack next loop time
          pre_k <- k
        }

        if (round(dlim,2) > 0) {

          nb_grp=length(unique(data$grp))
          if(nb_grp>1){
            #More than one patch so we separate the data
            grp_tmp <- c()

            for (a in 1:nb_grp){
              # a take the different value corresponding to  the numero of cluster

              # eval permit to run the text into (in order to run, in the loop, data1 and data2 and not dataa)
              # parse permit to pass text to expression in eval(...)
              # Separtion of data into sub dataset (data1, data2, ...) order by the cluster vector Pch$n
              eval(parse(text=paste0("data",a,"=data[data$grp==",a,",]")))

              # Run spatialpatche for each of the new dataset (data1, data2, ...)
              # if number of element i is sup to 2
              if(eval(parse(text=paste0("length(data",a,"[,1])>3")))){
                eval(parse(text=paste0("Pch",a," <- spatialpatches2(data",a,"[,1],data",a,"[,2],data",a,"[,3],w = rep(1 , length(data",a,"[,2])),Lim.D = dlim)")))
              }else{
                eval(parse(text=paste0("Pch",a,"<-NULL")))
                eval(parse(text=paste0("Pch",a,"$n<-rep(1,length(data",a,"[,1]))")))
              }

              #recuperation of the cluster vector for each sub dataset (data1, data2, ...)
              eval(parse(text=paste0("data",a,"$grp=Pch",a,"$n")))
              #On ajoute a en dizaine au nombre pour ne pas generer de doublons entre les differents a
              eval(parse(text=paste0("data",a,"$grp=as.numeric(paste(",a,",data",a,"$grp,sep=''))")))

              #Si 3 groupes.
              if( eval(parse(text=paste0("length(unique(data",a,"$grp)) == 3"))) ){
                eval(parse(text=paste0("XX <- data",a,"[,1]")))
                eval(parse(text=paste0("YY <- data",a,"[,2]")))
                eval(parse(text=paste0("ZZ <- data",a,"[,3]")))

                eval(parse(text=paste0("WW <- rep(1 , length(data",a,"[,2]))")))
                eval(parse(text=paste0("xg <- tapply(WW*ZZ*XX,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                eval(parse(text=paste0("yg <- tapply(WW*ZZ*YY,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                diff1_2 <- abs(xg[1]-xg[2])
                diff1_3 <- abs(xg[1]-xg[3])
                diff2_3 <- abs(xg[2]-xg[3])
                if(diff2_3 > diff1_2 && diff2_3 > diff1_3){
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"2,",a,"2,",a,"1)")))
                }else{
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"1,",a,"1,",a,"2)")))
                }
                nextdlim <- dlim - (pas/3)
                #pass take value 1 in order to re rsun one time the loop witjhout change k and replace dlim by nextdlim.
                #This nextdlim permit to separate in two times (one dlim and one nextdlim near to dlim) the 3 patchs.
                pass <- 1
              }
              #Si plus de 3 groupes.
              if( eval(parse(text=paste0("length(unique(data",a,"$grp)) >= 4"))) ){
                eval(parse(text=paste0("XX <- data",a,"[,1]")))
                eval(parse(text=paste0("YY <- data",a,"[,2]")))
                eval(parse(text=paste0("ZZ <- data",a,"[,3]")))

                eval(parse(text=paste0("WW <- rep(1 , length(data",a,"[,2]))")))
                eval(parse(text=paste0("xg <- tapply(WW*ZZ*XX,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                eval(parse(text=paste0("yg <- tapply(WW*ZZ*YY,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
                diff1_2 <- abs(xg[1]-xg[2])
                diff1_3 <- abs(xg[1]-xg[3])
                diff2_3 <- abs(xg[2]-xg[3])
                if(diff2_3 > diff1_2 && diff2_3 > diff1_3){
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"2,",a,"2,",a,"1)")))
                }else{
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"1,",a,"1,",a,"2)")))
                }
              }
              #fusion of the cluster vectors
              eval(parse(text=paste0( "grp_tmp <- rbind(grp_tmp,data",a,")" )))
            }

            grp_tmp <- grp_tmp[order(grp_tmp$id,decreasing=F), ]

            #change the cluster names (11, 21, ...) into new (1, 2, ...) more simple
            namegrp<-c()
            namegrp<-as.factor(grp_tmp$grp)
            levels(namegrp) <- c(1:length(levels(namegrp)))
            data$grp<-as.numeric(namegrp)

          }else{
            # only one patch in all so we continu on all the data
            Pch <- spatialpatches2(X , Y , Z , w = rep(1 , length(Y)) , Lim.D = dlim)
            grp <- Pch$n
            data$grp=grp
          }

          if(round(dlim,1) > 0){
            j=length(historique[1,])
            #compare last and new compo
            v <- data$grp==historique[,j]
            somme <- sum(v %in% FALSE)
            if(somme != 0){
              #if data$grp is different than the last compo hist
              #add the dlim to name and the hist
              names <- c(names , round(dlim,2))
              historique <- cbind(historique , data$grp)
            }else{
              if(round(dlim,2) < names[length(names)]){
                #IF no difference : replace the last dlim name by the new
                names[length(names)] <- round(dlim,2)
              }
            }
          }
        }
        if(pass == 1){
          #if pass egal to 1 don't increment K and re run one time the while loop. pass egal 0 for next time of loop
          pass <- 0
        }else{
          k = k + 1
        }
      }#end while

      dlim <- as.numeric(colnames(historique)[length(colnames(historique))])

      historique <- cbind(grp1,historique)
      names <- c(round(bsup,2),names)
      historique <- as.matrix(historique)
      colnames(historique) <- names
      res$dlim <- names


      eval_class <- "test"
      output$log <- renderPrint({
        eval_class
      })

      #silhouette
      Average_Sil_Width <- c()
      for(i in 1:length(historique[1,])){
        cluster <- historique[,i]
        if(length(unique(cluster))>1){
          sil <- silhouette(cluster,DiMatrix)
          avg <- mean(sil[(length(cluster)*2):(length(cluster)*3)])
          Average_Sil_Width <- c(Average_Sil_Width,avg)
        }
      }
      number <- apply(historique,2,unique)
      number <- lapply(number,length)

      Number_of_groups <- as.vector(unlist(number))[number>1]

      Average_Sil_Width <- rbind(Number_of_groups,Average_Sil_Width)
      values[["sil_width"]] <- Average_Sil_Width

      historique <- cbind(id,historique)
      hist<-c()

      cpy_historique <- historique
      cpy_historique<-cbind(label,cpy_historique)

      #On trie l'historique par la colonne du dlim minimal
      historique<-historique[order(historique[,length(historique[1,])],decreasing=F), ]
      cpy_historique<-cpy_historique[order(as.numeric(cpy_historique[,length(cpy_historique[1,])]),decreasing=F), ]


      label_ini_grp <- c()
      label_tmp <- c()
      label_avant <- c()
      label_grp <- c()

      if(extenddend){

        for(b in 2:(length(cpy_historique[,1]))){
          c=b-1
          if(!(FALSE %in% (historique[b,2:length(historique[1,])]==historique[c,2:length(historique[1,])]))){
            #Doublon
            label_tmp <- as.character(cpy_historique[b,1])
            label_avant <- as.character(cpy_historique[c,1])
            if(is.null(label_grp)){
              label_grp <- c(label_avant,label_tmp)
            }else{
              label_grp <- c(label_grp,label_avant,label_tmp)
            }
          }else{
            #Pas doublon
            hist<-rbind(hist,historique[c,])
            label_grp <- c(label_grp,as.character(cpy_historique[c,1]))
            label_grp <- paste(unique(label_grp), collapse = ".")
            label_ini_grp<-c(label_ini_grp,label_grp)
            label_grp <- c()
          }
        }

        #add the last elt
        hist<-rbind(hist,historique[length(historique[,1]),])
        label_grp <- paste(unique(label_grp), collapse = ".")
        label_ini_grp<-c(label_ini_grp,label_grp)

      }else{
        #----------------
        for(b in 2:(length(cpy_historique[,1]))){
          c=b-1
          label_grp <- c(label_grp,as.character(cpy_historique[c,1]))
          label_ini_grp<-c(label_ini_grp,label_grp)
          label_grp <- c()
        }
        hist <- historique
        #add last elt
        label_grp <- unique(as.character(cpy_historique[length(historique[,1]),1]))
        label_ini_grp <- c(label_ini_grp,label_grp)
        #----------------
      }


      #We cut the label colunms (Not use here because we work on the entire cluster the label and id of
      # each members of one cluster are still save in data)
      hist<-as.matrix(hist)
      hist<-hist[,2:length(historique[1,])]

      ecart=ecartmatrice(hist)
      values[["hist"]] <- hist
      #if un seul groupe pas de div gerer ce cas

      ##Les segments
      #creation colonne x
      x <- c()
      for (b in 1:length(hist[,1])) {
        x <- c(x , b)
      }
      hist<-as.matrix(hist)
      #creation colonne y
      y <- rep(binf , length(hist[,1]))

      #creation d'une matrice ou on stockera les coordonnees pour les segments du drendrogramme
      coord <- cbind(x , y)

      #initialisation des segments
      segment <- c()

      #boucle se repetant (nombre de grp initiaux -1) fois 3 pour ecrire les segments
      #Pour 6 grps il faut (6-1)*3=15 segments. n grp il faut (n-1)*3 segments.
      for (a in 1:(length(hist[,1])-1)) {

        #on recherche min (premi?re fusion)
        mini <- min(ecart)
        #On recherche les indices
        index_vect <- which.min(ecart)
        index <- arrayInd(index_vect,c(length(ecart[,1]),length(ecart[,1])))
        grp1 <- index[1]
        grp2 <- index[2]

        #on recupere dlim dans l'objet res$dlim qui contient tous les dlim utilises. length(..)-mini pour
        # partir de la fin et remonter. mini est le nombre de dlim a passer vers la gauche.

        if(length(res$dlim)-mini!=0){
          dlim_tmp <- res$dlim[length(res$dlim)-mini]

          #on construit 3 segments (deux qui monte et un qui fusionne branche horizontal)

          # segments verticaux
          # x = l'abscisse du tab coord pour grp correspondant  y = coord[grp_,2]    xend = x    yend = dlim_tmp
          ######################################################################################################
          segment <- rbind(segment , c(coord[grp1,1] ,     coord[grp1,2] ,  coord[grp1,1] ,         dlim_tmp))
          segment <- rbind(segment , c(coord[grp2,1] ,     coord[grp2,2] ,  coord[grp2,1] ,         dlim_tmp))

          # segment horizontal
          ##################     x = coord[grp1,1]     y = dlim_tmp     xend = coord[grp2,1]    yend = dlim_tmp
          ######################################################################################################
          segment <- rbind(segment,c(coord[grp1,1] ,       dlim_tmp ,       coord[grp2,1] ,         dlim_tmp))


          ## on sauvegarde la hauteur (y / dlim) a laquelle les grps sont rendus
          #MAJ valeur de y (dlim) grp1
          coord[grp1,2] <- dlim_tmp
          #MAJ ligne grp1, valeur en X du nouveau grp (fusion) moyenne des deux
          coord[grp1,1] <- (coord[grp1,1]+coord[grp2,1])/2
          #on marque la ligne du grp2 en mettant x=0, pour ne plus la prendre en compte dans prochaine
          # fusion puisque ce grp a fusionne avec grp1
          coord[grp2,1] <- 0
          #on elimine la ligne/colonne du grp2 dans les valeurs (historique grp)
          ecart[,grp2] <- Inf
          ecart[grp2,] <- Inf
        }
      }
      groupes=1
      dlim=1
      xend=1
      yend=1
      #on rajoute un dernier segment pour la racine de l'arbre
      segment <- rbind(segment , c(max(coord[,1]) , max(coord[,2]) , max(coord[,1]) , (max(res$dlim)+pas)))
      segment <- data.frame(segment)
      #on prepare tableau segment
      names(segment) <- c("groupes" , "dlim" , "xend" , "yend")
      row.names(segment) <- NULL


      #on creer un tableau pour les noms (implant au bout des branches y=0)
      label <- matrix(ncol = 3 , nrow = length(hist[,1]))
      for (a in 1:length(hist[,1])) {
        label[a,1] <- a
        label[a,2] <- 0
        label[a,3] <- label_ini_grp[a]
      }
      label <- data.frame(label)

      values[["label"]] <- label
      values[["dlim"]]<-res$dlim

      #dendrogramme affiche avec indication pour le clic de cut.
      plot.dend$main <- ggplot(segment) +
        geom_segment(data = segment,aes(x = groupes , y = dlim , xend = xend , yend = yend )) +
        ylim(0, max(res$dlim) + pas)
      #Title subtitle caption
      plot.dend$main <- plot.dend$main + labs(title = "Hierarchical Clustering" , subtitle = "Spatialpatches")
      #axes x
      plot.dend$main <- plot.dend$main + scale_x_continuous(breaks = c())

      plot.dend$main <- plot.dend$main + theme(
        panel.grid = element_line(colour = "grey")
      )

      historique<-historique[order(historique[,1],decreasing=F), ]
      #WSS
      Sum_of_Square <- WSS(historique[,-1],data)
      Within_Sum_of_Square <- c()
      for (i in Sum_of_Square) {
        if(length(i)>1){
          Within_Sum_of_Square <- c(Within_Sum_of_Square,sum(i))
        }
      }

      WSS2 <- c()
      for (i in Sum_of_Square) {
        WSS2 <- c(WSS2,sum(i))
      }
      values[["WSS2"]] <- WSS2

      Within_Sum_of_Square <- rbind(Number_of_groups,Within_Sum_of_Square)
      values[["WSS"]] <- Within_Sum_of_Square

      if(multi == "Univariate"){
        Moran <- IndiceMoran(historique[,-1],data)
        values[["Moran"]] <- Moran
      }


      values[["historique"]] <- historique
      values[["data"]] <- data

      output$distPlot <- renderPlot({
        values[["dendplot"]] <- plot.dend$main + plot.dend$add + plot.dend$add2
        plot.dend$main + plot.dend$add + plot.dend$add2
      })

      output$avgsilPlot <- renderPlot({
        #plot eval sil
        dataplot <- t(values[["sil_width"]])
        dataplot <- as.data.frame(dataplot)
        colnames(dataplot) <- c("number_of_groups","Average_Sil_Width")
        minisil <- min(dataplot$Average_Sil_Width)
        if(minisil > 0 ){
          minisil <- 0
        }else{
          minisil <- -1
        }
        plot <- ggplot(data=dataplot, aes(x=number_of_groups, y=Average_Sil_Width))
        plot <- plot + geom_line(linetype = 3)
        plot <- plot +   geom_point(aes(color=Average_Sil_Width), size=3) + scale_colour_gradientn(colours=c("brown1","chartreuse3"))
        plot <- plot +   scale_x_continuous(breaks = dataplot$number_of_groups,
                                            labels = dataplot$number_of_groups )
        plot <- plot +   labs(title = "Average Silhouette Width" , subtitle = "Partition evaluation")
        plot <- plot + ylim(minisil,1)
        plot <- plot + theme(
          legend.position="bottom",
          plot.background = element_rect(
            colour = "#222222",
            linewidth = 1
          )
        )
        values[["avesilplot"]] <- plot
        plot
      })

      output$WSSPlot <- renderPlot({
        #plot WSS
        dataplot <- t(values[["WSS"]])
        dataplot <- as.data.frame(dataplot)
        colnames(dataplot) <- c("number_of_groups","Within_Sum_of_Square")
        maxwws <- max(dataplot$Within_Sum_of_Square)
        plot <- ggplot(data=dataplot, aes(x=number_of_groups, y=Within_Sum_of_Square))
        plot <- plot + geom_line(linetype = 3)
        plot <- plot + geom_point(aes(color=Within_Sum_of_Square), size=3) + scale_colour_gradientn(colours=c("chartreuse3","brown1"))
        plot <- plot + scale_x_continuous(breaks = dataplot$number_of_groups ,
                                          labels = dataplot$number_of_groups )
        plot <- plot + labs(title = "Within Sum of Square" , subtitle = "Partition evaluation")
        plot <- plot + theme(
          legend.position="bottom",
          plot.background = element_rect(
            colour = "#222222",
            linewidth = 1
          )
        )
        values[["WSSplot"]] <- plot
        plot
      })
      removeModal()
      showModal(cuttreeModal())
      #TODO : add download log graph map and all data
    }else{
      removeModal()
    }

  })

  ##dendrogramme click
  output$click_info <- renderPrint({

    if(is.null(values[["index"]])){
      cat("Hierarchical Divise Clustering")
    }else{
      multi <- values[["multi2"]]
      cat(multi, " ")
      nb_var <- values[["nb_var"]]
      cat(nb_var)

      size <- c()
      data <- values[["data"]]
      if(multi=="Univariate"){
        Z <- values[["Size"]]
      }else{
        Z <-values[["mdata"]]
      }
      cluster <- values[["historique"]][,values[["index"]]+1]
      nb <- length(unique(cluster))
      tot <- length(cluster)
      X_mean <- tapply(data[,1],cluster,mean)
      X_sd <- tapply(data[,1],cluster,sd)
      Y_mean <- tapply(data[,2],cluster,mean)
      Y_sd <- tapply(data[,2],cluster,sd)
      cat("The selected partition contains",nb,"clusters among",tot,"observations.\ncluster:\t")
      for (k in 1:nb){
        size <- c(size,length(cluster[cluster == k]))
        cat(k,"\t")
      }
      cat("\nSize:\t\t")
      for(y in 1:nb){
        cat(size[y],"\t")
      }
      cat("\nX_mean:\t\t")
      for(y in 1:nb){
        cat(round(X_mean[y],2),"\t")
      }
      cat("\nX_sd:\t\t")
      for(y in 1:nb){
        cat(round(X_sd[y],2),"\t")
      }
      cat("\nY_mean:\t\t")
      for(y in 1:nb){
        cat(round(Y_mean[y],2),"\t")
      }
      cat("\nY_sd:\t\t")
      for(y in 1:nb){
        cat(round(Y_sd[y],2),"\t")
      }
      if(multi == "Univariate"){
        cat("\nvar_mean:\t")
        var_mean <- tapply(Z,cluster,mean)
        for(y in 1:nb){
          cat(round(var_mean[y],2),"\t")
        }
        cat("\nvar_sd:\t\t")
        var_sd <- tapply(Z,cluster,sd)
        for(y in 1:nb){
          cat(round(var_sd[y],2),"\t")
        }
      }else{
        for(i in 1:nb_var){
          var_mean <- tapply(Z[,i],cluster,mean)
          cat("\nvar",i,"_mean:\t",sep="")
          for(y in 1:nb){
            cat(round(var_mean[y],2),"\t")
          }
        }
      }
    }
  })

  output$click_info2 <- renderPrint({

    multi <- values[["multi2"]]

    if(is.null(values[["index"]])){
      cat("Summary")
    }else{
      cat("> summary()")
      cat("\n  -----------------------------------\n  dlim\t  n\tWSS\tsil\tMoran\n  -----------------------------------\n")
      j <- 1
      for (i in 1:length(values[["dlim"]])){
        dlim <- values[["dlim"]][i]
        cluster <- values[["historique"]][,i+1]
        number_grp <- length(unique(cluster))
        if(number_grp == 1){
          sil <- NA
        }else{
          sil <- values[["sil_width"]][2,j]
          j <- j+1
        }
        WSS <- values[["WSS2"]][i]
        if(multi == "Univariate"){
          Moran <- mean(na.omit(unlist(values[["Moran"]][[i]][1,])))
        }
        cat(" ",dlim,"\t")
        cat(" ",number_grp,"\t")
        cat(round(WSS,2),"\t")
        cat(round(sil,2),"\t")
        if(multi == "Univariate"){
          cat(round(Moran,2),"\t")
        }
        cat("\n")
      }
      cat("  -----------------------------------\n")
    }
  })



  observeEvent(input$Plabel,{
    if(input$Plabel){
      label <- values[["label"]]
      if(!is.null(values[["index"]])){
        type <- values[["hist"]][,values[["index"]]]
      }else {
        type <- rep(1,length(values[["hist"]][,1]))
      }
      couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
                   "#222222", "darkmagenta", "mediumpurple4",
                   "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
                   "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
                   "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
      pal <- colorFactor(couleur,as.factor(type))
      col <- pal(as.factor(type))
      if(!is.null(plot.dend$add)){
        plot.dend$add2 <- geom_text(data = label, aes(x = as.numeric(paste(label[,1])) ,
                                                      y = as.numeric(paste(label[,2])) ,
                                                      label = as.character(label[,3])) ,
                                    fontface = "bold" , nudge_x = -0.12 , colour = col ,
                                    hjust = 0 , vjust = 0.25 , size = 4 , angle = 90)
      }else{
        plot.dend$add2 <- geom_text(data = label, aes(x = as.numeric(paste(label[,1])) ,
                                                      y = as.numeric(paste(label[,2])) ,
                                                      label = as.character(label[,3])) ,
                                    fontface = "bold" , nudge_x = -0.12 , colour = "royalBlue4" ,
                                    hjust = 0 , vjust = 0.25 , size = 4 , angle = 90)
      }
    }else{
      plot.dend$add2 <- NULL
    }

  })


  observeEvent(input$plot_click,{
    diff <- values[["dlim"]]-input$plot_click$y
    #which.max (<=0) return index of the first value <= 0
    if((TRUE %in% (diff <= 0))){
      index <- which.max(diff <= 0)
    }else{
      index <- which.min(diff)
    }
    values[["cutdlim"]] <- values[["dlim"]][index]
    values[["index"]] <- index
    showNotification(paste("cutdlim : ",values[["cutdlim"]]),type = "message",duration = 10)

    plot.dend$add
    plot.dend$add2
    plot.dend$add <- geom_hline(yintercept = values[["cutdlim"]] + 0.02 , col = "dimgrey" , lty = 2 ,lwd = 0.5)
    if (input$Plabel) {
      if(!is.null(values[["index"]])){
        type <- values[["hist"]][,values[["index"]]]
      }else {
        type <- rep(1,length(values[["hist"]][,1]))
      }
      couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
                   "#222222", "darkmagenta", "mediumpurple4",
                   "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
                   "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
                   "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
      pal <- colorFactor(couleur,as.factor(type))
      col <- pal(as.factor(type))
      label <- values[["label"]]
      plot.dend$add2 <- geom_text(data = label, aes(x = as.numeric(paste(label[,1])) ,
                                                    y = as.numeric(paste(label[,2])) ,
                                                    label = as.character(label[,3])) ,
                                                   fontface = "bold" , nudge_x = -0.12 , colour = col ,
                                                   hjust = 0 , vjust = 0.25 , size = 4 , angle = 90)
    }else{
      plot.dend$add2 <- NULL
    }
  })

  ##leaflet map
  output$mymap <- renderLeaflet({
    #leaflet
    if(input$select == 3){
      lonlat <- values[["lonlat"]]
    }else{
      lonlat <- TRUE
    }
    if(lonlat){
      esri <- structure(c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                          "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                          "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                          "Esri.WorldGrayCanvas"),
                        .Names = c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                                   "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                                   "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                                   "Esri.WorldGrayCanvas"))
      m <- leaflet()
      for (provider in esri) {
        m <- m %>% addProviderTiles(provider, group = provider)
      }
      m <- m %>%
        addLayersControl(baseGroups = names(esri),
                         options = layersControlOptions(collapsed = TRUE),
                         position = "topright"
        ) %>%
        addMiniMap(tiles = esri[[1]], toggleDisplay = TRUE,minimized = TRUE,
                   position = "bottomright"
        ) %>%
        htmlwidgets::onRender("
                              function(el, x) {
                                var myMap = this;
                                myMap.on('baselayerchange',
                                function (e) {
                                  myMap.minimap.changeLayer(L.tileLayer.provider(e.name));
                                })
                              }"
        )
    }else{
      m <- leaflet()
    }
    dlmap$map <- m
  })

  observe({
    if(input$select == 3){
      lonlat <- values[["lonlat"]]
    }else{
      lonlat <- TRUE
    }
    multi <- values[["multi2"]]
    couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
                 "#222222", "darkmagenta", "mediumpurple4",
                 "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
                 "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
                 "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
    if(is.null(values[["data"]])){
      data <- datarcheo
    }else{
      data <- values[["data"]]
    }
    if(!is.null(values[["index"]])){
      type <- values[["historique"]][,values[["index"]]+1]
    }else {
      type <- rep(1,240)
    }
    Size <- values[["Size"]]
    if(!is.null(Size)){
      sign = FALSE
      if(min(Size)<0){sign = TRUE}
      C <- values[["C"]]
      data[,3] <- as.numeric(Size)

      if(multi == "Univariate"){
        Size <- 10*(0.1+(abs(data[,3])/max(abs(data[,3]))))+5
      }

      pal <- colorFactor(couleur,as.factor(type))
      col <- pal(as.factor(type))
      col2 <- data.frame(col,type)
      col2 <- col2[order(col2[,2],decreasing=F), ]
      values[["col"]] <- as.character(col2[,1])
      leafletProxy("mymap", data = data) %>%
        clearMarkers() %>% clearControls
        if(input$Plabel){
          leafletProxy("mymap", data = data) %>% addCircleMarkers(lng = data[,1], lat =data[,2], popup = ~htmlEscape(values[["labelall"]]), radius = Size, fillColor =  col,
                                      fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=C
          )
        }else{
          leafletProxy("mymap", data = data) %>% addCircleMarkers(lng = data[,1], lat = data[,2], radius = Size, fillColor = col,
                                      fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=C
          )
        }
        leafletProxy("mymap", data = data) %>%
        fitBounds(lat1 = max(data[,2]), lng1 = max(data[,1]), lat2 = min(data[,2]), lng2 = min(data[,1])
        ) %>%
        addLegend("bottomleft", pal = pal, values = as.factor(type),
                  title = "Cluster",
                  opacity = 1
        )

      if(multi == "Univariate"){
        leafletProxy("mymap", data = data) %>%
          addLegendCustom(colors = c("black", "black", "black"),
                          labels = c(round(min(abs(data[,3])),2),
                                     round(min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2  ,2),
                                     round(max(abs(data[,3])),2)),
                          sizes = c(10*(0.1+( min(abs(data[,3])) /max(abs(data[,3]))))+5,
                                    10*(0.1+((min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2) /max(abs(data[,3]))))+5,
                                    10*(0.1+( max(abs(data[,3])) /max(abs(data[,3]))))+5),
                          title = "Size"
          )
      }
    }
    if(!is.null(Size)){
      if(sign){
        leafletProxy("mymap", data = data) %>%
          addLegendCustom(colors = c("white", "white"), labels = c("line: positive","dotted-line: negative"),
                          sizes = 1, title="Sign"
          )
      }
    }
    if(!is.null(Size)){
      if(lonlat){
        esri <- structure(c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                            "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                            "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                            "Esri.WorldGrayCanvas"),
                          .Names = c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                                     "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                                     "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                                     "Esri.WorldGrayCanvas"))
        m = leaflet()
        for (provider in esri) {
          m <- m %>% addProviderTiles(provider, group = provider)
        }
        m = m %>%
          addLayersControl(baseGroups = names(esri),
                           options = layersControlOptions(collapsed = TRUE),
                           position = "topright"
          ) %>%
          addMiniMap(tiles = esri[[1]], toggleDisplay = TRUE,minimized = TRUE,
                     position = "bottomright"
          ) %>%
          htmlwidgets::onRender("
                                function(el, x) {
                                var myMap = this;
                                myMap.on('baselayerchange',
                                function (e) {
                                myMap.minimap.changeLayer(L.tileLayer.provider(e.name));
                                })
                                }")
      }else{
        m <- leaflet()
      }

      m <- m %>%
      addCircleMarkers(lng = data[,1], lat = data[,2], radius = Size, fillColor =  col,
                       fill = T, fillOpacity = 0.8, opacity=1, stroke = T,weight = 1.5, color="black", dashArray=C
      ) %>%
      fitBounds(lat1 = max(data[,2]), lng1 = max(data[,1]), lat2 = min(data[,2]), lng2 = min(data[,1])
      ) %>%
      addLegend("bottomleft", pal = pal, values = as.factor(type),
                title = "Cluster",
                opacity = 1
      )
      if(multi == "Univariate"){
        m <- m %>%
          addLegendCustom(colors = c("black", "black", "black"),
                          labels = c(round(min(abs(data[,3])),2),
                                     round(min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2  ,2),
                                     round(max(abs(data[,3])),2)),
                          sizes = c(10*(0.1+( min(abs(data[,3])) /max(abs(data[,3]))))+5,
                                    10*(0.1+((min(abs(data[,3])) + (max(abs(data[,3]))-min(abs(data[,3])))/2) /max(abs(data[,3]))))+5,
                                    10*(0.1+( max(abs(data[,3])) /max(abs(data[,3]))))+5),
                          title = "Size"
          )
      }
      if(sign){
        m <- m %>%
          addLegendCustom(colors = c("white", "white"), labels = c("line: positive","dotted-line: negative"),
                          sizes = 1, title="Sign"
          )
      }
      dlmap$map <- m
    }

  })

  ##silhouette plot
  observe({
    if(!is.null(values[["index"]])){
      type <- values[["historique"]][,values[["index"]]+1]
      DiMatrix = values[["DiMatrix"]]
      if(length(as.numeric(type)) != dim(DiMatrix)[1]){
        output$silPlot <- renderPlot({},height = 1)
      }else{
        if(length(unique(type)) == 1){
          showNotification("You have to create more than one patch. The \"Silhouette plot\" can't be calculate for one cluster.", type="warning", duration=10)
          output$silPlot <- renderPlot({},height = 1)
        }else{
          output$silPlot <- renderPlot({
            cluster = type
            DiMatrix = values[["DiMatrix"]]
            sil <- silhouette(cluster,DiMatrix)
            plot(sil,col=values[["col"]])
            values[["silhouetteplot"]] <- sil
          },height = function(){2.5*length(type)})
        }
      }
    }else{
      output$silPlot <- renderPlot({},height = 1)
    }
  })



  #EXPORT BUTTON
  output$all.pdf <- downloadHandler(
    filename = "mapclust.pdf",
    content = function(file) {
      cairo_pdf(file,onefile=T)
      print(values[["dendplot"]])
      print(values[["avesilplot"]])
      print(values[["WSSplot"]])
      if(!is.null(values[["silhouetteplot"]])){
        plot(values[["silhouetteplot"]],col=values[["col"]])
      }
      dev.off()
    }
  )

  }

cerardat_app <- function() {
  appDir <- system.file("shiny-examples", "cerardat.app", package = "SPARTAAS")
  if (appDir == "") {
    stop("Could not find example directory. Try re-installing `SPARTAAS`.", call. = FALSE)
  }
  shiny::runApp(appDir, display.mode = "normal")
}

## Gestion des données manquantes

# TODO: function interface missMDA, Amelia, VIM
# missMDA: imputation simple ou multiple avec PCA avec NA
# Amelia: imputation multiple avec bootstrap
# VIM: Visualisation NA et combinaison de NA

estimateDensityMulti <- function(data){
  cat("Estimation of multidimensionnal density ..")
  #Smooth cross-validation bandwidth selector
  H.scv <- Hscv(data)
  fhat <- kde(data, H.scv, eval.points=data)
  res <- fhat$estimate*100
  cat(". Done")
  return(res)
}

IW <- function(x)
{
  #===============================================================================
  # SOMMES DES ECARTS AUX CARRE
  #===============================================================================
  sum(scale(x, scale = FALSE)^2)
}

spatialpatches2 <- function (x, y, z, w, Lim.D = 100, A.li = 0, modproj = NA, mlong = NA, mlat = NA)
{
  #===============================================================================
  # IDENTIFICATION OF SPATIAL PATCHES
  # AND COUNT OF CENTERS OF GRAVITY OF SPATIAL PATCHES
  #
  # Routine from EU program Fisboat, DG-Fish, STREP n? 502572
  # Authors : P.Petitgas (Ifremer), M.Woillez and J.Rivoirard (Mines-ParisTech)
  # Last update : 01 march 2008
  #
  # Arguments:
  # x	      The x-coordinate of the first population
  #	        Can be a vector, a matrix or an xyZ list (-,|,[]).
  # y	      The y-coordinates of the first population
  # z	      The regionalised variable of the first population.
  #	        If missing, the results of 'cgi' will concern the samples only.
  # w	      Optional. A weight or an area of influence of the first population.
  #	        Set to 1 if missing
  # Lim.D   Select minimum distance from sample to patch centre: to
  #         identify patches (units are those of coordinates)
  # A.li    Visualisation of gravity centres for those patches with
  #         abundance > A.li (in %)
  # modproj	Optional. Indicates the type of projection to perform.
  # mlong   mean longitude in DEGREES of the data set to be transformed
  # mlat    mean latitude in DEGREES of the data set to be transformed
  #	        See 'dg2nm' for precisions
  #
  #===============================================================================

  patch.id <- function(xx,yy,zz,ww,dli,ali,modproj,mlong,mlat)
  {
    ############################################################################
    # inputs : xx , yy , ww & zz ranked by decreasing order of zz
    #          dli is a distance limit from the patch gravity centre to define
    #          the patch border
    #
    # the function starts from the richest zz value and considers each sample
    # in decreasing order of zz. It tests whether the current value is spatially
    # close enough to the gravity centre of previously formed patches. if not a
    # new patch is considered. and so on until the last value. Patches of nul
    # values are returned with centres as NA and code 0 and their areas are
    # summed.
    #
    # outputs : the patch number for each sample,
    #           the gravity center for each patch,
    #           the percent abundance and area for each patch
    ############################################################################

    g <- c(1,rep(0,(length(xx)-1)))
    for (j in 1:(length(xx)-1)) {
      xg <- tapply(ww[1:j]*zz[1:j]*xx[1:j],g[1:j],sum,na.rm=T) /
        tapply(ww[1:j]*zz[1:j],g[1:j],sum,na.rm=T)
      yg <- tapply(ww[1:j]*zz[1:j]*yy[1:j],g[1:j],sum,na.rm=T) /
        tapply(ww[1:j]*zz[1:j],g[1:j],sum,na.rm=T)
      d <- sqrt(((xg-x1[j+1]))^2 + (yg-y1[j+1])^2)
      o <- order(d)
      if (d[o[1]] < dli){
        g[j+1] <- o[1]
      }else{
        g[j+1] <- max(g[1:j], na.rm=T)+1
      }
    }
    #second passage des obs pour affiner centres de gravite
    xg <- tapply(ww*zz*xx,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    yg <- tapply(ww*zz*yy,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    for(i in 1:length(xx)) {
      d <- sqrt(((xg-x1[i]))^2+(yg-y1[i])^2)
      o <- order(d)
      g[i] <- o[1]
    }

    xg <- tapply(ww*zz*xx,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    yg <- tapply(ww*zz*yy,g,sum,na.rm=TRUE)/tapply(ww*zz,g,sum,na.rm=TRUE)
    pb <- tapply(ww*zz,g,mean,na.rm=TRUE)
    inertiaTot <- sum(scale(zz, scale = FALSE)^2)
    InertiaWithin <- tapply(zz,g,IW)
    sel <- (tapply(ww*zz,g,sum,na.rm=TRUE)/sum(ww*zz,na.rm=TRUE))*100>=ali
    res <- max(unique(g)[sel],na.rm=TRUE)
    n <- sort(unique(g))
    selna <- is.na(xg)
    if (sum(selna) > 1) {
      n <- c(n[!selna],0)
      pb <- c(pb[!selna],0)
      pa <- c(pa[!selna],sum(pa[selna],na.rm=T))
      xg <- c(xg[!selna],NA)
      yg <- c(yg[!selna],NA)
      g[!is.element(g,n)] <- 0
    }
    return(list(n=g,mat=cbind(n=n,xg=arrondi(xg,2),yg=arrondi(yg,2),zg=arrondi(pb*100,2)),
                InertiaWithin=InertiaWithin,
                InertiaWithintot=sum(InertiaWithin),
                InertiaBetween=inertiaTot - sum(InertiaWithin),
                inertiaTot=inertiaTot,
                nsp=res))
  }

  # prepare data
  Xsta <- x
  Ysta <- y
  Zsta <- z
  Wsta <- w/sum(w,na.rm=TRUE)

  # exclude NA values
  SEL <- is.na(Xsta)+is.na(Ysta)+is.na(Zsta)+is.na(Wsta)
  Xsta <- Xsta[!SEL]
  Ysta <- Ysta[!SEL]
  Zsta <- Zsta[!SEL]
  Wsta <- Wsta[!SEL]

  # order data by decreasing order of value z
  zz1 <- sort(Zsta,decreasing=TRUE,index.return=TRUE)
  w1 <- Wsta[zz1$ix]
  x1 <- Xsta[zz1$ix]
  y1 <- Ysta[zz1$ix]
  z1 <- zz1$x

  # identify patches around high values
  SP <- patch.id(x1,y1,z1,w1,Lim.D,A.li,modproj=modproj,mlong=mlong,mlat=mlat)

  # graphical visualisation : patch identification
  #shows data points, circles for data values, number of patch &
  #crosses for patch gravity centres with abundance > A.li of total
  o <- sort(zz1$ix,index.return=TRUE)
  SP$n <- SP$n[o$ix]
  SP
}

WSS <- function(hist,data)
{
  #====================================================================================
  # WSS        Return the within cluster sum of square for each partition.
  #
  # hist       The historique of the partitions
  # data       The geographical data and third variable Z
  #====================================================================================

  out <- list()
  n <- length(hist[1,])
  Itot <- calctot(data)
  for (i in 1:n){
    WSS <- c()
    #recup compo of one partition i
    compo <- hist[,i]

    #calculer Within-cluster Sum of Square
    X <- data [,1]
    Y <- data [,2]
    Z <- data [,3]
    XG <- tapply(X * Z,compo,sum)/tapply(Z,compo,sum)
    YG <- tapply(Y * Z,compo,sum)/tapply(Z,compo,sum)
    nbgrp <- length(XG)
    for(j in 1:nbgrp){
      subX <- X[compo==j]
      subY <- Y[compo==j]
      subZ <- Z[compo==j]
      DX <- subX - XG[j]
      DY <- subY - YG[j]
      D <- sqrt(DX^2 + DY^2)
      INERT <- sum(subZ * (D^2))/sum(Z)
      WSS <- c(WSS,INERT)
    }
    out <- c(out,list(WSS))
  }
  out
}

calctot <- function(data)
{
  X <- data[,1]
  Y <- data[,2]
  Z <- data[,3]
  XG <- sum(X * Z)/sum(Z)
  YG <- sum(Y * Z)/sum(Z)
  DX <- X - XG
  DY <- Y - YG
  D <- sqrt(DX^2 + DY^2)
  INERT <- sum(Z * (D^2))/sum(Z)
  INERT
}

dist_m <- function(data)
{
  #===================================================================================
  # DISTANCE MATRIX   Weighted Eucidiean distance
  #
  # data      data.frame or matrix with 3 colunms (X, Y, Z)
  #===================================================================================
  square <-function(x){return(x*x)}
  Dmatrix <- matrix(ncol=length(data[,1]), nrow=length(data[,1]))
  for(i in 1:length(data[,1])){
    for (j in 1:length(data[,1])){
      #Weighted Eucidiean distance
      distance_pond <- sqrt( ( square(data[i,1]-data[j,1]) + square(data[i,2]-data[j,2]) ) *
                               square(data[i,3]-data[j,3])
      )
      Dmatrix[i,j] <- distance_pond
      Dmatrix[j,i] <- distance_pond
    }
  }
  return(Dmatrix)
}

transfo_z <- function(z)
{
  #====================================================================================
  # Transfo Z  Confidence interval on Empirical Cumulative Distribution Function (ECDF)
  #            We only can use z data positive like density, count or frequencies.
  #
  # z         vector of values
  #====================================================================================
  cat("Estimation of univariate density ..")
  dx <- density(z)
  xnew <- z # find density value on a new point
  dx.approx <- approx(dx$x,dx$y,xout=xnew)
  pi <- dx.approx$y*100 # densite estimee * 100 en chaque point du jeu de donnees
  cat(". Done")
  return(pi)
}

addLegendCustom <- function(map, colors, labels, sizes, opacity = 0.5, title)
{
  #===================================================================================
  # Custom legend for diffrent sizes of point in final map
  #
  # Arguments:
  # map       The leaflet map object
  # colors    The colors of the points
  # labels    The labels of the points
  # sizes     The sizes signification of the points in the map
  # opacity   The opacity of the background
  #
  # You can change the title and the position of the legend directly in the code. (last line of the function)
  #===================================================================================

  colorAdditions <- paste0(colors, ";border-radius: 50%;margin-top:10%; width:", 2*sizes, "px; height:", 2*sizes, "px")
  labelAdditions <- paste0("<div style='display: inline-block;height: ",
                           2*sizes, "px;margin-top: 4px;line-height: 0px;'>", labels, "</div>")

  return(addLegend(map, colors = colorAdditions, labels = labelAdditions, opacity = opacity,
                   title = title, position = "topright"))
}

mapclust <-
  function (coord, var, label = "Nolabel", iter = 20, Plabel = TRUE, lonlat = TRUE, positive_var = FALSE, n = NULL)
  {
    #===================================================================================
    # add two param: transfo_Z and k
    # DENDROGRAMME CUT AND MAP
    #
    # Author : A. Coulon (Universite de Nantes)
    # Last update : 17 April 2018
    #
    # Arguments:
    # x	      	      The vector with x-coordinate (latitude)
    # y	              The vector with y-coordinates (longitude)
    # var	      	    The regionalised variable of interest (vector if univariate analysis, data.frame or matrix max: 6dimension)
    # label(optional)	The names of the samples or an id (factor)
    # Plabel          logical parameter for activate or not the Print of labels on dendrogram
    #
    # Information:
    # dlim is the select minimum distance from sample to patch centre: to
    # identify patches (units are those of coordinates)
    #
    #===================================================================================

    cuttree2 <- function(classification, index, coul)
    {
      #===================================================================================
      # COUPE DENDROGRAMME
      #
      # Author : A. Coulon (Universite de Nantes)
      # Last update : 17 April 2018
      #
      # Arguments:
      # classification	  The object return by mapclust
      # index             The index of the cut dlim in the classification$dlim vector
      #
      #===================================================================================

      cuttree <- classification$dendrogram_ggplot
      cuttree <- cuttree + geom_hline(yintercept = classification$dlim[index] + 0.03 , col = "black" , lwd = 1)
      if(classification$Plabel){
        cuttree <- cuttree + suppressWarnings(theme(axis.text.x = element_text(color =  coul,
                                                          hjust = 1, vjust = 0.25, size = 9, angle = 90)))
      }else{
        cuttree <- cuttree + theme(axis.text.x = element_blank())
      }

      return(cuttree = cuttree)
    }

    index2 <- function(classification, historique, nb_grp){

        V_NB_GRP <- c()
        index <- NULL
        for(i in 1:length(classification$dlim)){
          V_NB_GRP <- c(V_NB_GRP,length(unique(historique[,i])))
          if(nb_grp == length(unique(historique[,i]))){
            cutdlim <- classification$dlim[i]
            index <- i
          }
        }

        if(is.null(index)){
          message("There is not this number of group in the partition. We cut to the closest. possible number:")
          print(unique(V_NB_GRP))
          diff <- V_NB_GRP - nb_grp
          index <- which.min(abs(diff))
          cutdlim <- classification$dlim[index]
        }
      return(index)
    }

    #mapclust-----------------------------------------------------
    if(iter <= 9){stop("You must run at least 10 iterations.")}
    if(iter > 101){message("You try to run a high number of iterations. This will take a long time.")}
    if(iter > 201){message("Only run if you are sure you need so many iterations.")}
    if(iter >= 1000){stop("The number of iterations is too high. (<1000)")}#rm?
    if(dim(coord)[2] != 2){stop("coord: must be a data.frame or a matrix with two columns!")}
    if(!is.numeric(coord[,1])){stop("coord: must be numeric!")}
    if(!is.numeric(coord[,2])){stop("coord: must be numeric!")}
    if(!is.null(n) && !is.numeric(n)){stop("n: must be numeric!")}
    if(!is.null(n) && n <= 1){stop("n: must be higher than 1!")}
    if(!is.logical(positive_var)){stop("positive_var: must be logical!")}
    if(dim(as.data.frame(var))[2] > 1){
      for(i in 1:dim(as.data.frame(var))[2]){
        if(!is.numeric(var[,i])){stop("var: must be numeric!")}
      }
    }else{
      if(!is.numeric(var)){stop("var: must be numeric!")}
    }
    if(!is.character(label) && !is.factor(label)){stop("label: must be character (factor)!")}
    X <- coord[,1]
    Y <- coord[,2]
    binf = 0
    bsup = max(dist(cbind(X,Y)))
    pas = (max(dist(cbind(X,Y)))/iter)
    #test param

    ref <- length(X)
    if(length(Y) != ref){stop("'x' and 'y' have not the same length")}

    if(dim(as.data.frame(var))[2] > 1){
      if(length(var[,1]) != ref){stop("'coord' and 'var' have not the same length")}
    }else{
      if(length(var) != ref){stop("'coord' and 'var' have not the same length")}
    }
    if(length(label) != ref && label[1] != "Nolabel"){stop("'x' and 'label' have not the same length")}

    save_label <- label

    if(dim(as.data.frame(var))[2] > 6){
      #if (rstudioapi::hasFun("showDialog")) {
      #  showDialog("warning","var can't have more than 6 dimension",url="")
      #}else{
      #  message("var can't have more than 6 dimension\n")
      #}
      stop("var can't have more than 6 dimension")
    }

    res <- NULL
    if(dim(as.data.frame(var))[2] > 1){
      #size uniform
      Size <- rep(1,length(var[,1]))
      multi = TRUE
    }else{
      Size <- var
      multi = FALSE
    }
    xg <- c()

    ctrl = positive_var

    if(!ctrl){
      if(dim(as.data.frame(var))[2] > 1){
        Z <- estimateDensityMulti(var)
      }else{
        Z <- transfo_z(var)
      }

    }else{

      if(dim(as.data.frame(var))[2] > 1){

          message("var is multidimensionnal. You must transform it!
                  var variables change forced!\n")

        Z <- estimateDensityMulti(var)
      }else{
        if(min(var)<0){

            message("There are negatives values in var. The data are not like a count, probability or density.
                    var change forced.\n")

          Z <- transfo_z(var)
        }else{
          Z <- var
          cat("Use var as density ... Done")
        }
      }
    }

    data <- data.frame(X,Y,Z)

    if (bsup <= binf) {stop("The upper bound (bsup) value must be higher than the lower bound (binf).")}

    slabel <- as.character(label)

    if (Plabel) {
      if ( label[1] == "Nolabel") {
        #Plabel <- FALSE
        label <- c(1:length(X))
      }else{
        #si l'utilisateur renseigne un nom
        label <- as.character(label)
      }
    }

    id <- c(1:length(X))
    data=cbind(data,id)
    data=as.data.frame(data)

    if( bsup > max(dist(data[,1:2])) ) {
      cat("The maximal value of dlim is: ",arrondi(max(dist(data[,1:2])),2))
      cat("the upper bound (bsup) have been changed in order to not run excessively high dlim.")
      bsup <- max(dist(data[,1:2]))
    }

    #remise a zero des objects
    names <- c()
    #initialisation matrice historique des groupes
    historique <- matrix(ncol=0, nrow=length(X))

    cat("\nInitialisation and calcul of the distance matrix ..")
    DiMatrix <- dist_m(data)
    cat(". Done\n")
    cat("Running Progress:\n")
    #creation barre de progression(pb)
    pb <- txtProgressBar( min = 0, max = arrondi((bsup - binf) / pas,0) , style = 3 )

    pre_k <- 0
    k <- 1
    pass <- 0
    grp <- c(1)
    data <- cbind(data,grp)
    dlim <- bsup
    #running spatialpatches (WOILLEZ et al., 2009)
    Pch <- spatialpatches2(X , Y , Z , w = rep(1 , length(Y)) , Lim.D = dlim)
    grp1 <- Pch$n

    names <- c(arrondi(dlim,2))
    historique <- cbind(historique,grp1)

    data$grp=grp

    if (!(length(unique(data$grp)) <= 1)) {
      cat("\nThe maximal value of dlim is not enough to create one patch at the top of the tree.
          The groups may be merged too early. Try higher bsup.\n")
    }


    while ( dlim >= binf) {
      # update progress bar
      setTxtProgressBar(pb, k)

      #if k have not been increment
      if(k == pre_k){
        dlim <- nextdlim
      }else{
        dlim = bsup - k * pas
        #save value of k for this loop. need for check next loop time
        pre_k <- k
      }

      if (arrondi(dlim,2) > 0.01) {

        nb_grp=length(unique(data$grp))
        if(nb_grp>1){
          #More than one patch so we separate the data
          grp_tmp <- c()

          for (a in 1:nb_grp){
            # a take the different value corresponding to  the numero of cluster

            # eval permit to run the text into (in order to run, in the loop, data1 and data2 and not dataa)
            # parse permit to pass text to expression in eval(...)
            # Separtion of data into sub dataset (data1, data2, ...) order by the cluster vector Pch$n
            eval(parse(text=paste0("data",a,"=data[data$grp==",a,",]")))

            # Run spatialpatche for each of the new dataset (data1, data2, ...)
            # if number of element i is sup to 2
            if(eval(parse(text=paste0("length(data",a,"[,1]) > 1")))){
              eval(parse(text=paste0("Pch",a," <- spatialpatches2(data",a,"[,1],data",a,"[,2],data",a,"[,3],w = rep(1 , length(data",a,"[,2])),Lim.D = dlim)")))
            }else{
              eval(parse(text=paste0("Pch",a,"<-NULL")))
              eval(parse(text=paste0("Pch",a,"$n<-rep(1,length(data",a,"[,1]))")))
            }

            #recuperation of the cluster vector for each sub dataset (data1, data2, ...)
            eval(parse(text=paste0("data",a,"$grp=Pch",a,"$n")))
            #On ajoute a en dizaine au nombre pour ne pas generer de doublons entre les differents a
            eval(parse(text=paste0("data",a,"$grp=as.numeric(paste(",a,",data",a,"$grp,sep=''))")))

            #Si 3 groupes.
            if( eval(parse(text=paste0("length(unique(data",a,"$grp)) == 3"))) ){
              eval(parse(text=paste0("XX <- data",a,"[,1]")))
              eval(parse(text=paste0("YY <- data",a,"[,2]")))
              eval(parse(text=paste0("ZZ <- data",a,"[,3]")))

              eval(parse(text=paste0("WW <- rep(1 , length(data",a,"[,2]))")))
              eval(parse(text=paste0("xg <- tapply(WW*ZZ*XX,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
              eval(parse(text=paste0("yg <- tapply(WW*ZZ*YY,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
              diff1_2 <- abs(xg[1]-xg[2])
              diff1_3 <- abs(xg[1]-xg[3])
              diff2_3 <- abs(xg[2]-xg[3])
              if(diff2_3 > diff1_2 && diff1_2 > diff1_3){
                eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"2,",a,"2,",a,"1)")))
              }else{
                if(diff2_3 > diff1_2 && diff1_2 < diff1_3){
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"3,",a,"2,",a,"1)")))
                }else{
                  eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"1,",a,"1,",a,"2)")))
                }
              }
              nextdlim <- dlim - (pas/3)
              #pass take value 1 in order to re run one time the loop witjhout change k and replace dlim by nextdlim.
              #This nextdlim permit to separate in two times (one dlim and one nextdlim near to dlim) the 3 patchs.
              pass <- 1
            }
            #Si plus de 3 groupes.
            if( eval(parse(text=paste0("length(unique(data",a,"$grp)) >= 4"))) ){
              eval(parse(text=paste0("XX <- data",a,"[,1]")))
              eval(parse(text=paste0("YY <- data",a,"[,2]")))
              eval(parse(text=paste0("ZZ <- data",a,"[,3]")))

              eval(parse(text=paste0("WW <- rep(1 , length(data",a,"[,2]))")))
              eval(parse(text=paste0("xg <- tapply(WW*ZZ*XX,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
              eval(parse(text=paste0("yg <- tapply(WW*ZZ*YY,data",a,"$grp,sum,na.rm=T)/tapply(WW*ZZ,data",a,"$grp,sum,na.rm=T)")))
              diff1_2 <- abs(xg[1]-xg[2])
              diff1_3 <- abs(xg[1]-xg[3])
              diff2_3 <- abs(xg[2]-xg[3])
              if(diff2_3 > diff1_2 && diff2_3 > diff1_3){
                eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"2,",a,"2,",a,"1)")))
              }else{
                eval(parse(text=paste0("data",a,"$grp <- ifelse(data",a,"$grp==",a,"1,",a,"1,",a,"2)")))
              }
            }
            #fusion of the cluster vectors
            eval(parse(text=paste0( "grp_tmp <- rbind(grp_tmp,data",a,")" )))
          }
          grp_tmp <- grp_tmp[order(grp_tmp$id,decreasing=F), ]

          #change the cluster names (11, 21, ...) into new (1, 2, ...) more simple
          namegrp <- c()
          namegrp <- as.factor(grp_tmp$grp)
          levels(namegrp) <- c(1:length(levels(namegrp)))
          data$grp <- as.numeric(namegrp)
        }else{
          # only one patch in all so we keep going with all the data
          Pch <- spatialpatches2(X , Y , Z , w = rep(1 , length(Y)) , Lim.D = dlim)
          grp <- Pch$n
          data$grp <- grp

          if( length(unique(data$grp)) >= 3 ){
            # if 3 or more groups
            XX <- data[,1]
            YY <- data[,2]
            ZZ <- data[,3]

            WW <- rep(1 , length(data[,2]))
            xg <- tapply(WW*ZZ*XX,data$grp,sum,na.rm=T)/tapply(WW*ZZ,data$grp,sum,na.rm=T)
            yg <- tapply(WW*ZZ*YY,data$grp,sum,na.rm=T)/tapply(WW*ZZ,data$grp,sum,na.rm=T)
            diff1_2 <- abs(xg[1]-xg[2])
            diff1_3 <- abs(xg[1]-xg[3])
            diff2_3 <- abs(xg[2]-xg[3])
            if(diff2_3 > diff1_2 && diff2_3 > diff1_3){
              data$grp <- ifelse(data$grp==2,2,1)
            }else{
              data$grp <- ifelse(data$grp==1,1,2)
            }
          }

        }

        if(arrondi(dlim,2) > 0){
          j <- length(historique[1,])
          #compare last and new compo
          v <- data$grp==historique[,j]
          somme <- sum(v %in% FALSE)
          if(somme != 0){
            #if data$grp is different than the last compo hist
            #add the dlim to name and the hist
            names <- c(names , arrondi(dlim,2))
            historique <- cbind(historique , data$grp)
          }else{
            if(arrondi(dlim,2) < names[length(names)]){
              #IF no difference : replace the last dlim name by the new
              names[length(names)] <- arrondi(dlim,2)
            }
          }
        }
      }
      if(pass == 1){
        #if pass egal to 1 don't increment K and re-run one time the while loop. pass egal 0 for next time of loop
        pass <- 0
      }else{
        k = k + 1
      }
    }#finwhile

    close(pb)
    dlim <- as.numeric(colnames(historique)[length(colnames(historique))])

    historique <- cbind(grp1,historique)
    names <- c(arrondi(bsup,2),names)
    historique <- as.matrix(historique)
    colnames(historique) <- names
    res$dlim <- names


    #silhouette
    Average_Sil_Width <- c()
    for(i in 1:length(historique[1,])){
      cluster <- historique[,i]
      if(length(unique(cluster))>1){
        sil <- silhouette(cluster,DiMatrix)
        avg <- mean(sil[,3])
        Average_Sil_Width <- c(Average_Sil_Width,avg)
      }
    }
    number <- apply(historique,2,unique)
    number <- lapply(number,length)

    Number_of_groups <- as.vector(unlist(number))[number>1]
    Average_Sil_Width <- rbind(Number_of_groups,Average_Sil_Width)

    historique <- cbind(id,historique)
    hist<-c()

    cpy_historique <- historique
    cpy_historique<-cbind(label,cpy_historique)

    #On trie l'historique par la colonne du dlim minimal
    historique <- historique[order(historique[,length(historique[1,])],decreasing=FALSE), ]
    cpy_historique <- cpy_historique[order(as.numeric(cpy_historique[,length(cpy_historique[1,])]),decreasing=F), ]


    label_ini_grp <- c()
    label_tmp <- c()
    label_avant <- c()
    label_grp <- c()

    #for(b in 2:(length(cpy_historique[,1]))){
    #  c=b-1
    #  if(!(FALSE %in% (historique[b,2:length(historique[1,])]==historique[c,2:length(historique[1,])]))){
    #    #Doublon
    #    label_tmp <- as.character(cpy_historique[b,1])
    #    label_avant <- as.character(cpy_historique[c,1])
    #    if(is.null(label_grp)){
    #      label_grp <- c(label_avant,label_tmp)
    #    }else{
    #      label_grp <- c(label_grp,label_avant,label_tmp)
    #    }
    #  }else{
    #    #Pas doublon
    #    hist<-rbind(hist,historique[c,])
    #    label_grp <- c(label_grp,as.character(cpy_historique[c,1]))
    #    label_grp <- paste(unique(label_grp), collapse = ".")
    #    label_ini_grp<-c(label_ini_grp,label_grp)
    #    label_grp <- c()
    #  }
    #}

    # add the last elt
    #hist <- rbind(hist,historique[length(historique[,1]),])
    #label_grp <- paste(unique(as.character(cpy_historique[length(historique[,1]),1])), collapse = ".")
    #label_ini_grp <- c(label_ini_grp,label_grp)
    #----------------
    for(b in 2:(length(cpy_historique[,1]))){
      c=b-1
      label_grp <- c(label_grp,as.character(cpy_historique[c,1]))
      label_ini_grp<-c(label_ini_grp,label_grp)
      label_grp <- c()
    }
    hist <- historique
    # add last elt
    label_grp <- unique(as.character(cpy_historique[length(historique[,1]),1]))
    label_ini_grp <- c(label_ini_grp,label_grp)
    #----------------

    #We cut the label colunms (Not use here because we work on the entire cluster the label and id of
    #each members of one cluster are still save in data)
    hist <- as.matrix(hist)
    hist <- hist[,2:length(historique[1,])]


    ecart <- ecartmatrice(hist)
    #if un seul groupe pas de div gerer ce cas

    ##Les segments
    #creation colonne x
    xcoord <- c()
    for (b in 1:length(hist[,1])) {
      xcoord <- c(xcoord , b)
    }
    hist <- as.matrix(hist)
    #creation colonne y
    ycoord <- rep(binf, length(hist[,1]))

    #creation d'une matrice ou on stockera les coordonnees pour les segments du drendrogramme
    coord <- cbind(x=xcoord, y=ycoord)

    #initialisation des segments
    segment <- c()

    #------------------------
    tmp_id <- -seq(1,length(hist[,1]))
    dend <- list(order=rep(NA,length(hist[,1]))
                 ,order.lab=rep(NA,length(hist[,1]))
                 ,height=rep(NA,length(hist[,1])-1)
                 ,dc=NA
                 ,merge=matrix(NA,nrow=length(hist[,1])-1,ncol=2)
    )
    #------------------------

    #boucle se repetant (nombre de grp initiaux -1)
    #Pour 6 grps il faut (6-1)*3=15 segments. n grp il faut (n-1)*3 segments.
    for (a in 1:(length(hist[,1])-1)) {

      #On recherche min (premiere fusion)
      mini <- min(ecart)
      #On recherche les indices
      index_vect <- which.min(ecart)
      index <- arrayInd(index_vect,c(length(ecart[,1]),length(ecart[,1])))
      grp1 <- index[1]
      grp2 <- index[2]

      #------------------------
      dend$height[a] <- res$dlim[length(res$dlim)-mini]
      dend$merge[a,1] <- tmp_id[grp1]
      dend$merge[a,2] <- tmp_id[grp2]
      tmp_id[grp1] <- tmp_id[grp2] <- a

      #ecart[,grp2] <- Inf
      #ecart[grp2,] <- Inf
      #------------------------

      #On recupere dlim dans l'objet res$dlim qui contient tous les dlim utilises. length(...)-mini pour
      #partir de la fin et remonter. mini est le nombre de dlim passer vers la gauche.

      if( length(res$dlim)-mini != 0){
        dlim_tmp <- res$dlim[length(res$dlim)-mini]

        #On construit 3 segments (deux qui monte et un qui fusionne branche)

        # segments verticaux
        # x = l'abscisse du tab coord pour grp correspondant  y = coord[grp_,2]    xend = x    yend = dlim_tmp
        segment <- rbind(segment , c(coord[grp1,1] ,  coord[grp1,2] ,  coord[grp1,1] ,    dlim_tmp))
        segment <- rbind(segment , c(coord[grp2,1] ,  coord[grp2,2] ,  coord[grp2,1] ,    dlim_tmp))

        # segment horizontal
        #                        x = coord[grp1,1]  y = dlim_tmp   xend = coord[grp2,1]  yend = dlim_tmp
        segment <- rbind(segment,  c(coord[grp1,1] ,    dlim_tmp ,        coord[grp2,1] ,    dlim_tmp))

        ## On sauvegarde la hauteur (y / dlim) a laquelle les grps sont rendus
        #MAJ valeur de y (dlim) grp1
        coord[grp1,2] <- dlim_tmp
        #MAJ ligne grp1, valeur en X du nouveau grp (fusion) moyenne des deux
        coord[grp1,1] <- (coord[grp1,1]+coord[grp2,1])/2
        #On marque la ligne du grp2 en mettant x=0, pour ne plus la prendre en compte dans prochaine
        # fusion puisque ce grp a fusionne avec grp1
        coord[grp2,1] <- 0
        #On elimine la ligne/colonne du grp2 dans les valeurs (historique grp)
        ecart[,grp2] <- Inf
        ecart[grp2,] <- Inf
      }
    }

    #------------------------
    dend$order <- seq(1,length(hist[,1]))
    dend$order.lab <- label_ini_grp
    dend$dc <- 0

    class(dend) <- c("diana","twins")
    #------------------------

    groupes=1
    dlim=1
    xend=1
    yend=1
    #On rajoute un dernier segment pour la racine de l'arbre
    segment <- rbind(segment , c(max(coord[,1]) , max(coord[,2]) , max(coord[,1]) , (max(res$dlim)+pas)))
    segment <- data.frame(segment)
    #On prepare tableau segment
    names(segment) <- c("groupes" , "dlim" , "xend" , "yend")
    row.names(segment) <- NULL


    #Plabel=F
    #On creer un tableau pour les noms (implant au bout des branches y = 0)
    label <- matrix(ncol = 3, nrow = length(hist[,1]))
    for (a in 1:length(hist[,1])) {
      label[a,1] <- a
      label[a,2] <- 0
      label[a,3] <- label_ini_grp[a]
    }
    label <- data.frame(label)

    plot <- NULL
    plot1 <- NULL

    #On dessine les segments et on ajout les noms
    auto.haut <- max(res$dlim) + pas / 10
    tmp <- 1
    for (b in 2:length(hist[,1])) {
      tmp <- c(tmp , b)
    }

    #Dendrogram
    plot1 <- ggplot() +
      geom_segment(data = segment , aes(x = groupes, y = dlim, xend = xend, yend = yend)) +
      ylim(0 , max(res$dlim) + pas)
    if (Plabel) {
      plot1 <- plot1 + scale_x_continuous(breaks = tmp ,
                                          labels = as.character(label[,3]) )
      plot1 <- plot1 + theme(axis.text.x = element_text(face = "bold" , hjust = 1 , vjust = 0.25 , size = 9 ,
                                                        angle = 90))
    }
    plot1 <- plot1 + labs(title = "Hierarchical Clustering", subtitle = "Mapclust")

    plot1 <- plot1 + theme(
      plot.margin = margin(0.5, 1, 0.5, 1, "cm")
    )
    res$dendrogram_ggplot <- plot1

    groupes=1
    dlim=1
    xend=1
    yend=1
    #Dendrogramme affiche avec indication pour le clic de cut.
    plot <- ggplot() +
      geom_segment(data = segment, aes(x = groupes, y = dlim, xend = xend, yend = yend)) +
      ylim(0 , max(res$dlim) + pas)
    #Title subtitle caption
    plot <- plot + labs(title = "Hierarchical Clustering", subtitle = "Mapclust")
    #Axes x
    plot <- plot + scale_x_continuous(breaks = c())

    if (Plabel) {
      plot <- plot + geom_text(data = label, aes(x = as.numeric(paste(label[,1])),
                                                 y = as.numeric(paste(label[,2])),
                                                 label = as.character(label[,3])),
                               fontface = "bold", nudge_x = -0.12, colour = "gray32",
                               hjust = 0, vjust = 0.25, size = 3, angle = 90)
    }
    plot <- plot + theme(
      panel.grid = element_line(colour = "grey"),
      plot.margin = margin(0.5, 1, 0.5, 1, "cm")
    )
    #print(plot)
    minisil <- min(Average_Sil_Width)
    if(minisil > 0 ){
      minisil <- 0
    }else{
      minisil <- -1
    }
    #plot eval sil
    dataplot <- t(Average_Sil_Width)
    dataplot <- as.data.frame(dataplot)
    dataplot <- na.omit(dataplot)
    colnames(dataplot) <- c("Number_of_groups","Average_Sil_Width")
    p <- ggplot(dataplot,aes(x=Number_of_groups, y=Average_Sil_Width)) +
      geom_point(aes(color=Average_Sil_Width)) + scale_colour_gradientn(colours=c("brown1","chartreuse3")) +
      geom_line(linetype = 3) +
      scale_x_continuous(breaks = dataplot$Number_of_groups,
                         labels = dataplot$Number_of_groups) +
      ylim(minisil,1) +
      labs(title = "Average Silhouette Width", subtitle = "Partition evaluation") +
      theme(legend.position="none")

    historique<-historique[order(historique[,1],decreasing=F), ]
    Sum_of_Square <- WSS(historique[,-1],data)
    Itot <- calctot(data)
    if(dim(as.data.frame(var))[2] > 1){
      Moran <- "Impossible to calculate Moran index for multivariate data"
    }else{
      Moran <- IndiceMoran(historique[,-1],data)
    }

    wss <- c()

    for (i in Sum_of_Square) {
      if(length(i)>1){
        wss <- c(wss,sum(i))
      }
    }

    names(Sum_of_Square) <- res$dlim

    Within_Sum_of_Square <- rbind(Number_of_groups,wss)
    color_wss <- elbow_finder(Number_of_groups,wss)
    #plot eval WSS
    dataplot <- t(Within_Sum_of_Square)
    dataplot <- as.data.frame(dataplot)
    colnames(dataplot) <- c("Number_of_groups","Within_Sum_of_Square")
    q <- ggplot(dataplot,aes(x=Number_of_groups, y=Within_Sum_of_Square)) +
      geom_point(aes(color=color_wss)) + scale_colour_gradientn(colours=c("brown1","chartreuse3")) +
      geom_line(linetype = 3) +
      scale_x_continuous(breaks = dataplot$Number_of_groups ,
                         labels = dataplot$Number_of_groups ) +
      ylim(0,Itot) +
      labs(title = "Within Sum of Square" , subtitle = "Partition evaluation") +
      theme(legend.position="none")

    # Nouvelle page
    grid.newpage()
    # Creer la mise en page : nrow = 2, ncol = 2
    pushViewport(viewport(layout = grid.layout(2, 2)))
    # Une fonction pour definir une region dans la mise en page
    define_region <- function(row, col){
      viewport(layout.pos.row = row, layout.pos.col = col)
    }
    # Arranger les graphiques dans la fenetre
    print(plot, vp = define_region(1:2, 1))
    print(p, vp = define_region(1, 2))
    print(q, vp = define_region(2, 2))

    if(is.null(n)){
      #if (rstudioapi::hasFun("showQuestion")) {
      #  showDialog("Cut Tree", "You must click on the dendrogram (plot window) to select where the tree should be cut.
      #         You will not be able to execute anything else until you have selected a dlim.",url="")
      #}


      message("
            |=======================================================|
            | You must click on the dendrogram (plot window)        |
            | to select where the tree should be cut.               |
            | You will not be able to execute anything else until   |
            | you have selected a dlim.                             |
            |=======================================================|
            ")
      OK = FALSE
      while(OK == FALSE){
        #coupe arbre
        point <- data.frame(0,-10)
        loc <- as.numeric(grid.locator("npc"))
        # On adapte les coord (de la fenetre) a l axes y.
        point <- data.frame( loc[1], ( (loc[2]-0.1055195)/0.7516234  ) * ( max(res$dlim) + pas) )
        names(point) <- c("x","dlim")

        if (is.na.data.frame(point[1,2])){point <- data.frame(0,-10)}
        if (point[1,2] == -10){point[1,2] <- median(res$dlim)+0.1}

        #dlim max parmi ceux inferieur au dlim pointe par user
        diff <- res$dlim-point[1,2]
        if((TRUE %in% (diff <= 0))){
          #return index of the first value <= 0
          index <- which.max(diff <= 0)
        }else{
          index <- which.min(diff)
        }

        cutdlim <- res$dlim[index]
        nb <- length(unique(historique[,index+1]))

        if (nb <= 1){
          message("The number of patchs must be greater than 1. Try again")
        }else{
          OK = TRUE
        }

      }

      if (cutdlim <= min(res$dlim)) {
        cutdlim <- min(res$dlim)
      }
      cat("Select dlim ...",cutdlim)
    }else{
      index <- index2(res,historique,n-1)

      cutdlim <- res$dlim[index]
      nb <- length(unique(historique[,index+1]))

      if (cutdlim <= min(res$dlim)) {
        cutdlim <- min(res$dlim)
      }
      cat("Select dlim ...",cutdlim)
    }


    cat("\nCut the dendrogram ..")
    res$Plabel <- Plabel

    tmp<-data.frame(lon=X, lat=Y)
    if(!lonlat){
      tmp$lon <- (tmp$lon - mean(tmp$lon)) / max(tmp$lon, tmp$lat)
      tmp$lat <- (tmp$lat - mean(tmp$lat)) / max(tmp$lon, tmp$lat)
    }

    lineType <- ifelse(Size<0,"2, 5","1")

    #leaflet
    df <- data.frame(
      lat <- tmp$lat,
      lng <- tmp$lon,
      size <- 10*(0.1+(abs(Size)/max(abs(Size))))+5,
      color <- as.factor(historique[,index+1])
    )
    if(slabel[1] != "Nolabel"){
      df <- cbind(df,name = as.character(slabel))
    }
    if(multi){df$size <- rep(7,length(df$size))}
    df$category <- as.factor(historique[,index+1])

    # Make up some random levels. (TODO: Better example)
    couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
                 "#222222", "darkmagenta", "mediumpurple4",
                 "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
                 "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
                 "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
    #plot(1:25,1:25,pch =15,cex=3,col=couleur)
    factpal <- colorFactor(couleur, df$category)
    coul <- factpal(as.factor(hist[,index]))

    cut <- cuttree2(res,index,coul)


    if(lonlat){
      esri <- structure(c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                          "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                          "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                          "Esri.WorldGrayCanvas"),
                        .Names = c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                                   "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                                   "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                                   "Esri.WorldGrayCanvas"))
      m = leaflet(df)
      for (provider in esri) {
        m <- m %>% addProviderTiles(provider, group = provider)
      }
      if(slabel[1] != "Nolabel"){
        m <- m %>% addCircleMarkers(lng = ~lng, lat = ~lat, popup = ~htmlEscape(name), radius = ~size, fillColor =  ~factpal(category),
                                    fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=lineType
        )
      }else{
        m <- m %>% addCircleMarkers(lng = ~lng, lat = ~lat, radius = ~size, fillColor =  ~factpal(category),
                                    fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=lineType
        )
      }

      m <- m %>%
        addLegend("bottomleft", pal = factpal, values = as.factor(historique[,index+1]),
                  title = "Cluster",
                  opacity = 1
        ) %>%
        addMiniMap(toggleDisplay = TRUE,
                   position = "bottomright"
        ) %>%
        addLayersControl(baseGroups = names(esri),
                         options = layersControlOptions(collapsed = TRUE),
                         position = "topright"
        ) %>%
        htmlwidgets::onRender("
                      function(el, x) {
                        var myMap = this;
                        myMap.on('baselayerchange',
                        function (e) {
                          myMap.minimap.changeLayer(L.tileLayer.provider(e.name));
                        })
                      }"
        )
      if(!multi){
        m <- m %>%
          addLegendCustom(colors = c("black", "black", "black"),
                          labels = c(arrondi(min(abs(Size)),2),
                                     arrondi(min(abs(Size)) + (max(abs(Size))-min(abs(Size)))/2  ,2),
                                     arrondi(max(abs(Size)),2)),
                          sizes = c(10*(0.1+( min(abs(Size)) / max(abs(Size))))+5,
                                    10*(0.1+((min(abs(Size)) + (max(abs(Size))-min(abs(Size)))/2) /max(abs(Size))))+5,
                                    10*(0.1+( max(abs(Size)) / max(abs(Size))))+5),
                          title = "Size"
          )
      }
      if(min(Size)<0){
        m <- m %>%
          addLegendCustom(colors = c("white", "white"), labels = c("line: positive","dotted-line: negative"),
                          sizes = 1, title="Sign"
          )
      }
    }else{
      m <- leaflet(df)
      m <- m %>% addCircleMarkers(lng = ~lng, lat = ~lat, radius = ~size, fillColor =  ~factpal(category),
                                  fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=lineType
      ) %>%
        addLegend("bottomleft", pal = factpal, values = as.factor(historique[,index+1]),
                  title = "Cluster",
                  opacity = 1
        )
    }


    cluster <- as.numeric(historique[,index+1])

    sil <- silhouette(cluster,DiMatrix)
    coull <- factpal(sort(cluster))
    if(slabel[1] != "Nolabel"){
      row.names(sil) <- slabel
    }
    plot(sil,col = coull, nmax = 500, cex.names=.8, max.strlen = 80,
         main="Silhouette plot (x = cluster, dist = Weighted Eucidiean Distance)")
    silPlot <- recordPlot()
    plot.new()
    print(cut)
    cat(". Done")
    print(m)

    out <- structure(list(X=X,
                          Y=Y,
                          var=var,
                          label=save_label,
                          density=Z,
                          cluster=cluster,
                          fullhist=historique,
                          dlim=res$dlim,
                          historique=hist,
                          Plabel=Plabel,
                          InertiaWithin = Sum_of_Square,
                          Moran=Moran,
                          dendrogram=as.hclust(dend),
                          dendrogram_ggplot=res$dendrogram_ggplot,
                          DiMatrix=DiMatrix,
                          cuttree=cut,
                          map=m,
                          silhouette=silPlot,
                          silhouetteData=sil,
                          AveSilData=Average_Sil_Width[2,],
                          AveSilPlot=p,
                          WSSPlot=q,
                          cutdlim=cutdlim,
                          lonlat=lonlat,
                          call=match.call()),class=c("mapclust_cl","list"))
    cat("\n\n\n
    Divisive hierarchical clustering

call: ")
    print(out$call)
    cat(sep = "","\npartition:
      number of cluster: ",length(unique(out$cluster)),"\n      Average_Sil = ",arrondi(mean(out$silhouetteData[,3]),2),", WSS = ",arrondi(sum(out$InertiaWithin[[which(out$dlim == out$cutdlim)]]),2),", cutdlim = ",out$cutdlim,
        "\nplot:
      .. $dendrogram      The global dendrogramm
      .. $cuttree         The cut dendrogramm
      .. $map             The map of the select partition
      .. $AveSilPlot      The average silhouette width plot (for each partition)
      .. $WSSPlot         The Within Sum of Square plot (for each partition)
      .. $silhouette      The silhouette of the select partition
value:
      .. $X               The x-coordinates data you used
      .. $Y               The y-coordinates data you used
      .. $var             The regionalized variable.s data you used
      .. $label           The label vector you used
      .. $density         The estimate density based on var. Equal to var if you used a unidimensionnal density variable
      .. $cluster         The vector of cluster of the select partition
      .. $fullhist        The composition cluster for each observation
      .. $hist            The composition cluster without duplicates (matches to leaf of the dendrogramm)
      .. $dlim            The vector of the different limit distances
      .. $cutdlim         The select dlim for the cut
      .. $InertiaWithin   The Sum of Square for each patch of each partition
      .. $DiMatrix        The matrix of Weighted Euclidiean distances
      .. $silhouetteData  The silhouette data of the select partition
      .. $AveSilData      The average silhouette value for each partition
      .. $Moran           The Moran index for each groups for each partitions
      .. $lonlat          Logical parameter if your coordinates are in longitude latitude format or not.

If you ran mapclust like this: classif <- mapclust(...)
    See summary of the silhouette data by running:
    > summary(classif$silhouetteData)

You can cut the dendrogram for a new dlim with: NewCut <- mapclust_cut_tree(...)
    See documentation by running: \n    > ?mapclust_cut_tree

documentation: \n> ?mapclust
        ")

    return(out)

  }

print.mapclust_cl <- function(x, ...){
  cat("
      Divisive hierarchical clustering | mapclust_cl

call: ")
  print(x$call)
  cat(sep = "","\npartition:
      number of cluster: ",length(unique(x$cluster)),"\n      Average_Sil = ",arrondi(mean(x$silhouetteData[,3]),2),", WSS = ",arrondi(sum(x$InertiaWithin[[which(x$dlim == x$cutdlim)]]),2),", cutdlim = ",x$cutdlim,
      "\nplot:
      .. $dendrogram      The global dendrogramm
      .. $cuttree         The cut dendrogramm
      .. $map             The map of the select partition
      .. $AveSilPlot      The average silhouette width plot (for each partition)
      .. $WSSPlot         The Within Sum of Square plot (for each partition)
      .. $silhouette      The silhouette of the select partition
value:
      .. $X               The x-coordinates data you used
      .. $Y               The y-coordinates data you used
      .. $var             The regionalized variable.s data you used
      .. $label           The label vector you used
      .. $density         The estimate density based on var. Equal to var if you used a unidimensionnal density variable
      .. $cluster         The vector of cluster of the select partition
      .. $fullhist        The composition cluster for each observation
      .. $hist            The composition cluster without duplicates (matches to leaf of the dendrogramm)
      .. $dlim            The vector of the different limit distances
      .. $cutdlim         The select dlim for the cut
      .. $InertiaWithin   The Sum of Square for each patch of each partition
      .. $DiMatrix        The matrix of Weighted Euclidiean distances
      .. $silhouetteData  The silhouette data of the select partition
      .. $AveSilData      The average silhouette value for each partition
      .. $Moran           The Moran index for each groups for each partitions
      .. $lonlat          Logical parameter if your coordinates are in longitude latitude format or not.

If you ran mapclust like this: classif <- mapclust(...)
    See summary of the silhouette data by running:
    > summary(classif$silhouetteData)

You can cut the dendrogram for a new dlim with: NewCut <- mapclust_cut_tree(...)
    See documentation by running: \n    > ?mapclust_cut_tree

documentation: \n> ?mapclust
      ")
}

"ecartmatrice" <-
  function (historique = "matrix composition of groups for each dlim")
  {
    #===================================================================================
    # ECART MATRICE
    #
    # Author : A. Coulon (Universite de Nantes)
    # Last update : 22 march 2018
    #
    # Arguments:
    # historique   The composition of groups for each dlim and each initial elements
    #
    #===================================================================================

    historique <- as.matrix(historique)
    #variable securite
    OK <- 1

    if(length(historique[,1]) <= 1){
      cat("No Cluster")
    }
    ecart <- matrix(ncol = length(historique[,1]), nrow = length(historique[,1]))
    #On parcours le tableau pour tous les j on les compare a tous les i (compare tous les elt entre eux)
    for (j in 1:length(historique[,1])) {
      for (i in 1:length(historique[,1])) {
        if(i > j){
          v <- historique[i,]==historique[j,]
          somme <- sum(v %in% FALSE)
          ecart[i,j] <- somme
        }
      }
    }
    ecart <- replace(ecart,is.na(ecart),Inf)
    ecart
  }

"mapclust_cut_tree" <- function(classification, nb_grp = NA, dlim = NA)
{
  #===================================================================================
  # CUT DENDROGRAMM
  #
  # Author : A. Coulon (Universite de Nantes)
  # Last update : 09 October 2018
  #
  # Arguments:
  # classification	  The object return by mapclust
  # dlim              The value of dlim where you want to cut the dendrogramm
  # Plabel            Logical parameter for activate or not the print of labels on dendrogram
  #
  #===================================================================================
  lonlat <- classification$lonlat
  if(!is.na(dlim) && !is.na(nb_grp)){
    if(nb_grp - arrondi(nb_grp) == 0){
      message("you have entered both parameters.")
      message("The parameter dlim will be ignore!")
      dlim <- NA
    }else{
      message("you have entered a non-integer group number: ",nb_grp)
      message("The parameter nb_grp will be ignore!")
      nb_grp <- NA
    }
  }
  if(!nb_grp - arrondi(nb_grp) == 0 && is.na(dlim)){
    dlim <- nb_grp
    nb_grp <- NA
    message("you have entered a non-integer group number: ",dlim,". We consider this value as a dlim.")
  }
  if(!is.na(dlim)){
    if(dlim < 0){
      message("Dlim must be positive We change it to the closest to 0.")
    }
    if(dlim > max(classification$dlim)){
      message("Your dlim is too high.")
    }
    #On recupere les variables de l'objet classification et des autres arguments
    diff <- classification$dlim-dlim
    if((TRUE %in% (diff <= 0))){
      #return index of the first value <= 0
      index <- which.max(diff <= 0)
    }else{
      index <- which.min(diff)
    }

    cutdlim <- classification$dlim[index]
  }else{
    if(!is.na(nb_grp)){
      V_NB_GRP <- c()
      index <- NULL
      for(i in 1:length(classification$dlim)){
        V_NB_GRP <- c(V_NB_GRP,length(unique(classification$historique[,i])))
        if(nb_grp == length(unique(classification$historique[,i]))){
          cutdlim <- classification$dlim[i]
          index <- i
        }
      }

      if(is.null(index)){
        message("There is not this number of group in the partition. We cut to the closest. possible number:")
        print(unique(V_NB_GRP)[-1])
        diff <- V_NB_GRP - nb_grp
        index <- which.min(abs(diff))
        cutdlim <- classification$dlim[index]
      }
    }else{
      stop("You must enter a dlim or a number of group in order to cut the dendrogram.")
    }
  }
  N <- length(unique(classification$fullhist[,index+1]))
  change <- 0
  while (N == 1){
    change <- 1
    index <- index + 1
    cutdlim <- classification$dlim[index]
    N <- length(unique(classification$fullhist[,index+1]))
  }
  if (change == 1){message("You can't create only one cluster. We use the next dlim which create at least two cluster : ",cutdlim)}

  if(dim(as.data.frame(classification$var))[2] > 1){
    Size <- classification$var[,1]
  }else{
    Size <- classification$var
  }
  cat("Running ..")
  #classification$dendrogramme$theme correspond au metadonnees du dendrogramme (concernant le theme)
  #c'est une liste et si il y a du texte alors la liste est de longueur deux

  Plabel = classification$Plabel

  multi <- dim(as.data.frame(classification$var))[2] > 1
  tmp <- data.frame(lon=classification$X,lat=classification$Y)

  lineType <- ifelse(Size < 0,"2, 5","1")
  #leaflet
  if(!lonlat){
    tmp$lon <- (tmp$lon - mean(tmp$lon)) / max(tmp$lon, tmp$lat)
    tmp$lat <- (tmp$lat - mean(tmp$lat)) / max(tmp$lon, tmp$lat)
  }
  lat <- tmp$lat
  lng <- tmp$lon
  size <- 10*(0.1+(abs(Size)/max(abs(Size))))+5
  color <- as.factor(classification$fullhist[,index+1])
  df <- data.frame(
    lat,
    lng,
    size,
    color
  )

  if(multi){df$size <- rep(7,length(df$size))}
  # Make up some random levels. (TODO: Better example)
  couleur <- c("slateblue2", "blue", "chartreuse1","darkolivegreen2",
               "#222222", "darkmagenta", "mediumpurple4",
               "magenta","cyan3", "plum4","darkorange1", "pink1", "red",
               "lightgoldenrod", "violetred3", "brown4", "tan1","yellow", "gray50",
               "#967272","deepskyblue4","salmon", "cyan","palegreen", "green3")
  #plot(1:25,1:25,pch =15,cex=3,col=couleur)
  df$category <- as.factor(classification$fullhist[,index+1])
  factpal <- colorFactor(couleur, df$category)
  coul <- factpal(classification$historique[,index])

  if(lonlat){
    esri <- structure(c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                        "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                        "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                        "Esri.WorldGrayCanvas"),
                      .Names = c("Esri.WorldStreetMap", "Esri.WorldTopoMap",
                                 "Esri.WorldImagery", "Esri.WorldTerrain", "Esri.WorldShadedRelief",
                                 "Esri.WorldPhysical", "Esri.OceanBasemap", "Esri.NatGeoWorldMap",
                                 "Esri.WorldGrayCanvas"))
    m = leaflet(df)
    for (provider in esri) {
      m <- m %>% addProviderTiles(provider, group = provider)
    }

    m <- m %>% addCircleMarkers(lng = ~lng, lat = ~lat, radius = ~size, fillColor =  ~factpal(category),
                                fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=lineType
    ) %>%
      addLegend("bottomleft", pal = factpal, values = as.factor(classification$fullhist[,index+1]),
                title = "Cluster",
                opacity = 1
      ) %>%
      addMiniMap(toggleDisplay = TRUE,
                 position = "bottomright"
      ) %>%
      addLayersControl(baseGroups = names(esri),
                       options = layersControlOptions(collapsed = TRUE),
                       position = "topright"
      ) %>%
      htmlwidgets::onRender("
                            function(el, x) {
                              var myMap = this;
                              myMap.on('baselayerchange',
                              function (e) {
                                myMap.minimap.changeLayer(L.tileLayer.provider(e.name));
                              })
                            }"
        )
    if(!multi){
      m <- m %>%
        addLegendCustom(colors = c("black", "black", "black"),
                        labels = c(arrondi(min(abs(Size)),2),
                                   arrondi(min(abs(Size)) + (max(abs(Size))-min(abs(Size)))/2  ,2),
                                   arrondi(max(abs(Size)),2)),
                        sizes = c(10*(0.1+( min(abs(Size)) / max(abs(Size))))+5,
                                  10*(0.1+((min(abs(Size)) + (max(abs(Size))-min(abs(Size)))/2) /max(abs(Size))))+5,
                                  10*(0.1+( max(abs(Size)) / max(abs(Size))))+5),
                        title = "Size"
        )
    }
    if(min(Size)<0){
      m <- m %>%
        addLegendCustom(colors = c("white", "white"), labels = c("line: positive","dotted-line: negative"),
                        sizes = 1, title="Sign"
        )
    }
  }else{
    m <- leaflet(df)
    m <- m %>% addCircleMarkers(lng = ~lng, lat = ~lat, radius = ~size, fillColor =  ~factpal(category),
                                fill = TRUE, fillOpacity = 0.8, opacity=1, stroke = TRUE,weight = 1.5, color="black", dashArray=lineType
    ) %>%
      addLegend("bottomleft", pal = factpal, values = as.factor(classification$fullhist[,index+1]),
                title = "Cluster",
                opacity = 1
      )
  }

  cluster <- as.numeric(classification$fullhist[,index+1])

  cuttree <- classification$dendrogram_ggplot
  cuttree <- cuttree + geom_hline(yintercept = cutdlim + 0.03 , col = "black" , lwd = 1)
  print(Plabel)
  if(Plabel){
    cuttree <- cuttree + suppressWarnings(theme(axis.text.x = element_text(color = coul,
                                                          hjust = 1, vjust = 0.25, size = 9, angle = 90)))
  }else{
    cuttree <- cuttree + theme(axis.text.x = element_blank())
  }

  sil <- silhouette(cluster,classification$DiMatrix)
  coull <- factpal(sort(cluster))
  plot(sil,col = coull)
  silPlot <- recordPlot()
  plot.new()
  # Nouvelle page
  grid.newpage()
  # Creer la mise en page : nrow = 2, ncol = 2
  pushViewport(viewport(layout = grid.layout(2, 2)))
  # Une fonction pour definir une region dans la mise en page
  define_region <- function(row, col){
    viewport(layout.pos.row = row, layout.pos.col = col)
  }
  # Arranger les graphiques dans la fenetre
  print(cuttree, vp = define_region(1:2, 1))
  print(classification$AveSilPlot, vp = define_region(1, 2))
  print(classification$WSSPlot, vp = define_region(2, 2))
  print(m)

    out <- structure(list(X=classification$X,
                        Y=classification$Y,
                        var=classification$var,
                        label=classification$label,
                        density=classification$density,
                        cluster=cluster,
                        fullhist=classification$fullhist,
                        dlim=classification$dlim,
                        historique=classification$historique,
                        Plabel=classification$Plabel,
                        InertiaWithin=classification$InertiaWithin,
                        Moran=classification$Moran,
                        dendrogram=classification$dendrogram,
                        dendrogram_ggplot=classification$dendrogram_ggplot,
                        DiMatrix=classification$DiMatrix,
                        cuttree=cuttree,
                        map=m,
                        silhouette=silPlot,
                        silhouetteData=sil,
                        AveSilData=classification$AveSilData,
                        AveSilPlot=classification$AveSilPlot,
                        WSSPlot=classification$WSSPlot,
                        cutdlim=cutdlim,
                        lonlat=lonlat,
                        call=match.call()),class=c("mapclust_cl","list"))
    cat(". Done")
    cat("\n\n\n
      Divisive hierarchical clustering

call: ")
    print(out$call)
    cat(sep = "","\npartition:
      number of cluster: ",length(unique(out$cluster)),"\n      Average_Sil = ",arrondi(mean(out$silhouetteData[,3]),2),", WSS = ",arrondi(sum(out$InertiaWithin[[which(out$dlim == out$cutdlim)]]),2),", cutdlim = ",out$cutdlim,
        "\nplot:
      .. $dendrogram      The global dendrogramm
      .. $cuttree         The cut dendrogramm
      .. $map             The map of the select partition
      .. $AveSilPlot      The average silhouette width plot (for each partition)
      .. $WSSPlot         The Within Sum of Square plot (for each partition)
      .. $silhouette      The silhouette of the select partition
value:
      .. $X               The x-coordinates data you used
      .. $Y               The y-coordinates data you used
      .. $var             The regionalized variable(s) data you used
      .. $label           The label vector you used
      .. $density         The estimate density based on var. Equal to var if you used a unidimensionnal density variable
      .. $cluster         The vector of cluster of the select partition
      .. $fullhist        The composition cluster for each observation
      .. $hist            The composition cluster without duplicates (matches to leaf of the dendrogramm)
      .. $dlim            The vector of the different limit distances
      .. $cutdlim         The select dlim for the cut
      .. $InertiaWithin   The Sum of Square for each patch of each partition
      .. $DiMatrix        The matrix of Weighted Euclidiean distances
      .. $silhouetteData  The silhouette data of the select partition
      .. $AveSilData      The average silhouette value for each partition
      .. $Moran           The Moran index for each groups for each partitions
      .. $lonlat          Logical parameter if your coordinates are in longitude latitude format or not.

If you ran mapclust like this: classif <- mapclust(...)
    See summary of the data by running:
    > summary(classif$silhouetteData)

You can cut the dendrogram for a new dlim with: NewCut <- mapclust_cut_tree(...)
    See documentation by running:
    > ?mapclust_cut_tree

documentation: \n> ?mapclust
        ")
  return(out)
}


summary.mapclust_cl <- function(object, ...){
  plot(object$cuttree)
  cat("\n mapclust: Divise hierarchical clustering")
  if(dim(as.data.frame(object$var))[2] == 1){
    cat("\n -------------------------------------
  dlim\t  n\t WSS\t sil\tMoran
 -------------------------------------
")
  }else{
    cat("\n -------------------------------
  dlim\t  n\t WSS\t sil
 -------------------------------
")
  }
  j <- 1
  for (i in 1:length(object$dlim)){
    dlim <- object$dlim[i]
    number_grp <- length(unique(object$historique[,i]))
    if(number_grp == 1){
      sil <- NA
    }else{
      sil <- object$AveSilData[j]
      j <- j+1
    }
    WSS <- sum(object$InertiaWithin[[i]])
    if(dim(as.data.frame(object$var))[2] == 1){
      Moran <- mean(na.omit(unlist(object$Moran[[i]][1,])))
    }


    cat(" ",dlim,"\t")
    cat(" ",number_grp,"\t")

    cat(arrondi(WSS,2),"\t")
    cat(arrondi(sil,2),"\t")
    aste <- FALSE
    if(dim(as.data.frame(object$var))[2] == 1){
      cat(arrondi(Moran,2))
      if(anyNA(object$Moran[[i]][1,]) == TRUE){
        nbTOT <- length(object$Moran[[i]][1,])
        nbNA <- length(is.na(object$Moran[[i]][1,])[is.na(object$Moran[[i]][1,])==TRUE])
        aste <- TRUE
        cat("\t* ",nbNA,"NA /",nbTOT)
      }
    }
    cat("\n")
  }
  if(dim(as.data.frame(object$var))[2] == 1){
    cat(" -------------------------------------")
  }else{
    cat(" -------------------------------")
  }
  if(aste){
    cat("\n* Some groups in this partition have less than 3 observations and we can't calculate the Moran index for these groups. There is at least one of these groups in the partition. In this case the mean value is calculated only with the groups with more than 2 observations.")
  }
  nb <- length(unique(object$cluster))
  tot <- length(object$X)
  X_mean <- tapply(object$X,object$cluster,mean)
  Y_mean <- tapply(object$Y,object$cluster,mean)
  size <- c()
  cat("\n\n")
  cat("The selected partition contains",nb,"clusters among",tot,"observations.\ncluster:\t")
  for (k in 1:nb){
    size <- c(size,length(object$cluster[object$cluster == k]))
    cat(k,"\t")
  }
  cat("\nSize:\t\t")
  for(y in 1:nb){
    cat(size[y],"\t")
  }
  cat("\nX_mean:\t\t")
  for(y in 1:nb){
    cat(arrondi(X_mean[y],2),"\t")
  }
  cat("\nY_mean:\t\t")
  for(y in 1:nb){
    cat(arrondi(Y_mean[y],2),"\t")
  }
  if(dim(as.data.frame(object$var))[2] == 1){
    var_mean <- tapply(object$var,object$cluster,mean)
    cat("\nvar_mean:\t")
    for(y in 1:nb){
      cat(arrondi(var_mean[y],2),"\t")
    }
  }else{
    for(i in 1:dim(as.data.frame(object$var))[2]){
      var_mean <- tapply(object$var[,i],object$cluster,mean)
      cat("\nvar",i,"_mean:\t",sep="")
      for(y in 1:nb){
        cat(arrondi(var_mean[y],2),"\t")
      }
    }
  }
  object$map
}

IndiceMoran <- function(hist,data) {
  out <- list()
  X <- data[,1]
  Y <- data[,2]
  Z <- data[,3]

  n <- length(hist[1,])
  for (i in 1:n){
    #recup compo of one partition i
    compo <- hist[,i]

    nb <- length(unique(compo))

    for(j in 1:nb){
      subX <- X[compo==j]
      subY <- Y[compo==j]
      subZ <- Z[compo==j]

      if(length(subX) > 3){

        if(length(unique(subZ)) > 2 && length(unique(subX)) > 2 && length(unique(subY)) > 2){
          data.dists <- as.matrix(dist(cbind(subX, subY)))
          Moran <- Moran.I(subZ, data.dists)
        }else{
          Moran <- list(observed= NA, p.value = NA)
        }

      }else{
        Moran <- list(observed= NA, p.value = NA)
      }

      if(!exists("M.I")){
        M.I <- data.frame(grp=c(Moran$observed,Moran$p.value),row.names = c("Moran.I","p.value"))
      }else{
        M.I <- cbind(M.I,grp=c(Moran$observed,Moran$p.value))
      }
    }
    out <- c(out,list(M.I))
    rm(M.I)
  }
  out
}

elbow_finder <- function(x_values, y_values) {
  # Max values to create line
  max_x_x <- max(x_values)
  max_x_y <- y_values[which.max(x_values)]
  max_y_y <- max(y_values)
  max_y_x <- x_values[which.max(y_values)]
  max_df <- data.frame(x = c(max_y_x, max_x_x), y = c(max_y_y, max_x_y))

  # Creating straight line between the max values
  fit <- lm(max_df$y ~ max_df$x)

  # Distance from point to line
  distances <- c()
  for(i in 1:length(x_values)) {
    distances <- c(distances,
                   abs( coef(fit)[2]*x_values[i] - y_values[i] + coef(fit)[1] ) / sqrt( coef(fit)[2]^2 + 1^2 ))
  }

  # Max distance point
  x_max_dist <- x_values[which.max(distances)]
  y_max_dist <- y_values[which.max(distances)]
  return(distances)
  #return(c(x_max_dist, y_max_dist))
}

CAdist <- function(df, nPC = NULL, graph = TRUE){

  #===============================================================================
  # GENERATE DISTANCE MATRIX WITH CORRESPONDENCE ANALYSiS (CA) (Fr: Analyse Factorielle des Correspondances)
  #
  # Authors : A.COULON
  # Last update : 23 october 2018
  #
  # Arguments:
  # df	      The ceramic contingence table
  # nPC       The number of principal component to keep
  # graph     Logical for print or not the plot of the CA
  #
  #===============================================================================

  if(!is.table(df)){
    if(is.matrix(df)){df <- as.data.frame(df)}
    if(!is.data.frame(df)){stop("df is not a data frame.")}
  }

  if(FALSE %in% (df == trunc(df))){stop("The data frame must contain integers.")}
  max <- min( nrow(df)-1,ncol(df)-1)

  if(!is.null(nPC)){
    if(nPC == "max"){nPC <- max}
    if(nPC > max){
      message("nPC must be less than: ",max+1,".")
      nPC <- max
    }
  }else{
    tmp <- FactoMineR::CA(df, graph=graph)

    x <- paste0(rownames(tmp$eig)," (",arrondi(tmp$eig[,3],1),"%)")
    Eigen_value <- tmp$eig[,1]
    df2 <- data.frame(x,Eigen_value)
    df2 <- within(df2,Principal_Component <- factor(x,levels = x))

    p <- plotly::plot_ly(df2,
      x = ~Principal_Component,
      y = ~Eigen_value,
      type = "bar"
    )
    print(p)
    if(interactive()){
      OK <- TRUE
      while(OK){
        nPC <- as.integer(readline("Select the number of axes:"))
        if(is.na(nPC) || nPC < 1 || nPC > max){
          message("must be a integer between 1 and ",max)
        }else{
          OK <- FALSE
        }
      }
    }else{
      stop("Need a nPC value.")
    }

  }
  df.CA <- FactoMineR::CA(df, ncp = nPC, graph = graph)
  dist <- dist(df.CA$row$coord) / max(dist(df.CA$row$coord))
  #dist <- dist(scale(df.CA$row$coord)) / max(dist(scale(df.CA$row$coord)))
  return(D=dist)
}

adjacency <- function(network){

  #===============================================================================
  # GENERATE DISTANCE MATRIX WITH NETWORK
  #
  # Authors : A.COULON
  # Last update : 23 october 2018
  #
  # Arguments:
  # raw	      The data frame with the stratigraphic element and the relation with the other
  #
  #===============================================================================

  #data.frame ?
  if(!is.data.frame(network)){stop("network data must be a data.frame.")}
  #number of col  ?
  if(length(network[1,])!=2){stop("network must have 2 colunms.")}
  M <- matrix(0,ncol = length(network[,1]),nrow = length(network[,1]),dimnames = list(network[,1],network[,1]))
  diag(M) <- 1
  for(i in 1:length(network[,1])){
    rel <- as.character(network[i,2])
    rel <- strsplit(rel,",")
    for(elt in rel[[1]]){
      j <- which(network[,1] == elt)
      M[i,j] <- 1
      M[j,i] <- 1
    }
  }
  D <- 1 - M
  return(D=as.dist(D))
}

overlap <- function(temporal){

  #===============================================================================
  # GENERATE DISSIMILARITY MATRIX WITH TEMPORAL
  #
  # Authors : A.COULON
  # Last update : 23 october 2018
  #
  # Arguments:
  # temporal	      The data frame
  #
  #===============================================================================

  indice <- function(inf1,sup1,inf2,sup2){
    if(inf2 == inf1 && inf1 == sup2 && inf1 == sup1){
      return(1)
    }
    if(sup1 < inf1 || inf2 > sup2){
      return(99)
    }
    #we define the bound of the total overlap as follow : min of lower bounds and max of upper bounds
    total_overlap_inf <- min(inf1,inf2)
    total_overlap_sup <- max(sup1,sup2)

    #The overlap is the difference between the bounds
    total_overlap <- total_overlap_sup - total_overlap_inf

    #we define the bound of the within overlap as follow : max of lower bounds and min of upper bounds
    within_overlap_inf <- max(inf1,inf2)
    within_overlap_sup <- min(sup1,sup2)

    #The overlap is the difference between the bounds
    within_overlap <- within_overlap_sup - within_overlap_inf

    #the overlap index is the ratio between the within overlap and the total overlap
    i <- within_overlap / total_overlap
    return(i)
  }

  df <- as.data.frame(temporal)
  #we create a square matrix of the same size as the number of rows
  overlap_matrix <- matrix(nrow = length(df[,1]), ncol = length(df[,1]))

  #we look each couple i j and compute the overlap index for this couple and save the value in the matrix[i,j]
  for(i in 1:(length(df[,1])-1)){
    for(j in (i+1):length(df[,1])){
      #recuperation of the elements rows i and j
      ens1 <- df[i,]
      ens2 <- df[j,]
      #compute the overlap index
      ind <- indice(ens1[1,2],ens1[1,3],ens2[1,2],ens2[1,3])
      #if(ind < 0){ind <- 0}
      if(ind==99){stop("The upper and lower bound are not in the correct column (sup1 < inf1 || inf2 > sup2).")}
      #if(ind <= 0){ind <- 0}
      #if(ind > 0){ind <- 1}
      #fill the matrix
      overlap_matrix[i,j] <- overlap_matrix[j,i] <- ind
    }
  }

  #fill the diag with 1
  diag(overlap_matrix) <- 1
  #[-1,1] to [0,2] (2 mean identical time range)
  overlap_matrix <- overlap_matrix + 1
  #inversion of values (high values now mean distant elements)
  overlap_matrix <- 2 - overlap_matrix
  #[0,2] to [0,1]
  overlap_matrix <- overlap_matrix / 2
  rownames(overlap_matrix) <- colnames(overlap_matrix) <- df[,1]
  return(D=overlap_matrix)
}

perioclust <- function(D1,D2,alpha="EstimateAlphaForMe",k=NULL,title="notitle",method="complete",suppl_plot=TRUE) {
  #===============================================================================
  # CAH CLASSIFICATION USING TWO DIST MATRIX
  #
  # Authors : A.COULON, L.BELLANGER and P.HUSI
  # Last update : 23 october 2018
  #
  # Arguments:
  # D1	       The fisrt distance matrix. Archeological context: stratigraphic or timerange distance.
  # D2	       The second distance matrix. Archeological context: ceramic distance.
  # alpha	     The mixing parameter.
  # k          The number of clusters.
  # title	     The title to print on the dendrogram (optionnal).
  # method     The method to use in hclust (see doc).
  # suppl_plot Logical for plot the WSS and average sil plot.
  #
  #===============================================================================

  sort = TRUE
  if(is.null(labels(D1))){
    sort <- FALSE
  }
  if(is.null(labels(D2))){
    sort <- FALSE
  }

  METHODS <- c("ward.D","single","complete","average","mcquitty","median","centroid","ward.D2")
  if (method == "ward") {
    message("The \"ward\" method has been renamed to \"ward.D\"; note new avaible method \"ward.D2\"")
    method <- "ward.D"
  }
  i.meth <- pmatch(method, METHODS)
  if (is.na(i.meth))
    stop("invalid clustering method", paste("", method))
  if (i.meth == -1)
    stop("ambiguous clustering method", paste("", method))

  method <- METHODS[i.meth]
  cont <- NULL
  Number_of_groups <- c()
  y <- c()
  yend <- c()
  xend <- c()
  if(!"dist" %in% class(D2)){
    #matrix ?
    if(!is.matrix(D2)){
      if(length(D2[1,]) == 3){
        message("generation of a temporal dissimilarities matrix.")
        D2 <- overlap(D2)
      }else{
        message("generation of a stratigraphic dissimilarities matrix.")
        D2 <- adjacency(D2)
        #diag = 0 ?
        if(sum(diag(D2)) != 0){
          stop("D2: The diagonal must be equal to 0.")
        }
      }
    }

    #symetric ?
    if(!isSymmetric(D2)){
      stop("D2 is not symmetric.")
    }
    if(!dim(D2)[1] == dim(D2)[2]){stop("D2 not a square matrix.")}
  }

  if(!"dist" %in% class(D1)) {
    #matrix ?
    if(!is.matrix(D1)){
      if(is.data.frame(D1)){cont <- D1;EPPM <- TRUE}else{EPPM <- FALSE}
      message("generation of a ceramic distances matrix.")
      D1 <- as.matrix(CAdist(D1,nPC="max",graph=FALSE))
    }
    #diag = 0 ?
    if(sum(diag(D1)) != 0){
      stop("D1: The diagonal must be equal to 0.")
    }
    #symetric ?
    if(!isSymmetric(D1)){
      stop("D1 is not symmetric.")
    }
    if(!dim(D1)[1] == dim(D1)[2]){stop("D1 not a square matrix.")}
  }

  D1 <- as.matrix(D1)
  max <- dim(D1)[1]
  D2 <- as.matrix(D2)

  if(FALSE %in% (sort(rownames(D1)) == sort(rownames(D2)))){
    warning("The names of D1 and D2 are not the same.")
    sort <- FALSE
  }
  if(FALSE %in% (sort(colnames(D1)) == sort(colnames(D2)))){
    warning("The names of D1 and D2 are not the same.")
    sort <- FALSE
  }
  if(FALSE %in% (sort(rownames(D1)) == sort(colnames(D2)))){
    warning("The names of D1 and D2 are not the same.")
    sort <- FALSE
  }

  if(!length(D1) == length(D2)){stop("D1 and D2 have not the same size.")}
  #dist (positive matrix) ?
  if(min(D1) < 0){stop("D1 is not positive.")}
  if(min(D2) < 0){stop("D2 is not positive.")}

  #normalization
  if(max(D1) != 0){
    D1 <- D1 / max(D1)
  }
  if(max(D2) != 0){
    D2 <- D2 / max(D2)
  }
  #create mixing matrix
  D1 <- as.dist(D1)
  D2 <- as.dist(D2)
  D1 <- as.matrix(D1)
  D2 <- as.matrix(D2)

  #sort labels
  if (sort) {
    D1.Tmp <- D1[sort(labels(D1)[[1]]),sort(labels(D1)[[1]])]
    D2.Tmp <- D2[sort(labels(D2)[[1]]),sort(labels(D2)[[1]])]
    D1 <- as.dist(D1.Tmp)
    D2 <- as.dist(D2.Tmp)
  }

  #no alpha
  if(alpha == "EstimateAlphaForMe"){
    # sa <- hclustcompro_select_alpha(D1, D2, method = method, resampling = FALSE)
    weight = optim_select_alpha(data =  list(D1, D2), method = method)
    alpha = weight$solution[1]
    # alpha <- sa$alpha[1]
    # sa <- list(alpha.plot = NULL,conf = NULL)
  }else{
    # sa <- list(alpha.plot = NULL,conf = NULL)
  }
  #alpha 0-1 ?
  if(alpha > 1 || alpha < 0){stop("alpha must be between 0 and 1.")}
  

  #mixt matrix
  D_alpha <- (alpha)*D1 + (1-alpha)*D2
  message("\nThe mixing parameter is ",alpha,". The mixing matrix is made of ",arrondi((alpha)*100,2),"% of D1 and ",
      arrondi((1-alpha)*100,2),"% of D2.\n\n")
  D_alpha <- as.dist(D_alpha)

  #CAH
  tree <- SPARTAAS::hclust(d=list(as.dist(D1),as.dist(D2)),method=method,weight=c(alpha,1-alpha))

  if(title == "notitle"){
    title <- paste("Dendrogram with alpha =",alpha)
  }
  subtitle <- paste("The mixing matrix is made of ",arrondi((alpha)*100,2),"% of D1 and ",
                    arrondi((1-alpha)*100,2),"% of D2.")

  D <- as.matrix(D_alpha)
  n <- length(D[,1])

  #wss sil value
  if(suppl_plot){
    if(length(D[,1]) > 20){n <- 20}
    wss <- c()
    sil <- c()
    for ( i in 2:(n-1)){
      cutree <- stats::cutree(tree,i)
      res <- fpc::cluster.stats(D,cutree)
      wss <- c(wss,res$within.cluster.ss)
      sil <- c(sil,res$avg.silwidth)
    }
    #wss
    Within_Sum_of_Square <- rbind(seq(2,(n-1)),wss)
    color_wss <- elbow_finder(seq(2,(n-1)),wss)
    dataplot <- t(Within_Sum_of_Square)
    dataplot <- as.data.frame(dataplot)
    colnames(dataplot) <- c("Number_of_groups","Within_Sum_of_Square")
    q <- ggplot2::ggplot(dataplot,aes(x=Number_of_groups, y=Within_Sum_of_Square)) +
      ggplot2::geom_point(aes(color=color_wss)) +
      ggplot2::scale_colour_gradientn(colours=c("brown1", "chartreuse3")) +
      ggplot2::geom_line(linetype = 3) +
      ggplot2::scale_x_continuous(breaks = dataplot$Number_of_groups ,
                                  labels = dataplot$Number_of_groups ) +
      ylim(0,max(wss)+1) +
      labs(title = "Within Sum of Square" , subtitle = "Partition evaluation") +
      ggplot2::theme(legend.position="none")

    #silhouette
    Average_Sil_Width <- rbind(seq(2,(n-1)),sil)
    dataplot <- t(Average_Sil_Width)
    dataplot <- as.data.frame(dataplot)
    dataplot <- na.omit(dataplot)
    colnames(dataplot) <- c("Number_of_groups", "Average_Sil_Width")
    p <- ggplot2::ggplot(dataplot,aes(x = Number_of_groups, y = Average_Sil_Width)) +
      ggplot2::geom_point(aes(color=Average_Sil_Width)) +
      ggplot2::scale_colour_gradientn(colours=c("brown1", "chartreuse3")) +
      ggplot2::geom_line(linetype = 3) + ylim(0,1) +
      ggplot2::scale_x_continuous(breaks = dataplot$Number_of_groups,
                                  labels = dataplot$Number_of_groups) +
      labs(title = "Average Silhouette Width", subtitle = "Partition evaluation") +
      ggplot2::theme(legend.position="none")

    # Nouvelle page
    grid::grid.newpage()
    # Creer la mise en page : nrow = 2, ncol = 2
    grid::pushViewport(viewport(layout = grid.layout(2, 2)))
    # Une fonction pour definir une region dans la mise en page
    define_region <- function(row, col){
      grid::viewport(layout.pos.row = row, layout.pos.col = col)
    }

    # Arranger les graphiques dans la fenetre
    hc <- tree
    hcdata <- ggdendro::dendro_data(tree, type="rectangle")
    dendrogram <- ggplot2::ggplot() + ggplot2::ggtitle(title) + ylim(-0.2, max(tree$height)) + ylab("height") + xlab("") +
      ggplot2::geom_segment(data=segment(hcdata), aes(x=x, y=y, xend=xend, yend=yend)) +
      ggplot2::geom_text(data=label(hcdata), aes(x=x,y=y,label=label,hjust=1,angle=90,fontface="bold",cex=1.5),
                         size=3) +
      ggplot2::theme(axis.title.x=element_blank(),
                     axis.text.x=element_blank(),
                     axis.ticks.x=element_blank())

    print(dendrogram, vp = define_region(1:2, 1))
    print(p, vp = define_region(1, 2))
    print(q, vp = define_region(2, 2))

    if(is.null(k)){
      if(interactive()){
        OK <- TRUE
        while(OK){
          k <- as.integer(readline("Select the number of clusters:"))
          if(is.na(k) || k < 1 || k > max){
            message("must be a integer between 1 and ",max)
          }else{
            OK <- FALSE
          }
        }
      }else{
        stop("No interactive session. Use parameter k to set clusters.")
      }
    }
  }else{
    if(is.null(k)){
      plot(tree,h = -1,main = title,sub = subtitle,xlab = paste0("D_alpha, method: \"",method,"\""))
      if(interactive()){
        OK <- TRUE
        while(OK){
          k <- as.integer(readline("Select the number of clusters:"))
          if(is.na(k) || k < 1 || k > max){
            message("must be a integer between 1 and ",max)
          }else{
            OK <- FALSE
          }
        }
      }else{
        stop("No interactive session. Use parameter k to set clusters.")
      }
    }
  }


  plot(tree,h = -1, main = title,sub = subtitle,xlab = paste0("D_alpha, method: \"",method,"\""))
  stats::rect.hclust(tree, k = k, border = colorspace::rainbow_hcl(k, c = 80, l = 60, start = 0, end = 360 * (k-1) / k))
  cluster <- stats::cutree(tree, k = k)
  height <- tree$height[length(tree$height) - k + 2]
  order <- unique(cluster[tree$order])
  for(i in 1:length(order)){
    if(i != order[i] && i < order[i]){
      cluster[cluster == i] <- 0
      cluster[cluster == order[i]] <- i
      cluster[cluster == 0] <- order[i]
      order <- unique(cluster[tree$order])
    }
  }
  col <- colorspace::rainbow_hcl(length(unique(cluster)),
                     c = 80, l = 60, start = 0,
                     end = 360*(length(unique(cluster))-1)/(length(unique(cluster))))
  for(i in 1:length(unique(cluster))){
    x <- mean(which(cluster[tree$order] == sort(unique(cluster))[i]))

    if(length(unique(cluster)) > 338){
      symb_letter <- sort(unique(cluster))[i]
    }else{
      symb_letter <- Int2Alpha(sort(unique(cluster))[i])
    }
    mtext(symb_letter, side = 1, line = .5, at = x, col = col[i], cex = 1.5)
  }
  cutree <- grDevices::recordPlot()

  return(
    structure(
      list(
        D1 = D1,
        D2 = D2,
        D_alpha = D_alpha,
        alpha = alpha,
        alpha.plot = NULL,
        conf_alpha = NULL,
        tree = tree,
        cluster = cluster,
        cutree = cutree,
        call = match.call(),
        cont = cont,
        wss = q,
        avg_sil = p
      ),
      class = c("perioclust_cl","list")
    )
  )
}

hclustcompro_cutree <- function(tree, k, main="", sub="", xlab="", ylab="", reorder=FALSE, letter=FALSE){
  method <- tree$method
  if(main==""){    main <- "Cluster Dendrogram"  }
  if(sub==""){     sub  <- paste0("method: \"",method,"\"")  }
  if(xlab==""){    xlab <- "d"  }
  if(ylab==""){    ylab <- "Height"  }
  plot(tree,h = -1, main = main, sub = sub, xlab = xlab)
  stats::rect.hclust(tree, k = k, border = colorspace::rainbow_hcl(k, c = 80, l = 60, start = 0, end = 360 * (k-1) / k))
  cluster <- stats::cutree(tree, k = k)

  if(reorder){
    order <- unique(cluster[tree$order])
    for(i in 1:length(order)){
      if(i != order[i] && i < order[i]){
        cluster[cluster == i] <- 0
        cluster[cluster == order[i]] <- i
        cluster[cluster == 0] <- order[i]
        order <- unique(cluster[tree$order])
      }
    }
    if(letter){
      cluster <- unlist(lapply(cluster,Int2Alpha))
    }
    col <- colorspace::rainbow_hcl(length(unique(cluster)),
                      c = 80, l = 60, start = 0,
                      end = 360*(length(unique(cluster))-1)/(length(unique(cluster)))
    )
    for(i in 1:length(unique(cluster))){
      x <- mean(which(cluster[tree$order] == sort(unique(cluster))[i]))
      mtext(unique(cluster[tree$order])[i], side = 1, line = .5, at = x, col = col[i], cex = 1.5)
    }
  }else{
    if(letter){
      cluster <- unlist(lapply(cluster,Int2Alpha))
    }
    col <- colorspace::rainbow_hcl(length(unique(cluster)),
                     c = 80, l = 60, start = 0,
                     end = 360*(length(unique(cluster))-1)/(length(unique(cluster)))
    )
    for(i in 1:length(unique(cluster))){
      x <- mean(which(cluster[tree$order] == unique(cluster[tree$order])[i]))
      mtext(unique(cluster[tree$order])[i], side = 1, line = .5, at = x, col = col[i], cex = 1.5)
    }
  }
  return(cluster)
}


perioclust_subdivide <- function(perioclust_cl,cluster,nb_class){

  # test
  if("perioclust_cl" != class(perioclust_cl)[1]){stop("Not a hclustcompro object.")}
  if(length(cluster) != length(nb_class)){stop("cluster and nb_cluster must have the same length.")}
  title <- paste("Cluster Dendrogram with alpha =",perioclust_cl$alpha)
  subtitle <- paste("The mixing matrix is made of ",arrondi((perioclust_cl$alpha)*100,2),"% of D1 and ",
                    arrondi((1-perioclust_cl$alpha)*100,2),"% of D2.")
  xlab <- paste("D_alpha, method: \"ward.D2\"")
  if(!is.null(perioclust_cl$oldcluster)){
    perioclust_cl$cluster <- perioclust_cl$oldcluster
  }
  k <- length(unique(perioclust_cl$cluster))
  color <- colorspace::rainbow_hcl(k, c = 80, l = 60, start = 0,
                       end = 360*(k-1)/(k))
  plot(perioclust_cl$tree, h=-1, main = title, sub = subtitle, xlab=xlab)
  stats::rect.hclust(perioclust_cl$tree,k = k,border = color)
  clusters <- c()

  cmpt <- 0
  for(i in 1:length(cluster)){
    where <- cluster[i]
    sk <- nb_class[i]
    number <- length(perioclust_cl$cluster[perioclust_cl$cluster == where])
    if(!sk %in% seq(2,(number-1)) ){stop("You try to create more sub cluster than possible.")}
    #col2 <- rainbow_hcl((k+sk-1), c = 80, l = 60, start = 0, end = 360*((k+sk-1)-1)/(k+sk-1)) 
    # add
    col <- colorspace::rainbow_hcl(k, c = 80, l = 60, start = 0,
                                end = 360*(k-1)/(k))
    col <- col[where]
    color <- append(color, rep(col,sk-1), after = where+cmpt)
    cmpt <- cmpt + sk-1
    #end
    if(length(perioclust_cl$cluster[perioclust_cl$cluster == where]) < sk){
      stop("There is not enought element in this class to create ",sk," sub-classes.")
    }
    subdiv <- sub_div(perioclust_cl$tree,perioclust_cl$cluster,where,sk)
    step <- subdiv$step
    clusters <- fusion(clusters,subdiv$cluster)

    #for color 
    nbcluster <- stats::cutree(perioclust_cl$tree,h=perioclust_cl$tree$height[step])
    order <- unique(nbcluster[perioclust_cl$tree$order])
    for(i in 1:length(order)){
      if(i != order[i] && i < order[i]){
        nbcluster[nbcluster == i] <- 0
        nbcluster[nbcluster == order[i]] <- i
        nbcluster[nbcluster == 0] <- order[i]
        order <- unique(nbcluster[perioclust_cl$tree$order])
      }
    }
    
    nb <- which(perioclust_cl$cluster == where)
    nb <- nbcluster[nb]
    which <- sort(unique(nb))
    stats::rect.hclust(perioclust_cl$tree,h = perioclust_cl$tree$height[step+1],which = which,
                border = col)  #col2[seq(where,where+sk-1)])
  }
  #col2 <- rainbow_hcl(length(unique(clusters)), c = 80, l = 60, start = 0,
  #                   end = 360*(length(unique(clusters))-1)/(length(unique(clusters))))
  for(i in 1:length(unique(clusters))){
    col <- color[i]
    x <- mean(which(clusters[perioclust_cl$tree$order] == sort(unique(clusters))[i]))
    # col2[i], cex=1.5)
    graphics::mtext(Int2Alpha(sort(unique(clusters))[i]), side = 1, line = .5, at = x, col = col, cex=1.5)
  }
  cutree <- grDevices::recordPlot()
  return(
    structure(
      list(
        D1 = perioclust_cl$D1,
        D2 = perioclust_cl$D2,
        D_alpha = perioclust_cl$D_alpha,
        alpha = perioclust_cl$alpha,
        tree = perioclust_cl$tree,
        cluster = clusters,
        oldcluster = perioclust_cl$cluster,
        cutree = cutree,
        call = match.call(),
        cont = perioclust_cl$cont
      ),
      class = c("perioclust_cl","list")
    )
  )
}

seriograph <- function(cont, order = NULL, insert = NULL, show = "both", permute = TRUE){

  show <- tolower(show)
  SHOWS <- c("both", "frequency", "eppm")
  i.show <- pmatch(show, SHOWS)
  if (is.na(i.show))
    stop("invalid show method", paste("", show))
  if (i.show == -1)
    stop("ambiguous show method", paste("", show))

  show <- SHOWS[i.show]
  if(show == "eppm"){
    show <- "EPPM"
  }

  if(class(cont)[1] == "perioclust_cl"){
    if(is.null(cont$cont)){
      stop("We don't find contingency table in perioclust_cl object.")
    }else{
      perioclust_cl <- cont
      cont <- perioclust_cl$cont
    }
    remove <- Int2Alpha(sort(unique(perioclust_cl$cluster)))
    if(is.null(order)){
      order <- Int2Alpha(sort(unique(perioclust_cl$cluster)))
    }
    remove <- remove[!remove %in% order]
    if(length(remove) >= 1){
      remove <- Alpha2Int(remove)
    }
    Cluster <- c(0)

    split <- Alpha2Int(order)
    seq <- perioclust_cl$cluster
    if(length(remove) >= 1){
      removeseq <- which(seq %in% remove)
      for(i in 1:length(removeseq)){
        seq[removeseq[i]] <- 900 + i
      }
    }

    c <- 0
    d <- 1
    pelt <- 0
    for (i in 1:length(split)) {
      elt <- as.numeric(split[i])
      felt <- as.numeric(split[i+1])
      if(elt != as.integer(elt)){
        if(as.integer(elt) != as.integer(pelt)){
          c <- c + 1
          d <- 1
        }
        seq[perioclust_cl$cluster == elt] <- as.numeric(paste0(c,".",d))
        d <- d + 1
      }else{
        c <- c + 1
        seq[perioclust_cl$cluster == elt] <- c
        d <- 1
      }
      pelt <- elt
    }
    message("Distribution of elements in the different clusters:\n")

    if(length(remove) >= 1){
      removeseq <- which(seq >= 900)
      seq2 <- seq[-removeseq]
      print(seq2)
    }else{
      print(seq)
    }

    cont <- cont[sort(labels(cont)[[1]]),]
    cont2 <- dplyr::mutate(cont, Cluster = factor(seq))
    cont2 <- dplyr::group_by(cont2, Cluster)
    cont2 <- dplyr::summarise_all(cont2, list(sum=sum))
    cont2 <- as.data.frame(cont2)
    rownames(cont2) <- cont2[,1]
    cont2 <- cont2[,-1]
    if(length(remove) >= 1){
      remove <- which(rownames(cont2) >= 900)
      cont2 <- cont2[-remove,]
    }
    cont2 <- cont2[order(as.numeric(rownames(cont2)),decreasing = T),]
    rownames(cont2) <- order

    if(!is.null(insert)){
      #insert hiatus
      ignore <- FALSE
      if(is.list(insert)){
        if(!is.numeric(insert[[1]])){
          if(!is.numeric(insert[[2]])){
            warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
            ignore <- TRUE
          }else{
            tmp <- insert[[1]]
            insert[[1]] <- insert[[2]]
            insert[[2]] <- tmp
          }
        }
        if(!ignore){
          if(FALSE %in% (insert[[1]] == trunc(insert[[1]])) ){
            warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
            ignore <- TRUE
          }else{
            if(!is.character(insert[[2]])){
              warning("Insert must contain labels as a character vector in the list.")
              ignore <- TRUE
            }
          }
        }
        if(!ignore){
          insert[[1]] <- length(cont2[,1]) - insert[[1]]
          label <- make.names(insert[[2]],unique=TRUE)
          for (i in length(insert[[1]]):1){
            after <- sort(insert[[1]])[i]
            if(after < length(cont2[,1]) && after >= 1 && after == trunc(after)){
              cont2 <- rbind(cont2[1:after,], label = rep(0,length(cont2[1,])),
                             cont2[(after+1):length(cont2[,1]),])
              row.names(cont2)[after+1] <- label[i]
            }else{
              warning("Some positions are outside the range. Ex: ",(length(cont2[,1]) - after))
            }
          }
        }
      }else{
        if(is.vector(insert)){
          if(!is.numeric(insert)){
            warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
            ignore <- TRUE
          }
          if(!ignore){
            if(FALSE %in% (insert[[1]] == trunc(insert[[1]])) ){
              warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
              ignore <- TRUE
            }
          }
          if(!ignore){
            insert <- length(cont2[,1]) - insert
            label <- "Hiatus"
            insert <- sort(insert)
            for (i in length(insert):1){
              if(insert[i] < length(cont2[,1]) && insert[i] >= 1 && insert[i] == trunc(insert[i])){
                cont2 <- rbind(cont2[1:insert[i],], label = rep(0,length(cont2[1,])),
                               cont2[(insert[i]+1):length(cont2[,1]),])
                if(length(insert)== 1){
                  row.names(cont2)[insert[i]+1] <- label
                }else{
                  row.names(cont2)[insert[i]+1] <- paste0(label, ".", (length(insert) + 1 - i) )
                }
              }else{
                warning("Some positions are outside the range. Ex: ",(length(cont2[,1]) - insert[i]))
              }
            }
          }
        }else{
          stop("insert must be a list of two vector (position after which to insert and label)
               or a vector (position)")
        }
      }
    }
    res <- plot_EPPM(cont2,show,permute)
    return(
      structure(
        list(
          seriograph = res$seriograph,
          contingency = res$contingency,
          frequency = res$frequency,
          EPPM = res$EPPM,
          EPM = res$EPM
        ),
        class = "seriograph"
      )
    )

  }else{
    #test data.frame
    if(!is.data.frame(cont)){
      cont <- as.data.frame(cont)
      if(!is.data.frame(cont)){stop("cont must be a data frame.")}
      if(!FALSE %in% (cont == trunc(cont))){stop("A contingency table must contains integers values.")}
    }
    cont2 <- cont

    #order
    if(!is.null(order)){
      new_order <- rep(NA,length(order))

      for(i in 1:length(order)){
        new_order[i] <- which(labels(cont2)[[1]] == order[i])
      }
      cont2 <- cont2[new_order,]

      #for (i in 1:length(order)){
      #  order[i] <- which(rownames(cont2) == order[i])
      #}
      #cont2 <- cont2[order(as.numeric(order),decreasing = F),]
    }

    if(!is.null(insert)){
      #insert hiatus
      ignore <- FALSE
      if(is.list(insert)){
        if(!is.numeric(insert[[1]])){
          if(!is.numeric(insert[[2]])){
            warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
            ignore <- TRUE
          }else{
            tmp <- insert[[1]]
            insert[[1]] <- insert[[2]]
            insert[[2]] <- tmp
          }
        }
        if(!ignore){
          if(FALSE %in% (insert[[1]] == trunc(insert[[1]])) ){
            warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
            ignore <- TRUE
          }else{
            if(!is.character(insert[[2]])){
              warning("Insert must contain labels as a character vector in the list.")
              ignore <- TRUE
            }
          }
        }
        if(!ignore){
          insert[[1]] <- length(cont2[,1]) - insert[[1]]
          label <- make.names(insert[[2]],unique=TRUE)
          for (i in length(insert[[1]]):1){
            after <- sort(insert[[1]])[i]
            if(after < length(cont2[,1]) && after >= 1 && after == trunc(after)){
              cont2 <- rbind(cont2[1:after,], label = rep(0,length(cont2[1,])),
                             cont2[(after+1):length(cont2[,1]),])
              row.names(cont2)[after+1] <- label[i]
            }else{
              warning("Some positions are outside the range. Ex: ",(length(cont2[,1]) - after))
            }
          }
        }
      }else{
        if(is.vector(insert)){
          if(!is.numeric(insert)){
            warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
            ignore <- TRUE
          }
          if(!ignore){
            if(FALSE %in% (insert[[1]] == trunc(insert[[1]])) ){
              warning("Insert must contain integers corresponding to the position where to insert Hiatus.")
              ignore <- TRUE
            }
          }
          if(!ignore){
            insert <- length(cont2[,1]) - insert
            label <- "Hiatus"
            insert <- sort(insert)
            for (i in length(insert):1){
              if(insert[i] < length(cont2[,1]) && insert[i] >= 1 && insert[i] == trunc(insert[i])){
                cont2 <- rbind(cont2[1:insert[i],], label = rep(0,length(cont2[1,])),
                               cont2[(insert[i]+1):length(cont2[,1]),])
                if(length(insert)== 1){
                  row.names(cont2)[insert[i]+1] <- label
                }else{
                  row.names(cont2)[insert[i]+1] <- paste0(label,".",(length(insert)+1 - i))
                }
              }else{
                warning("Some positions are outside the range. Ex: ",(length(cont2[,1]) - insert[i]))
              }
            }
          }
        }else{
          stop("insert must be a list of two vector (position after which to insert and label)
               or a vector (position)")
        }
      }
    }
    res <- plot_EPPM(cont2,show,permute)
    return(
      structure(
        list(
          seriograph = res$seriograph,
          contingency = res$contingency,
          frequency = res$frequency,
          EPPM = res$EPPM,
          EPM = res$EPM
        ),
        class = "seriograph"
      )
    )
  }
}


print.perioclust_cl <- function(x, ...){

  cat("     perioclust | perioclust_cl

Alpha:",x$alpha,"

call: ")
  print(x$call)
  cat("
value:
      .. $D1          The first dissimilarities matrix
      .. $D2          The second dissimilarities matrix
      .. $D_alpha     The mixing dissimilarities matrix
      .. $alpha       The mixing parameter
      .. $tree        An object of class hclust (see ?hclust)

If you ran perioclust like this: classif <- perioclust(...)
select another number of cluster on the plot:
> plot(classif$tree, h = -1)
> rect.hclust(classif$tree, k = k) #with k the number of cluster

documentation:
> ?perioclust
      ")
}

hclustcompro_detail_resampling <- function(D1, D2 = NULL, acc = 2, method = "complete", iter = 5){

  #===============================================================================
  # CAH CLASSIFICATION USING TWO DIST MATRIX
  #
  # Authors : A.COULON, L.BELLANGER and P.HUSI
  # Last update : 23 october 2018
  #
  # Arguments:
  # D1	                  The fisrt distance matrix. Archeological context: stratigraphic distance.
  # D2	                  The second distance matrix. Archeological context: ceramic distance.
  # acc	                  The accuracy to round.
  # method                The method to use in hclust (see doc)
  # iter                   The number of replication for each element
  #
  #===============================================================================
  METHODS <- c("ward.D", "single", "complete", "average", "mcquitty",
               "median", "centroid", "ward.D2")
  if (method == "ward") {
    message("The \"ward\" method has been renamed to \"ward.D\"; note new avaible method \"ward.D2\"")
    method <- "ward.D"
  }
  i.meth <- pmatch(method, METHODS)
  if (is.na(i.meth))
    stop("invalid clustering method", paste("", method))
  if (i.meth == -1)
    stop("ambiguous clustering method", paste("", method))

  method <- METHODS[i.meth]
  if(!is.numeric(acc)) {stop("acc must be numeric!")}
  if(acc < 1) {
    acc <- 1;
    message("The minimal value allowed for acc is 1.")
  }

  if(class(D1)[1] == "perioclust_cl"){
    if(is.null(D1$D1) || is.null(D1$D2)){
      stop("We don't find dissmilarities matrices in perioclust_cl object.")
    }else{
      D2 <- D1$D2
      D1 <- D1$D1
    }
  }else{
    if(is.null(D1) || is.null(D2)){
      stop("We don't find dissmilarities in D1 and D2 arguments.")
    }
    #test
    if(!"dist" %in% class(D1)){
      #matrix ?
      if(!is.matrix(D1)){
        message("generation of a ceramic distances matrix.")
        D1 <- as.matrix(CAdist(D1, nPC="max", graph=FALSE))
      }
      #diag = 0 ?
      if(sum(diag(D1)) != 0){
        stop("D1: in distance matrix the diagonal is equal to 0.")
      }
      #symetric ?
      if(!isSymmetric(D1)){
        stop("D1 is not symmetric.")
      }
    }
    if(!"dist" %in% class(D2)){
      #matrix ?
      if(!is.matrix(D2)){
        if(length(D2[1,]) == 3){
          message("generation of a temporal dissimilarities matrix.")
          D2 <- overlap(D2)
        }else{
          message("generation of a stratigraphic dissimilarities matrix.")
          D2 <- adjacency(D2)
          #diag = 0 ?
          if(sum(diag(D2)) != 0){
            stop("D2: The diagonal must be equal to 0.")
          }
        }
      }

      #symetric ?
      if(!isSymmetric(D2)){
        stop("D2 is not symmetric.")
      }
    }
    if(!length(D1[,1]) == length(D1[1,])){stop("D1 not a square matrix.")}
    if(!length(D2[,1]) == length(D2[1,])){stop("D2 not a square matrix.")}
    #dist (positive matrix) ?
    if(min(D1) < 0){stop("D1 not a matrix dissimilarity (positive).")}
    if(min(D2) < 0){stop("D2 not a matrix distance (positive).")}
    if(median(D2) != 0 && median(D2) != 1){
      if(median(D1) == 0 || median(D1) == 1){
        message("We switch the two matrices. Stratigraphic data must be in D1 and ceramic data in D2.")
        tmp <- D1
        D1 <- D2
        D2 <- tmp
      }
    }
    #normalization
    if(max(D1) != 0){
      D1 <- D1 / max(D1)
    }
    if(max(D2) != 0){
      D2 <- D2 / max(D2)
    }
    #create mixing matrix
    D1 <- as.dist(D1)
    D2 <- as.dist(D2)
    D1 <- as.matrix(D1)
    D2 <- as.matrix(D2)
    #sort labels
    D1.Tmp <- D1[sort(labels(D1)[[1]]), sort(labels(D1)[[1]])]
    D2.Tmp <- D2[sort(labels(D2)[[1]]), sort(labels(D2)[[1]])]
    D1 <- as.matrix(D1.Tmp)
    D2 <- as.matrix(D2.Tmp)
  }

  save_lines <- NULL
  alpha_seq <- seq(0,1,0.01)

  D1save <- as.matrix(D1)
  D2save <- as.matrix(D2)

  #echantillonnage
  if(iter == "max"){iter <- length(D1save[1,])}
  if(!is.numeric(iter)){stop("iter must be numeric.")}
  if(iter < 1){iter <- 1; message("iter must be higher than 1.")
  warning("iter: must be between 1 and the number of elements.")}
  if(iter > length(D1save[1,])-1){
    if(iter > length(D1save[1,])){
      message("iter can't be higher than the number of element: ",length(D1save[1,]),".")
      warning("iter must be between 1 and the number of elements.")
    }
    iter <- length(D1save[1,])-1
  }

  cat("Resampling process:\n")
  #creation barre de progression(pb)
  pb <- utils::txtProgressBar(min = 0, max = length(D1save[1,]), style = 3)
  df <- NULL
  for (i in 1:length(D1save[1,])){
    for(j in sample((1:(length(D1save[,1])))[-i],iter,replace=FALSE)){
      if(iter == 1 && i == j){
        if(j == 1){j <- j + 1}else{j <- j - 1}
      }
      list <- generateClone(D1save, D2save, i, j)
      D1 <- list[[1]]
      D2 <- list[[2]]
      dist.dend <- c()
      for(alpha in alpha_seq){
        Mdist <- (alpha) * D1 + (1-alpha) * D2
        mixt.dist <- as.dist(Mdist)
        tree <- fastcluster::hclust(mixt.dist, method=method)
        new <- corCriterion(tree, D1, D2)
        dist.dend <- c(dist.dend, new)
      }
      if(is.null(df)){
        df <- data.frame(alpha = alpha_seq, height = dist.dend)
      }else{
        df <- cbind(df,dist.dend)
      }
    }
    mean <- unlist(lapply(as.data.frame(t(df[-1])),mean))
    if(is.null(save_lines)){
      save_lines <- cbind(df$alpha,mean)
    }else{
      save_lines <- cbind(save_lines,mean)
    }
    df <- NULL
    utils::setTxtProgressBar(pb, i)
  }
  close(pb)

  colnames(save_lines) <- c("alpha",rownames(D1save))
  rownames(save_lines) <- c()

  p <- plotly::plot_ly()
  for (k in 2:length(colnames(save_lines))) {
    p <- plotly::add_lines(p,x=save_lines[,1],y=save_lines[,k],name=colnames(save_lines)[k])
  }
  p
}

print.selectAlpha_obj <- function(x, ...){

  cat("        select Alpha | selectAlpha_obj

Estimated alpha:",x$alpha,"

call: ")
  print(x$call)
  cat("
value:
      .. $alpha             The estimate value of the parameter alpha (min CorCrit_alpha).
      .. $alpha.plot        The CorCrit for all the possible alpha.
    if resampling = TRUE
      .. $sd                The standard deviation.
      .. $conf              The confidence interval of alpha.
      .. $boxplot           The boxplot of the distribution.
      .. $values            All the potential alpha values obtained from clones.

documentation:
> ?hclustcompro_select_alpha
")
}

summary.selectAlpha_obj <- function(object, ...){
  cat("Estimate Alpha:",object$alpha,"    ( sd:",object$sd,")
Confidence interval: [",object$conf[1],";",object$conf[2],"]

Estimation calculated with",NROW(object$values),"observations.

?hclustcompro_select_alpha for details.
")
}

print.seriograph <- function(x, ...){
  cat("")
}

hclustcompro_select_alpha <- function(D1,D2, acc = 2, method = "complete", resampling = TRUE, iter = 5, suppl_plot = TRUE){

  #===============================================================================
  # CAH CLASSIFICATION USING TWO OR MORE DIST MATRIX
  #
  # Authors : A.COULON, L.BELLANGER and P.HUSI
  # Last update : 20 octobre 2025
  #
  # Arguments:
  # D1	                  The fisrt distance matrix.
  # D2	                  The second distance matrix.
  # acc             	    The accuracy to round.
  # resampling            Logical if we have to calculate confidence interval with resampling strategy
  # method                The method to use in hclust (see doc)
  #
  #===============================================================================

  METHODS <- c("ward.D", "single", "complete", "average", "mcquitty",
               "median", "centroid", "ward.D2")
  if (method == "ward") {
    message("The \"ward\" method has been renamed to \"ward.D\"; note new avaible method \"ward.D2\"")
    method <- "ward.D"
  }
  i.meth <- pmatch(method, METHODS)
  if (is.na(i.meth))
    stop("invalid clustering method", paste("", method))
  if (i.meth == -1)
    stop("ambiguous clustering method", paste("", method))
  method <- METHODS[i.meth]

  cor_ <- function(alpha) {
    c1 <- c()
    c2 <- c()
    for (elt in alpha){
      Mdist <- (elt)*D1 + (1-elt)*D2
      mixt.dist <- as.dist(Mdist)
      a_tree <- fastcluster::hclust(mixt.dist, method=method)
      da <- stats::cophenetic(a_tree)
      c1 <- c(c1, cor(as.dist(D1), da))
      c2 <- c(c2, cor(as.dist(D2), da))
    }
    return(list(c1,c2))
  }

  corCriterion_clone <- function(alpha, a, b) {
    res <- c()
    D1bis <- rbind(D1,clone=D1[a,])
    D1bis <- cbind(D1bis,clone=c(D1[a,],0))
    D2bis <- rbind(D2,clone=D2[b,])
    D2bis <- cbind(D2bis,clone=c(D2[b,],0))
    for (elt in alpha){
      Mdist <- (elt)*D1bis + (1-elt)*D2bis
      mixt.dist <- as.dist(Mdist)
      a_tree <- fastcluster::hclust(mixt.dist,method=method)
      da <- stats::cophenetic(a_tree)
      res <- c(res,abs(cor(as.dist(D1bis),da) - cor(as.dist(D2bis),da)))
    }
    return(res)
  }

  if(!is.numeric(acc)) {stop("acc must be numeric!")}
  if(acc < 1) {
    acc <- 1;
    message("The minimal value allowed for acc is 1.")
  }
  if(acc > 3) {
    message("If this take too much time reduce the acc. Try default acc=2.")
  }
  seq_acc <- 10^(-acc)
  #test
  if(!"dist" %in% class(D1)){
    #matrix ?
    if(!is.matrix(D1)){
      message("generation of distances matrix.")
      D1 <- as.matrix(CAdist(D1, nPC="max", graph=FALSE))
    }
    #diag = 0 ?
    if(sum(diag(D1)) != 0){
      stop("D1: The diagonal must be 0.")
    }
    #symetric ?
    if(!isSymmetric(D1)){
      stop("D1 is not symmetric.")
    }
    if(!length(D1[,1]) == length(D1[1,])){stop("D1 not a square matrix.")}
  }
  if(!"dist" %in% class(D2)){
    #matrix ?
    if(!is.matrix(D2)){
      if(length(D2[1,]) == 3){
        message("generation of a temporal dissimilarities matrix.")
        D2 <- overlap(D2)
      }else{
        message("generation of a stratigraphic dissimilarities matrix.")
        D2 <- adjacency(D2)
        #diag = 0 ?
        if(sum(diag(D2)) != 0){
          stop("D2: The diagonal must be 0.")
        }
      }
    }
    #symetric ?
    if(!isSymmetric(D2)){
      stop("D2 is not symmetric.")
    }
    if(!length(D2[,1]) == length(D2[1,])){stop("D2 not a square matrix.")}
  }

  D1 <- as.matrix(D1)
  D2 <- as.matrix(D2)
  #dist (positive matrix) ?
  if(min(D1) < 0){stop("D1 not a dissimilarity matrix (positive).")}
  if(min(D2) < 0){stop("D2 not a distance matrix (positive).")}
  if(median(D2) != 0 && median(D2) != 1){
    if(median(D1) == 0 || median(D1) == 1){
      message("We switch the two matrices. Stratigraphic data must be in D1 and ceramic data in D2.")
      tmp <- D1
      D1 <- D2
      D2 <- tmp
    }
  }
  #normalization
  if(max(D1) != 0){
    D1 <- D1 / max(D1)
  }
  if(max(D2) != 0){
    D2 <- D2 / max(D2)
  }
  #create mixing matrix
  D1 <- as.dist(D1)
  D2 <- as.dist(D2)
  D1 <- as.matrix(D1)
  D2 <- as.matrix(D2)
  #sort labels
  D1.Tmp <- D1[sort(labels(D1)[[1]]), sort(labels(D1)[[1]])]
  D2.Tmp <- D2[sort(labels(D2)[[1]]), sort(labels(D2)[[1]])]
  D1 <- as.matrix(D1.Tmp)
  D2 <- as.matrix(D2.Tmp)

  #resampling check
  if(iter == "max"){iter <- length(D1[1,])}
  if(!is.numeric(iter)){stop("iter must be numeric.")}
  if(!is.logical(resampling)){stop("resampling must be TRUE or FALSE.")}
  if(iter < 1){iter <- 1; message("iter must be higher than 1.")
  warning("iter: must be between 1 and the number of elements.")}
  if(iter > length(D1[1,])-1){
    if(iter > length(D1[1,])){
      message("iter can't be higher than the number of element: ",length(D1[1,]),".")
      warning("iter: must be between 1 and the number of elements.")
    }
    iter <- length(D1[1,])-1
  }

  if(suppl_plot){
    #estime alpha
    ## TODO: use cor_ and calc corcrit
    correlation_curve <- cor_(seq(0, 1, seq_acc))
    #corCrit <- c()
    #for(i in seq(0,1,seq_acc)){corCrit <- c(corCrit,corCriterion_(i))}
    corCrit <- abs(correlation_curve[[1]] - correlation_curve[[2]])

    index <- which(corCrit == min(corCrit))
    alpha <- seq(0,1,seq_acc)[index]

    add_text_resamp <- ""
    if(resampling){
      #echantillonnage
      cat("\nResampling process:\n")
      pb <- utils::txtProgressBar(min = 0, max = length(D1[1,]), style = 3)
      set <- c()
      res <- c()
      for (i in 1:length(D1[1,])){
        for(j in sample((1:(length(D1[,1])))[-i], iter, replace=FALSE)){
          if(i!=j){
            tmp <- stats::optimize(corCriterion_clone, lower=0, upper=1, a=i, b=j)
            set <- c(set, tmp$minimum)
          }
        }
        utils::setTxtProgressBar(pb, i)
      }

      res <- set
      sd <- arrondi(sd(res), acc)
      box <- quantile(res, c(0,0.025,0.25,0.5,0.75,0.975,1))
      conf <- c(arrondi(box[2], acc), arrondi(box[6], acc))
      values <- arrondi(res, acc)

      BoxPlotValue <- boxplot(
        res,
        horizontal = TRUE,
        range = 0,
        main="boxplot: Distribution of all the potentials values of alpha.",
        xlab=expression(alpha),
        xaxt="n"
      )
      axis(
        side = 1,
        at = c(box[1], box[3], box[4], box[5], box[7]),
        labels = arrondi(c(box[1], box[3], box[4], box[5], box[7]), acc)
      )
      points(arrondi(mean(res), acc), 1, col="red")
      text(arrondi(mean(res),acc), 1.3,
           bquote(hat(alpha)^"*" == .(arrondi(mean(res),acc))))
      boxplot <- grDevices::recordPlot()

      add_text_resamp <- paste(" The IC95% is calculated with",iter,'*',length(D1[1,]),
                               "clones out of",length(D1[1,])-1,"*",length(D1[1,]),"available.")
    }

    ## TODO: re-use calc corcrit instead of running twice corCriterion_
    plot(
      seq(0,1,seq_acc),corCrit,type="l",
      xlab = expression(alpha),
      ylab = expression(CorCrit[alpha]),
      ylim=c(0,1),
      axes = TRUE,
      main = "Select alpha",
      sub = paste0("method: ", method, ".", add_text_resamp),
      xaxp = c(0, 1, 5),
      mgp= c(2, 1, 0),
      col = "slateblue2",
      xaxt= "n",
      lwd=1
    )
    ## TODO: re-use value from calc corcrit
    #correlation_curve <- cor_(seq(0,1,seq_acc))
    lines(seq(0, 1, seq_acc), correlation_curve[[1]], lty=3, col="grey60")
    lines(seq(0, 1, seq_acc), correlation_curve[[2]], lty=3, col="grey60")
    axis(
      side = 1,
      at = c(.1,.2,.3,.4,.5,.6,.7,.8,.9),
      labels = rep("", 9),
      col = "grey"
    )
    axis(
      side = 1,
      at = arrondi(alpha, acc),
      labels = rep("", length(alpha)),
      col = "slateblue2"
    )
    if(resampling){
      axis(
        side = 1,
        at = c(0, arrondi(box[2], acc), arrondi(box[6], acc), 1),
        labels = c(0, arrondi(box[2], acc), arrondi(box[6], acc), 1)
      )
      abline(v = arrondi(box[6], acc), col = "gray50", lty=3, lwd=2)
      abline(v = arrondi(box[2], acc), col = "gray50", lty=3, lwd=2)
      close(pb)
    }else{
      axis(
        side = 1,
        at = c(0,1),
        labels = c(0,1)
      )
    }

    for(i in 1:length(alpha)){
      graphics::mtext(arrondi(alpha[i],acc),side = 1,line = .4,at = arrondi(alpha[i],acc),col = "slateblue2")
    }
    alpha.plot <- grDevices::recordPlot()

    if(resampling){
      return(structure(
        list(alpha = arrondi(alpha, acc),
             alpha.plot = alpha.plot,
             sd = sd,
             conf = conf,
             boxplot = boxplot,
             values = values
        ),
        class = c("selectAlpha_obj","list")
      )
      )
    }else{
      return(structure(
        list(alpha = arrondi(alpha,acc),
             alpha.plot = alpha.plot,
             more = "Need resampling = TRUE!"
        ),
        class = c("selectAlpha_obj", "list")
      )
      )
    }
  }else{

    #estime alpha
    correlation_curve <- cor_(seq(0, 1, seq_acc))
    #corCrit <- c()
    #for(i in seq(0,1,seq_acc)){corCrit <- c(corCrit,corCriterion_(i))}
    corCrit <- abs(correlation_curve[[1]] - correlation_curve[[2]])

    index <- which(corCrit == min(corCrit))
    alpha <- seq(0, 1, seq_acc)[index]

    if(resampling){
      pb <- utils::txtProgressBar(min = 0, max = length(D1[1,]), style = 3)
      set <- c()
      res <- c()
      for (i in 1:length(D1[1,])){
        for(j in sample((1:(length(D1[,1])))[-i], iter, replace=FALSE)){
          if(i!=j){
            tmp <- stats::optimize(corCriterion_clone, lower=0, upper=1, a=i, b=j)
            set <- c(set, tmp$minimum)
          }
        }
        utils::setTxtProgressBar(pb, i)
      }
      conf <- arrondi(quantile(set, c(0.025,0.975)), acc)
      close(pb)
      return(structure(
        list(alpha = arrondi(alpha, acc),
             conf = conf,
             more = "Need suppl_plot = TRUE!"
        ),
        class = c("selectAlpha_obj","list")
      )
      )
    }else{
      return(structure(
        list(alpha = arrondi(alpha, acc),
             more = "Need suppl_plot = TRUE!"
        ),
        class = c("selectAlpha_obj","list")
      )
      )
    }


  }

}

optim_select_alpha <- function(data = list(), acc = 2, method = "complete", algo = "NLOPT_GN_ORIG_DIRECT_L"){

  #============================================================================
  # Optimisation (NLOPT) des poids de chaque source
  #
  # Authors : A.COULON
  # Last update : 25 november 2025
  #
  # Arguments:
  # data      liste des matrices de distances
  # acc       nombre de chiffre après la virgule
  # method    stratégie d'aggrégation utilisée par hclust()
  #
  #============================================================================

  ALGO <- c("NLOPT_GN_ISRES", "NLOPT_GN_ORIG_DIRECT_L")
  i.algo <- pmatch(algo, ALGO)
  if (is.na(i.algo))
    stop("invalid algorithm", paste("", algo))
  if (i.algo == -1)
    stop("ambiguous algorithm", paste("", algo))
  algo <- ALGO[i.algo]

  METHODS <- c("ward.D", "single", "complete", "average", "mcquitty",
               "median", "centroid", "ward.D2")
  if (method == "ward") {
    message("The \"ward\" method has been renamed to \"ward.D\"; note new avaible method \"ward.D2\"")
    method <- "ward.D"
  }
  i.meth <- pmatch(method, METHODS)
  if (is.na(i.meth))
    stop("invalid clustering method", paste("", method))
  if (i.meth == -1)
    stop("ambiguous clustering method", paste("", method))
  method <- METHODS[i.meth]

  if(!is.numeric(acc)) {stop("acc must be numeric!")}
  if(acc < 1) {
    acc <- 1;
    message("The minimal value allowed for acc is 1.")
  }

  ## number of sources
  K <- length(data)

  for(i in 1:K){

    #test
    if(!"dist" %in% class(data[[i]])){
      #matrix
      if(!"matrix" %in% class(data[[i]])){
        stop("data contain objets that are neither matrix nor dist objet.")
      }
      #diag = 0 ?
      if(sum(diag(data[[i]])) != 0){
        stop("Distance matrix diagonal must be 0.")
      }
      #symetric ?
      if(!isSymmetric(data[[i]])){
        stop("data is not symmetric.")
      }
      if(!length(data[[i]][,1]) == length(data[[i]][1,])){stop("Not a square matrix.")}
    }

    #normalization of matrix
    if(max(data[[i]]) != 0){
      data[[i]] <- data[[i]] / max(data[[i]])
    }
    #sort labels
    data[[i]] <- as.matrix(data[[i]])
    D_Tmp <- data[[i]][sort(labels(data[[i]])[[1]]),sort(labels(data[[i]])[[1]])]
    data[[i]] <- as.dist(D_Tmp)
  }

  ## objective function
  eval_f0 <- function(x, data, method) {
    x_tmp <- c(x, 1-sum(x) )

    ## Map multiply the weight (here x_tmp) on each matrix (here data), result is a list of weighted matrices
    DIST <- Map(`*`, data, x_tmp)
    ## Reduce apply + operator to the first two matrices of the list, then adds the thirds, and so on (sum)
    DIST <- Reduce(`+`, DIST)

    ## hclust and cophenetic distances
    tree <- stats::hclust(d=as.dist(DIST), method=method)
    D_coph <- stats::cophenetic(tree)

    ## correlation between each sources and D_coph
    correlation <- sapply(data, cor, y = D_coph)
    ## symetric matrix: absolute difference between each sources
    diff_matrix <- abs(outer(correlation, correlation, `-`))
    ## lower triangle
    diff_matrix_triangle <- diff_matrix[lower.tri(diff_matrix, diag=F)]
    ## sum of each abs diff between sources
    res <- sum( diff_matrix_triangle )

    return(res)
  }

  ## constraint function
  eval_g0 <- function(x, data, method) {
    ## Inequality: sum(x) < 1
    # g(x) must be < 0 when good -> so, sum(x)-1 < 0
    return(sum(x)-1)
  }

  # note: we are searching for x values (weight), x length is K-1, (K: number of sources)
  # because the last weight is deduced by 1-sum(x)
  res <- nloptr::nloptr(
    x0 = rep(1/(K), K-1),   ## weight 1/K for each of K-1 first sources
    eval_f = eval_f0,       # i.e. for 2 sources: rep(1/2, 1); for 3 rep(1/3, 2)...
    lb = rep(0, K-1),
    ub = rep(1, K-1),
    eval_g_ineq = eval_g0,
    opts = list("algorithm" = algo, ## NLOPT_GN_ISRES NLOPT_GN_ORIG_DIRECT_L
                "xtol_rel" = 1.0e-3,
                "maxeval" = 10000),
    data = data,
    method = method
  )

  ## first K-1 weight and the last (1-sum(x))
  solution = c(res$solution, 1 - sum(res$solution))

  return(list(
      res_nlopt = res,
      solution = arrondi(solution, acc)
    )
  )
}

timerange <- function(data, cluster = NULL, add = NULL, density = NULL, color = NULL, reorder = FALSE){

  ## test
  if(is.null(cluster)){cluster <- rep(1,dim(data)[1])}
  if(!is.data.frame(data)){stop("data must be a data.frame.")}
  if(!is.null(add)){
    if(!is.data.frame(add)){stop("add must be a data.frame (even if there is only one column).")}
  }
  if(!is.vector(cluster)){stop("cluster must be a vector.")}
  if(!is.null(density)){
    if(!is.vector(density)){stop("cluster must be a vector.")}
  }

  ## call fonction overlap plot
  df <- cbind(data,cluster)
  res <- overlap_plot(df,add,density,color,reorder_color = reorder)

  return(structure(list(order = res$order, plot = res$plot, density = res$density, cluster = res$reorder_cluster), class = c("temp_obj", "list")))
}


hclustcompro <- function(d, method = "complete", weight = NULL){

  #============================================================================
  # hclustcompro multivues
  #
  # Authors : A.COULON
  # Last update : 25 november 2025
  #
  #============================================================================

  METHODS <- c("ward.D", "single", "complete", "average", "mcquitty",
               "median", "centroid", "ward.D2")
  if (method == "ward") {
    message("The \"ward\" method has been renamed to \"ward.D\"; note new avaible method \"ward.D2\"")
    method <- "ward.D"
  }
  i.meth <- pmatch(method, METHODS)
  if (is.na(i.meth))
    stop("invalid clustering method", paste("", method))
  if (i.meth == -1)
    stop("ambiguous clustering method", paste("", method))
  method <- METHODS[i.meth]

  if("list" %in% class(d)){
    ## number of sources
    K <- length(d)

    ## If the last weight is missing we can deduce him (as sum(weight)=1)
    if(length(weight) == K-1){
      weight = c(weight,1-sum(weight))
    }

    if(length(weight) != K){
      message("weight were missing or not the same length as d. We calculate them base on corcrit.")
      weight = optim_select_alpha(d,2,method)$solution
    }

    if(sum(weight)!=1){
      message("sum of weight not equal to 1. We recalculate them base on corcrit.")
      weight = optim_select_alpha(d,2,method)$solution
    }

    for(i in 1:K){

      #test
      if(!"dist" %in% class(d[[i]])){
          stop("d not a dist object or a list of dist object.")
      }

      #normalization of matrix
      if(max(d[[i]]) != 0){
        d[[i]] <- d[[i]] / max(d[[i]])
      }
      #sort labels
      d[[i]] <- as.matrix(d[[i]])
      D_Tmp <- d[[i]][sort(labels(d[[i]])[[1]]),sort(labels(d[[i]])[[1]])]
      d[[i]] <- as.dist(D_Tmp)
    }

    ## Map multiply the weight on each matrix (here list d), result is a list of weighted matrices
    DIST <- Map(`*`, d, weight)
    ## Reduce apply + operator to the first two matrices of the list, then adds the thirds, and so on (sum)
    DIST <- Reduce(`+`, DIST)

    ## hclust and cophenetic distances
    d=as.dist(DIST)
  }
  fastcluster::hclust(d, method)
}


hclust <- function(d, method = "complete", members = NULL, weight = NULL){

  #============================================================================
  # hclustcompro multivues
  #
  # Authors : A.COULON
  # Last update : 25 november 2025
  #
  #============================================================================

  METHODS <- c("ward.D", "single", "complete", "average", "mcquitty",
               "median", "centroid", "ward.D2")
  if (method == "ward") {
    message("The \"ward\" method has been renamed to \"ward.D\"; note new avaible method \"ward.D2\"")
    method <- "ward.D"
  }
  i.meth <- pmatch(method, METHODS)
  if (is.na(i.meth))
    stop("invalid clustering method", paste("", method))
  if (i.meth == -1)
    stop("ambiguous clustering method", paste("", method))
  method <- METHODS[i.meth]

  if("list" %in% class(d)){
    ## number of sources
    K <- length(d)

    ## If the last weight is missing we can deduce him (as sum(weight)=1)
    if(length(weight) == K-1){
      weight = c(weight,1-sum(weight))
    }

    if(length(weight) != K){
      message("weight were missing or not the same length as d. We calculate them base on corcrit.")
      weight = optim_select_alpha(d,2,method)$solution
    }

    if(sum(weight)!=1){
      message("sum of weight not equal to 1. We recalculate them base on corcrit.")
      weight = optim_select_alpha(d,2,method)$solution
    }

    for(i in 1:K){

      #test
      if(!"dist" %in% class(d[[i]])){
          stop("d not a dist object or a list of dist object.")
      }

      #normalization of matrix
      if(max(d[[i]]) != 0){
        d[[i]] <- d[[i]] / max(d[[i]])
      }
      #sort labels
      d[[i]] <- as.matrix(d[[i]])
      D_Tmp <- d[[i]][sort(labels(d[[i]])[[1]]),sort(labels(d[[i]])[[1]])]
      d[[i]] <- as.dist(D_Tmp)
    }

    ## Map multiply the weight on each matrix (here list d), result is a list of weighted matrices
    DIST <- Map(`*`, d, weight)
    ## Reduce apply + operator to the first two matrices of the list, then adds the thirds, and so on (sum)
    DIST <- Reduce(`+`, DIST)

    ## hclust and cophenetic distances
    d=as.dist(DIST)
  }
  fastcluster::hclust(d, method, members)
}

cerardat = function(df, row.sup, date, nf = NULL, confidence = 0.95, graph = T){

  ## test
  # df is data.frame
  if(!is.data.frame(df)){stop("df is not a data frame.")}
  # row.sup is vector
  if(!is.vector(row.sup)){stop("row.sup is not a vector.")}
  # row.sup is interger
  #if(!is.integer(row.sup)){stop("row.sup is not integer.")}
  # date is vector
  if(!is.vector(date)){stop("date is not a vector.")}
  # date is na or interger
  #if(!is.integer(date)){stop("date is not integer.")}
  # nf is integerdate
  #if(!is.integer(nf)){stop("nf is not integer.")}
  # max(row.sup) < 280 size df
  if(max(row.sup) > dim(df)[1]){stop("row.sup must contain integers between 1 and the number of lines in df.")}
  # min(row.sup) > 0
  if(min(row.sup) <= 0){stop("row.sup must contain integers between 1 and the number of lines in df.")}
  # length(date) = nrow(df)
  if(!length(date) == nrow(df)){stop("date must have the same number of observations as the number of lines in df. Complete with NA if necessary.")}
  # nf > 0
  if(!is.null(nf)){
    if(nf < 0){stop("nf must be positive.")}
  }

  # confidence < 1
  if(confidence > 0.99){stop("Confidence must be between 0 and 0.99.")}
  # confidence >= 0
  if(confidence < 0){stop("Confidence must be between 0 and 0.99.")}

  lwr = 0
  upr = 0

  #vector for ref data (every not in row.sup)
  row.ref = which(!(1:length(df[,1]) %in% row.sup))

  df_ref = df[row.ref,]
  df_sup = df[row.sup,]
  date = c(date[row.ref],date[row.sup])
  rownames = c(dimnames(df)[1][row.ref],dimnames(df)[1][row.sup])

  clean = TRUE
  GT_rm_names = c()
  row_rm_names = c()
  while(clean){
    #nettoyage des GT ie GT<5
    GT_rm = which(colSums(df_ref)<5)
    if(sum(GT_rm)!=0){
      GT_rm_names = c(GT_rm_names,names(df_ref)[which(colSums(df_ref)<5)])
      df_ref = df_ref[,-c(GT_rm)]
      df_sup = df_sup[,-c(GT_rm)]
    }

    #si row < 5
    df_tmp = rbind(df_ref,df_sup)
    row_rm = which(rowSums(df_tmp)<5)
    if(sum(row_rm)!=0){
      row_rm_names = c(row_rm_names,which(rowSums(df_tmp)<5))
      df_ref = df_ref[!rowSums(df_ref)<5,]
      df_sup = df_sup[!rowSums(df_sup)<5,]
      date = date[-c(row_rm)]
      rownames = rownames[-c(row_rm)]
    }

    if(sum(GT_rm)==0 & sum(row_rm)==0){
      clean = FALSE
    }
  }

  row.ref = 1:length(df_ref[,1])
  row.sup = (length(df_ref[,1])+1):(length(df_ref[,1])+length(df_sup[,1]))

  data_ref = df_ref
  data_sup = df_sup

  if(length(row_rm_names)>0){
    warning(paste0("The sums of ",length(row_rm_names)," rows are less than 5. They were suppressed from the analysis."))
  }
  if(length(GT_rm_names)>0){
    warning(paste0("The sums of ",length(GT_rm_names)," columns in the reference data are less than 5. They were suppressed from the analysis."))
  }

  #DOF Degrees of freedom
  DOF = min(ncol(data_ref)-1,nrow(data_ref)-1)

  ## test
  # nf <= DOF

  if(is.null(nf)){
    k=cerardat_estim_nf(rbind(data_ref,data_sup), row.sup, date)$nf[1]
  }else{
    k=nf
  }

  #Correspondance analysis
  ca.NMI <- ade4::dudi.coa(data_ref, scannf=FALSE, nf=k)


  res.CA = FactoMineR::CA(data_ref, ncp = k, graph = graph)

  #passer par FactoMineR::CA()
  obs_ca_eval = res.CA$row$cos2
  #obs_ca_eval = t(rowSums(res.CA$col$cos2))

  #df with coord ens (col) de reference + date monnaie
  DATA_REF = cbind(ca.NMI$li, date=date[row.ref])
  #here

  #genrate col coord for sup data
  data_sup_proj_col = ade4::suprow(ca.NMI, data_sup)$lisup
  #df with coord GT (col) supplementaire + date monnaie
  DATA_SUP = cbind(data_sup_proj_col, date=date[row.sup])

  #final data
  DATA_TOT = rbind(DATA_REF, DATA_SUP)
  #DATA_TOT = DATA_TOT[sort(c(row.ref,row.sup)),]

  #linear regression
  #DATA_REF_lm = MASS::steapAIC(lm(date~., data=DATA_REF, na.action=na.omit),trace=0)
  DATA_REF_lm = lm(date~.,data=DATA_REF, na.action=na.omit)

  R_adj = arrondi(summary(DATA_REF_lm)$adj.r.squared,3)
  R_sq = arrondi(summary(DATA_REF_lm)$r.squared,3)
  sigma = arrondi(summary(DATA_REF_lm)$sigma,2)

  if(graph == 1){
    plot(DATA_REF_lm,which=c(1))
    plot(DATA_REF_lm,which=c(2))
    MASS::boxcox(DATA_REF_lm,data=DATA_REF)
  }


  ## residus student
  mod1.rstudent<-as.data.frame(which(abs(rstudent(DATA_REF_lm))>2))
  dimnames(mod1.rstudent)[[2]]<-c("abs(rstudent)")

  # H0 Normalite des residus : Shapiro-Wilks Test
  Shapiro=shapiro.test(rstudent(DATA_REF_lm))
  if (Shapiro$p.value<0.05) {warning("The Shapiro-Wilks test indicates a problem with the normality of the residuals.",sep="")}
  # H0 : pas Autocorrelation D-W  test
  DW=lmtest::dwtest(date~.,data=DATA_REF,alternative = "two.sided")
  if (DW$p.value<0.05) {warning("The Durbin-Watson test indicates a first order autocorrelation problem.",sep="")}
  # H0 : Homoscedasticite, B-P test
  BP=lmtest::bptest(date~.,data=DATA_REF)
  if (BP$p.value<0.05) {warning("The Breusch-Pagan test indicates a heteroskedasticity problem.",sep="")}

  #eval model
  mod1.diagGl = data.frame(
    cbind(
      R_adj, R_sq, sigma,
      arrondi(shapiro.test(rstudent(DATA_REF_lm))$p.value,3),
      arrondi(lmtest::dwtest(date~.,data=DATA_REF, alternative = "two.sided")$p.value, 3),
      arrondi(lmtest::bptest(date~.,data=DATA_REF)$p.value,3)
    )
  )
  dimnames(mod1.diagGl)[[2]]<-c("R2_aj", "R2", "sigma", "Shapiro p-value", "D-W p-value", "B-P p-value")

  #prediction date ensemble
  predict_obj_row = predict(DATA_REF_lm, newdata=DATA_TOT, se.fit=TRUE, interval="confidence", level=confidence)

  #prediction date GT
  dimnames(ca.NMI$co)[[2]] = dimnames(ca.NMI$li)[[2]]
  predict_obj_col = predict(DATA_REF_lm, newdata=ca.NMI$co, se.fit=TRUE, interval="confidence", level=confidence)

  ##### 95% des dates des GT
  date_gt = arrondi(predict_obj_col$fit,0)
  date_gt = cbind(colSums(df_ref),date_gt)
  dimnames(date_gt)[[2]]<-c("Tot_count","Fit_dateEv","lower","upper")

  #matrix proportion GT
  cont = rbind(data_ref,data_sup)


  cont.gt = cont

  for(j in 1:nrow(cont))
  {
    cont.gt[j,]<-cont[j,]/as.matrix(rowSums(cont))[j]
  }

  median.norMix <- function(x) nor1mix::qnorMix(1/2,x)

  median_tab = as.data.frame(matrix(0, nrow(cont),3))
  dimnames(median_tab)[[1]] = dimnames(cont)[[1]]
  dimnames(median_tab)[[2]] = c("Median_dateAc","lower","upper")

  date_gt_sd = arrondi(cbind(as.data.frame(predict_obj_col$fit)[,1], as.data.frame(predict_obj_col$se.fit)),1)

  for(i in 1:nrow(cont))
  {
    ex = nor1mix::norMix(mu = date_gt_sd[,1],w=unlist(as.vector(cont.gt[i,])),sigma= date_gt_sd[,2])
    median_tab[i,] = cbind(median.norMix(ex), nor1mix::qnorMix(0.025,ex), nor1mix::qnorMix(0.975,ex))
  }
  cont_cat = cont
  cont_cat[!cont_cat == 0] = 1
  #here
  date_predict = arrondi(cbind(
    rowSums(cont),
    rowSums(cont_cat),
    date[c(row.ref,row.sup)],
    predict_obj_row$fit,
    median_tab
  ),0)
  dimnames(date_predict)[[1]]<-dimnames(predict_obj_row$fit)[[1]]
  dimnames(date_predict)[[2]]<-c("Tot_count","Category_number","date","Fit_dateEv","lower_Ev","upper_Ev","Median_dateAc","lower_Ac","upper_Ac")

  tmp = date_predict[row.ref,]
  tmp = tmp[!is.na(tmp$date),]

  df = data.frame(
    names = rownames(tmp),
    date = tmp$date,
    lwr = tmp$lower_Ev,
    upr = tmp$upper_Ev,
    fit = tmp$Fit_dateEv
  )

  #generation du graph
  check_ref = ggplot(df) +
    geom_point(aes(x = names, y = date),colour="red",shape=3, size=3) +
    geom_errorbar( aes(x=names, ymin=lwr, ymax=upr), width=.2, colour="#3f0b18", alpha=0.9, linewidth =0.8)+
    ylab("Date (year)") + xlab("Site") + ggtitle("Comparison of estimated dates (black) with actual dates (red)") +
    theme_classic() + coord_flip() +
    scale_y_continuous(breaks = scales::pretty_breaks(n = 10)) +
    theme(
      panel.grid.major.x = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                        linewidth = .1,
                                        linetype = 2),
      panel.grid.major.y = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                        linewidth = .1,
                                        linetype = 2),
      axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)
    )

  tmp2 = date_predict[row.sup,]
  tmp2 = tmp2[!is.na(tmp2$date),]

  df2 = data.frame(
    names = rownames(tmp2),
    date = tmp2$date,
    lwr = tmp2$lower_Ev,
    upr = tmp2$upper_Ev,
    fit = tmp2$Fit_dateEv
  )

  #generation du graph
  check_sup = ggplot(df2) +
    geom_point(aes(x = names, y = date),colour="red",shape=3, size=3) +
    geom_errorbar( aes(x=names, ymin=lwr, ymax=upr), width=.2, colour="#3f0b18", alpha=0.9, linewidth=0.8)+
    ylab("Date (year)") + xlab("Site") + ggtitle("Comparison of estimated dates (black) with actual dates (red)") +
    theme_classic() + coord_flip() +
    scale_y_continuous(breaks = scales::pretty_breaks(n = 10)) +
    theme(
      panel.grid.major.x = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                        linewidth = .1,
                                        linetype = 2),
      panel.grid.major.y = element_line(color = rgb(0.5,0.5,0.5,0.3),
                                        linewidth = .1,
                                        linetype = 2),
      axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)
    )

  print(summary(DATA_REF_lm))

  return(
    structure(
      list(
        prediction          = date_predict,
        date_gt             = date_gt,
        lm                  = DATA_REF_lm,
        k                   = k,
        predict_obj_row     = predict_obj_row,
        predict_obj_col     = predict_obj_col,
        cont_gt             = cont,
        statistical_summary = mod1.diagGl,
        obs_ca_eval         = obs_ca_eval,
        check_ref           = check_ref,
        check_sup           = check_sup,
        Shapiro_Wilks       = Shapiro,
        Durbin_Watson       = DW,
        Breusch_Pagan       = BP,
        call                = match.call()
      ),
      class = c("cerardat_obj","list")
    )
  )

}

cerardat_estim_nf <- function(df, row.sup, date, option=list(method = "LOOCV")){

  press <- function(linear.model){
    # calculate the predictive residuals
    pr <- residuals(linear.model)/(1-lm.influence(linear.model)$hat)
    # calculate the PRESS
    PRESS <- sum(pr^2)
    return(PRESS)
  }

  p.value=0
  test=0

  # df is data.frame
  if(!is.data.frame(df)){stop("df is not a data frame.")}
  #
  # row.sup is vector
  if(!is.vector(row.sup)){stop("row.sup is not a vector.")}
  #
  # row.sup is interger
  #if(!is.integer(row.sup)){stop("row.sup is not integer.")}
  #
  # date is vector
  if(!is.vector(date)){stop("date is not a vector.")}
  #
  # date is na or interger
  #if(!is.integer(date)){stop("date is not integer.")}
  #
  # nf is integerdate
  #if(!is.integer(nf)){stop("nf is not integer.")}
  #
  # max(row.sup) < 280 size df
  if(max(row.sup) > dim(df)[1]){stop("row.sup must contain integers between 1 and the number of lines in df.")}
  # min(row.sup) > 0
  if(min(row.sup) <= 0){stop("row.sup must contain integers between 1 and the number of lines in df.")}
  #
  # length(date) = 280
  if(!length(date) == nrow(df)){stop("date must have the same number of observations as the number of lines in df. Complete with NA if necessary.")}
  #

  nf = 0

  #vector for ref data (every not in col.sup)
  row.ref = which(!(1:length(df[,1]) %in% row.sup))

  df_ref = df[row.ref,]
  df_sup = df[row.sup,]
  date = c(date[row.ref],date[row.sup])
  rownames = c(dimnames(df)[1][row.ref],dimnames(df)[1][row.sup])

  clean = TRUE
  GT_rm_names = c()
  row_rm_names = c()
  while(clean){
    #clean, threshold: 5
    #nettoyage des GT ie GT<5
    GT_rm = which(colSums(df_ref)<5)
    if(sum(GT_rm)!=0){
      GT_rm_names = c(GT_rm_names,names(df_ref)[which(colSums(df_ref)<5)])
      df_ref = df_ref[,-c(GT_rm)]
      df_sup = df_sup[,-c(GT_rm)]
    }

    #si row < 5
    df_tmp = rbind(df_ref,df_sup)
    row_rm = which(rowSums(df_tmp)<5)
    if(sum(row_rm)!=0){
      row_rm_names = c(row_rm_names,which(rowSums(df_tmp)<5))
      df_ref = df_ref[!rowSums(df_ref)<5,]
      df_sup = df_sup[!rowSums(df_sup)<5,]
      date = date[-c(row_rm)]
      rownames = rownames[-c(row_rm)]
    }

    if(sum(GT_rm)==0 & sum(row_rm)==0){
      clean = FALSE
    }
  }

  row.ref = 1:length(df_ref[,1])
  row.sup = (length(df_ref[,1])+1):(length(df_ref[,1])+length(df_sup[,1]))

  data_ref = df_ref
  data_sup = df_sup

  if(length(row_rm_names)>0){
    warning(paste0("The sums of ",length(row_rm_names)," rows are less than 5. They were suppressed from the analysis."))
  }
  if(length(GT_rm_names)>0){
    warning(paste0("The sums of ",length(GT_rm_names)," columns in the reference data are less than 5. They were suppressed from the analysis."))
  }


  #DOF Degrees of freedom
  DOF = min(ncol(data_ref)-1,nrow(data_ref)-1)

  #Correspondance analysis
  ca.NMI <- ade4::dudi.coa(data_ref,scannf=FALSE,nf=DOF)


  #df with coord ens (col) de reference + date monnaie
  DATA_REF = cbind(ca.NMI$li, date=date[row.ref])

  max = min(length(DATA_REF[!is.na(DATA_REF$date),1])-2,DOF)

  #specify the cross-validation method
  ctrl = caret::trainControl(method = "cv", number=10)
  if(option[[1]] == "LOOCV"){
    ctrl <- caret::trainControl(method = option[[1]])
  }
  if(option[[1]] == "cv"){
    ctrl <- caret::trainControl(method = option[[1]], number=option[[2]])
  }
  if(option[[1]] == "repeatedcv"){
    ctrl <- caret::trainControl(method = option[[1]], number=option[[2]], repeats=option[[3]])
  }

  formula <- "date ~"

  PRESS <- c()
  R_sq <- c()
  RMSE <- c()
  MAE <- c()

  SW <- c()
  DW <- c()
  BP <- c()

  for(i in 1:max){
    if(i == 1){
      formula <- paste0(formula, ' Axis',i)
    }else{
      formula <- paste0(formula, ' + Axis',i)
    }
    model <- caret::train(as.formula(formula), data = DATA_REF, method = "lm", trControl = ctrl, na.action=na.omit)
    
    RMSE <- c(RMSE,model$results$RMSE)
    MAE <- c(MAE,model$results$MAE)
    R_sq <- c(R_sq,model$results$Rsquared)

    # Calcul du PRESS
    loocv_preds <- model$pred    # PRESS = Sum((Observed - Predicted_LOOCV)^2)
    press_value <- sum((loocv_preds$obs - loocv_preds$pred)^2)

    lm = lm(as.formula(formula), data = DATA_REF)
    PRESS <- c(PRESS, press(lm))

    SW = c(SW,arrondi(shapiro.test(rstudent(lm))$p.value,3))
    DW = c(DW,arrondi(lmtest::dwtest(as.formula(formula),data = DATA_REF, alternative = "two.sided")$p.value,3))
    BP = c(BP,arrondi(lmtest::bptest(as.formula(formula),data = DATA_REF)$p.value,3))
  }

  data2 = data.frame(
    nf = 1:max,
    MAE = MAE,
    RMSE = RMSE,
    PRESS = PRESS,
    R_sq = R_sq
  )

  data3 = data.frame(
    nf = rep(1:max,3),
    p.value=c(SW,DW,BP),
    test=c(rep("Shapiro-Wilks",max),rep("Durbin-Watson",max),rep("Breusch-Pagan",max))
  )

  pTest = ggplot(data3) +
    ylab("p.value") + xlab("Number of component in lm()") +
    ggtitle("hypothesis tests p.value") +
    geom_point(shape=1,aes(x = nf, y = p.value, col=test)) +
    geom_line(aes(x = nf, y = p.value, col=test)) +
    geom_line(aes(x = nf, y = 0.05)) +
    theme_classic()

  pMAE = ggplot(data2) +
    ylab("MAE") + xlab("Number of component in lm()") +
    ggtitle("Mean Absolute Error (MAE)") +
    geom_point(shape=1,aes(x = nf,y = MAE)) +
    geom_line(aes(x = nf,y = MAE)) +
    annotate("point",shape=16,cex=1.1,x = which(MAE == min(MAE)),y = min(MAE)) +
    geom_vline(xintercept = which(MAE == min(MAE)), linetype = "dotted",
               color = "grey", linewidth=1) +
    scale_y_log10(labels = scientific) +
    annotation_logticks(sides="l") +
    theme_classic()

  pRMSE = ggplot(data2) +
    ylab("RMSE") + xlab("Number of component in lm()") +
    ggtitle("Root Mean Squared Error (RMSE)") +
    geom_point(shape=1,aes(x = nf,y = RMSE)) +
    geom_line(aes(x = nf,y = RMSE)) +
    annotate("point",shape=16,cex=1.1,x = which(RMSE == min(RMSE)),y = min(RMSE)) +
    geom_vline(xintercept = which(RMSE == min(RMSE)), linetype = "dotted",
               color = "grey", linewidth=1) +
    scale_y_log10(labels = scientific) +
    annotation_logticks(sides="l") +
    theme_classic()

  pPRESS = ggplot(data2) +
    ylab("PRESS") + xlab("Number of component in lm()") +
    ggtitle("Predicted Residual Error Sum of Squares (PRESS)") +
    geom_point(shape=1,aes(x = nf,y = PRESS)) +
    annotate("point",shape=16,cex=1.1,x = which(PRESS == min(PRESS)),y = min(PRESS)) +
    geom_line(aes(x = nf,y = PRESS)) +
    scale_y_log10(labels = scientific) +
    annotation_logticks(sides="l") +
    geom_vline(xintercept = which(PRESS == min(PRESS)), linetype = "dotted",
               color = "grey", linewidth=1) +
    theme_classic()

  min_R = min(data2$R_sq)
  if(min(data2$R_sq) > 0)min_R= 0
  if(min(data2$R_sq) > 0.25)min_R= 0.25
  if(min(data2$R_sq) > 0.5)min_R= 0.5
  if(min(data2$R_sq) > 0.75)min_R= 0.75

  pR_sq = ggplot(data2) +
    ylab("adj.R_squared") + xlab("Number of component in lm()") +
    ggtitle("adj.R_squared") + ylim(min_R,1) +
    geom_point(shape=1,aes(x = nf,y = R_sq)) +
    geom_line(aes(x = nf,y = R_sq)) +
    annotate("point",shape=16,cex=1.1,x = which(R_sq == max(R_sq)),y = max(R_sq)) +
    geom_vline(xintercept = which(R_sq == max(R_sq)), linetype = "dotted",
               color = "grey", linewidth=1) +
    theme_classic()

  return(
    structure(
      list(
        nf= which(data2$PRESS == min(data2$PRESS)),
        PRESS = pPRESS,
        RMSE = pRMSE,
        MAE = pMAE,
        adj.R_sq = pR_sq,
        Pvalue = pTest,
        data = data2,
        data_stat = data3
      ),
      class = c("list")
    )
  )

}

print.cerardat_obj <- function(x, ...){
  ## test
  #
  # is cerardat
  if(class(x)[1] != "cerardat_obj"){stop("x is not a cerardat object.")}

  cat("cerardat
call: ")
  print(x$call)
  cat("\n")
  print(x$statistical_summary)
  cat("\n")
  head(x$date_predict)
}

plot.cerardat_obj = function(x,
                             which=NULL,
                             main="model dateEv (red) and dateAc (black)",
                             xlab="Date",
                             ylab="Density",
                             col1=rgb(0.93, 0.23, 0.23, 0.5),
                             col2="black",
                             xlim=NULL,
                             ylim=NULL,...){
  ## test

  # is cerardat
  if(class(x)[1] != "cerardat_obj"){stop("x is not a cerardat object.")}

  if(is.null(xlim)){
    ecart = max(x$date_gt)-min(x$date_gt)
    xlim = c(min(x$date_gt-ecart*0.07),max(x$date_gt+ecart*0.07))
  }

  AUTO_TICK = trunc((xlim[2]-xlim[1])/100)

  if(is.null(which)){
    which = 1:nrow(x$cont_gt)
  }else{
    if(is.character(which)){
      which = which(rownames(x$cont_gt) %in% which)
    }
  }

  if(length(which) > 1){
    pb <- utils::txtProgressBar(min = 0, max = length(which), style = 3)
  }

  ens_date_sd = arrondi(cbind(
    Pred=data.frame(x$predict_obj_row$fit)[,1],
    data.frame(E_T=x$predict_obj_row$se.fit)
  ), 1)

  GT_date_sd = arrondi(cbind(
    Pred=data.frame(x$predict_obj_col$fit)[,1],
    data.frame(E_T=x$predict_obj_col$se.fit)
  ), 1)

  if(is.null(ylim)){
    tmp_ = c()
    for(i in which)
    {
      date_evenement = nor1mix::norMix(ens_date_sd[i,1],sigma=ens_date_sd[i,2])
      date_evenement_density = nor1mix::dnorMixL(date_evenement,xlim=xlim)
      date_accumulation <- nor1mix::norMix(mu = GT_date_sd[,1], w = unlist(as.vector(x$cont_gt[i,])), sigma= GT_date_sd[,2])
      date_accumulation_density = nor1mix::dnorMixL(date_accumulation, xlim=xlim)

      tmp_ = c(tmp_,max(date_accumulation_density$y+0.005),max(date_evenement_density$y+0.005))
    }
    ylim=c(0,max(tmp_))
  }

  for(i in which)
  {
    date_accumulation <- nor1mix::norMix(mu = GT_date_sd[,1], w = unlist(as.vector(x$cont_gt[i,])), sigma= GT_date_sd[,2])
    date_accumulation_density = nor1mix::dnorMixL(date_accumulation, xlim=xlim)

    date_evenement = nor1mix::norMix(ens_date_sd[i,1],sigma=ens_date_sd[i,2])
    date_evenement_density = nor1mix::dnorMixL(date_evenement,xlim=xlim)

    ymax = max(ylim) + 0.01
    date_accumulation_density$y[date_accumulation_density$y > ymax] = ymax
    date_evenement_density$y[date_evenement_density$y > ymax] = ymax

    date_accumulation_density$y[1] = 0
    date_evenement_density$y[1] = 0

    date_accumulation_density$y[length(date_accumulation_density$y)] = 0
    date_evenement_density$y[length(date_accumulation_density$y)] = 0
    default = par()
    par(mar=c(5.1,5.1,4.1,2.1))
    plot(date_accumulation_density,xlim=xlim,ylim=ylim,xlab=xlab,col="black",
         ylab=ylab,type="n",xaxt="n",main=main,...)

    graphics::mtext(dimnames(x$cont_gt)[[1]][i], side=2, line=4)
    par(mar=default$mar)
    
    if(!i > length(x$obs_ca_eval[,1]) & main != ""){
      subtitle=paste("Quality of row representation (cos2) in correspondence analysis:",arrondi(sum(x$obs_ca_eval[i,1:x$k]),2))
      graphics::mtext(subtitle, side = 3, line = 0)
    }

    graphics::axis(side=1, col="black", at=pretty(seq(xlim[1],xlim[2]), AUTO_TICK))

    graphics::polygon(date_accumulation_density,col=col2,border=col2)
    graphics::polygon(date_evenement_density,col=col1,border=NA)

    if(length(which) > 1)
      utils::setTxtProgressBar(pb, i)
  }
}

plot_custom_cerardat_obj = function(x,
                             which = NULL,
                             main="model dateEv (red) and dateAc (black)",
                             xlab="Date",
                             ylab="Density",
                             col1=rgb(0.93,0.23,0.23,0.5),
                             col2="black",
                             xlim=NULL,
                             ylim=NULL,
                             xaxt='s',...){
  ## test
  #
  # is cerardat
  if(class(x)[1] != "cerardat_obj"){stop("x is not a cerardat object.")}

  if(is.null(xlim)){
    ecart = max(x$date_gt)-min(x$date_gt)
    xlim = c(min(x$date_gt-ecart*0.07),max(x$date_gt+ecart*0.07))
  }

  AUTO_TICK = trunc((xlim[2]-xlim[1])/100)

  if(is.null(which)){
    which = 1:nrow(x$cont_gt)
  }else{
    if(is.character(which)){
      which = which(rownames(x$cont_gt) %in% which)
    }
  }

  if(length(which) > 1){
    pb <- utils::txtProgressBar(min = 0, max = length(which), style = 3)
  }

  ens_date_sd = arrondi(cbind(
    Pred=data.frame(x$predict_obj_row$fit)[,1],
    data.frame(E_T=x$predict_obj_row$se.fit)
  ),1)

  GT_date_sd = arrondi(cbind(
    Pred=data.frame(x$predict_obj_col$fit)[,1],
    data.frame(E_T=x$predict_obj_col$se.fit)
  ),1)

  for(i in which)
  {
    date_accumulation <- nor1mix::norMix(mu = GT_date_sd[,1], w = unlist(as.vector(x$cont_gt[i,])), sigma= GT_date_sd[,2])
    date_accumulation_density = nor1mix::dnorMixL(date_accumulation, xlim=xlim)

    date_evenement = nor1mix::norMix(ens_date_sd[i,1],sigma=ens_date_sd[i,2])
    date_evenement_density = nor1mix::dnorMixL(date_evenement, xlim=xlim)

    ymax = max(ylim) + 0.01
    date_accumulation_density$y[date_accumulation_density$y > ymax] = ymax
    date_evenement_density$y[date_evenement_density$y > ymax] = ymax

    date_accumulation_density$y[1] = 0
    date_evenement_density$y[1] = 0

    date_accumulation_density$y[length(date_accumulation_density$y)] = 0
    date_evenement_density$y[length(date_accumulation_density$y)] = 0
    default = par()
    par(mar=c(0,4.1,0,2.1), las=1)
    plot(date_accumulation_density,xlim=xlim, ylim=ylim,xlab=xlab,col="black",
         ylab='',type= "l",xaxt="n",main=main,...)
    graphics::mtext(dimnames(x$cont_gt)[[1]][i], side=2, line=4, las=1)
    par(mar=default$mar, las=default$las)

    if(xaxt != 'n'){
      graphics::axis(side=1,line=0,outer=TRUE,col="black",at=pretty(seq(xlim[1],xlim[2]),AUTO_TICK))
    }
    graphics::polygon(date_accumulation_density,col=col2,border=col2)
    graphics::polygon(date_evenement_density,col=col1,border=NA)
  }
}

arrangePlot = function(x,
                       which=1:2,
                       main="model dateEv (red) and dateAc (black)",
                       xlab="Date",
                       ylab="Density",
                       col1=rgb(0.93,0.23,0.23,0.5),
                       col2="black",
                       xlim=NULL,
                       ylim=NULL,
                       date=NULL,
                       col_date=NULL,
                       lty_date=NULL,...){
  plotcerar = function(x, which, main, xlab, ylab, col1, col2, xlim, ylim, date, col_date, lty_date, xaxt,...){
    plot_custom_cerardat_obj(x, which=which, main="", xlab=xlab, ylab=ylab, 
    col1=col1, col2=col2, xlim=xlim, ylim=ylim, xaxt=xaxt,...)
    if(!is.null(date)){
      for(j in 1:length(date)){
        abline(v=date[j],col=col_date[j],lty=lty_date[j],...)
      }
    }
  }
  # is cerardat
  if(class(x)[1] != "cerardat_obj"){stop("x is not a cerardat object.")}

  if(is.null(which)){
    which = 1:nrow(x$cont_gt)
  }else{
    if(is.character(which)){
      which = match(which, rownames(x$cont_gt))
    }
  }

  nb_plot = length(which)

  if(is.null(ylim)){
    ens_date_sd = arrondi(cbind(
      Pred=data.frame(x$predict_obj_row$fit)[,1],
      data.frame(E_T=x$predict_obj_row$se.fit)
    ),1)

    GT_date_sd = arrondi(cbind(
      Pred=data.frame(x$predict_obj_col$fit)[,1],
      data.frame(E_T=x$predict_obj_col$se.fit)
    ),1)
    
    tmp_ = c()
    for(i in which)
    {
      date_accumulation <- nor1mix::norMix(mu = GT_date_sd[,1], w = unlist(as.vector(x$cont_gt[i,])), sigma= GT_date_sd[,2])
      date_accumulation_density = nor1mix::dnorMixL(date_accumulation, xlim=xlim)
      date_evenement = nor1mix::norMix(ens_date_sd[i,1],sigma=ens_date_sd[i,2])
      date_evenement_density = nor1mix::dnorMixL(date_evenement,xlim=xlim)
      tmp_ = c(tmp_,max(date_accumulation_density$y+0.005),max(date_evenement_density$y+0.005))
    }
    ylim=c(0,max(tmp_)+0.01)
  }

  if(is.null(col_date)){
    col_date = 2:(length(date)+1)
  }

  if(length(col_date) != length(date)){
    col_date = rep(col_date,ceiling(length(date)/length(col_date)))
  }

  if(is.null(lty_date)){
    lty_date = rep(2,length(date))
  }

  if(length(lty_date) != length(date)){
    lty_date = rep(lty_date,ceiling(length(date)/length(lty_date)))
  }

  plotlist = list()
  for(i in 1:nb_plot){
    if(i == nb_plot){
      plotlist[[i]] = ggplotify::as.grob(function() plotcerar(x, which=which[i], main="",xlab="",
       ylab=ylab, col1=col1, col2=col2, xlim=xlim, ylim=ylim,date,col_date,lty_date,xaxt="s",...))
    }else{
      plotlist[[i]] = ggplotify::as.grob(function() plotcerar(x, which=which[i], main="",xlab="",
       ylab=ylab, col1=col1, col2=col2, xlim=xlim, ylim=ylim,date,col_date,lty_date,xaxt="n",...))
    } 
  }

  gridExtra::grid.arrange(
    grobs = plotlist,
    nrow = nb_plot,
    top = grid::textGrob(main, vjust=-1, gp=gpar(fontsize=18)),
    bottom = grid::textGrob(xlab, vjust=4),
    left = grid::textGrob(ylab, vjust=-3.5, rot = 90),
    vp = grid::viewport(width=0.9, height=0.8)
  )
}

extract_results = function(cerardat,
                            width = 480,
                            height = 480,
                            path = "figures",
                            main="model dateEv (red) and dateAc (black)",
                            xlab="Date",
                            ylab="Density",
                            col1 = rgb(0.93,0.23,0.23,0.5),
                            col2 = "black",
                            xlim = NULL,
                            ylim = NULL){
  ## test
  #
  # is cerardat
  if(class(cerardat)[1] != "cerardat_obj"){stop("cerardat is not a cerardat object.")}

  if(is.null(xlim)){
    ecart = max(cerardat$date_gt)-min(cerardat$date_gt)
    xlim = c(min(cerardat$date_gt-ecart*0.07),max(cerardat$date_gt+ecart*0.07))
  }

  AUTO_TICK = trunc((xlim[2]-xlim[1])/100)

  pb <- utils::txtProgressBar(min = 0, max = length(cerardat$cont_gt[1,]), style = 3)

  if(!dir.exists(path))
    dir.create(path)

  if(!dir.exists(paste0(path,"/ref")))
    dir.create(paste0(path,"/ref"))

  if(!dir.exists(paste0(path,"/sup")))
    dir.create(paste0(path,"/sup"))

  ens_date_sd = arrondi(cbind(
    Pred=data.frame(cerardat$predict_obj_row$fit)[,1],
    data.frame(E_T=cerardat$predict_obj_row$se.fit)
  ),1)

  GT_date_sd = arrondi(cbind(
    Pred = data.frame(cerardat$predict_obj_col$fit)[,1],
    data.frame(E_T=cerardat$predict_obj_col$se.fit)
  ),1)

  if(is.null(ylim)){
    tmp_ = c()
    for(i in 1:nrow(cerardat$cont_gt) )
    {
      date_evenement = nor1mix::norMix(ens_date_sd[i,1],sigma=ens_date_sd[i,2])
      date_evenement_density = nor1mix::dnorMixL(date_evenement,xlim=xlim)
      date_accumulation <- nor1mix::norMix(mu = GT_date_sd[,1], w = unlist(as.vector(cerardat$cont_gt[i,])), sigma= GT_date_sd[,2])
      date_accumulation_density = nor1mix::dnorMixL(date_accumulation, xlim=xlim)
      tmp_ = c(tmp_,max(date_accumulation_density$y+0.005),max(date_evenement_density$y+0.005))
    }
    ylim=c(0,max(tmp_))
  }

  for(i in 1:nrow(cerardat$cont_gt) )
  {
    date_accumulation = nor1mix::norMix(mu=GT_date_sd[,1],w=unlist(as.vector(cerardat$cont_gt[i,])),sigma=GT_date_sd[,2])
    date_accumulation_density = nor1mix::dnorMixL(date_accumulation,xlim=xlim)

    date_evenement = nor1mix::norMix(ens_date_sd[i,1],sigma=ens_date_sd[i,2])
    date_evenement_density = nor1mix::dnorMixL(date_evenement,xlim=xlim)

    ymax = max(ylim) + 0.01
    date_accumulation_density$y[date_accumulation_density$y > ymax] = ymax
    date_evenement_density$y[date_evenement_density$y > ymax] = ymax

    date_accumulation_density$y[1] = 0
    date_evenement_density$y[1] = 0

    date_accumulation_density$y[length(date_accumulation_density$y)] = 0
    date_evenement_density$y[length(date_accumulation_density$y)] = 0
    
    if(i <= length(cerardat$obs_ca_eval[,1])){
      grDevices::jpeg(file=path.expand(paste(path,"/ref/",
        dimnames(cerardat$cont_gt)[[1]][i],".jpeg",sep="")),
        width = width, height = height,units = "px", pointsize = 12)
      default = par()
      par(mar=c(5.1,5.1,4.1,2.1))
      plot(date_accumulation_density, xlim=xlim, ylim=ylim, xlab=xlab, col="black",
        ylab=ylab, type="l", xaxt="n", main=main)
      graphics::mtext(dimnames(cerardat$cont_gt)[[1]][i], side=2, line=4)
      subtitle=paste("Quality of row representation (cos2) in correspondence analysis:",arrondi(sum(cerardat$obs_ca_eval[i,1:cerardat$k]),2))
      graphics::mtext(subtitle, side = 3, line = 0)
      par(mar=default$mar)
    }else{
      grDevices::jpeg(file=path.expand(paste(path,"/sup/",
        dimnames(cerardat$cont_gt)[[1]][i],".jpeg",sep="")),
        width = width, height = height,units = "px", pointsize = 12)
      default = par()
      par(mar=c(5.1,5.1,4.1,2.1))
      plot(date_accumulation_density,xlim=xlim, ylim=ylim,xlab=xlab,col="black",
        ylab=ylab,type= "l",xaxt="n",
        main=main)
      graphics::mtext(dimnames(cerardat$cont_gt)[[1]][i], side=2, line=4)
      par(mar=default$mar)
    }
    graphics::axis(side=1,col="black",at=pretty(seq(xlim[1],xlim[2]),AUTO_TICK))
    graphics::polygon(date_accumulation_density,col=col2,border=col2)
    graphics::polygon(date_evenement_density,col=col1,border=NA)

    grDevices::graphics.off()

    utils::setTxtProgressBar(pb,i)
  }
  print(paste0(getwd(),"/",path))
}

\name{extract_results}
\alias{extract_results}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Generates all plots (in jpeg format) in a subfolder
}
\description{
Generates all plots (in jpeg format) in a subfolder (relative path from the working directory).
}
\usage{
extract_results(cerardat, width=480, height=480, path="figures",
                  main="model dateEv (red) and dateAc (black)", 
                  xlab="Date", ylab="Density",
                  col1 = rgb(0.93,0.23,0.23,0.5), col2 = "black",
                  xlim=NULL, ylim=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{cerardat}{
    a \link[SPARTAAS]{cerardat} output.
}
  \item{width}{
  the width of the graphics region in pixels.
}
  \item{height}{
  the width of the graphics region in pixels.
}
  \item{path}{
  directory where plots are exported (relative path from the working directory). Automatically generates two sub-folders 'ref' for the reference plot and 'sup' for the supplementary  plot.
}
  \item{main}{
  an overall title for the plot: see \link[graphics]{title}.
}
  \item{xlab}{
  a title for the x axis: see \link[graphics]{title}.
}
  \item{ylab}{
  a title for the y axis: see \link[graphics]{title}.
}
  \item{col1}{
  Color of the the Event curve (color name, hex code, \link[grDevices]{rgb} function \{grDevices\} for transparency).
}
  \item{col2}{
  Color of the the Accumulation curve (color name, hex code, \link[grDevices]{rgb} function \{grDevices\} for transparency).
}
  \item{xlim}{
  Two numeric values, specifying the left limit and the right limit of the scale.
}
  \item{ylim}{
  Two numeric values, specifying the lower limit and the upper limit of the scale.
}
}


\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
\donttest{
data("datacerardat")

resultat = cerardat(df = datacerardat$df,
           row.sup = datacerardat$row.sup,
           date = datacerardat$date,
           nf = NULL,
           confidence = 0.95,
           graph = TRUE
        )
#Generates all plots (in jpeg format) in a folder 'figures'
#You will find the jpeg in 'ref' and 'sup' subfolder
#extract_results(resultat,width=480, height=480, path="figures",
#     col1=rgb(0.93,0.23,0.23,0.5),
#     col2="black",
#     xlim=NULL,
#     ylim=c(0,0.03)
#    )

#You can extract the plots and find them in the directory :
paste0(getwd(),"/figures")
}
}

\name{cerardat}
\alias{cerardat}

\title{
cerardat
}
\description{
The methodology is based on a statistical and visual approach using two estimated density curves to date each archaeological context. The statistical procedure required two steps, each leading to the construction of a density curve. The first allowed us to estimate a date corresponding to the terminus post quem of the context, a cursor reflecting an event dated in calendar time. This statistical tool allows the archaeologist to easily visualise and analyse chronological patterns.
}
\usage{
cerardat(df, row.sup, date, nf = NULL, confidence = 0.95, graph = T)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{df}{
The data (data.frame) is a contingency table with the observations in the rows and the technical groups in the columns.
}
  \item{row.sup}{
Index of supplementary rows in df (vector).
}
  \item{date}{
The dates of each observation or NA (vector).
}
  \item{nf}{
an integer representing the number of axes retained in the correspondence analysis. If NULL, it is automatically chosen see \link[SPARTAAS]{cerardat_estim_nf}.
}
  \item{confidence}{
The desired confidence interval (0.95 for 95\%).
}
  \item{graph}{
logical to display the plots or not.
}


}
\details{
The corpus data is a contingency table with the observations in the rows and the technical groups in the columns. There are two types of observations: the reference corpus observations and the supplementary observations. The supplementary rows (observations) are identified by the argument `row.sup`.

\strong{step 1: modelling events dated in calendar time (dateEv)}\cr
This step involves estimating the date of an event recorded in the ground (an archaeological context for the archaeologist) from the pottery assemblage of which it is composed, by fitting a regression model that relates a known date in calendar time, such as the date of issue of a coin, to its pottery profile. The reference corpus used to fit the regression model. We then used the previously fitted model to calculate a predicted value for contexts not included in the reference corpus, sometimes stratigraphically separated or poorly documented, with a 95\% confidence interval for the predicted date.

A correspondence analysis (\link[FactoMineR]{CA}) was carried out to summarize the information in the reference corpus data. We then kept only the first factorial axes. In this way, our contingency table becomes a reduced size table, an incomplete reconstruction of the data. This principle is used in many factor analysis techniques to reduce the number of explanatory variables in the linear regression model.

After estimating the beta parameters of the model using the classical results of multiple regression analysis and checking that the model fits the data correctly, we can deduce the estimated date of an observation and also predict the date of another observation that has no coins and is therefore not dated.

\strong{step 2: from event time (dateEv) to accumulation time (dateAc)}\cr
We used the results of the first step and the properties of the CA to obtain an estimate of the date of each fabric. We could then define the archaeological time represented as dateAc, in other words the accumulation time of a context, as the weighted sum of the fabric dates; the weights being the proportions of MINVC of each fabric in the context. Assuming that the random variables dateEvj are independent, the distribution of the accumulation time of each context can be approximated by the Gaussian mixture. In this way, for each context, we obtained a plurimodal density curve representing the estimated law of accumulation time based on the mixture of normal densities (dates of each tissue). By definition, the area under the density curve has a value of 1 (i.e. 100\%).

\strong{date}\cr
In order to estimate a date for the context, it is essential to refer to objects that have been dated by another source, such as coins. These contexts were selected on a very strict basis for their chronostratigraphic reliability, level of domestic occupation or enclosures with long urban stratigraphic sequences, thereby minimising any bias associated with the disparity between the date of the coin and that of the context.

See \link[SPARTAAS]{plot.cerardat_obj} and \link[SPARTAAS]{arrangePlot} for visualisation.

}

\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
\item{prediction }{Estimated date for archaeological context (event: dateEV and accumulation: dateAc) with confidence interval. The first two columns show the total count and the number of categories per line (only for columns used in CA). Then a date column shows the known dates. Each dateEv and dateAC model has three columns (value, lower bound, upper bound).}
\item{date_gt }{Estimated date for technical groups with confidence interval (use for dateAc). The first column show the total count per category in the reference data (only for rows used in CA).}
\item{lm }{Linear model on the components of the correspondance analysis.}

\item{predict_obj_row }{date prediction of archaeological contexts (rows) using \link[stats]{predict.lm}.}
\item{predict_obj_col }{date prediction of technical groups (columns) using \link[stats]{predict.lm}.}
\item{cont_gt }{Contingency table of the reference corpus.}

\item{statistical.summary }{Statistical summary of the model:\cr
Adjusted R-squared\cr
R-squared\cr
sigma (Residual standard error)\cr
The Shapiro-Wilks test is used to verify the normality of the residuals.\cr
The Durbin-Watson test checks for first order autocorrelation.\cr
The Breusch-Pagan test checks for heteroscedasticity.
}

\item{obs_ca_eval }{Quality of row representation in the correspondence analysis.}

\item{check_ref }{Plot of estimated dates (with confidence interval) and real dates of reference data. Only when the real date is known.}
\item{check_sup }{Plot of estimated dates (with confidence interval) and real dates of supplementary data. Only when the real date is known.}

\item{Shapiro_Wilks }{Summary of the Shapiro-Wilks test. see \link[stats]{shapiro.test}.}
\item{Durbin_Watson }{Summary of the Durbin-Watson test. see \link[lmtest]{dwtest}.}
\item{Breusch_Pagan }{Summary of the Breusch-Pagan test. see \link[lmtest]{bptest}.}



%% ...
}
\references{
Bellanger L. and Husi P. (2012) Statistical tool for dating and interpreting archaeological contexts using pottery. Journal of Archaeological Science, Elsevier, 39 (4), pp.777-790. doi:10.1016/j.jas.2011.06.031.
}
\author{
A. COULON

L. BELLANGER

P. HUSI
}
\examples{
\donttest{
data("datacerardat")

resultat = cerardat(df = datacerardat$df,
  row.sup = datacerardat$row.sup,
  date = datacerardat$date,
  nf = NULL,
  confidence = 0.95,
  graph = TRUE
)

resultat
#The Shapiro-Wilks test is used to verify the normality of the residuals.
#The Durbin-Watson test checks for first order autocorrelation.
#The Breusch-Pagan test checks for heteroscedasticity.

#See the first plot
plot(resultat, which = 1)
#We can add the date (when there is one) and the dateEv and dateAc
abline(v=resultat$prediction$date[1], col="grey40", lty=1)
abline(v=resultat$prediction$Fit_dateEv[1], col="red", lty=2)
abline(v=resultat$prediction$Median_dateAc[1], col="black", lty=3)
legend("topright",
  legend=c("Date", "DateEv", "DateAc"),
  col=c("grey40", "red", "black"),
  lty=c(1, 2, 3)
)

#You can also create a plot with multiple context (which can be index/integer or rownames)
arrangePlot(resultat,
  which = c("Z067", "Z070", "Z074"),
  main = "Site: Neuvy-Le-Roi.", 
  xlim = c(300, 1300), 
  date = resultat$prediction[c("Z067","Z070","Z074"),]$Fit_dateEv, 
  col_date = c("red", rgb(0.20, 0.40, 0.30), 4)
)

##See the first 10 plots
#plot(resultat, which = 1:10)

#arrange the first 10 plots (with date we can add a grid)
arrangePlot(resultat,
  which = 1:10,
  col1 = rgb(0.7, 0.7, 0.7, 0.6), # the last arg of rgb is alpha for opacity
  col2 = "black",
  xlim = c(200, 1000),      # limit of the x axis between 200 and 1000
  date = seq(250, 950, 50), # add one line every 50 years between 250 and 950
  col_date = "grey80",
  lty_date = 3,      # dotted line
  bty = 'n',         # remove box
  cex.axis = 0.8     # control axis text size (only y axis is impacted in arrangePlot) 
)

##See all the plots
#plot(resultat)

##You can extract the plots and find them in the directory :
#paste0(getwd(),"/figures")
##With the 'extract_results' function
#extract_results(resultat, width=480, height=480, path="figures")
}
}

\name{timerange}
\alias{timerange}

%- Also NEED an '\alias' for EACH other topic documented here.

\title{
Plot the time range of observations sorted by cluster.
}
\description{
Cluster time range visualisation.}
\usage{
timerange(
  data,
  cluster = NULL,
  add = NULL,
  density = NULL,
  color = NULL,
  reorder = FALSE
)
}

%- maybe also 'usage' for other objects documented here.

\arguments{
  \item{data}{
data.frame containing the identifier, lower and upper bound (id, inf, sup) for each observation}
  \item{cluster}{
vector containing the cluster number of each observation.}
  \item{add}{
data.frame containing the information to be displayed on hover.}
  \item{density}{
vector of the density for each observation.}
  \item{color}{
vector of the colors for each observation (if you want the colors to correspond to something else than clusters). Character vector of the same length as the number of observations.}
  \item{reorder}{
Logical to rearrange the colors. If TRUE, the first color corresponds to the leftmost cluster on the plot. If FALSE, the first color is that of cluster number 1, wherever it is.}
}

\value{
The function returns a list.
  \item{plot }{The timerange plot.}
}

\author{
A. COULON

B. MARTINEAU

L. BELLANGER

P. HUSI
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
data <- data.frame(
  id = c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20),
  Lower_bound = c(400,401,401,350,500,460,482,432,399,
    489,750,740,704,700,758,789,802,755,750,820),
  Upper_bound = c(550,689,755,700,780,700,700,699,650,
    850,1100,1100,1010,889,999,999,1050,1002,1000,1100)
)

cluster = c(1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2)

add <- data.frame(
  Site = c("Angers","Angers","Angers","Angers","Angers",
    "Angers","Angers","Angers","Angers","Angers",
    "Blois","Blois","Blois","Blois","Blois",
    "Blois","Blois","Blois","Blois","Blois")
)

timerange(data, cluster, add)

## with sub group (cluster 1 is sub divided in 2: 1.1 and 1.2)
cluster_with_sub = c(1.1,1.1,1.1,1.1,1.1,1.2,1.2,1.2,1.2,1.2,2,2,2,2,2,2,2,2,2,2)

timerange(data, cluster_with_sub, add)

## with density
density <- c(32,34,35,19,9,25,19,29,28,18,10,13,9,10,9,6,3,7,7,1)
timerange(data=data, cluster=cluster, density=density)

}

\name{seriograph}
\alias{seriograph}
\title{
Plot seriograph (B. DESACHY).
}

\description{
  Visualization of contingency data over time. \strong{Rows} must be individuals (archaeological site,...) and \strong{columns} must be categories (type,...).
}

\usage{
seriograph(cont, order, insert, show, permute)
}

\arguments{
  \item{cont}{
    Contingency table or hclustcompro object. Note: Your contingency table must have the rows sorted in chronological order (the order parameter allows you to change the order of the rows if necessary).
}
  \item{order}{
     Vector to change the order of the rows (use row names or cluster names if cont is a hclustcompro object, as a character vector). The oldest (at the bottom) must be at the end of the vector. Missing names are not plotted. You can remove a row by simply removing the name in the vector.
}
  \item{show}{
     The element to be plotted. This should be (a unique abbreviation of) one of 'both', 'EPPM' or 'frequency'.
}
  \item{permute}{
     Logical for permute columns in order to show seriation.
}

  \item{insert}{
     Vector with the position after which you want to insert one or more hiatuses. Could be a list with two vectors: position and label to be printed instead of hiatus (see last examples).
}
}
\details{
  \strong{Seriograph}\cr\cr
  We have chosen the serigraph (DESACHY 2004). This tool makes it possible to highlight the evolution of ceramics over time as well as to understand the commercial relations thanks to the imported ceramics. The percentages of each category of ceramics per set are displayed. The percentages are calculated independently for each set (row). The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).

  We can generalized this representation for other contingency data or with \link[SPARTAAS]{hclustcompro} object.

  The visualization of positive deviations from the average percentage allows us to observe a series that results from changes in techniques and materials dedicated to ceramic manufacturing over time.

  In order to facilitate the exploitation of the data tables, we propose here a computerised graphic processing tool (EPPM serigraph - for Ecart Positif aux Pourcentages Moyens - positive deviation from the average percentage), which does not require specialised statistical skills and is adapted to the case of stratified sites, where the study of the evolution of artefacts can be based on the relative chronology provided by the excavation.

The treatment consists firstly of transforming this table of counts into a table of percentages, the total number in each set (each row) being reduced to 100; these are the proportions, or frequencies, of the types in the sets are thus compared.

The display of positive deviations from the mean percentages (EPPM) shows in black on a lighter background the percentage shares that are higher than the mean percentage of the variable, so as to highlight the most significant part of the values in the table.This display is simply adapted to the seriograph: when a percentage is greater than the average percentage of the type, the excess share (called here EPPM: positive deviation from the average percentage) is shown in black, centred around the axis of the type, on the grey background of the percentage bar.

The table is then transformed into a graphic matrix where these percentages are expressed, for each type, by horizontal bars centred on the column axis. When the rows are ordered chronologically, the silhouette formed by the superposition of these frequency bars bars makes it possible to visualise the evolution over time of the type concerned.

The display of the percentages allows comparison of the different sets but does not provide information on the differences in numbers. To fill this gap, the proportion of the numbers in each class is displayed on the seriograph (weight column).

The processing technique applies to sets whose chronological order is not known; the lines of the graph are to be reorganised so as to obtain boat-shaped silhouettes following the hypothesis of a chronological evolution corresponding to the seriation model.

  \strong{Positive deviation from the average percentage (EPPM in French)}\cr\cr
  The average percentage is calculated for each ceramic category (columns) on the total number of accounts (all classes combined). From the average percentage we recover for each category and for each rows the difference between the percentage of the category in the class with the average percentage. The EPPM corresponds to the notion of independence deviation (between rows and columns, between categories and time classes) in a chi-square test approach. Although this approach is fundamental in statistical analysis, independence deviations are here purely indicative and are not associated with a p_value that could determine the significance of deviations.

    \strong{Weight}\cr\cr
  Weight is the number of observations divided by the total number of observations. It indicates for each row the percentage of the data set used to calculate the frequencies of the elements (row).

  \strong{Permutation}\cr\cr
  order argument:\cr
  The rows of the contingency table are initially in the order of appearance (from top to bottom). It must be possible to re-order the classes in a temporal way (You can also order as you want your contingency table).

  permute argument:\cr
  In addition, it is possible to swap ceramic categories (contingency table columns) in order to highlight a serialization phenomenon. Matrix permutation uses an algorithm called "reciprocal averages". Each line is assigned a rank ranging from 1 to n the number of lines. A barycentre is calculated for each column by weighting according to the row rank. Finally, the columns are reorganized by sorting them by their barycentre.

  \strong{Insert}\cr\cr
  It's possible to insert a row in the seriograph in order to represent a archeological hiatus or other temporal discontinuities.
}
\value{
The function returns a list (class: seriograph).
\item{seriograph }{The seriograph plot}
\item{dendrogram}{If cont is a hclustcompro object return the dendrogram with the period order as label}
\item{contingency }{Data frame of the contingencies data (reorder)}
\item{frequency }{Data frame of the frequencies data}
\item{EPPM }{Data frame of EPPM (Positive Deviation from the Average Pourcentage)}
\item{EPM }{Data frame of EPM (Deviation from the Average Pourcentage)}
}

\references{
Desachy B. (2004). Le sériographe EPPM : un outil informatisé de sériation graphique pour tableaux de comptages. In: Revue archéologique de Picardie, n°3-4, 2004. Céramiques domestiques et terres cuites architecturales. Actes des journées d'étude d'Amiens (2001-2002-2003) pp. 39-56 \doi{10.3406/pica.2004.2396}
}
\author{
B. DESACHY

A. COULON

L. BELLANGER

P. HUSI
}

\examples{
\donttest{
data(datangkor)

## contingency table
cont <- datangkor$contingency

## default
seriograph(cont)

seriograph(cont, show = "EPPM")
seriograph(cont, show = "frequency")

## Don't allow permutation of columns
seriograph(cont, permute = FALSE)

## insert Hiatus (position, 1 -> after first row from bottom: oldest)
seriograph(cont,insert = 2)
seriograph(cont,insert = c(2,3))

## insert costum label element
insert <- list(
  position = c(2,3),
  label = c("Hiatus.100years","Missing data")
)
seriograph(cont, insert = insert)

## change order with cluster name (letters on dendrogram) to sort them in a chronological order
seriograph(cont,order=c("AI03","AI09","AI01","AI02","AI04","APQR01","AO05",
"AO03","AI05","AO01","APQR02","AI07","AI08","AO02","AI06","AO04","APQR03"))
## by omitting the row names, you delete the corresponding rows
seriograph(cont,order=c("AI09","AI06","AI08","AI05","AI07","AI04","AI03"))

## Compiègne data from Desachy (2004)
compiegne = data.frame(
A=c(13740, 13540, 12490, 6940, 6490),
B=c(375, 1520, 5255, 2880, 2350),
C=c(8270, 10110, 4220, 5800, 6900),
D=c(250, 740, 980, 3400, 2745),
E=c(20, 1230, 1395, 1510, 670),
F=c(10, 635, 1415, 2280, 3020),
G=c(40, 265, 440, 1080, 950),
H=c(80, 400, 680, 2840, 6700),
I=c(0, 0, 30, 620, 340),
J=c(5, 105, 580, 2075, 2660),
K=c(1740, 7210, 6750, 2130, 1080),
L=c(460, 1785, 5930, 2410, 570),
M=c(0, 410, 350, 910, 740),
N=c(1510, 565, 160, 410, 190),
O=c(350, 310, 10, 310, 985),
P=c(0, 450, 275, 410, 50)
)
rownames(compiegne) <- c("Période 5", "Période 4", "Période 3", "Période 2", "Période 1")

serio = seriograph(compiegne)
serio$contingency
serio$frequency
serio$EPPM
serio$EPM
}
}

\name{dataceram}
\alias{dataceram}
\docType{data}
\title{
Data set of archeology
}
\description{
This important dataset comes from the Collective Research Project (CRP) on Medieval and Modern pottery in the Middle Loire Basin. This is a long-term project, since it began in 1996 and has already been the subject of two books on the subject (Husi dir. 2003 and 2013), as well as an online logical publication (Husi dir. 2022).
}
\author{
Husi P. (dir.) – La céramique médiévale et moderne du bassin de la Loire moyenne, chrono-typologie et transformation des aires culturelles dans la longue durée (6e —19e s.), XXe Supplément à la Revue Archéologique du Centre de la France, FERACF, Tours, 2022.
}
\usage{data("dataceram")}
\format{
  List of three objects with 226 observations.
  \describe{
\item{$contingency}{(data.frame) Contingency table of the quantities of 183 types of pottery sherds in the 226 sets.}
\item{$timerange}{(data.frame) The first column corresponds to the identifier (sets), the second and the third to the lower and upper limits of the temporal range of the observations.}
\item{$geographic_area}{Vector of the geographical area of each observation.}
  }
}

\examples{
  data(dataceram)
  str(dataceram)
  str(dataceram$contingency)
  head(dataceram$timerange)
  head(dataceram$geographic_area)
}

\name{CAdist}
\alias{CAdist}
\title{
Distance matrix based on correspondence analysis results}
\description{
Perform a correspondence analysis on a contingency table and then return the distance matrix of the coordinates (you can choose the number of axes to use to build the distance matrix with the nCP parameter).
}
\usage{
CAdist(df, nPC = NULL, graph = TRUE)
}
\arguments{
  \item{df}{
Data.frame, matrix or table with the data for the correspondence analysis.}
  \item{nPC}{
Number of principal components to be retained for the construction of the distance matrix. Must be between 1 and the minimum of ncol - 1 and nrow - 1. Could also be "max".}
  \item{graph}{
Logical parameter for plotting the Correspondence Analysis (Axis1, Axis2).}
}

\value{
\item{D }{The distance matrix}
}
\author{
A. COULON

L. BELLANGER

P. HUSI
}
\examples{
data(datangkor)

## contingency table
cont <- datangkor$contingency

distance <- CAdist(cont, nPC = "max")
distance

## run without printing the plot
distance <- CAdist(cont, nPC = "max", graph=FALSE)
}

\name{perioclust}
\alias{perioclust}
\title{perioclust}
\usage{
perioclust(
  D1,
  D2,
  alpha="EstimateAlphaForMe",
  k=NULL,
  title="notitle",
  method="complete",
  suppl_plot=TRUE
)
}
\description{
Compromised Hierarchical bottom-up clustering method.
The method uses two sources of information. The merging of the two data sources is done by a parameter (\eqn{\alpha}) that allows to weight each source.
\deqn{D_\alpha = \alpha D_1 + (1-\alpha) D_2}\cr
}

\arguments{
  \item{D1}{
First dissimilarity matrix (square matrix) or distance matrix. Could be a contingency table (see \link[SPARTAAS]{CAdist}). A factorial correspondence analysis is performed using the distances (chi-square metric).
}
  \item{D2}{
Second dissimilarity matrix (square matrix), same size as D1, or distance matrix.
}
  \item{alpha}{
The mixing parameter in order to generate the \eqn{D_\alpha} matrix (in [0;1]). Formula: \eqn{D_\alpha = \alpha D_1 + (1-\alpha) D_2}
}

  \item{k}{
The number of clusters you want.
}

  \item{title}{
The title to be displayed on the dendogram plot.
}

  \item{method}{
The agglomeration method to be used. This should be (an unambiguous abbreviation of) one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
}

  \item{suppl_plot}{
Logical defines whether additional plots are to be displayed (WSS and average sil plot).
}
}

\details{
\strong{CAH}\cr
Data fusion (parameter \eqn{\alpha} optimal value see \link[SPARTAAS]{hclustcompro_select_alpha}). It is necessary to define the appropriate proportion for each data source. This is the first sensitive point of the method that the user has to consider. A tool is provided to help him in his decision.

\strong{Cut dendrogram}\cr
The division into classes and subclasses is the second crucial point. It has to be done based on the knowledge of the study area and some decision support tools such as the cluster silhouette or the calculation of the intra-cluster variability (WSS: Within Sum of Square). You can use \link[SPARTAAS]{perioclust_subdivide} to subdivide a cluster into sub-clusters.
}

\value{
The function returns a list (class: perioclust_cl):
\item{D1 }{First dissimilarity matrix (square matrix)}
\item{D2 }{Second dissimilarity matrix (square matrix)}
\item{D_alpha }{The matrix use in the CAH resulting from the mixing of the two matrices (D1 and D2)}
\item{alpha }{Alpha}
\item{tree }{An object of class hclust, describing the tree generated by the clustering process (see \link[stats]{hclust})}
\item{cluster }{The cluster number vector of the selected partition}
\item{cutree }{Plot of the cut dendrogram}
\item{call }{How you call the function}
\item{cont }{Original contingency data (if D1 is a contingency table)}
}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
data(datangkor)

#network stratigraphic data (Network)
network <- datangkor$stratigraphy

#contingency table
cont <- datangkor$contingency

#obtain the dissimilarities matrices
distance <- CAdist(cont, nPC = 11)
constraint <- adjacency(network)

#You can also run hclustcompro with the dist matrix directly
perioclust(D1 = distance, D2 = constraint, alpha = 0.7, k = 4, method = "ward.D2")
}

\name{datacerardat}
\alias{datacerardat}
\docType{data}
\title{
Data set of archaeology.
}
\description{
Data sets on Tours pottery.
Contains three objects:
- Pottery sherd contingency table (data.frame 280 sets and 391 technical groups)
- The date (if known) of the sets (vector)
- Column index for supplementary  sets (vector)
}
\usage{data("datacerardat")}
\format{
  A list with 3 objects.
  \describe{
    \item{\code{df}}{Ceramic contingency table. a integer data.frame}
    \item{\code{date}}{The date (if known) of the sets. a numeric or NA vector}
    \item{\code{col.sup}}{Column index for supplementary  sets. a numeric vector}
  }
}
\examples{
data(datacerardat)
datacerardat
## maybe str(datacerardat);
}
\keyword{datasets}

\name{hclustcompro_app}
\alias{hclustcompro_app}
\title{
Launch the shiny application.
}
\description{
see \link[SPARTAAS]{hclustcompro}, \link[SPARTAAS]{hclustcompro_select_alpha}, \link[SPARTAAS]{seriograph}.
You can also check the wiki on the application.}
\usage{
hclustcompro_app()
}

\value{No return value}

\author{
A. COULON

L. BELLANGER

P. HUSI
}



\examples{
\donttest{
if(interactive()){
  hclustcompro_app()
}
}

}

\name{arrangePlot}
\alias{arrangePlot}

\title{
arrangePlot
}
\description{
plot muliple cerardat output
}
\usage{
arrangePlot(x,
            which=1:2,
            main="model dateEv (red) and dateAc (black)",
            xlab="Date",
            ylab="Density",
            col1=rgb(0.93,0.23,0.23,0.5),
            col2="black",
            xlim=NULL,
            ylim=NULL,
            date=NULL,
            col_date=NULL,
            lty_date=NULL,...
)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{x}{
a \link[SPARTAAS]{cerardat} output.
}
  \item{which}{
  Vector containing the plots you want (rownames or row number on the contingence table: numeric).
}
  \item{main}{
  an overall title for the plot: see title \link[graphics]{title}.
}
  \item{xlab}{
  a title for the x axis: see \link[graphics]{title}.
}
  \item{ylab}{
  a title for the y axis: see \link[graphics]{title}.
}
  \item{col1}{
  Color of the Event curve (color name, hex code, \link[grDevices]{rgb} function \{grDevices\} for transparency).
}
  \item{col2}{
  Color of the Accumulation curve (color name, hex code, \link[grDevices]{rgb} function \{grDevices\} for transparency).
}
  \item{xlim}{
  Two numeric values, specifying the left limit and the right limit of the scale.
}
  \item{ylim}{
  Two numeric values, specifying the lower limit and the upper limit of the scale.
}
  \item{date}{
  x coordinate to highlight with a vertical line accross all plots (vector).
}
  \item{col_date}{
  Vector of color for vertical lines (one for each date, could be numeric, hexadecimal, rgb).
}
  \item{lty_date}{
  Vector of line type (see lty in \link[graphics]{par}, don't use lty with arrangePlot) for vertical lines.
}
  \item{...}{
  other parameters to be passed through to plotting functions (bty, lwd, cex.axis see \link[graphics]{par}).
}
  
}

\author{
A. COULON

L. BELLANGER

P. HUSI
}
\examples{
\donttest{
data("datacerardat")

resultat = cerardat(df = datacerardat$df,
  row.sup = datacerardat$row.sup,
  date = datacerardat$date,
  nf = NULL,
  confidence = 0.95,
  graph = TRUE
)

#Plot the first 5 with their dateEv fit as vertical line
arrangePlot(resultat,
  which = 1:5,
  date = resultat$prediction$Fit_dateEv[1:5]
)
arrangePlot(resultat,
  which = 1:5,
  xlim = c(300, 600),
  date = resultat$prediction$Fit_dateEv[1:5]
)

#Plot using row names and highlighting custom dates
arrangePlot(resultat,
  which = c("Z067","Z070","Z074"),
  main = "Neuvy-Le-Roi", 
  xlim = c(300, 1300), 
  date = c(669, 745, 850, 1115),
  col_date = c("red", rgb(0.20, 0.40, 0.30), 4, '#5614be')
)

#You can change the line type (lty) and the color of the curve
arrangePlot(resultat,
  which = c("Z067", "Z070", "Z074"),
  main = "Neuvy-Le-Roi", 
  col1 = rgb(0.7, 0.7, 0.7, 0.6), # the last arg of rgb is alpha for opacity
  col2 = "black",
  xlim = c(300, 1300), 
  date = c(669, 745, 850, 1115), 
  col_date = "grey50",
  lty_date = c(1, 2, 3, 4)    # 1: line, 2: dashed line, 3: dotted line, 4:mixed dotted dashed line)
)

#Default first 10 plot
arrangePlot(resultat, which = 1:10)

#You can remove the box arround the plot region
arrangePlot(resultat,
  which = 1:10,
  xlim = c(200, 1000),      # fix the x axis between 200 and 1000
  date = seq(250, 950, 50), # add one line every 50 years between 250 and 950
  col_date = "grey80",
  lty_date = 3,      # dotted line (1: line, 2: dashed line)
  bty = 'n',         # remove box ('L': keep bottom and left) 
  cex.axis = 0.8     # control axis text size (only y axis here) 
)

#minimalist version (remove y-axis)
arrangePlot(resultat,
  which = 1:10,
  xlim = c(200, 1000),       # fix the x axis between 200 and 1000
  date = seq(250, 950, 50),  # add one line every 50 years between 250 and 950
  col_date = "grey80",
  lty_date = 3,              # dotted line (1: line, 2: dashed line)
  ylab = '',                 # remove the y-axis name ('Density')
  fg = rgb(0, 0, 0, 0),      # color box transparent (invisible) 
  col.axis = rgb(0, 0, 0, 0) # color axis text (only y axis here) transparent (invisible)
)

}
}

\name{hclustcompro_select_alpha}
\alias{hclustcompro_select_alpha}
\title{
Estimate the optimal value(s) of the \eqn{\alpha} parameter.}
\description{
The following criterion "balances" the weight of \eqn{D_1} and \eqn{D_2} in the final clustering. The \eqn{\alpha} value is only a point estimate but the confidence interval gives a range of possible values.
\cr\cr
Based on a resampling process, we generate clones and recalculate the criteria according to \eqn{\alpha} (see below).}

\usage{
hclustcompro_select_alpha(
    D1,
    D2,
    acc=2,
    method="complete",
    resampling=TRUE,
    iter=5,
    suppl_plot=TRUE
)
}
\arguments{
  \item{D1}{
    First dissimilarity matrix or contingency table (square matrix).
  }
  \item{D2}{
    Second dissimilarity matrix or network data (square matrix) of the same size as D1.
  }
  \item{acc}{
    Number of digits after the decimal point for the alpha value.
  }
  \item{method}{
    The agglomeration method to be used. This should be (an unambiguous abbreviation of) one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
  }
  \item{resampling}{
    Logical for estimating the confidence interval with a resampling strategy. If you have a lot of data, you can save computation time by setting this option to FALSE.
  }
  \item{iter}{
    The number of clones checked per observation. (200 observations iter=1: ~30 sec, 1000 observations iter=1: ~40 min).
  }
  \item{suppl_plot}{
    Logical defines whether additional plots should be displayed.
  }
}
\details{
\strong{Definition of the criterion:}\cr\cr

A criterion for choosing \eqn{\alpha \in [0;1]} must be determined by balancing the weights between the two sources of information in the final classification. To obtain \eqn{\alpha}, we define the following criterion
\deqn{CorCrit_\alpha = |Cor(dist_{cophenetic},D_1) - Cor(dist_{cophenetic},D_2)|}  \deqn{Equation (1)}
The \eqn{CorCrit_\alpha} criterion in (1) represents the difference in absolute value between two cophenetic correlations (cophenetic correlation is defined as the correlation between two distance matrices. It is calculated by considering the half distance matrices as vectors. It measures how faithfully a dendrogram preserves the pairwise distances between the original unmodeled data points). The first correlation is related to the comparison between \eqn{D_1} and the ultrametric distances of the clustering with \eqn{\alpha} fixed, while the second compares \eqn{D_2} and the ultrametric distances of the clustering with \eqn{\alpha} fixed. Then, in order to compromise between the information provided by \eqn{D_1} and \eqn{D_2}, we decided to estimate \eqn{\alpha} with \eqn{\hat{\alpha}} such that:
\deqn{\hat{\alpha} = min CorCrit_\alpha}
\deqn{Equation (2)}

\strong{Resampling strategy:}\cr\cr

This is done by creating a set of "clones" for each observation \eqn{i}. A clone \eqn{c} of observation \eqn{i} is a copy of observation \eqn{i} for which the distances from the second source have been modified. The modification is made by copying the distances for the second source from another observation \eqn{j}. A clustering is then performed using the combination defined in (1) with \eqn{D_1^{(c)}} an \eqn{(n+1)\times(n+1)} matrix where observations \eqn{i} and \eqn{c} are identical and \eqn{D_2^{(c)}} an \eqn{(n+1)\times(n+1)} matrix where the clone \eqn{c} of \eqn{i} has different distances from those of \eqn{i}. A set of clones is generated by varying \eqn{j} for all observations except \eqn{i}. We can generate a set of \eqn{n-1} clones for each element \eqn{i} in \eqn{n}, so \eqn{n(n-1)} clones in total.

Intuitively, by varying \eqn{\alpha} between 0 and 1, we will be able to identify when the clone and the original observation are separated on the dendrogram. This moment will correspond to the value of alpha above which the weight given to the information about the connection between observations contained in \eqn{D_2} has too much influence on the results compared to that of \eqn{D_1}.

Let \eqn{CorCrit_\alpha^{(c)}} define the same criterion as in (1), where \eqn{D_1} and \eqn{D_2} are replaced by \eqn{D_1^{(c)}} and \eqn{D_2^{(c)}} respectively.
The estimated \eqn{\alpha} is the mean of the estimated values for each clone.\cr
For each clone \eqn{c}:
\deqn{\hat{\alpha}^{(c)} = min   CorCrit_\alpha^{(c)}}
\deqn{Equation (3)}
\eqn{\hat{\alpha}^*} is the mean of \eqn{\hat{\alpha}^{(c)}}. In the same spirit as confidence intervals based on bootstrap percentiles (Efron & Tibshirani, 1993), a percentile confidence interval based on replication is also be obtained using the empirical percentiles of the distribution of \eqn{\hat{\alpha}^{(c)}}.
\deqn{\hat{\alpha}^* = \frac{1}{n(n-1)} \sum{ \hat\alpha^{(c)} }}
\deqn{Equation (4)}
\deqn{c  \in [1; n(n-1)]}

\strong{Warnings:}\cr\cr
It is possible to observe an \eqn{\alpha} value outside the confidence interval. In some cases, this problem can be solved by increasing the number of iterations or by changing the number of axes used to construct the matrix D1 after the correspondence analysis. If the \eqn{\alpha} value remains outside the interval, it means that the data are noisy and the resampling procedure is affected.
}


\value{
The function returns a list (class: selectAlpha_obj).
  \item{alpha }{The estimated value of the alpha parameter (min CorCrit_alpha)}
  \item{alpha.plot }{The CorCrit curve for alpha between 0 and 1}
If resampling = TRUE
  \item{sd }{The standard deviation}
  \item{conf }{The confidence interval of alpha}
  \item{boxplot }{The boxplot of alpha estimation with resampling}
  \item{values }{All potential alpha values obtained from each clone}
%%  \item{comp2 }{Description of 'comp2'}
}

\author{
A. COULON

L. BELLANGER

P. HUSI
}


\examples{
data(datangkor)

#network stratigraphic data (Network)
network <- datangkor$stratigraphy

#contingency table
cont <- datangkor$contingency

dissimilarity <- CAdist(cont,nPC="max",graph=FALSE)
constraint <- adjacency(network)

hclustcompro_select_alpha(D1 = dissimilarity, D2 = constraint, method = "ward.D2")
hclustcompro_select_alpha(D1 = dissimilarity, D2 = constraint, acc = 3,
                          method = "ward.D2", resampling = TRUE)
}

\name{datacancer}
\alias{datacancer}
\docType{data}
\title{
Data set of cancerology.
}
\description{
Longitude, latitude, number of thyroid cancers. The data concern two French departments (Loire-Atlantique and Vendee) between 1998 and 2012. For confidentiality reasons, the data are simulated.
}
\author{
M. Karakachoff (IR CHU - l'institut du Thorax INSERM UMR 1087 - CNRS UMR 6291) Nantes, France

F. Molinie (resp. Loire-Atlantique-Vendee cancer registry -
\href{https://www.registre-cancers-44-85.fr/}{\code{registre-des-cancers}}) France
}

\usage{data("datacancer")}
\format{
  List of two objects with 359 observations on the following 3 variables.

  \strong{$coord (data.frame):}
  \describe{
    \item{\code{longitude}}{a numeric vector: geographical coordinate}
    \item{\code{latitude}}{a numeric vector: geographical coordinate}
  }
  \strong{$var (vector):}
  \describe{
    \item{\code{var}}{a numeric vector: number of thyroid cancers (simulated)}
  }
}


\examples{
  data(datacancer)
  str(datacancer)
  head(datacancer$coord)
  str(datacancer$var)
}

\name{mapclust_cut_tree}
\alias{mapclust_cut_tree}
\encoding{utf-8}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Function to cut the dendrogram for a new height (distance limit) or a new number of clusters and map the new partition
}
\description{
The function returns the new map, one dendrogram with the cut line, the silhouette width and the new vector cluster. You must call \link[SPARTAAS]{mapclust} beforehand to get a \code{mapclust_cl} object that can be used by mapclust_cut_tree.

}
\usage{
mapclust_cut_tree(classification, nb_grp = NA, dlim = NA)
}
\arguments{
  \item{classification}{
The object return by mapclust  \code{Class: mapclust_cl}
}
  \item{nb_grp}{
The number of groups you want to have on the partition. Must be an integer. (don't use dlim in this case)
}
  \item{dlim}{
The value of dlim at which you wish to cut the dendrogram. You can enter any value (numerical) and the function will select the nearest lower dlim with the same partition. (do not use nb_grp in this case).
}
}

\value{
the function returns a list.

\strong{Plot:}
\item{dendrogram}{The global dendrogram (hclust object)}
\item{dendrogram_ggplot}{The global dendrogram (ggplot2 object)}
\item{cuttree}{The cut dendrogram}
\item{map}{The map of the selected partition}
\item{AveSilPlot}{The average silhouette width plot (for each partition)}
\item{WSSPlot}{The Within Sum of Square plot (for each partition)}
\item{silhouette}{The silhouette plot of the selected partition}
\strong{Value:}
\item{X}{The x-coordinate data you have used}
\item{Y}{The y-coordinate data you have used}
\item{var}{The regionalised variable data you have used}
\item{label}{The label vector you have used}
\item{density}{The estimated density based on var. Equal to var if you used a unidimensional density variable}
\item{cluster}{The cluster vector of the selected partition}
\item{Plabel}{Logical parameter to activate or not the printing of labels on the dendrogram}
\item{fullhist}{The compositional cluster for each observation}
\item{hist}{The compositional cluster without duplicates (corresponds to the split on the dendogram)}
\item{dlim}{The vector of the different limit distances}
\item{cutdlim}{The select dlim for the cut of the current partition}
\item{DiMatrix}{The weighted euclidean distance matrix}
\item{silhouetteData}{The silhouette data of the selected partition}
\item{AveSilData}{The average silhouette value for each partition}
\item{Moran}{The Moran index for each group for each partition}
\item{lonlat}{Logical parameter, whether your coordinates are in latitude-longitude format or not}
}

\author{
A. COULON
L. BELLANGER
P. HUSI
}

\examples{
\donttest{
## loading data
data(datarcheo)

##First you need to run the mapclust function.
#This function allow you to obtain one partition
# object <- mapclust( coord = ..., var = ..., label = ...)
OldClassif <- mapclust(datarcheo$coord, datarcheo$var, datarcheo$label, n = 4)

##In order to cut the dendrogram for another dlim
NewCut <- mapclust_cut_tree(classification = OldClassif, dlim = 0.37)
##In order to cut the dendrogram for another number of cluster
NewCut2 <- mapclust_cut_tree(classification = OldClassif, nb_grp = 4)

#See evaluation using Silhouette width by running:
NewCut$silhouette
#If the plot is empty try to increase the height of the window (full screen).

#See summary of the data by running:
summary(NewCut$silhouetteData)
}
}

\name{perioclust_subdivide}
\alias{perioclust_subdivide}
\title{
Subdivide a cluster after running perioclust.
}

\description{
Allow the user to split one cluster into sub-clusters. This function only works with 'perioclust_cl' object returned by the perioclust function.
}

\usage{
perioclust_subdivide(perioclust_cl,cluster,nb_class)
}

\arguments{
  \item{perioclust_cl }{A perioclust_cl object}
  \item{cluster }{The number of the cluster. Numbered from left to right on the dendrogram (1, 2, ...)}
  \item{nb_class }{The number of sub-clusters you want}
}

\value{
  see \link[SPARTAAS]{perioclust}
}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
data(datangkor)

#network stratigraphic data (Network)
network <- datangkor$stratigraphy

#contingency table
cont <- datangkor$contingency

#obtain the dissimilarities matrices
distance <- CAdist(cont, nPC = 11)
constraint <- adjacency(network)

#You can also run hclustcompro with the dist matrix directly (number of cluster 7)
clustering <- perioclust(D1 = distance, D2 = constraint, alpha = 0.7,k=7,method="ward.D2")
clustering <- perioclust_subdivide(clustering,cluster = 5, nb_class = 2)

#subdivide more than one cluster (number of cluster 7)
clustering2 <- perioclust(D1 = distance, D2 = constraint,0.7,k=7, method = "ward.D2")
clustering2 <- perioclust_subdivide(clustering2,cluster = c(5,7), nb_class = c(2,2))
}

\name{adjacency}
\alias{adjacency}
\title{
Dissimilarity matrix based on connectivity information.
}
\description{
From the data of a network, we build a contiguity matrix. Based on this matrix, we generate a dissimilarity matrix.
The matrix contains only 0 or 1, 1 if there is no relationship and 0 if there is a relationship.
The network object is a data frame with two columns. The first column contains the elements of the network and the second column contains a list of all other elements related to it. The list is a string consisting of the names of the elements separated by commas (see example).
}
\usage{
adjacency(network)
}
\arguments{
  \item{network}{
Data frame with 2 columns. The first contains all elements (nodes) and the second a string with all related elements (links).
}
}
\value{
\item{D }{Dissimilarity matrix based on adjacency.}
}
\author{
A. COULON

L. BELLANGER

P. HUSI
}
\examples{
data(datangkor)

## network stratigraphic data (Network)
network <- datangkor$stratigraphy

dissimilarity <- adjacency(network)
dissimilarity
}

\name{datangkor}
\alias{datangkor}
\docType{data}
\title{
Data set of archeology
}
\description{
The archaeological data come from excavations carried out at Angkor Thom (Cambodia), the capital of the Khmer Empire between the 9th and 15th centuries (Gaucher, 2004). The dataset consists of the pottery assemblages (quantities of different types of pottery sherds contained in the sets - ..$contingency) and the stratigrpahy of the sets from 3 disconnected archaeological sites (..$stratigraphy).
}
\author{
Gaucher, J. (2004). Angkor Thom, une utopie réalisée ? Structuration de l’espace et modèle indien d’urbanisme dans le Cambodge ancien. Arts Asiatiques, Volume 59, pp. 58-86.
}
\usage{data("datangkor")}
\format{
  List of two objects with 17 observations.
  \describe{
\item{$contingency}{(data.frame) Contingency table of the quantities of 12 types of pottery sherds in the 17 sets.}
\item{$stratigraphy}{(data.frame) Saves the stratigraphic network. The first column corresponds to the nodes (sets) and the second to the edges by listing the nodes connected to it.}
  }
}

\examples{
  data(datangkor)
  str(datangkor)
  str(datangkor$contingency)
  str(datangkor$stratigraphy)
}

\name{SPARTAAS-package}
\alias{SPARTAAS-package}
\alias{SPARTAAS}
\docType{package}
\title{
SPARTAAS
}
\description{
\strong{S}tatistical \strong{Pa}ttern \strong{R}ecognition and da\strong{T}ing using \strong{A}rcheological \strong{A}rtefacts assemblage\strong{S}.
}
\details{
\packageDescription{SPARTAAS}
}
\author{
\packageAuthor{SPARTAAS}

Maintainer: \packageMaintainer{SPARTAAS}
}
\references{
Bellanger L., Coulon A., Husi P., 2021 – Perioclust: a new Hierarchical agglomerative clustering method including temporal or spatial ordering constraints. Springer Series, Studies in Classification, Data Analysis, and Knowledge Organization. <doi: 10.1007/978-3-030-60104-1>

Bellanger L., Husi P., Laghzali Y. (2015). Spatial statistic analysis of dating using pottery: an aid to the characterization of cultural areas in West Central France. In : Traviglia A. ed., Across Space and Time, Proceedings of the 41th International Conference on Computer Applications and Quantitative Methods in Archaeology (CAA-2013), Perth (Australie), Amsterdam University Press : 276-282.
}

\name{hclustcompro}
\alias{hclustcompro}
\title{hclustcompro}
\usage{
hclustcompro(d, method = "complete", weight = NULL)
}
\description{
Compromised Hierarchical bottom-up clustering method.
The method uses at least two sources of information. The merging of the data sources is done by weigth that allows to weight each source.
Exemple for two sources: \deqn{D_\alpha = \alpha D_1 + (1-\alpha) D_2}\cr
}

\arguments{
  \item{d}{
a list of dissimilarity structure as produced by dist.
}
  \item{method}{
the agglomeration method to be used. This should be (an unambiguous abbreviation of) one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
}
  \item{weight}{
The parameter in order to generate a mixed matrix on which the classical hclust method is applied (same length as d). \cr Formula: \deqn{D_\alpha = w_1 D_1 + w_2 D_2 + ...}\cr where \eqn{w_1} is the first value of vector weight, \eqn{D_1} the first matrix of d and \eqn{D_\alpha} the mixed matrix used in hclust. Sum of weight must be 1.
}
}

\details{
\strong{CAH}\cr
Data fusion (parameter optimal value see \link[SPARTAAS]{hclustcompro_select_alpha}). It is necessary to define the appropriate proportion for each data source. This is the first sensitive point of the method that the user has to consider. A tool is provided to help him in his decision.

\strong{Cut dendrogram}\cr
The division into classes (see \link[SPARTAAS]{hclustcompro_cutree}) is the second crucial point. It has to be done based on the knowledge of the study area and some decision support tools such as the cluster silhouette or the calculation of the intra-cluster variability (WSS: Within Sum of Square).
}

\value{
See \link[stats:hclust]{stats::hclust} or \link[fastcluster:hclust]{fastcluster::hclust}
}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
data(datangkor)

#network stratigraphic data (Network)
network <- datangkor$stratigraphy

#contingency table
cont <- datangkor$contingency

#obtain the dissimilarities matrices
distance <- CAdist(cont, nPC = 11)
constraint <- adjacency(network)

#You can also run hclustcompro with the dist matrix directly
tree = hclustcompro(d = list(distance, constraint), weight = c(0.7, 0.3), method = "ward.D2")
cluster = hclustcompro_cutree(tree, k=5, reorder=TRUE, letter=TRUE)
}

\name{datarcheo}
\alias{datarcheo}
\docType{data}
\title{
Data set of archeology
}
\description{
Latitude, longitude, absolute difference between two dates and the name of the archaeological site. The data concern the dating of archaeological contexts in West-central France based on a large collection of medieval pottery finds. Two original statistical models are developed to estimate context dates using pottery. The  absolute difference is calculated for each context. The data are based on a collective research on medieval pottery directed by P. Husi (\href{https://citeres.univ-tours.fr/contrat/la-ceramique-medievale-dans-la-vallee-de-la-loire-moyenne/}{\code{"La céramique médiévale dans la vallée de la Loire moyenne"}}).
}
\author{
P. Husi IR CNRS, UMR CITERES-LAT, CNRS/Tours University, France :
}
\usage{data("datarcheo")}
\format{
  List of three objects with 240 observations on the following 4 variables.
  \describe{
\strong{$coord (data.frame):}
    \item{\code{longitude}}{a numeric vector: geographical coordinate}
    \item{\code{latitude}}{a numeric vector: geographical coordinate}
    \strong{$var (vector):}
    \item{\code{regionalized_var}}{a numeric vector: difference between two dating model}
    \strong{$label (vector):}
    \item{\code{noms}}{a character vector(factor): name of archeological site}
  }
}

\examples{
  data(datarcheo)
  str(datarcheo)
  head(datarcheo$coord)
  str(datarcheo$var)
  levels(datarcheo$label)
}

\name{mapclust}
\alias{mapclust}
\encoding{utf-8}
\title{
Divise hierarchical Clustering using Spatialpatches algorithm.
}

\description{

This function performs a divisive hierarchical clustering on a regionalised variable using the spatial patches algorithm (Woillez et al. 2007; Woillez, Rivoirard and Petitgas 2009). It is a top-down hierarchical clustering with a geographical constraint. It is possible to cut the tree by clicking on the dendrogram at the desired level. The results include a description of the clusters and graphics. When slicing the dendrogram, you can look at the two plots (WSSPlot and AveSilPlot) that show the relatively good quality of the partitions. The first shows the Within Sum of Square (WSS) for each partition and you can use the elbow approach to select a partition. The second graph shows the average silhouette width. This index is between -1 and 1. The closer it is to 1, the better the partition. See \link[cluster]{silhouette}.

If you want to cut the dendogram to a different dlim or number of clusters, you can do so without re-running  \code{mapclust() with \link[SPARTAAS]{mapclust_cut_tree}}.
}
\usage{
mapclust(
    coord,
    var,
    label = "Nolabel",
    iter = 20,
    Plabel = TRUE,
    lonlat = TRUE,
    positive_var = FALSE,
    n = NULL
)
}

\arguments{
  \item{coord}{
The x (longitude) and y (latitude) coordinates of the data.frame or matrix dimension 2.
}
  \item{var}{
The regionalised variable(s) of interest
}
  \item{label}{
(optional) The names of the samples or an identifier. Must be a factor.
}
  \item{iter}{
The number of iterations. The number of different dlim you want to test (must be greater than 10).
}
  \item{Plabel}{
Logical parameter to activate or not the printing of labels on the dendrogram.
}
  \item{lonlat}{
Logical parameter to activate or not the cartography in lonlat system with leaflet (base map).
}
  \item{positive_var}{
logical parameter that defines whether your variable of interest is positive or not.
}
  \item{n}{
Number of groups. If NULL, you can select the number of groups by clicking on the dendrogram.
}
}
\details{
Dlim is the selected minimum distance from the sample to the patch centre: to identify patches (units are those of coordinates). The dlim is automatically initialised and does not need to be set by the user. The minimum data is a data frame or matrix with at least 3 columns.
}
\value{
the function returns a list.

\strong{Plot:}
\item{dendrogram}{The global dendrogram (hclust object)}
\item{dendrogram_ggplot}{The global dendrogram (ggplot2 object)}
\item{cuttree}{The cut dendrogram}
\item{map}{The map of the selected partition}
\item{AveSilPlot}{The average silhouette width plot (for each partition)}
\item{WSSPlot}{The Within Sum of Square plot (for each partition)}
\item{silhouette}{The silhouette plot of the selected partition}
\strong{Value:}
\item{X}{The x-coordinate data you have used}
\item{Y}{The y-coordinate data you have used}
\item{var}{The regionalized variable(s) data you have used}
\item{label}{The label vector you have used}
\item{density}{The estimated density based on var. Equal to var if you used a unidimensional density variable.}
\item{cluster}{The cluster vector of the selected partition}
\item{Plabel}{Logical parameter to activate or not the printing of labels on the dendrogram}
\item{fullhist}{The compositional cluster for each observation}
\item{hist}{The compositional cluster without duplicates (corresponds to the split on the dendogram)}
\item{dlim}{The vector of the different limit distances}
\item{cutdlim}{The select dlim for the cut of the current partition}
\item{DiMatrix}{The weighted euclidean distance matrix}
\item{silhouetteData}{The silhouette data of the selected partition}
\item{AveSilData}{The average silhouette value for each partition}
\item{Moran}{The Moran index for each group for each partition}
\item{lonlat}{Logical parameter, whether your coordinates are in latitude-longitude format or not}
}

\author{
A. COULON
L. BELLANGER
P. HUSI
}

\references{
Bellanger L., Coulon A. and Husi P. (2021) Determination of cultural areas based on medieval pottery using an original divisive hierarchical clustering method with geographical constraint (MapClust), Journal of Archaeological Science, Volume 132 \doi{10.1016/j.jas.2021.105431}.

Bellanger L., Husi P., Laghzali Y. (2015). Spatial statistic analysis of dating using pottery: an aid to the characterization of cultural areas in West Central France. In : Traviglia A. ed., Across Space and Time, Proceedings of the 41th International Conference on Computer Applications and Quantitative Methods in Archaeology (CAA-2013), Perth (Australie), Amsterdam University Press : 276-282.

Woillez M., Poulard J.C., Rivoirard J., Petitgas P., Bez N. (2007). Indices for capturing spatial patterns and their evolution in time,
with application to European hake (Merluccius merluccius) in the
Bay of Biscay. ICES J. Mar. Sci. 64, 537-550.

Woillez M., Rivoirard J. and Petitgas P. (2009) Notes on survey-based spatial indicators for monitoring fish populations, Aquatic Living Resources, 22 :155-164.
}

\examples{
\donttest{
###################################
## loading data
 data(datarcheo)
 data(datacancer)

###################################
### Example: 1
## Function "mapclust"
# object <- mapclust( coord = ..., var = ..., label = ...)

classification <- mapclust(datarcheo$coord, datarcheo$var, datarcheo$label, n=4)

#Global dendrogram
 classification$dendrogram
#Cut dendrogram
 classification$cuttree
#silhouette of selected partition
 classification$silhouette


#You can cut the dendrogram for another dlim
 NewCut <- mapclust_cut_tree(classification, dlim=0.30)

#See evaluation using Silhouette width by running:
 NewCut$silhouette
 #If the plot is empty try to increase the height of the window (full screen)

#See summary of the data by running:
 summary(NewCut$silhouetteData)


###################################
## kmeans comparison
# pepare data (only geographical data)
 datakmeans <- datarcheo$coord

#kmeans
 number_cluster <- 4
 cl <- kmeans(datakmeans, number_cluster)
 plot(datakmeans, col = cl$cluster)


}
}

\name{overlap}
\alias{overlap}
\title{
Temporal overlap index
}
\description{
The overlap index is the ratio between internal overlap and total overlap over time. We define the limit of total overlap as: the minimum of the lower limits of the pair of individuals and the maximum of the upper limits. We define the internal overlap limit as the maximum of the lower limits and the minimum of the upper limits.
}

\details{
The lower and upper limits must be numeric.

The dissimilarity between time periods is calculated as the ratio of the overlap of the time periods (distance in the case of disjoint time periods) to the cumulative extent of the two time periods.

As the ratio is bounded between -1 and 1, we add 1 to make it positive and normalise it so that it is between 0 and 1.

This overlap index then needs to be transformed into a dissimilarity index between sets. To do this we use the 1 - ratio. It is equal to 0 if the two time periods are identical and 1 if they are infinitely different.
}
\usage{
overlap(temporal)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{temporal}{
A data frame with tree columns: the name of the element, the lower limit and the upper limit.
}
}

\value{

\item{D }{The dissimilarity matrix base on the overlap index.}

}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
data(dataceram)
dist <- overlap(dataceram$timerange)

}

\name{mapclust_app}
\alias{mapclust_app}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Shiny application for Mapclust method
}
\description{
see \link[SPARTAAS]{mapclust}
You can also check the wiki on the application.
}
\usage{
mapclust_app()
}


\value{No return value}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
## open application
\donttest{
if(interactive()){
  mapclust_app()
}
}

}

\name{optim_select_alpha}
\alias{optim_select_alpha}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Optimized selection of the weights for the sources (hclustcompro)
}
\description{
Description of the function.
}
\usage{
optim_select_alpha(data = list(), 
                   acc = 2, 
                   method = "complete", 
                   algo = "NLOPT_GN_ORIG_DIRECT_L"
)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
liste des matrices de distances
}
  \item{acc}{
nombre de chiffre après la virgule
}
  \item{method}{
stratégie d'aggrégation utilisée par hclustcompro
}
  \item{algo}{
Algorithme utilisé par nloptr pour trouver les poids optimaux
}
}
\details{
See NLOPT website
}
\value{

\item{solution }{vector of weigths}
\item{res_nlopt }{object return by nloptr}

}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
## The function is currently defined as
}

\name{cerardat_estim_nf}
\alias{cerardat_estim_nf}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Number of axes to keep
}
\description{
Estimate the correct number of axes to keep in the regression model (see \link[SPARTAAS]{cerardat}). We use a resampling procedure see \link[caret]{train}.
}
\usage{
cerardat_estim_nf(df, row.sup, date, option=list(method = "LOOCV"))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{df}{
The data (data.frame) is a contingency table with the observations in rows and the technical groups in columns.
}
  \item{row.sup}{
Index of supplementary rows in df (vector).
}
  \item{date}{
The dates of each observation or NA (vector).
}
  \item{option}{
A list of values that define how the resampling method acts. See detail.
}
}

\details{
We implement a resampling procedure (see \link[caret]{train}) and calculate several metrics in order to determine the number of axes to retain (nf).

The metrics are: PRESS (always use LOOCV for calculations), RMSE, MAE, and adjusted R².
In addition, hypothesis tests for normality (\link[stats]{shapiro.test}), heteroscedasticity (\link[lmtest]{bptest}) and autocorrelation (\link[lmtest]{dwtest}) are carried out for each value of nf.

You can give a list of options, there are 3 diffrents way of resampling implemented :

Leave-One-Out: `option=list(method="LOOCV")` (Default)

K-fold cross-validation: `option=list(method="cv", number=5)`

repeated K-fold cross-validation: `option=list(method="repeatedcv", number=5, repeats=10)`

Warning: For the CV and repeated CV methods, ensure you have sufficient data per fold. These approaches are particularly useful when the computation time for LOOCV becomes too long (For small datasets, they may generate warnings).

Note: when using the LOOCV method, the values for the Root Mean Square Error (RMSE) are essentially the same as those for PRESS, with the difference that RMSE is the mean rather than the sum.
}

\value{
\item{nf }{Number of axes to keep (default minimal PRESS value)}
\item{PRESS }{plot of the Predicted Residual Error Sum of Squares.}
\item{RMSE }{plot of the Root Mean Squared Error.}
\item{MAE }{plot of the Mean Absolute Error.}
\item{adj.R_sq }{plot of the Coefficient of determination R².}
\item{Pvalue }{plot of p-values from statistical tests on the hypotheses.}
\item{data }{data frame of MSE, PRESS and R_sq values.}
\item{data_stat }{data frame of the p-values.}

}

\references{
Bellanger L. and Husi P. (2012) Statistical tool for dating and interpreting archaeological contexts using pottery. Journal of Archaeological Science, Elsevier, 39 (4), pp.777-790. doi:10.1016/j.jas.2011.06.031.
}
\author{
A. COULON

L. BELLANGER

P. HUSI
}


\examples{
\donttest{
data(datacerardat)
res = cerardat_estim_nf(datacerardat$df, datacerardat$row.sup, datacerardat$date)

#Number of axes to keep (minimal PRESS value)
res$nf

#the plots
res$PRESS
res$MAE
res$adj.R_sq

#p-value
res$Pvalue
}
}

\name{hclustcompro_cutree}
\alias{hclustcompro_cutree}
\title{
Cut a Tree into Groups of Data.
}

\description{
Cuts the tree and draws rectangles around the branches of the dendrogram highlighting the corresponding clusters. This function is a combination of \link[stats]{cutree} and \link[stats]{rect.hclust}.
}

\usage{
hclustcompro_cutree(tree, k, main="", sub="", xlab="", ylab="",
    reorder=FALSE, letter=FALSE
)
}

\arguments{
  \item{tree }{a tree as produced by hclust. hclustcompro_cutree() only expects a list with components merge, height, and labels, of appropriate content each.}
  \item{k }{an integer scalar or vector with the desired number of groups}
  \item{main }{an overall title for the plot: see \link[graphics]{title}.}
  \item{sub }{a subtitle for the plot: see \link[graphics]{title}.}
  \item{xlab }{a title for the x axis: see \link[graphics]{title}.}
  \item{ylab }{a title for the y axis: see \link[graphics]{title}.}
  \item{reorder }{logical, reorder the number cluster (from left to right on the dendrogram).}
  \item{letter }{logical, replace number by letter (1->A; 2->B...).}
}

\value{
hclustcompro_cutree returns a vector with group memberships.
}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
data(datangkor)

#network stratigraphic data (Network)
network <- datangkor$stratigraphy

#contingency table
cont <- datangkor$contingency

#obtain the dissimilarities matrices
distance <- CAdist(cont, nPC = 11)
constraint <- adjacency(network)

#You can also run hclustcompro with the dist matrix directly (number of cluster 7)
tree <- hclustcompro(d = list(distance, constraint), weight = 0.7,method="ward.D2")
clustering <- hclustcompro_cutree(tree, k = 5)

}

\name{cerardat_app}
\alias{cerardat_app}
\title{
  Launch the shiny application.
}
\description{
  see \link[SPARTAAS]{cerardat}.
  }
\usage{
  cerardat_app()
}

\value{No return value}

\author{
  A. COULON

  L. BELLANGER

  P. HUSI
}



\examples{
  \donttest{
    if(interactive()){
      cerardat_app()
    }
  }

}

\name{hclust}
\alias{hclust}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Hierarchical clustering for multiple datasets (Compromised clustering).
}
\description{
Implementation of \link[stats:hclust]{stats::hclust} for dealing with multiple dissimilarities matrices.
We use \link[fastcluster:hclust]{fastcluster::hclust}, another implementation of hclust with faster algorithms.
}
\usage{
hclust(d, method = "complete", members = NULL, weight = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{d}{
a dissimilarity structure as produced by dist or a list of dissimilarity structure.
}
  \item{method}{
the agglomeration method to be used. This should be (an unambiguous abbreviation of) one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
}
  \item{members}{
NULL or a vector with length size of d. See \link[stats:hclust]{stats::hclust} ‘Details’ section.
}
  \item{weight}{
The parameter in order to generate a mixed matrix on which the classical hclust method is applied (same length as d). \cr Formula: \deqn{D_\alpha = w_1 D_1 + w_2 D_2 + ...}\cr where \eqn{w_1} is the first value of vector weight, \eqn{D_1} the first matrix of d and \eqn{D_\alpha} the mixed matrix used in hclust. Sum of weight must be 1.
}
}
\details{
Data fusion (parameter weigth): it is necessary to define the appropriate proportion for each data source. This is the first sensitive point of the method that the user has to consider. A tool is provided to help him in his decision see \link[SPARTAAS]{hclustcompro_select_alpha} (case with only 2 matrices) or \link[SPARTAAS]{optim_select_alpha}.
}
\value{
See \link[stats:hclust]{stats::hclust} or \link[fastcluster:hclust]{fastcluster::hclust}
}

\author{
The stats::hclust function is based on Fortran code contributed to STATLIB by F. Murtagh.
The fastcluster::hclust function implements hierarchical clustering with the same interface as hclust from the stats package but with much faster algorithms.

A. COULON

L. BELLANGER

P. HUSI
}

\examples{
data(datangkor)

#network stratigraphic data (Network)
network <- datangkor$stratigraphy

#contingency table
cont <- datangkor$contingency

#obtain the dissimilarities matrices
distance <- CAdist(cont, nPC = 11)
constraint <- adjacency(network)

D1 = distance
D2 = constraint

tree = hclust(d = list(D1, D2), method = "ward.D2", weight = c(0.7,0.3))
plot(tree, hang=-1)

}

\name{plot.cerardat_obj}
\alias{plot.cerardat_obj}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
plot cerardat model
}
\description{
plot cerardat model
}
\usage{
\method{plot}{cerardat_obj}(x, 
  which = NULL,
  main="model dateEv (red) and dateAc (black)",
  xlab = "Date", 
  ylab = "Density",
  col1 = rgb(0.93,0.23,0.23,0.5), 
  col2 = "black",
  xlim=NULL, 
  ylim=NULL,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
a \link[SPARTAAS]{cerardat} output.
}
  \item{which}{
  Vector containing the plots you want (rownames or row number on the contingence table: numeric).
}
  \item{main}{
  an overall title for the plot: see \link[graphics]{title}.
}
  \item{xlab}{
  a title for the x axis: see \link[graphics]{title}.
}
  \item{ylab}{
  a title for the y axis: see \link[graphics]{title}.
}
  \item{col1}{
  Color of the Event curve (color name, hex code, \link[grDevices]{rgb} function \{grDevices\} for transparency).
}
  \item{col2}{
  Color of the Accumulation curve (color name, hex code, \link[grDevices]{rgb} function \{grDevices\} for transparency).
}
  \item{xlim}{
  Two numeric values, specifying the left limit and the right limit of the scale.
}
  \item{ylim}{
  Two numeric values, specifying the lower limit and the upper limit of the scale.
}
  \item{...}{
  other parameters to be passed through to plotting functions (bty, cex.axis, las).
}
}


\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
\donttest{
data("datacerardat")

resultat = cerardat(df = datacerardat$df,
  row.sup = datacerardat$row.sup,
  date = datacerardat$date,
  nf = NULL,
  confidence = 0.95,
  graph = TRUE
)

#See the first plot
plot(resultat, which = 1)
plot(resultat, 
  which = 1, 
  xlim = c(200, 700),
  ylim = c(0, 0.03),
  col1 = rgb(0.7, 0.7, 0.7, 0.6), 
  col2 = "black", 
  las = 1
)
abline(v = resultat$prediction$date[1], col="grey60", lty=1)
abline(v = resultat$prediction$Fit_dateEv[1], col="grey60", lty=2)
abline(v = resultat$prediction$Median_dateAc[1], col="grey60", lty=3)
legend("topright", 
  legend=c("Date coin", "DateEv", "DateAc"),
  col="grey60",
  lty=c(1, 2, 3)
)

#See the first 10 plots
#plot(resultat, which = 1:10) 

#See all plots
#plot(resultat)
}
}

\name{arrondi}
\alias{arrondi}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Returns the rounded value.
}
\description{
Always returns the upper value if the next digit is 5.
}
\usage{
arrondi(x, acc)
}
\arguments{
  \item{x}{
The number to round.
}
  \item{acc}{
Accuracy (number of digits). A negative number of digits means rounding to the power of ten, e.g. arrondi(x, digits = -2) will round to the nearest hundred.}
}
\value{
  \item{res }{Value or vector of values rounded.}
}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
  x1 <- c(15,25,35,45,55)
  round(x1,-1)
  arrondi(x1,-1)

  x2 <- c(-15,-25,-35,-45,-55)
  round(x2,-1)
  arrondi(x2, -1)

  x3 <- 1.125
  round(x3,2)
  arrondi(x3, 2)

  x4 <- seq(-0.55,0.55,0.1)
  data.frame(
    val = x4,
    round = round(x4,1),
    arrondi = arrondi(x4, 1),
    equal = (arrondi(x4, 1) == round(x4,1))
  )

}
